@@ -25,7 +25,7 @@
  */
 #ifndef lint
 static char RCSid[] =
-      "@(#)$Header: /amnt/maui/tangram/u/william/X11/TGIF2/RCS/align.c,v 2.40 1994/04/04 00:49:06 william Exp $";
+      "@(#)$Header: /export/bourbon/multimedia/william/X11/TGIF2/RCS/align.c,v 2.41 1994/11/26 00:35:55 william Exp $";
 #endif
 
 #include <stdio.h>
@@ -830,8 +830,8 @@ void AlignSelObjs ()
             !(topVSel->obj->type==OBJ_POLYGON && topVSel->v_index[0]==0))) &&
             horiAlign!=ALIGN_N && horiAlign!=ALIGN_S &&
             vertAlign!=ALIGN_N && vertAlign!=ALIGN_S)
-         if (YesNoCancel ("Okay to move all vertices to one point? [ync](y)",
-               CONFIRM_YES) != CONFIRM_YES)
+         if (MsgBox ("Okay to move all vertices to one point? [ync](y)",
+               TOOL_NAME, YNC_MB) != MB_ID_YES)
             return;
 
       StartCompositeCmd ();
@@ -1100,8 +1100,8 @@ void AlignSelToPage ()
             !(topVSel->obj->type==OBJ_POLYGON && topVSel->v_index[0]==0))) &&
             horiAlign!=ALIGN_N && horiAlign!=ALIGN_S &&
             vertAlign!=ALIGN_N && vertAlign!=ALIGN_S)
-         if (YesNoCancel ("Okay to move all vertices to one point? [ync](y)",
-               CONFIRM_YES) != CONFIRM_YES)
+         if (MsgBox ("Okay to move all vertices to one point? [ync](y)",
+               TOOL_NAME, YNC_MB) != MB_ID_YES)
             return;
 
       StartCompositeCmd ();
