@@ -25,12 +25,15 @@
  */
 #ifndef lint
 static char RCSid[] =
-      "@(#)$Header: /amnt/maui/tangram/u/william/X11/TGIF2/RCS/arc.c,v 2.79 1994/05/02 00:53:02 william Exp $";
+      "@(#)$Header: /export/bourbon/multimedia/william/X11/TGIF2/RCS/arc.c,v 2.86 1994/11/30 06:13:15 william Exp $";
 #endif
 
 #include <math.h>
 #include <stdio.h>
+#include <sys/types.h>
 #include <X11/Xlib.h>
+#include <X11/Xutil.h>
+#include <X11/keysym.h>
 #include "const.h"
 #include "types.h"
 
@@ -419,14 +422,12 @@ void DumpArcArrow (FP,Angle,Dir,X,Y,XRad,YRad,AW,AH,Pen,ColorIndex)
       else
          fprintf (FP, "   pat%1d %s\n", Pen, patternStr);
    }
-
    if (!(colorDump && Pen==BACKPAT))
    {
       fprintf (FP, "   newpath\n");
       fprintf (FP, "      %1d %1d %1d %1d %1d %1d tgifarrowtip\n", X, Y, AW, AH,
             dx, dy);
    }
-
    if (colorDump)
    {
       switch (Pen)
@@ -826,15 +827,17 @@ struct ObjRec * CreateArcObj (xc,yc,x1,y1,x2,y2,dir,ltx,lty,w,h,angle1,angle2)
    obj_ptr->fattr = obj_ptr->lattr = NULL;
    return (obj_ptr);
 }
+
+static XComposeStatus	c_stat;
  
 static
 void ContinueArc (OrigX, OrigY)
    int	OrigX, OrigY;
 {
-   int 			grid_x, grid_y, first_x = 0, first_y = 0;
+   int 			grid_x, grid_y, first_x=0, first_y=0;
    int 			end_x, end_y, saved_x, saved_y, dx, dy, radius;
-   int 			done = FALSE, drawing_arc = FALSE;
-   int			dir = INVALID, ltx, lty, w, h, angle1, angle2=0;
+   int 			done=FALSE, drawing_arc=FALSE;
+   int			dir=INVALID, ltx, lty, w, h, angle1, angle2=0;
    char			msg[80], buf[80];
    struct ObjRec	* obj_ptr;
    XGCValues		values;
@@ -1018,6 +1021,39 @@ void ContinueArc (OrigX, OrigY)
          MarkRulers (grid_x, grid_y);
          while (XCheckMaskEvent (mainDisplay, PointerMotionMask, &ev)) ;
       }
+      else if (input.type == KeyPress)
+      {
+         KeySym	key_sym;
+         char	s[80];
+
+         XLookupString (&(input.xkey), s, 80-1, &key_sym, &c_stat);
+         TranslateKeys (s, &key_sym);
+         if (s[0] == '\033' && (key_sym & 0xff) == '\033')
+         {
+            XUngrabPointer (mainDisplay, CurrentTime);
+            if (drawing_arc)
+            {
+               XDrawArc (mainDisplay, drawWindow, drawGC, ltx, lty, w, h,
+                     angle1, angle2);
+               sprintf (buf, "r=%1d,degree=%1d", w>>1, abs(angle2>>6));
+            }
+            else
+            {
+               XDrawLine (mainDisplay, drawWindow, drawGC, OrigX, OrigY,
+                     saved_x, saved_y);
+               dx = saved_x - OrigX;
+               dy = saved_y - OrigY;
+               radius = (int)(sqrt((double)(dx*dx+dy*dy)));
+               w = h = (radius<<1);
+               sprintf (buf, "r=%1d,[%+1d,%+1d]", radius,
+                     ABS_X(saved_x), ABS_Y(saved_y));
+            }
+            EndShowMeasureCursor (saved_x, saved_y, buf, TRUE);
+            done = TRUE;
+            angle2 = 0;
+            Msg ("");
+         }
+      }
    }
    RestoreStatusStrings ();
    SetMouseStatus (NULL, NULL, NULL);
@@ -1081,7 +1117,7 @@ void ReadArcObj (Inbuf, ObjPtr)
    struct ObjRec	* * ObjPtr;
 {
    register struct ArcRec	* arc_ptr;
-   char				color_str[20], * s, msg[MAXSTRING];
+   char				color_str[40], * s, msg[MAXSTRING];
    int				fill, width=0, pen, dash, ltx, lty, w, h, id=0;
    int				rotation, new_alloc, style, locked=FALSE;
    int				aw=origArrowHeadW[0], ah=origArrowHeadH[0];
@@ -1090,7 +1126,7 @@ void ReadArcObj (Inbuf, ObjPtr)
    *ObjPtr = NULL;
 
    s = FindChar ('(', Inbuf);
-   s = ParseStr (s, ',', color_str);
+   s = ParseStr (s, ',', color_str, sizeof(color_str));
 
    InitScan (s, ", \t\n");
 
@@ -1297,7 +1333,7 @@ void ReadArcObj (Inbuf, ObjPtr)
    UpdArcBBox (*ObjPtr);
 
    (*ObjPtr)->type = OBJ_ARC;
-   (*ObjPtr)->color = QuickFindColorIndex (color_str, &new_alloc);
+   (*ObjPtr)->color = QuickFindColorIndex (color_str, &new_alloc, TRUE);
    (*ObjPtr)->dirty = FALSE;
    (*ObjPtr)->id = id;
    (*ObjPtr)->rotation = rotation;
@@ -1320,11 +1356,11 @@ int ParseArcSpec (spec, seperator, radius, dir, theta1, theta2, error_str)
 
    strcpy (tmp_buf, spec);
 
-   s = ParseStr (tmp_buf, seperator, buf);
+   s = ParseStr (tmp_buf, seperator, buf, sizeof(buf));
    if (*s != '\0' && (*radius = atoi (buf)) <= 0) *s = '\0';
    if (*s == '\0') { strcpy (error_str, "radius"); return (FALSE); }
 
-   s = ParseStr (s, seperator, buf);
+   s = ParseStr (s, seperator, buf, sizeof(buf));
    switch (*buf)
    {
       case '+': *dir = ARC_CW; break;
@@ -1333,7 +1369,7 @@ int ParseArcSpec (spec, seperator, radius, dir, theta1, theta2, error_str)
    }
    if (*s == '\0') { strcpy (error_str, "dir"); return (FALSE); }
 
-   s = ParseStr (s, seperator, buf);
+   s = ParseStr (s, seperator, buf, sizeof(buf));
    if (*s == '\0') { strcpy (error_str, "theta1"); return (FALSE); }
    *theta1 = atoi (buf);
    *theta2 = atoi (s);
