@@ -25,11 +25,12 @@
  */
 #ifndef lint
 static char RCSid[] =
-      "@(#)$Header: /amnt/maui/tangram/u/william/X11/TGIF2/RCS/attr.c,v 2.77 1994/04/04 00:49:06 william Exp $";
+      "@(#)$Header: /export/bourbon/multimedia/william/X11/TGIF2/RCS/attr.c,v 2.89 1994/12/12 00:11:40 william Exp $";
 #endif
 
 #include <stdio.h>
 #include <string.h>
+#include <sys/types.h>
 #include <X11/Xlib.h>
 #include <X11/Xutil.h>
 #include <X11/keysym.h>
@@ -59,6 +60,7 @@ static char RCSid[] =
 #include "msg.e"
 #include "names.e"
 #include "obj.e"
+#include "pattern.e"
 #include "raster.e"
 #include "rect.e"
 #include "ruler.e"
@@ -245,6 +247,87 @@ void DelAllAttrs (AttrPtr)
    }
 }
 
+struct AttrRec * AddAttrByNameAndValue (ObjPtr, AttrName, AttrValue)
+   struct ObjRec	* ObjPtr;
+   char			* AttrName, * AttrValue;
+{
+   struct ObjRec * text_obj_ptr;
+   struct TextRec * text_ptr;
+   struct StrRec * s_ptr;
+   struct AttrRec * attr_ptr;
+   char s[MAXSTRING+1];
+
+   sprintf (s, "%s%s", AttrName, AttrValue);
+
+   text_obj_ptr = (struct ObjRec *) calloc (1, sizeof(struct ObjRec));
+   text_ptr = (struct TextRec *) calloc (1, sizeof(struct TextRec));
+   attr_ptr = (struct AttrRec *) calloc (1, sizeof(struct AttrRec));
+   if (text_obj_ptr == NULL || text_ptr == NULL || attr_ptr == NULL)
+      fprintf (stderr, "Can not calloc().\n");
+
+   text_ptr->lines = 1;
+   text_ptr->cached_bitmap = None;
+   text_ptr->cached_zoom = 0;
+   text_ptr->cached_rotate = ROTATE0;
+   text_ptr->image = NULL;
+
+   text_ptr->font = curFont;
+   text_ptr->style = curStyle;
+   text_ptr->size = curSize;
+   text_ptr->just = textJust;
+   text_ptr->v_space = textVSpace;
+   text_ptr->rotate = ROTATE0;
+   text_ptr->pen = penPat;
+   text_ptr->fill = objFill;
+
+   text_ptr->asc = defaultFontAsc;
+   text_ptr->des = defaultFontDes;
+
+   text_ptr->read_only = FALSE;
+   text_ptr->orig_w = text_ptr->orig_h = 0;
+   text_ptr->font_name = NULL;
+
+   text_ptr->attr = NULL;
+
+   text_ptr->first = text_ptr->last = s_ptr = (struct StrRec *) calloc (1,
+         sizeof(struct StrRec));
+
+   strcpy (s_ptr->s, s);
+   s_ptr->next = s_ptr->prev = NULL;
+
+   text_obj_ptr->x = ObjPtr->obbox.ltx;
+   text_obj_ptr->y = ObjPtr->obbox.rby;
+   text_obj_ptr->type = OBJ_TEXT;
+   text_obj_ptr->color = ObjPtr->color;
+   text_obj_ptr->dirty = FALSE;
+   text_obj_ptr->id = objId++;
+   text_obj_ptr->rotation = 0;
+   text_obj_ptr->locked = FALSE;
+   text_obj_ptr->detail.t = text_ptr;
+
+   UpdTextBBox (text_obj_ptr);
+
+   attr_ptr->shown = FALSE;
+   attr_ptr->nameshown = TRUE;
+   attr_ptr->inherited = FALSE;
+   attr_ptr->obj = text_obj_ptr;
+   attr_ptr->owner = ObjPtr;
+   strcpy (attr_ptr->name, AttrName);
+   strcpy (attr_ptr->s, AttrValue);
+
+   text_ptr->attr = attr_ptr;
+
+   attr_ptr->prev = NULL;
+   attr_ptr->next = ObjPtr->fattr;
+   if (ObjPtr->fattr == NULL)
+      ObjPtr->lattr = attr_ptr;
+   else
+      ObjPtr->fattr->prev = attr_ptr;
+   ObjPtr->fattr = attr_ptr;
+   AdjObjBBox (ObjPtr);
+   return (attr_ptr);
+}
+
 static
 struct AttrRec	* NewAttr (OwnerObjPtr, ObjPtr, Inherited)
    struct ObjRec	* ObjPtr;
@@ -304,7 +387,7 @@ void DupAttrs (FromObjPtr, ToObjPtr)
 }
 
 static
-void AddAttr(ObjPtr, TextObjPtr)
+void AddAttr (ObjPtr, TextObjPtr)
    struct ObjRec	* ObjPtr, * TextObjPtr;
 {
    struct AttrRec	* attr_ptr;
@@ -1184,7 +1267,7 @@ int MoveOneAttr (ObjPtr, AttrPtr)
                   HighLightReverse ();
                   PrepareToReplaceAnObj (ObjPtr);
                   AttrPtr->shown = TRUE;
-                  MoveObj (text_obj_ptr, GRID_ABS_SIZE(dx), GRID_ABS_SIZE(dy));
+                  MoveObj (text_obj_ptr, ABS_SIZE(dx), ABS_SIZE(dy));
                   AdjObjBBox (ObjPtr);
                   RecordReplaceAnObj (ObjPtr);
                   return (TRUE);
@@ -1449,27 +1532,27 @@ void CopyAndUpdateAttrs (ToObjPtr, FromObjPtr)
       }
       if (to_attr_ptr == NULL)
       {  /* can not find the same attribute in the new one */
-         confirm_status = CONFIRM_NO;
+         confirm_status = MB_ID_NO;
 
          if (from_attr_ptr->inherited)
          {  /* obsolete attribute? */
-            confirm_status = (dropObsIconAttrWhenUpdate) ? CONFIRM_YES :
-                  CONFIRM_CANCEL;
+            confirm_status = (dropObsIconAttrWhenUpdate) ? MB_ID_YES :
+                  MB_ID_CANCEL;
             if (*from_attr_ptr->name == '\0')
                sprintf (msg, "Is the attribute '%s' %s for icon '%s'?  [yn](y)",
                      from_attr_ptr->s, "obsolete", FromObjPtr->detail.r->s);
             else
                sprintf (msg, "Is the attribute '%s' %s for icon '%s'?  [yn](y)",
                      from_attr_ptr->name, "obsolete", FromObjPtr->detail.r->s);
-            while (confirm_status == CONFIRM_CANCEL)
+            while (confirm_status == MB_ID_CANCEL)
             {
-               confirm_status = YesNoCancel (msg, CONFIRM_YES);
-               if (confirm_status == CONFIRM_CANCEL)
+               confirm_status = MsgBox (msg, TOOL_NAME, YN_MB);
+               if (confirm_status == MB_ID_CANCEL)
                   Msg ("CANCEL is not a valid option.");
             }
          }
 
-         if (confirm_status == CONFIRM_NO)
+         if (confirm_status == MB_ID_NO)
          {  /* new attribute */
             to_attr_ptr = (struct AttrRec *) calloc (1, sizeof(struct AttrRec));
             to_attr_ptr->owner = ToObjPtr;
@@ -1509,7 +1592,7 @@ void ToggleNamedAttrShown (attr_name)
    { Msg ("Invalid format in shortcut specification."); return; }
 
    *paran_ptr = '\0';
-   if ((attr_ptr = FindAttrWithName (topSel->obj, attr_name)) == NULL)
+   if ((attr_ptr = FindAttrWithName (topSel->obj, attr_name, NULL)) == NULL)
    {
       sprintf (msg, "Can not find a '%s' attribute for the selected object.",
             attr_name);
@@ -1755,6 +1838,8 @@ int EditAttrNames (TopStr, ObjAttrDspList)
 #else
    XSetWMNormalHints (mainDisplay, nameBaseWin, &sizehints);
 #endif
+   sprintf (buf, "%s - Edit Attribute Names", TOOL_NAME);
+   XStoreName (mainDisplay, nameBaseWin, buf);
 
    XSetTransientForHint (mainDisplay, nameBaseWin, mainWindow);
 
@@ -2020,9 +2105,9 @@ void EditFileAttrs ()
       return;
    }
    dsp_ptr = FileAttrNameListing (tgifObj);
-   ignoreDirectoryFlagInMakeNameDspItemArray = TRUE;
+   ignoreDirectoryFlag = TRUE;
    nameDspPtr = MakeNameDspItemArray (nameEntries, dsp_ptr);
-   ignoreDirectoryFlagInMakeNameDspItemArray = FALSE;
+   ignoreDirectoryFlag = FALSE;
    nameFirst = 0;
    nameMarked = 0;
    if (EditAttrNames ("Edit File Attributes...", NULL))
@@ -2063,13 +2148,19 @@ void EditFileAttrs ()
          RecordReplaceAnObj (topObj);
          recordCmdIncludeTgifObj = FALSE;
 
+         RemoveAllSel ();
          UnlinkObj (topObj);
 
          SetFileModified (TRUE);
          Msg ("File attribute updated.");
       }
       else
+      {
          AbortPrepareCmd (CMD_REPLACE);
+
+         RemoveAllSel ();
+         UnlinkObj (topObj);
+      }
    }
    cfree (dsp_ptr);
    cfree (*nameDspPtr);
@@ -2096,9 +2187,9 @@ void EditAttrs ()
       return;
    }
    dsp_ptr = FileAttrNameListing (topSel->obj);
-   ignoreDirectoryFlagInMakeNameDspItemArray = TRUE;
+   ignoreDirectoryFlag = TRUE;
    nameDspPtr = MakeNameDspItemArray (nameEntries, dsp_ptr);
-   ignoreDirectoryFlagInMakeNameDspItemArray = FALSE;
+   ignoreDirectoryFlag = FALSE;
    nameFirst = 0;
    nameMarked = 0;
    if (EditAttrNames ("Edit Object Attributes...", dsp_ptr))
