@@ -25,10 +25,11 @@
  */
 #ifndef lint
 static char RCSid[] =
-      "@(#)$Header: /amnt/maui/tangram/u/william/X11/TGIF2/RCS/auxtext.c,v 2.50 1994/03/21 04:53:29 william Exp $";
+      "@(#)$Header: /export/bourbon/multimedia/william/X11/TGIF2/RCS/auxtext.c,v 2.56 1994/12/12 06:32:45 william Exp $";
 #endif
 
 #include <stdio.h>
+#include <sys/types.h>
 #include <X11/Xlib.h>
 #include <X11/Xutil.h>
 #include <X11/keysym.h>
@@ -39,6 +40,8 @@ static char RCSid[] =
 #include "auxtext.e"
 #endif
 #include "color.e"
+#include "cursor.e"
+#include "dialog.e"
 #include "file.e"
 #include "font.e"
 #include "msg.e"
@@ -283,10 +286,10 @@ void MakeCachedTextBitmap (ObjPtr)
    struct ObjRec	* ObjPtr;
 {
    register int		r, c;
-   int			w=0, h=0, num_cols=0, num_rows=0;
-   int			yinc, y, rotate, max_str_w=0, str_w;
+   int			w=0, h=0, num_cols=0, num_rows=0, watch_cursor;
+   int			yinc, y, rotate, max_str_w=0, str_w, do_msg;
    int			flat=FALSE, skinny=FALSE, start_row, start_col;
-   char			* s, msg[MAXSTRING];
+   char			* s;
    struct MtrxRec	mtrx;
    Pixmap		dest_bitmap;
    XImage		* src_image, * dest_image;
@@ -363,13 +366,20 @@ void MakeCachedTextBitmap (ObjPtr)
       textBackingPixmap = XCreatePixmap (mainDisplay,mainWindow,tmp_w,tmp_h,1);
       if (textBackingPixmap == None)
       {
-         sprintf (msg,"Can not allocate pixmap of size %1dx%1d.",tmp_w,tmp_h);
-         Msg (msg);
+         sprintf(gszMsgBox, "Can not allocate pixmap of size %1dx%1d.",
+               tmp_w, tmp_h);
+         MsgBox(gszMsgBox, TOOL_NAME, INFO_MB);
          return;
       }
       textBackingPixmapW = tmp_w;
       textBackingPixmapH = tmp_h;
    }
+   watch_cursor = watchCursorOnMainWindow;
+   if (!watch_cursor)
+   {
+      SetWatchCursor (drawWindow);
+      SetWatchCursor (mainWindow);
+   }
 
    XSetForeground (mainDisplay, rotateGC, 0);
    XFillRectangle (mainDisplay,textBackingPixmap,rotateGC,0,0,w,h);
@@ -447,6 +457,13 @@ void MakeCachedTextBitmap (ObjPtr)
          num_rows = 1;
       }
    }
+   do_msg = ((num_rows*num_cols)>=0x4000);
+   if (do_msg) {
+      SaveStatusStrings();
+      SetStringStatus("Caching text bitmap...");
+      XSync(mainDisplay, False);
+   }
+
    dest_bitmap = XCreatePixmap (mainDisplay,mainWindow,num_cols,num_rows,1);
    XSetForeground (mainDisplay, rotateGC, 0);
    XFillRectangle (mainDisplay,dest_bitmap,rotateGC,0,0,num_cols,num_rows);
@@ -471,6 +488,13 @@ void MakeCachedTextBitmap (ObjPtr)
       {
          float	part_x, part_y;
 
+         if (do_msg && ((r & 0xf) == 0)) {
+            int percent=(r*10000/num_rows)/100;
+
+            sprintf(gszMsgBox, "Progress: %1d%%", percent);
+            SetStringStatus(gszMsgBox);
+            XSync(mainDisplay, False);
+         }
          part_x = (r+start_row)*mtrx.rev_m[1][0];
          part_y = (r+start_row)*mtrx.rev_m[1][1];
          for (c = 0; c < num_cols; c++)
@@ -484,14 +508,26 @@ void MakeCachedTextBitmap (ObjPtr)
          }
       }
    }
+   if (do_msg) {
+      SetStringStatus("Finishing caching text bitmap...");
+      XSync(mainDisplay, False);
+   }
    XPutImage (mainDisplay, dest_bitmap, rotateGC, dest_image, 0, 0, 0, 0,
          num_cols, num_rows);
+   if (do_msg) RestoreStatusStrings();
+
    text_ptr->cached_bitmap = dest_bitmap;
    text_ptr->cached_zoomed = zoomedIn;
    text_ptr->cached_zoom = zoomScale;
    text_ptr->cached_rotate = rotate;
    XDestroyImage (src_image);
    XDestroyImage (dest_image);
+
+   if (!watch_cursor)
+   {
+      SetDefaultCursor (mainWindow);
+      ShowCursor ();
+   }
 }
 
 static
@@ -535,6 +571,7 @@ struct ObjRec * SubFindTextObj (ObjPtr, XOff, YOff)
             if ((found_text_obj = SubFindTextObj (obj_ptr, XOff, YOff)) != NULL)
             {
                sel_ptr = (struct SelRec *) calloc (1, sizeof(struct SelRec));
+               if (sel_ptr == NULL) fprintf(stderr, "Can not calloc().\n");
                sel_ptr->obj = obj_ptr;
                sel_ptr->prev = NULL;
                sel_ptr->next = outerSel;
@@ -610,6 +647,7 @@ struct ObjRec * FindTextObj (XOff, YOff)
             if ((found_text_obj = SubFindTextObj (obj_ptr, XOff, YOff)) != NULL)
             {
                sel_ptr = (struct SelRec *) calloc (1, sizeof(struct SelRec));
+               if (sel_ptr == NULL) fprintf(stderr, "Can not calloc().\n");
                sel_ptr->obj = obj_ptr;
                sel_ptr->prev = NULL;
                sel_ptr->next = outerSel;
@@ -776,7 +814,7 @@ void DumpTextObj (FP, ObjPtr)
 {
    int			color_index, fill;
    int			ltx, lty, rbx, rby;
-   char			font_str[MAXSTRING], msg[MAXSTRING];
+   char			font_str[MAXSTRING];
    struct TextRec	* text_ptr = ObjPtr->detail.t;
 
    if (text_ptr->pen == NONEPAT && text_ptr->fill == NONEPAT) return;
@@ -898,9 +936,9 @@ void DumpTextObj (FP, ObjPtr)
                "May cause error!");
       else
       {
-         sprintf (msg, "Warning:  %s",
+         sprintf (gszMsgBox, "Warning:  %s",
                "Printing Courier fonts with non-solid pen in color.");
-         TwoLineMsg (msg, "    May cause error!");
+         TwoLineMsg (gszMsgBox, "    May cause error!");
       }
    }
 
@@ -1007,7 +1045,6 @@ char * ReadString (Str)
    char	*	Str;
 {
    register char	* s;
-   char			msg[MAXSTRING];
 
    for (s = Str; *s != '\0'; s++)
       if (*s == '"')
@@ -1033,9 +1070,10 @@ char * ReadString (Str)
                         s[1], s[2], s[3]);
                else
                {
-                  sprintf (msg, "Bad octal string \\%c%c%c encountered.\n",
+                  sprintf (gszMsgBox,
+                        "Bad octal string \\%c%c%c encountered.\n",
                         s[1], s[2], s[3]);
-                  Msg (msg);
+                  Msg (gszMsgBox);
                }
                strcpy (s, s+1);
             }
@@ -1048,6 +1086,98 @@ char * ReadString (Str)
    return (s);
 }
 
+struct ObjRec * FormTextObjectFromFile (FP, AbsX, AbsY)
+   FILE	* FP;
+   int	AbsX, AbsY;
+{
+   struct StrRec	* first_str, * last_str, *str_ptr;
+   struct ObjRec	* obj_ptr;
+   struct TextRec	* text_ptr;
+   char			buf[MAXSTRING+1];
+   int			num_lines=0, max_len=0, len, w;
+
+   text_ptr = (struct TextRec *) calloc (1, sizeof(struct TextRec));
+   if (text_ptr == NULL) fprintf(stderr, "Can not calloc().\n");
+   text_ptr->font = curFont;
+   text_ptr->style = curStyle;
+   text_ptr->attr = NULL;
+   text_ptr->size = curSize;
+   text_ptr->just = textJust;
+   text_ptr->v_space = textVSpace;
+   text_ptr->rotate = curRotate;
+   text_ptr->pen = penPat;
+   text_ptr->fill = objFill;
+   text_ptr->asc = canvasFontAsc;
+   text_ptr->des = canvasFontDes;
+
+   text_ptr->read_only = FALSE;
+   text_ptr->orig_w = text_ptr->orig_h = 0;
+   text_ptr->font_name = NULL;
+
+   first_str = last_str = NULL;
+   if (FP != NULL)
+   {
+      while (fgets (buf, MAXSTRING, FP) != NULL)
+      {
+         num_lines++;
+         str_ptr = (struct StrRec *) calloc (1, sizeof(struct StrRec));
+         if (str_ptr == NULL)
+         {
+            fprintf(stderr, "Can not calloc().\n");
+            break;
+         }
+         len = strlen (buf);
+         if (len > 0 && buf[len-1] == '\n') buf[--len] = '\0';
+         strcpy (str_ptr->s, buf);
+
+         str_ptr->prev = last_str;
+         str_ptr->next = NULL;
+         if (last_str == NULL)
+            first_str = str_ptr;
+         else
+            last_str->next = str_ptr;
+         last_str = str_ptr;
+
+         w = XTextWidth (canvasFontPtr, str_ptr->s, strlen (str_ptr->s));
+         if (w > max_len) max_len = w;
+      }
+   }
+   if (num_lines == 0)
+   {
+      num_lines++;
+      str_ptr = (struct StrRec *) calloc (1, sizeof(struct StrRec));
+      if (str_ptr == NULL) fprintf(stderr, "Can not calloc().\n");
+      *str_ptr->s = '\0';
+
+      str_ptr->prev = last_str;
+      str_ptr->next = NULL;
+      first_str = last_str = str_ptr;
+
+      w = XTextWidth (canvasFontPtr, str_ptr->s, strlen (str_ptr->s));
+      if (w > max_len) max_len = w;
+   }
+   text_ptr->lines = num_lines;
+   text_ptr->first = first_str;
+   text_ptr->last = last_str;
+
+   obj_ptr = (struct ObjRec *) calloc (1, sizeof(struct ObjRec));
+   if (obj_ptr == NULL) fprintf(stderr, "Can not calloc().\n");
+   obj_ptr->x = AbsX;
+   obj_ptr->y = AbsY;
+   obj_ptr->type = OBJ_TEXT;
+   obj_ptr->color = colorIndex;
+   obj_ptr->id = objId++;;
+   obj_ptr->dirty = FALSE;
+   obj_ptr->rotation = 0;
+   obj_ptr->detail.t = text_ptr;
+   obj_ptr->fattr = obj_ptr->lattr = NULL;
+
+   SetTextBBox (obj_ptr, textJust, max_len,
+         num_lines*textCursorH+(num_lines-1)*textVSpace, curRotate);
+   AdjObjBBox (obj_ptr);
+   return (obj_ptr);
+}
+
 static
 int PaintLeftText (Str, Just, Rotate, LtX, LtY)
    char	* Str;
