@@ -25,11 +25,12 @@
  */
 #ifndef lint
 static char RCSid[] =
-      "@(#)$Header: /amnt/maui/tangram/u/william/X11/TGIF2/RCS/button.c,v 2.45 1994/03/21 04:53:36 william Exp $";
+      "@(#)$Header: /export/bourbon/multimedia/william/X11/TGIF2/RCS/button.c,v 2.51 1994/11/30 06:13:15 william Exp $";
 #endif
 
 #include <stdio.h>
 #include <string.h>
+#include <sys/types.h>
 #include <X11/Xlib.h>
 #include <X11/Xutil.h>
 #include "const.h"
@@ -54,7 +55,7 @@ int ButtonWidth(Str, Len)
    char	* Str;
    int	Len;
 {
-   return (defaultFontWidth * max(strlen (Str) + 2, Len));
+   return (defaultFontWidth * max(((int) strlen (Str)) + 2, Len));
 }
 
 void DisplayButton (Win, Str, Len, BBox, Normal)
@@ -63,7 +64,7 @@ void DisplayButton (Win, Str, Len, BBox, Normal)
    int		Len;
    struct BBRec	* BBox;
    short	Normal;
-   /* display a button in Win at location ((*BBox).ltx,(*BBox).lty), Str is */
+   /* Display a button in Win at location ((*BBox).ltx,(*BBox).lty), Str is */
    /*    centered in the button.  The width of the button is the width of */
    /*    Str + 2 character widths or the width of Len number of characters, */
    /*    whichever is bigger.  *BBox will be set with the bounding */
@@ -91,10 +92,47 @@ void DisplayButton (Win, Str, Len, BBox, Normal)
             (*BBox).lty+defaultFontAsc+2, Str, len_of_str);
    }
    else
-   {
       XFillRectangle (mainDisplay, Win, revDefaultGC, (*BBox).ltx, (*BBox).lty,
             button_w, button_h);
+}
+
+void DisplayButtonInBBox (Win, Str, Len, BBox, Normal, HighLight, Width)
+   Window	Win;
+   char		* Str;
+   int		Len, HighLight, Width;
+   struct BBRec	* BBox;
+   short	Normal;
+   /* Display a button in Win at location ((*BBox).ltx,(*BBox).lty), Str is */
+   /*    centered in the button.  The width of the button is given in BBox. */
+   /* Len must be strlen(Str) */
+   /* If HighLight is TRUE, Width is used to draw an outline around the box. */
+{
+   int	button_w, button_h, left, top, text_w;
+
+   button_w = BBox->rbx - BBox->ltx;
+   button_h = BBox->rby - BBox->lty;
+   text_w = defaultFontWidth * Len;
+   left = ((button_w - text_w)>>1);
+   top = ((button_h - defaultFontHeight)>>1);
+
+   if (Normal)
+   {
+      XSetForeground (mainDisplay, defaultGC, myBgPixel);
+      XFillRectangle (mainDisplay, Win, defaultGC, BBox->ltx, BBox->lty,
+            button_w, button_h);
+      XSetForeground (mainDisplay, defaultGC, myFgPixel);
+
+      XDrawRectangle (mainDisplay, Win, defaultGC, BBox->ltx, BBox->lty,
+            button_w, button_h);
+      XDrawString (mainDisplay, Win, defaultGC, BBox->ltx+left,
+            BBox->lty+defaultFontAsc+2, Str, Len);
    }
+   else
+      XFillRectangle (mainDisplay, Win, revDefaultGC, BBox->ltx, BBox->lty,
+            button_w, button_h);
+   if (HighLight)
+      XDrawRectangle (mainDisplay, Win, defaultGC, BBox->ltx-Width,
+            BBox->lty-Width, button_w+(Width<<1), button_h+(Width<<1));
 }
 
 static char * confirmStr[] = { "YES", "NO", "CANCEL" };
@@ -175,6 +213,8 @@ int YesNoCancel (Str, DefaultReturn)
 #else
    XSetWMNormalHints (mainDisplay, confirm_win, &sizehints);
 #endif
+   sprintf (buf, "%s - Confirmation", TOOL_NAME);
+   XStoreName (mainDisplay, confirm_win, buf);
 
    XSetTransientForHint (mainDisplay, confirm_win, mainWindow);
 #ifdef MAPBEFORESELECT
@@ -301,3 +341,4 @@ int YesNoCancel (Str, DefaultReturn)
             (int)(ZOOMED_SIZE(drawWinW)>>1), (int)(ZOOMED_SIZE(drawWinH)>>1));
    return (choice);
 }
+
