@@ -25,7 +25,7 @@
  */
 #ifndef lint
 static char RCSid[] =
-      "@(#)$Header: /amnt/maui/tangram/u/william/X11/TGIF2/RCS/choice.c,v 2.70 1994/04/23 05:01:36 william Exp $";
+      "@(#)$Header: /export/bourbon/multimedia/william/X11/TGIF2/RCS/choice.c,v 2.78 1995/01/05 06:29:20 william Exp $";
 #endif
 
 #include <stdio.h>
@@ -41,6 +41,8 @@ static char RCSid[] =
 #endif
 #include "color.e"
 #include "cursor.e"
+#include "dialog.e"
+#include "drawing.e"
 #include "edit.e"
 #include "file.e"
 #include "font.e"
@@ -49,6 +51,7 @@ static char RCSid[] =
 #include "mainmenu.e"
 #include "mark.e"
 #include "msg.e"
+#include "navigate.e"
 #include "oval.e"
 #include "page.e"
 #include "pattern.e"
@@ -167,6 +170,16 @@ void CleanUpChoices ()
    XFreePixmap (mainDisplay, choiceBackingPixmap);
 }
 
+void ShowWhereToPrint ()
+{
+   if (colorDump)
+      XSetForeground (mainDisplay, rasterGC, colorPixels[colorIndex]);
+   XSetStipple (mainDisplay, rasterGC, whereToPrintPixmap[whereToPrint]);
+   XFillRectangle (mainDisplay, choiceWindow, rasterGC, PRINT_COL*choiceImageW,
+         PRINT_ROW*choiceImageH, choiceImageW, choiceImageH);
+   if (colorDump) XSetForeground (mainDisplay, rasterGC, myFgPixel);
+}
+
 void ShowMode ()
 {
    XSetStipple (mainDisplay, rasterGC, choicePixmap[curChoice]);
@@ -410,16 +423,6 @@ void ShowDash ()
          DASH_ROW*choiceImageH, choiceImageW, choiceImageH);
 }
 
-void ShowWhereToPrint ()
-{
-   if (colorDump)
-      XSetForeground (mainDisplay, rasterGC, colorPixels[colorIndex]);
-   XSetStipple (mainDisplay, rasterGC, whereToPrintPixmap[whereToPrint]);
-   XFillRectangle (mainDisplay, choiceWindow, rasterGC, PRINT_COL*choiceImageW,
-         PRINT_ROW*choiceImageH, choiceImageW, choiceImageH);
-   if (colorDump) XSetForeground (mainDisplay, rasterGC, myFgPixel);
-}
-
 void ShowFile ()
 {
    switch (pageStyle)
@@ -649,8 +652,14 @@ void SetCurChoice (NewChoice)
    ShowMode ();
    UpdateSubMenu (MENU_MODE);
 
-   SetMouseStatus (choiceMouseStatus[curChoice].l,
-         choiceMouseStatus[curChoice].m, choiceMouseStatus[curChoice].r);
+   if (curChoice == NOTHING && inHyperSpace)
+      SetMouseStatus ("(none)",
+            choiceMouseStatus[curChoice].m, choiceMouseStatus[curChoice].r);
+   else
+      SetMouseStatus (choiceMouseStatus[curChoice].l,
+            choiceMouseStatus[curChoice].m, choiceMouseStatus[curChoice].r);
+
+   if (inHyperSpace && curChoice != NOTHING) ToggleHyperSpace ();
 }
 
 static int	prevChoice = NOTHING;
@@ -1149,7 +1158,7 @@ int ChoiceEventHandler (input)
                   if (curChoice != NOTHING) SetCurChoice (NOTHING);
                   colorIndex = (colorIndex+maxColors+delta) % maxColors;
                   ShowColor (TRUE);
-                  if (topSel != NULL) ChangeAllSelColor (colorIndex);
+                  if (topSel != NULL) ChangeAllSelColor (colorIndex, TRUE);
                }
             }
             SetMouseStatus (colorMouseStatus[0].l,
@@ -1383,7 +1392,7 @@ int ChoiceEventHandler (input)
                lineWidth = (lineWidth+maxLineWidths+delta) % maxLineWidths;
                ShowLineWidth ();
                UpdateSubMenu (MENU_LINEWIDTH);
-               if (topSel != NULL) ChangeAllSelLineWidth (lineWidth);
+               if (topSel != NULL) ChangeAllSelLineWidth (lineWidth, TRUE);
             }
             SetMouseStatus (lineWidthMouseStatus[0].l,
                   lineWidthMouseStatus[0].m, lineWidthMouseStatus[0].r);
@@ -1396,7 +1405,7 @@ int ChoiceEventHandler (input)
                lineStyle = (lineStyle+MAXLINESTYLES+delta) % MAXLINESTYLES;
                ShowLineStyle ();
                UpdateSubMenu (MENU_LINESTYLE);
-               if (topSel != NULL) ChangeAllSelLineStyle (lineStyle);
+               if (topSel != NULL) ChangeAllSelLineStyle (lineStyle, TRUE);
             }
             SetMouseStatus (lineStyleMouseStatus[0].l,
                   lineStyleMouseStatus[0].m, lineStyleMouseStatus[0].r);
@@ -1409,7 +1418,7 @@ int ChoiceEventHandler (input)
                curSpline = (curSpline+MAXLINETYPES+delta) % MAXLINETYPES;
                ShowLineType ();
                UpdateSubMenu (MENU_LINETYPE);
-               if (topSel != NULL) ChangeAllSelLineType (curSpline);
+               if (topSel != NULL) ChangeAllSelLineType (curSpline, TRUE);
             }
             SetMouseStatus (lineTypeMouseStatus[0].l,
                   lineTypeMouseStatus[0].m, lineTypeMouseStatus[0].r);
@@ -1422,7 +1431,7 @@ int ChoiceEventHandler (input)
                curDash = (curDash+MAXDASHES+delta) % MAXDASHES;
                ShowDash ();
                UpdateSubMenu (MENU_LINEDASH);
-               if (topSel != NULL) ChangeAllSelDashes (curDash);
+               if (topSel != NULL) ChangeAllSelDashes (curDash, TRUE);
             }
             SetMouseStatus (lineDashMouseStatus[0].l,
                   lineDashMouseStatus[0].m, lineDashMouseStatus[0].r);
@@ -1513,7 +1522,7 @@ int ChoiceEventHandler (input)
                }
                objFill = (objFill+MAXPATTERNS+delta) % MAXPATTERNS;
                ShowFill ();
-               if (topSel != NULL) ChangeAllSelFill (objFill);
+               if (topSel != NULL) ChangeAllSelFill (objFill, TRUE);
             }
             SetMouseStatus (fillMouseStatus[0].l,
                   fillMouseStatus[0].m, fillMouseStatus[0].r);
@@ -1536,7 +1545,7 @@ int ChoiceEventHandler (input)
                }
                penPat = (penPat+MAXPATTERNS+delta) % MAXPATTERNS;
                ShowPen ();
-               if (topSel != NULL) ChangeAllSelPen (penPat);
+               if (topSel != NULL) ChangeAllSelPen (penPat, TRUE);
             }
             SetMouseStatus (penMouseStatus[0].l,
                   penMouseStatus[0].m, penMouseStatus[0].r);
