@@ -25,7 +25,7 @@
  */
 #ifndef lint
 static char RCSid[] =
-      "@(#)$Header: /amnt/maui/tangram/u/william/X11/TGIF2/RCS/cmd.c,v 2.26 1994/03/21 04:53:44 william Exp $";
+      "@(#)$Header: /export/bourbon/multimedia/william/X11/TGIF2/RCS/cmd.c,v 2.29 1994/11/26 02:30:16 william Exp $";
 #endif
 
 #include <stdio.h>
@@ -51,7 +51,7 @@ static char RCSid[] =
 #include "stk.e"
 
 int		recordCmdIncludeTgifObj=FALSE;
-int		historyDepth=(-1), historyCount=0;
+int		historyDepth=(-1), historyCount=0, defaultHistoryDepth=(-1);
 struct CmdRec	* firstCmd=NULL, * lastCmd=NULL, * curCmd=NULL;
 
 static
@@ -74,91 +74,103 @@ void InsertCmd (PrevCmd, NextCmd, CmdPtr)
 }
 
 static
-void DeleteARedoRecord (CmdPtr)
-   struct CmdRec	* CmdPtr;
+void DeleteARedoRecord(CmdPtr, percent_start, percent_end)
+   struct CmdRec *CmdPtr;
+   float percent_start, percent_end;
 {
-   register struct SelRec	* sel_ptr, * next_sel;
-   struct CmdRec		* cmd_ptr, * next_cmd;
+   register struct SelRec *sel_ptr, *next_sel;
+   struct CmdRec *cmd_ptr, *next_cmd;
+   int num_records=0;
+   float inc=0;
 
-   if (CmdPtr->pos_before != NULL) cfree (CmdPtr->pos_before);
-   if (CmdPtr->pos_after != NULL) cfree (CmdPtr->pos_after);
+   if (CmdPtr->pos_before != NULL) cfree(CmdPtr->pos_before);
+   if (CmdPtr->pos_after != NULL) cfree(CmdPtr->pos_after);
 
-   switch (CmdPtr->type)
-   {
-      case CMD_NEW:
-         for (sel_ptr=CmdPtr->top_after; sel_ptr!=NULL; sel_ptr=next_sel)
-         {
-            next_sel = sel_ptr->next;
-            if (CmdPtr->undone) FreeObj (sel_ptr->obj);
-            cfree (sel_ptr);
+   if (percent_start >= 0.0) {
+      sprintf(gszMsgBox, "Flushing undo buffers... %1d%%",
+            round(percent_start));
+      SetStringStatus(gszMsgBox);
+      CheckInterrupt();
+   }
+   switch (CmdPtr->type) {
+   case CMD_NEW:
+      for (sel_ptr=CmdPtr->top_after; sel_ptr!=NULL; sel_ptr=next_sel) {
+         next_sel = sel_ptr->next;
+         if (CmdPtr->undone) FreeObj(sel_ptr->obj);
+         cfree(sel_ptr);
+      }
+      break;
+   case CMD_DELETE:
+      for (sel_ptr=CmdPtr->top_before; sel_ptr!=NULL; sel_ptr=next_sel) {
+         next_sel = sel_ptr->next;
+         if (!CmdPtr->undone) FreeObj(sel_ptr->obj);
+         cfree(sel_ptr);
+      }
+      break;
+   case CMD_MOVE:
+      for (sel_ptr=CmdPtr->top_after; sel_ptr!=NULL; sel_ptr=next_sel) {
+         next_sel = sel_ptr->next;
+         cfree(sel_ptr);
+      }
+      break;
+   case CMD_STRETCH:
+      for (sel_ptr=CmdPtr->top_before; sel_ptr!=NULL; sel_ptr=next_sel) {
+         next_sel = sel_ptr->next;
+         FreeObj(sel_ptr->obj);
+         cfree(sel_ptr);
+      }
+      break;
+   case CMD_REPLACE:
+      for (sel_ptr=CmdPtr->top_before; sel_ptr!=NULL; sel_ptr=next_sel) {
+         next_sel = sel_ptr->next;
+         FreeObj(sel_ptr->obj);
+         cfree(sel_ptr);
+      }
+      break;
+   case CMD_ONE_TO_MANY:
+      for (sel_ptr=CmdPtr->top_before; sel_ptr!=NULL; sel_ptr=next_sel) {
+         next_sel = sel_ptr->next;
+         if (!CmdPtr->undone) FreeObj(sel_ptr->obj);
+         cfree(sel_ptr);
+      }
+      for (sel_ptr=CmdPtr->top_after; sel_ptr!=NULL; sel_ptr=next_sel) {
+         next_sel = sel_ptr->next;
+         if (CmdPtr->undone) FreeObj(sel_ptr->obj);
+         cfree(sel_ptr);
+      }
+      break;
+   case CMD_MANY_TO_ONE:
+      for (sel_ptr=CmdPtr->top_before; sel_ptr!=NULL; sel_ptr=next_sel) {
+         next_sel = sel_ptr->next;
+         if (!CmdPtr->undone) FreeObj(sel_ptr->obj);
+         cfree(sel_ptr);
+      }
+      for (sel_ptr=CmdPtr->top_after; sel_ptr!=NULL; sel_ptr=next_sel) {
+         next_sel = sel_ptr->next;
+         if (CmdPtr->undone) FreeObj(sel_ptr->obj);
+         cfree(sel_ptr);
+      }
+      break;
+   case CMD_COMPOSITE:
+      if (percent_start >= 0) {
+         for (cmd_ptr=CmdPtr->first; cmd_ptr!=NULL; cmd_ptr=cmd_ptr->next) {
+            num_records++;
          }
-         break;
-      case CMD_DELETE:
-         for (sel_ptr=CmdPtr->top_before; sel_ptr!=NULL; sel_ptr=next_sel)
-         {
-            next_sel = sel_ptr->next;
-            if (!CmdPtr->undone) FreeObj (sel_ptr->obj);
-            cfree (sel_ptr);
+         if (num_records > 0) {
+            inc = (percent_end-percent_start)/((float)num_records);
          }
-         break;
-      case CMD_MOVE:
-         for (sel_ptr=CmdPtr->top_after; sel_ptr!=NULL; sel_ptr=next_sel)
-         {
-            next_sel = sel_ptr->next;
-            cfree (sel_ptr);
+      }
+      for (cmd_ptr=CmdPtr->first; cmd_ptr!=NULL; cmd_ptr=next_cmd) {
+         next_cmd = cmd_ptr->next;
+         if (percent_start >= 0) {
+            DeleteARedoRecord(cmd_ptr, percent_start,
+                  min(percent_start+inc,100.0));
+            percent_start += inc;
+         } else {
+            DeleteARedoRecord(cmd_ptr, percent_start, percent_end);
          }
-         break;
-      case CMD_STRETCH:
-         for (sel_ptr=CmdPtr->top_before; sel_ptr!=NULL; sel_ptr=next_sel)
-         {
-            next_sel = sel_ptr->next;
-            FreeObj (sel_ptr->obj);
-            cfree (sel_ptr);
-         }
-         break;
-      case CMD_REPLACE:
-         for (sel_ptr=CmdPtr->top_before; sel_ptr!=NULL; sel_ptr=next_sel)
-         {
-            next_sel = sel_ptr->next;
-            FreeObj (sel_ptr->obj);
-            cfree (sel_ptr);
-         }
-         break;
-      case CMD_ONE_TO_MANY:
-         for (sel_ptr=CmdPtr->top_before; sel_ptr!=NULL; sel_ptr=next_sel)
-         {
-            next_sel = sel_ptr->next;
-            if (!CmdPtr->undone) FreeObj (sel_ptr->obj);
-            cfree (sel_ptr);
-         }
-         for (sel_ptr=CmdPtr->top_after; sel_ptr!=NULL; sel_ptr=next_sel)
-         {
-            next_sel = sel_ptr->next;
-            if (CmdPtr->undone) FreeObj (sel_ptr->obj);
-            cfree (sel_ptr);
-         }
-         break;
-      case CMD_MANY_TO_ONE:
-         for (sel_ptr=CmdPtr->top_before; sel_ptr!=NULL; sel_ptr=next_sel)
-         {
-            next_sel = sel_ptr->next;
-            if (!CmdPtr->undone) FreeObj (sel_ptr->obj);
-            cfree (sel_ptr);
-         }
-         for (sel_ptr=CmdPtr->top_after; sel_ptr!=NULL; sel_ptr=next_sel)
-         {
-            next_sel = sel_ptr->next;
-            if (CmdPtr->undone) FreeObj (sel_ptr->obj);
-            cfree (sel_ptr);
-         }
-         break;
-      case CMD_COMPOSITE:
-         for (cmd_ptr=CmdPtr->first; cmd_ptr!=NULL; cmd_ptr=next_cmd)
-         {
-            next_cmd = cmd_ptr->next;
-            DeleteARedoRecord (cmd_ptr);
-         }
-         break;
+      }
+      break;
    }
    cfree (CmdPtr);
 }
@@ -177,7 +189,7 @@ void ClearRedoRecords (CmdPtr)
    for ( ; cmd_ptr != NULL; cmd_ptr = next_cmd)
    {
       next_cmd = cmd_ptr->next;
-      DeleteARedoRecord (cmd_ptr);
+      DeleteARedoRecord (cmd_ptr, (-1.0), (-1.0));
       historyCount--;
    }
    if ((lastCmd = curCmd) == NULL) firstCmd = NULL;
@@ -185,17 +197,49 @@ void ClearRedoRecords (CmdPtr)
 
 void CleanUpCmds ()
 {
-   register struct CmdRec	* cmd_ptr, * next_cmd;
+   register struct CmdRec *cmd_ptr, *next_cmd;
+   int num_records=0;
 
-   for (cmd_ptr = firstCmd; cmd_ptr != NULL; cmd_ptr = next_cmd)
-   {
-      next_cmd = cmd_ptr->next;
-      DeleteARedoRecord (cmd_ptr);
+   for (cmd_ptr=firstCmd; cmd_ptr != NULL; cmd_ptr=cmd_ptr->next) {
+      num_records++;
+   }
+   if (num_records > 0) {
+      float inc=(100.0/((float)num_records)), percent_start=0.0;
+
+      ShowInterrupt(1);
+      SaveStatusStrings();
+      for (cmd_ptr = firstCmd; cmd_ptr != NULL; cmd_ptr=next_cmd,
+            percent_start += inc) {
+         next_cmd = cmd_ptr->next;
+         DeleteARedoRecord(cmd_ptr, percent_start,
+               min(percent_start+inc,100.0));
+      }
+      RestoreStatusStrings();
+      HideInterrupt();
    }
    firstCmd = lastCmd = curCmd = NULL;
    historyCount = 0;
 }
 
+void RestoreDefaultHistoryDepth()
+{
+   CleanUpCmds();
+   historyDepth = defaultHistoryDepth;
+   historyCount = 0;
+   firstCmd = lastCmd = curCmd = NULL;
+}
+
+void DisableUndo()
+{
+   CleanUpCmds();
+   historyDepth = 0;
+}
+
+void EnableUndo()
+{
+   RestoreDefaultHistoryDepth();
+}
+
 static
 void CopySel (from_top_sel, count, to_top_sel, to_bot_sel)
    struct SelRec	* from_top_sel;
@@ -837,7 +881,7 @@ void EndCompositeCmd ()
 
          new_first_cmd->prev = NULL;
          firstCmd->next = NULL;
-         DeleteARedoRecord (firstCmd);
+         DeleteARedoRecord (firstCmd, (-1.0), (-1.0));
          historyCount--;
          firstCmd = new_first_cmd;
       }
@@ -974,7 +1018,7 @@ void RecordCmd (CmdType, SubCmdPtr, TopSel, BotSel, NumObjs)
 
       new_first_cmd->prev = NULL;
       firstCmd->next = NULL;
-      DeleteARedoRecord (firstCmd);
+      DeleteARedoRecord (firstCmd, (-1.0), (-1.0));
       historyCount--;
       firstCmd = new_first_cmd;
    }
