@@ -25,10 +25,12 @@
  */
 #ifndef lint
 static char RCSid[] =
-      "@(#)$Header: /amnt/maui/tangram/u/william/X11/TGIF2/RCS/color.c,v 2.68 1994/05/01 23:22:57 william Exp $";
+      "@(#)$Header: /export/bourbon/multimedia/william/X11/TGIF2/RCS/color.c,v 2.83 1995/01/02 01:40:43 william Exp $";
 #endif
 
 #include <stdio.h>
+#include <string.h>
+#include <sys/types.h>
 #include <X11/Xlib.h>
 #include <X11/Xutil.h>
 #include "const.h"
@@ -39,6 +41,7 @@ static char RCSid[] =
 #ifndef _NO_EXTERN
 #include "color.e"
 #endif
+#include "dialog.e"
 #include "drawing.e"
 #include "mainloop.e"
 #include "mark.e"
@@ -50,6 +53,7 @@ static char RCSid[] =
 #include "select.e"
 #include "setup.e"
 #include "text.e"
+#include "util.e"
 #include "xpixmap.e"
 
 extern int	atoi ARGS_DECL((char *));
@@ -72,6 +76,10 @@ XColor	* tgifRequestedColors=NULL;
 XColor	myBgColor;
 int	maxRGB = 0;
 int	colorDump = FALSE;
+int	useLocalRGBTxt=FALSE;
+int	printUsingRequestedColor=FALSE;
+
+static int	canChangeAttrColor=FALSE;
 
 static int	defaultColorIndexInXDefaults = FALSE;
 
@@ -110,6 +118,36 @@ void DefaultColorArrays (Entries, ForePixels, Valid, InitRV, StatusStr)
    }
 }
 
+struct LocalColorRec {
+   char *name, *spec;
+   int len;
+};
+
+static struct LocalColorRec *privateColorInfo=NULL;
+
+int TgifParseColor(psz_color, p_color)
+   char *psz_color;
+   XColor *p_color;
+{
+   struct LocalColorRec *ptr;
+   int len;
+
+   if (!useLocalRGBTxt || *psz_color == '#') {
+      return (int)XParseColor(mainDisplay, mainColormap, psz_color, p_color);
+   }
+   len = strlen(psz_color);
+   for (ptr=privateColorInfo; ptr->name != NULL; ptr++) {
+      if (len == ptr->len && strcmp(psz_color, ptr->name) == 0) {
+         if (ptr->spec != NULL) {
+            return (int)XParseColor(mainDisplay, mainColormap, ptr->spec,
+                  p_color);
+         }
+         break;
+      }
+   }
+   return (int)XParseColor(mainDisplay, mainColormap, psz_color, p_color);
+}
+
 static
 int ParseAndAllocColorByName (colorname, color, red_req, green_req, blue_req)
    char			* colorname;
@@ -118,7 +156,7 @@ int ParseAndAllocColorByName (colorname, color, red_req, green_req, blue_req)
 {
    Colormap	colormap;
 
-   if (!XParseColor(mainDisplay, mainColormap, colorname, color))
+   if (!TgifParseColor(colorname, color))
    {
       fprintf (stderr, "Warning:  can not parse color '%s'\n", colorname);
       return (FALSE);
@@ -168,10 +206,92 @@ int MyStrCmp (s1, s2)
    return (0);
 }
 
+static
+void InitLocalRGBTxt()
+{
+   char *c_ptr;
+
+   useLocalRGBTxt = FALSE;
+   if ((c_ptr=XGetDefault(mainDisplay,TOOL_NAME,"LocalRGBTxt")) != NULL) {
+      char *fname=UtilStrDup(c_ptr);
+      FILE *fp;
+
+      if (fname == NULL) { FailAllocMessage(); return; }
+      if ((fp=fopen(fname, "r")) != NULL) {
+         int num_lines=0, ok=TRUE, line_num=0, info_index=0;
+         char *buf;
+
+         while (fgets(gszMsgBox, sizeof(gszMsgBox), fp) != NULL) {
+            int len=strlen(gszMsgBox);
+
+            if (len > 0 && gszMsgBox[len-1] != '\n') {
+               while (fgets(gszMsgBox, sizeof(gszMsgBox), fp) != NULL) {
+                  len = strlen(gszMsgBox);
+                  if (len > 0 && gszMsgBox[len-1] == '\n') break;
+               }
+            }
+            num_lines++;
+         }
+         rewind(fp);
+         if ((privateColorInfo=(struct LocalColorRec *)calloc(num_lines+1,
+               sizeof(struct LocalColorRec))) == NULL) {
+            FailAllocMessage();
+            ok = FALSE;
+         }
+         while ((buf=UtilGetALine(fp)) != NULL) {
+            char *red_str, *green_str, *blue_str, *color_name;
+
+            line_num++;
+            if ((red_str=strtok(buf, " \t\n\r")) != NULL &&
+                  (green_str=strtok(NULL, " \t\n\r")) != NULL &&
+                  (blue_str=strtok(NULL, " \t\n\r")) != NULL &&
+                  (color_name=strtok(NULL, "\t\n\r")) != NULL) {
+               struct LocalColorRec *color_info_ptr;
+               int red, green, blue;
+
+               color_info_ptr = (&privateColorInfo[info_index]);
+               while (*color_name == ' ' || *color_name == '\t') color_name++;
+               if (sscanf(red_str, "%d", &red) == 1 &&
+                     sscanf(green_str, "%d", &green) == 1 &&
+                     sscanf(blue_str, "%d", &blue) == 1 &&
+                     red >= 0 && red <= 0xff && green >= 0 && green <= 0xff &&
+                     blue >= 0 && blue <= 0xff && *color_name != '\0') {
+                  color_info_ptr->name = UtilStrDup(color_name);
+                  color_info_ptr->len = strlen(color_info_ptr->name);
+                  sprintf(gszMsgBox, "#%02x%02x%02x", red, green, blue);
+                  color_info_ptr->spec = UtilStrDup(gszMsgBox);
+                  info_index++;
+               } else {
+                  fprintf(stderr, "Malformed line %1d in '%s'.\n", line_num,
+                        fname);
+               }
+            } else {
+               fprintf(stderr, "Malformed line %1d in '%s'.\n", line_num,
+                     fname);
+            }
+            privateColorInfo[info_index].name = NULL;
+            privateColorInfo[info_index].len = 0;
+            cfree(buf);
+         }
+         fclose(fp);
+         if (ok) useLocalRGBTxt = TRUE;
+      } else {
+         fprintf(stderr, "Invalid file '%s' specified by X default %s*%s.\n",
+               fname, TOOL_NAME, "LocalRGBTxt");
+      }
+      cfree(fname);
+   }
+   printUsingRequestedColor = FALSE;
+   if ((c_ptr=XGetDefault(mainDisplay,TOOL_NAME,"PrintUsingRequestedColor")) !=
+         NULL && (strcmp(c_ptr, "true") == 0 || strcmp(c_ptr, "True") == 0)) {
+      printUsingRequestedColor = TRUE;
+   }
+}
+
 void InitColor ()
 {
    register int	i, index;
-   XColor	color, exact_def;
+   XColor	color;
    char		buf[80], * c_ptr, fg_color[80], bg_color[80], brdr_color[80];
    long		bg_gray=(long)0;
    int		looking_for_default_color;
@@ -179,6 +299,11 @@ void InitColor ()
    int		bg_allocated=FALSE, fg_allocated=FALSE, brdr_allocated=FALSE;
    int		tmp_max;
 
+   InitLocalRGBTxt();
+
+   strcpy (fg_color, "Black");
+   strcpy (bg_color, "White");
+   strcpy (brdr_color, "Black");
    if (cmdLineRV != INVALID)
       reverseVideo = cmdLineRV;
    else if ((c_ptr = XGetDefault(mainDisplay,TOOL_NAME,"ReverseVideo")) != NULL)
@@ -324,10 +449,6 @@ void InitColor ()
             exit (-1);
          }
 
-         exact_def.red = color.red;
-         exact_def.green = color.green;
-         exact_def.blue = color.blue;
-
          if (i != index) strcpy (colorMenuItems[index], colorMenuItems[i]);
 
          colorPixels[index] = color.pixel;
@@ -476,7 +597,7 @@ void InitColor ()
       colorIndex = defaultColorIndex;
    }
    else
-   {
+   {  /* !colorDisplay */
       if (!ParseAndAllocColorByName (fg_color, &color, NULL, NULL, NULL))
       {
          fprintf (stderr, "Fail to allocate the '%s' color!  Abort!\n",
@@ -505,7 +626,29 @@ void InitColor ()
 
       for (i = 0; i < maxColors; i++)
       {
-         colorPixels[i] = myFgPixel;
+         unsigned short	red_req, green_req, blue_req;
+
+         if (!ParseAndAllocColorByName (colorMenuItems[i], &color,
+               &red_req, &green_req, &blue_req))
+         {
+            fprintf (stderr, "%s %1d:  '%s'!  %s aborted!\n",
+                  "Fail to allocate color number", i, colorMenuItems[i],
+                  TOOL_NAME);
+            exit (-1);
+         }
+
+         tgifColors[i].red = color.red;
+         tgifColors[i].green = color.green;
+         tgifColors[i].blue = color.blue;
+
+         tgifRequestedColors[i].red = red_req;
+         tgifRequestedColors[i].green = green_req;
+         tgifRequestedColors[i].blue = blue_req;
+
+         if (MyStrICmp (colorMenuItems[i], bg_color) == 0)
+            colorPixels[i] = myBgPixel;
+         else
+            colorPixels[i] = myFgPixel;
          xorColorPixels[i] = myFgPixel ^ myBgPixel;
       }
       colorIndex = defaultColorIndex;
@@ -515,7 +658,7 @@ void InitColor ()
       fprintf (stderr, "Fail to allocate the 'black' color!  Abort!\n");
       exit (-1);
    }
-   tmp_max = max(color.red,max(color.green,color.blue));
+   tmp_max = max(((int)color.red),max(((int)color.green),((int)color.blue)));
 #ifndef DONTFREECOLORS
    XFreeColors (mainDisplay, mainColormap, &(color.pixel), 1, 0);
 #endif
@@ -525,7 +668,7 @@ void InitColor ()
       fprintf (stderr, "Fail to allocate the 'white' color!  Abort!\n");
       exit (-1);
    }
-   tmp_max = max(color.red,max(color.green,color.blue));
+   tmp_max = max(((int)color.red),max(((int)color.green),((int)color.blue)));
 #ifndef DONTFREECOLORS
    XFreeColors (mainDisplay, mainColormap, &(color.pixel), 1, 0);
 #endif
@@ -540,6 +683,15 @@ void InitColor ()
       xorOne = myFgPixel^myBgPixel;
 
    xorZero = myBgPixel;
+   if (myFgPixel == myBgPixel)
+      fprintf (stderr, "%s.\n%s.\n",
+            "Warning: foreground and background colors are the same",
+            "         You may not be able to see much");
+
+   canChangeAttrColor = FALSE;
+   if ((c_ptr=XGetDefault(mainDisplay,TOOL_NAME,"CanChangeAttrColor")) != NULL)
+      if ((strcmp (c_ptr, "True") == 0) || (strcmp (c_ptr, "true") == 0))
+         canChangeAttrColor = TRUE;
 }
 
 int OneColorObject (ObjPtr, ColorIndex)
@@ -710,6 +862,7 @@ int ChangeObjColor (ObjPtr, ColorIndex)
    int			ColorIndex;
 {
    register struct ObjRec	* obj_ptr;
+   register struct AttrRec	* attr_ptr;
    int				changed=FALSE, icon_color_index;
 
    switch (ObjPtr->type)
@@ -748,11 +901,20 @@ int ChangeObjColor (ObjPtr, ColorIndex)
                   changed = TRUE;
          break;
    }
+   if (canChangeAttrColor)
+   {
+      for (attr_ptr=ObjPtr->fattr; attr_ptr != NULL; attr_ptr=attr_ptr->next)
+         if (attr_ptr->obj->color != ColorIndex)
+         {
+            attr_ptr->obj->color = ColorIndex;
+            changed = TRUE;
+         }
+   }
    return (changed);
 }
 
-void ChangeAllSelColor (ColorIndex)
-   int	ColorIndex;
+void ChangeAllSelColor (ColorIndex, HighLight)
+   int	ColorIndex, HighLight;
 {
    register struct SelRec	* sel_ptr;
    int				text_obj_created, text_cursor_shown;
@@ -781,7 +943,7 @@ void ChangeAllSelColor (ColorIndex)
    XChangeGC (mainDisplay, drawGC,
          GCForeground | GCFunction | GCFillStyle, &values);
 
-   HighLightReverse ();
+   if (HighLight) HighLightReverse ();
    StartCompositeCmd ();
    for (sel_ptr = botSel; sel_ptr != NULL; sel_ptr = sel_ptr->prev)
    {
@@ -803,7 +965,7 @@ void ChangeAllSelColor (ColorIndex)
             selLtY-GRID_ABS_SIZE(1), selRbX+GRID_ABS_SIZE(1),
             selRbY+GRID_ABS_SIZE(1));
    }
-   HighLightForward ();
+   if (HighLight) HighLightForward ();
 }
 
 void SetUpColorMenuPixmap (fore_colors, init_rv, pixmap, rows, cols)
@@ -851,7 +1013,7 @@ int ColorMenu (X, Y, TrackMenubar)
          TrackMenubar);
    cfree (pixmap);
 
-   if (index >= 0) ChangeAllSelColor (index);
+   if (index >= 0) ChangeAllSelColor (index, TRUE);
    if (desc != NULL)
    {
       for (i=0; i < maxColors; i++)
@@ -863,8 +1025,17 @@ int ColorMenu (X, Y, TrackMenubar)
 
 void CleanUpColors ()
 {
-   register int		i;
+   register int	i;
 
+   if (privateColorInfo != NULL) {
+      struct LocalColorRec *ptr;
+
+      for (ptr=privateColorInfo; ptr->name != NULL; ptr++) {
+         cfree(ptr->name);
+         if (ptr->spec != NULL) cfree(ptr->spec);
+      }
+      cfree(privateColorInfo);
+   }
    cfree (colorPixels);
    cfree (xorColorPixels);
    cfree (tgifColors);
@@ -909,7 +1080,8 @@ int UpdatePixel (ObjPtr, OldColorStr)
          for (i = 0; i < ncolors; i++)
          {
             from_pixels[i] = xpm_ptr->pixels[i];
-            index = QuickFindColorIndex (xpm_ptr->color_str[i], &new_alloc);
+            index = QuickFindColorIndex (xpm_ptr->color_str[i], &new_alloc,
+                  TRUE);
             if (index == INVALID)
             {
                sprintf (msg, "Can not allocate color '%s', use '%s' instead.",
@@ -960,7 +1132,8 @@ int UpdatePixel (ObjPtr, OldColorStr)
          cfree (to_pixels);
          break;
       default:
-         index = QuickFindColorIndex (OldColorStr[ObjPtr->color], &new_alloc);
+         index = QuickFindColorIndex (OldColorStr[ObjPtr->color], &new_alloc,
+               TRUE);
          if (index != ObjPtr->color) picture_changed = TRUE;
          if (index == INVALID)
          {
@@ -1048,3 +1221,28 @@ int FlushColormap ()
    }
    return (FALSE);
 }
+
+void AddColor()
+{
+   char spec[MAXSTRING+1], *c_ptr;
+
+   MakeQuiescent();
+   *spec = '\0';
+   Dialog("Please enter colors to add:",
+         "( <CR>: accept, <ESC>: cancel )", spec);
+   UtilTrimBlanks(spec);
+   for (c_ptr=strtok(spec, ", \t\n\r"); c_ptr != NULL && *c_ptr != '\0';
+         c_ptr=strtok(NULL, ", \t\n\r")) {
+      int new_alloc;
+      int index=QuickFindColorIndex(c_ptr, &new_alloc, FALSE);
+
+      if (index == INVALID) {
+         sprintf(gszMsgBox, "%s '%s' %s.",
+               "Fail to allocate the", c_ptr, "color");
+         MsgBox(gszMsgBox, TOOL_NAME, INFO_MB);
+         return;
+      } else {
+         ChangeAllSelColor(index, FALSE);
+      }
+   }
+}
