@@ -23,7 +23,7 @@
  * OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
  * PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /amnt/maui/tangram/u/william/X11/TGIF2/RCS/const.h,v 2.99 1994/04/18 04:48:32 william Exp $
+ * @(#)$Header: /export/bourbon/multimedia/william/X11/TGIF2/RCS/const.h,v 2.120 1994/12/20 05:42:41 william Exp $
  */
 
 #ifndef _TGIF_CONST_H_
@@ -360,9 +360,10 @@
 #define MENU_FILL 17
 #define MENU_PEN 18
 #define MENU_COLOR 19
-#define MENU_SPECIAL 20
+#define MENU_NAVIGATE 20
+#define MENU_SPECIAL 21
 
-#define MAXMENUS 21
+#define MAXMENUS 22
 
 /* file menu */
 
@@ -374,23 +375,26 @@
 #define FILE_IMPORTXBM 5
 #define FILE_IMPORTXPM 6
 #define FILE_IMPORTEPS 7
-#define FILE_DOMAIN 8
-#define FILE_DUMP 9
-#define FILE_USR_DUMP 10
-#define FILE_DUMPSELECTED 11
-#define FILE_PRINTONE 12
-#define FILE_INPUT_POLY 13
-#define FILE_INPUT_POLYGON 14
-#define FILE_SOLVE 15
-#define FILE_SIMULATE 16
-#define FILE_PROBE 17
-#define FILE_ANIMATE 18
-#define FILE_ESCAPE 19
-#define FILE_SAVESELAS 20
-#define FILE_SAVESYMINLIB 21
-#define FILE_QUIT 22
+#define FILE_EMBEDEPS 8
+#define FILE_BROWSEXBM 9
+#define FILE_BROWSEXPM 10
+#define FILE_DOMAIN 11
+#define FILE_DUMP 12
+#define FILE_USR_DUMP 13
+#define FILE_DUMPSELECTED 14
+#define FILE_PRINTONE 15
+#define FILE_INPUT_POLY 16
+#define FILE_INPUT_POLYGON 17
+#define FILE_SOLVE 18
+#define FILE_SIMULATE 19
+#define FILE_PROBE 20
+#define FILE_ANIMATE 21
+#define FILE_ESCAPE 22
+#define FILE_SAVESELAS 23
+#define FILE_SAVESYMINLIB 24
+#define FILE_QUIT 25
 
-#define FILEMENUENTRIES 23
+#define FILEMENUENTRIES 26
 
 /* edit menu */
 
@@ -420,8 +424,10 @@
 #define EDIT_SMOOTH_HINGE 23
 #define EDIT_MAKE_REGULAR 24
 #define EDIT_BREAK_TEXT 25
+#define EDIT_SET_SEL_LINEWIDTH 26
+#define EDIT_ADD_COLOR 27
 
-#define MAXEDITMENUS 26
+#define MAXEDITMENUS 28
 
 /* layout menu */
 
@@ -447,8 +453,9 @@
 #define LAYOUT_TOGGLE_SHOW_MEASURE 19
 #define LAYOUT_TOGGLE_SHOW_MENUBAR 20
 #define LAYOUT_TOGGLE_SHOW_STATUS 21
+#define LAYOUT_TOGGLE_ONEMOTION_SELMOVE 22
 
-#define MAXLAYOUTMENUS 22
+#define MAXLAYOUTMENUS 23
 
 /* move mode stuff */
 
@@ -485,6 +492,18 @@
 
 #define MAXARRANGEMENUS 23
 
+/* nagivate menu */
+
+#define NAVIGATE_BACK 0
+#define NAVIGATE_FORWARD 1
+#define NAVIGATE_REFRESH 2
+#define NAVIGATE_HOTLIST 3
+#define NAVIGATE_ADD 4
+#define NAVIGATE_HISTORY 5
+#define NAVIGATE_HYPERSPACE 6
+
+#define MAXNAVIGATEMENUS 7
+
 /* special menu */
 
 #define SPECIAL_SYM 0
@@ -508,8 +527,12 @@
 #define SPECIAL_DETACHFILEATTR 18
 #define SPECIAL_EDITFILEATTR 19
 #define SPECIAL_UPDATESYMS 20
+#define SPECIAL_IMPORT_ATTRS 21
+#define SPECIAL_EXPORT_ATTRS 22
+#define SPECIAL_MERGE_WITH_TABLE 23
+#define SPECIAL_EXPORT_TO_TABLE 24
 
-#define MAXSPECIALMENUS 21
+#define MAXSPECIALMENUS 25
 
 /* page menu */
 
@@ -521,14 +544,16 @@
 #define PAGE_ADDAFTER 5
 #define PAGE_DEL 6
 #define PAGE_PRINT_ONE_IN_STACK 7
+#define PAGE_PAPER_SIZE_IN_STACK 8
 
-#define MAXPAGESTACKMENUS 8
+#define MAXPAGESTACKMENUS 9
 
 #define PAGE_TOGGLELINE 0
 #define PAGE_SIZING 1
 #define PAGE_PRINT_ONE 2
+#define PAGE_PAPER_SIZE 3
 
-#define MAXPAGETILEMENUS 3
+#define MAXPAGETILEMENUS 4
 
 #define PAGE_STACK 0
 #define PAGE_TILE 1
@@ -543,9 +568,13 @@
 #ifdef ultrix
 #define PUT_A_POINT(dpy,win,gc,x,y) XDrawPoint(dpy,win,gc,x,y)
 #else
+#ifdef _USE_XDRAWPOINT_TO_PUT_A_POINT
+#define PUT_A_POINT(dpy,win,gc,x,y) XDrawPoint(dpy,win,gc,x,y)
+#else
 #define PUT_A_POINT(dpy,win,gc,x,y) XDrawLine(dpy,win,gc,x,y,x,y)
 #endif
 #endif
+#endif
 
 /* file stuff */
 
@@ -597,4 +626,77 @@
 
 #define MAX_STATUS_BTNS 3
 
+/* copypaste stuff */
+
+#define TGIF_HEADER 0x80
+
+/* interrupt stuff */
+
+#define MAXINTRS 2
+
+/* remote status */
+
+#define TG_REMOTE_STATUS_OK 0
+#define TG_REMOTE_STATUS_INTR 1 /* user interrupt */
+#define TG_REMOTE_STATUS_MEM 2 /* memory allocation failed */
+#define TG_REMOTE_STATUS_WRITE 3 /* write failed */
+#define TG_REMOTE_STATUS_READ 4 /* read failed */
+#define TG_REMOTE_STATUS_NET 5 /* unexpect network error */
+#define TG_REMOTE_STATUS_HOST 6 /* gethostbyname failed */
+#define TG_REMOTE_STATUS_FORMAT 7 /* format error */
+#define TG_REMOTE_STATUS_TERM 8 /* terminated by the server */
+#define TG_REMOTE_STATUS_FILE 9 /* fail to open file/socket */
+
+/* corner stuff */
+
+#define CORNER_NONE 0
+#define CORNER_LT 1
+#define CORNER_TOP 2
+#define CORNER_RT 3
+#define CORNER_RIGHT 4
+#define CORNER_RB 5
+#define CORNER_BOTTOM 6
+#define CORNER_LB 7
+#define CORNER_LEFT 8
+
+/* msgbox stuff */
+
+#define MB_ID_FAILED 0
+#define MB_ID_OK 1
+#define MB_ID_CANCEL 2
+#define MB_ID_YES 3
+#define MB_ID_NO 4
+#define MB_ID_EXTRA 5
+
+#define MB_BTN_OK		1
+#define MB_BTN_YESNOCANCEL	2
+#define MB_BTN_OKCANCEL		3
+#define MB_BTN_YESNO		4
+#define MB_BTN_EXTRA		0x0008
+
+#define MB_ICON_STOP		0x0010
+#define MB_ICON_QUESTION	0x0020
+#define MB_ICON_INFORMATION	0x0040
+
+#define MB_BTNMASK		0x0007
+#define MB_ICONMASK		0x00f0
+
+#define INFO_MB (MB_BTN_OK | MB_ICON_INFORMATION)
+#define YNC_MB  (MB_BTN_YESNOCANCEL | MB_ICON_QUESTION)
+#define YN_MB   (MB_BTN_YESNO | MB_ICON_QUESTION)
+#define STOP_MB (MB_BTN_OK | MB_ICON_STOP)
+
+#define MB_PIXMAP_STOP 0
+#define MB_PIXMAP_QUESTION 1
+#define MB_PIXMAP_INFORMATION 2
+
+#define MAX_MB_ICONS 3
+
+/* expr stuff */
+
+#define NULL_VAL 0
+#define INT_VAL 1
+#define DBL_VAL 2
+#define STR_VAL 3
+
 #endif /*_TGIF_CONST_H_*/
