@@ -26,7 +26,7 @@
  */
 #ifndef lint
 static char RCSid[] =
-      "@(#)$Header: /amnt/maui/tangram/u/william/X11/TGIF2/RCS/copypaste.c,v 2.52 1994/03/21 04:53:52 william Exp $";
+      "@(#)$Header: /export/bourbon/multimedia/william/X11/TGIF2/RCS/copypaste.c,v 2.58 1994/12/23 09:20:34 william Exp $";
 #endif
 
 #include <sys/types.h>
@@ -63,8 +63,6 @@ static char RCSid[] =
 #include "special.e"
 #include "text.e"
 
-#define TGIF_HEADER 0x80
-
 extern char	* mktemp ARGS_DECL((char *Template));
 #ifndef _NO_EXTERN
 extern int	write ARGS_DECL((int fd, char *buf, int nbyte));
@@ -200,7 +198,6 @@ void CutToCutBuffer ()
    }
 }
 
-static
 void PasteString (CutBuffer)
    char	* CutBuffer;
 {
@@ -273,7 +270,7 @@ void PasteString (CutBuffer)
       w = XTextWidth (canvasFontPtr, str_ptr->s, strlen (str_ptr->s));
       if (w > max_len) max_len = w;
 
-      if (*c_ptr == '\n' || *c_ptr == '\r') c_ptr++;
+      while (*c_ptr == '\n' || *c_ptr == '\r') c_ptr++;
    }
 
    text_ptr->lines = num_lines;
@@ -354,6 +351,14 @@ void AssignNewObjIds (ObjPtr)
       AssignNewObjIds (attr_ptr->obj);
 }
 
+char * FetchCutBuffer (pn_buf_sz)
+   int	* pn_buf_sz;
+{
+   char	* cut_buffer=(char *) XFetchBytes (mainDisplay, pn_buf_sz);
+
+   return (cut_buffer);
+}
+
 int PasteFromCutBuffer ()
    /* returns FALSE if pasting in text mode and non-tgif bytes are */
    /*	in the cut buffer -- this is interpreted as an attempt to */
@@ -362,11 +367,12 @@ int PasteFromCutBuffer ()
    FILE 		* fp;
    int 			len, ltx, lty, rbx, rby, dx, dy, read_status;
    char 		tmpfile[MAXSTRING], * cut_buffer, msg[MAXSTRING];
+   char 		* orig_cut_buffer;
    unsigned char 	header = TGIF_HEADER;
    struct ObjRec	* obj_ptr, * saved_top_obj, * saved_bot_obj, * tmp_obj;
    struct ObjRec	* tmp_top_obj, * tmp_bot_obj;
 
-   cut_buffer = (char *) XFetchBytes (mainDisplay, &len);
+   orig_cut_buffer = cut_buffer = (char *) FetchCutBuffer (&len);
    if (len == 0)
    {
       Msg ("Cut buffer is empty");
@@ -378,6 +384,7 @@ int PasteFromCutBuffer ()
       {
          XEvent	ev;
 
+         XFree (cut_buffer);
          pasteInDrawTextMode = TRUE;
          ev.type = KeyPress;
          DrawText (&ev);
@@ -385,6 +392,7 @@ int PasteFromCutBuffer ()
       }
       Msg ("Paste from a non-tgif tool.");
       PasteString (cut_buffer);
+      XFree (cut_buffer);
       return (TRUE);
    }
    cut_buffer++;
@@ -396,6 +404,7 @@ int PasteFromCutBuffer ()
    {
       sprintf (msg, "Can not open %s for write.", tmpfile);
       Msg (msg);
+      XFree (orig_cut_buffer);
       return (TRUE);
    }
    writeFileFailed = FALSE;
@@ -406,10 +415,12 @@ int PasteFromCutBuffer ()
       sprintf (msg, "FWRITE error in writing to '%s'.  Paste aborted!",
             tmpfile);
       Msg (msg);
+      XFree (orig_cut_buffer);
       return (TRUE);
    }
    fflush (fp);
    rewind (fp);
+   XFree (orig_cut_buffer);
 
    SetWatchCursor (drawWindow);
    SetWatchCursor (mainWindow);
@@ -507,57 +518,63 @@ int PasteFromFile ()
    /*	paste into the current text */
 {
    char	file_name[MAXPATHLENGTH+1];
+   FILE	* fp;
+   char	msg[MAXSTRING+1], inbuf[MAXSTRING+1], * cut_buffer=NULL;
+   int	size=0, pos=0;
 
-   if (SelectFileNameToPaste ("Please select a file to PASTE ...", file_name) !=
+   if (SelectFileNameToPaste ("Please select a file to PASTE ...", file_name) ==
          INVALID)
    {
-      FILE	* fp;
-      char	msg[MAXSTRING+1], inbuf[MAXSTRING+1], * cut_buffer=NULL;
-      int	size=0, pos=0;
-
-      if (curChoice == DRAWTEXT)
-      {
-         XEvent	ev;
-
-         pasteInDrawTextMode = TRUE;
-         pasteFromFileInDrawTextMode = TRUE;
-         strcpy (pasteFromFileName, file_name);
-         ev.type = KeyPress;
-         DrawText (&ev);
-         return (FALSE);
-      }
-      if ((fp = fopen (file_name, "r")) == NULL)
-      {
-         sprintf (msg, "Can not open '%s' for read.", file_name);
-         Dialog (msg, NULL, NULL);
-         return (TRUE);
-      }
-      while (fgets (inbuf, MAXSTRING, fp) != NULL) size += strlen (inbuf);
-      fclose (fp);
-      if (size == 0)
-      {
-         sprintf (msg, "File '%s' is empty.", file_name);
-         Dialog (msg, NULL, NULL);
-         return (TRUE);
-      }
-      cut_buffer = (char *) calloc (size+2, sizeof(char));
-      if (cut_buffer == NULL)
-      {
-         sprintf (msg, "Can not malloc %1d bytes.", size+2);
-         Dialog (msg, NULL, NULL);
-         return (TRUE);
-      }
-      if ((fp = fopen (file_name, "r")) == NULL)
-      {
-         sprintf (msg, "Can not open '%s' for read.", file_name);
-         Dialog (msg, NULL, NULL);
-         return (TRUE);
-      }
-      while (fgets (&cut_buffer[pos], MAXSTRING, fp) != NULL)
-         pos += strlen (&cut_buffer[pos]);
-      fclose (fp);
-      PasteString (cut_buffer);
+      return (TRUE);
    }
+   else if (FileIsRemote (file_name))
+   {
+      Dialog ("Pasting remote file not supported.", NULL, NULL);
+      return (TRUE);
+   }
+   if (curChoice == DRAWTEXT)
+   {
+      XEvent	ev;
+
+      pasteInDrawTextMode = TRUE;
+      pasteFromFileInDrawTextMode = TRUE;
+      strcpy (pasteFromFileName, file_name);
+      ev.type = KeyPress;
+      DrawText (&ev);
+      return (FALSE);
+   }
+   if ((fp = fopen (file_name, "r")) == NULL)
+   {
+      sprintf (msg, "Can not open '%s' for read.", file_name);
+      Dialog (msg, NULL, NULL);
+      return (TRUE);
+   }
+   while (fgets (inbuf, MAXSTRING, fp) != NULL) size += strlen (inbuf);
+   fclose (fp);
+   if (size == 0)
+   {
+      sprintf (msg, "File '%s' is empty.", file_name);
+      Dialog (msg, NULL, NULL);
+      return (TRUE);
+   }
+   cut_buffer = (char *) calloc (size+2, sizeof(char));
+   if (cut_buffer == NULL)
+   {
+      sprintf (msg, "Can not malloc %1d bytes.", size+2);
+      Dialog (msg, NULL, NULL);
+      return (TRUE);
+   }
+   if ((fp = fopen (file_name, "r")) == NULL)
+   {
+      sprintf (msg, "Can not open '%s' for read.", file_name);
+      Dialog (msg, NULL, NULL);
+      return (TRUE);
+   }
+   while (fgets (&cut_buffer[pos], MAXSTRING, fp) != NULL)
+      pos += strlen (&cut_buffer[pos]);
+   fclose (fp);
+   PasteString (cut_buffer);
+
    return (TRUE);
 }
 
