@@ -25,10 +25,15 @@
  */
 #ifndef lint
 static char RCSid[] =
-      "@(#)$Header: /amnt/maui/tangram/u/william/X11/TGIF2/RCS/drawing.c,v 2.138 1994/04/09 05:08:42 william Exp $";
+      "@(#)$Header: /export/bourbon/multimedia/william/X11/TGIF2/RCS/drawing.c,v 2.180 1994/12/22 07:51:14 william Exp $";
 #endif
 
 #include <stdio.h>
+#include <sys/time.h>
+#include <sys/types.h>
+#ifdef _NO_GETTIMEOFDAY
+#include <sys/timeb.h>
+#endif /* _NO_GETTIMEOFDAY */
 #include <X11/Xlib.h>
 #include <X11/Xutil.h>
 #include "const.h"
@@ -51,6 +56,7 @@ static char RCSid[] =
 #include "dup.e"
 #include "edit.e"
 #include "eps.e"
+#include "exec.e"
 #include "file.e"
 #include "font.e"
 #include "grid.e"
@@ -59,6 +65,8 @@ static char RCSid[] =
 #include "mainloop.e"
 #include "menu.e"
 #include "msg.e"
+#include "navigate.e"
+#include "names.e"
 #include "obj.e"
 #include "oval.e"
 #include "page.e"
@@ -84,7 +92,9 @@ static char RCSid[] =
 #define O_INVIS 4
 #define O_GRID (O_VIS+O_INVIS)
 
-int		intrCheckInterval=10;
+#define DEF_CHECK_INTERVAL 1
+
+int		intrCheckInterval=DEF_CHECK_INTERVAL;
 int		pasteInDrawTextMode=FALSE;
 int		pasteFromFileInDrawTextMode=FALSE;
 char		pasteFromFileName[MAXPATHLENGTH+1];
@@ -93,10 +103,13 @@ int		numRedrawBBox=INVALID;
 int		numClipRecs=0;
 int		clipOrdering=Unsorted;
 XRectangle	clipRecs[4];
+int		checkBBox=TRUE;
 
-static int		checkBBox=TRUE;
 static struct BBRec	smallArea[2];
 
+static Pixmap	execAnimatePixmap=None;
+static int	execAnimatePixmapW=0, execAnimatePixmapH=0;
+
 void SetDefaultDrawWinClipRecs ()
 {
    SetRecVals (clipRecs[0], 0, 0, ZOOMED_SIZE(drawWinW), ZOOMED_SIZE(drawWinH));
@@ -191,37 +204,125 @@ int NeedToDraw (ObjBBox)
    return (TRUE);
 }
 
+#include "xbm/intr.xbm"
+#include "xbm/trek.xbm"
+
 static int	intrShown=FALSE;
 static int	checkCount=0;
+static int	savedCheckInterval=(-1);
+static int	intrIndex=(-1);
 
-void ShowInterrupt ()
+static long	intrTick=0L;
+
+static
+void RedrawInterrupt ()
 {
-   GC		gc;
-   XGCValues	values;
+   GC			gc;
+   XGCValues		values;
+   int			x, y;
+#ifdef _NO_GETTIMEOFDAY
+   struct timeb		now;
+#else /* ~_NO_GETTIMEOFDAY */
+   struct timeval	now;
+   struct timezone	zone;
+#endif /* _NO_GETTIMEOFDAY */
+   long			cur_tick;
 
-   if (intrCheckInterval <= 0) return;
+   if (!intrShown) return;
 
+#ifdef _NO_GETTIMEOFDAY
+   ftime(&now);
+   cur_tick = ((long)(((long)now.millitm) / 200)) +
+         ((long)(((long)now.time) * 5));
+#else /* ~_NO_GETTIMEOFDAY */
+   gettimeofday(&now, &zone);
+   cur_tick = ((long)(now.tv_usec / 200000)) + ((long)(now.tv_sec * 5));
+#endif /* _NO_GETTIMEOFDAY */
+   if (intrIndex != (-1) && intrTick == cur_tick) return;
+
+   intrTick = cur_tick;
+   if (++intrIndex == MAXINTRS) intrIndex = 0;
+
+   x = ((rulerW-intr_width)>>1);
+   y = ((rulerW-intr_height)>>1);
    values.foreground = myFgPixel;
    values.background = myBgPixel;
    values.fill_style = FillOpaqueStippled;
-   values.stipple = intrPixmap;
+   values.stipple = intrPixmap[intrIndex];
+   values.ts_x_origin = x;
+   values.ts_y_origin = y;
    gc = XCreateGC (mainDisplay, dummyWindow1,
-         GCForeground | GCBackground | GCFillStyle | GCStipple, &values);
-   XFillRectangle (mainDisplay, dummyWindow1, gc, 0, 0, scrollBarW, scrollBarW);
+         GCForeground | GCBackground | GCFillStyle | GCStipple |
+         GCTileStipXOrigin | GCTileStipYOrigin, &values);
+   XFillRectangle (mainDisplay, dummyWindow1, gc, x, y, intr_width,
+         intr_height);
    XFreeGC(mainDisplay, gc);
 
    XSync (mainDisplay, False);
-   intrShown = TRUE;
 }
 
-void HideInterrupt ()
+static
+void ShowHyperSpace ()
 {
-   XEvent	ev;
+   GC			gc;
+   XGCValues		values;
+   int			x, y;
 
-   while (XCheckWindowEvent (mainDisplay, dummyWindow1, ButtonPressMask, &ev)) ;
+   x = ((rulerW-trek_width)>>1);
+   y = ((rulerW-trek_height)>>1);
+   values.foreground = myFgPixel;
+   values.background = myBgPixel;
+   values.fill_style = FillOpaqueStippled;
+   values.stipple = trekPixmap;
+   values.ts_x_origin = x;
+   values.ts_y_origin = y;
+   gc = XCreateGC (mainDisplay, dummyWindow1,
+         GCForeground | GCBackground | GCFillStyle | GCStipple |
+         GCTileStipXOrigin | GCTileStipYOrigin, &values);
+   XFillRectangle (mainDisplay, dummyWindow1, gc, x, y, trek_width,
+         trek_height);
+   XFreeGC(mainDisplay, gc);
+}
+
+static int interruptLevel=0;
+
+void ShowInterrupt (CheckInterval)
+   int	CheckInterval;
+{
+   if (interruptLevel++ > 0) return;
+
+   if (CheckInterval > 0)
+   {
+      savedCheckInterval = intrCheckInterval;
+      intrCheckInterval = CheckInterval;
+   }
+   if (intrCheckInterval <= 0) return;
+
+   intrShown = TRUE;
+   intrIndex = (-1);
+   RedrawInterrupt ();
+}
+
+int HideInterrupt ()
+{
+   if (--interruptLevel > 0) return interruptLevel;
+   interruptLevel = 0; 
+   if (execAnimatePixmap == None) {
+      XEvent ev;
+
+      while (XCheckWindowEvent(mainDisplay,dummyWindow1,ButtonPressMask,&ev)) ;
+   }
    XClearWindow (mainDisplay, dummyWindow1);
    intrShown = FALSE;
    checkCount = 0;
+   if (savedCheckInterval > 0)
+   {
+      intrCheckInterval = savedCheckInterval;
+      savedCheckInterval = (-1);
+   }
+   if (inHyperSpace) ShowHyperSpace ();
+   XSync(mainDisplay, False);
+   return 0;
 }
 
 void RedrawDummyWindow1 ()
@@ -231,9 +332,11 @@ void RedrawDummyWindow1 ()
    while (XCheckWindowEvent (mainDisplay, dummyWindow1, ExposureMask, &ev)) ;
    while (XCheckWindowEvent (mainDisplay, dummyWindow1, ButtonPressMask, &ev)) ;
    if (intrShown)
-      ShowInterrupt ();
+      RedrawInterrupt ();
+   else if (inHyperSpace)
+      ShowHyperSpace ();
    else
-      HideInterrupt ();
+      HideInterrupt();
 }
 
 void DummiesEventHandler (input)
@@ -250,6 +353,13 @@ void DummiesEventHandler (input)
          else
             SetMouseStatus ("(none)", "(none)", "(none)");
       }
+      else if (input->type == ButtonPress)
+      {
+         if (!intrShown && execAnimatePixmap == None)
+            ToggleHyperSpace ();
+         else if (intrShown)
+            HideInterrupt ();
+      }
    }
    else if (input->xany.window == dummyWindow2)
       SetMouseStatus ("(none)", "(none)", "(none)");
@@ -257,11 +367,12 @@ void DummiesEventHandler (input)
 
 int CheckInterrupt ()
 {
-   if (intrCheckInterval <= 0) return (FALSE);
+   if (execAnimatePixmap == None && intrCheckInterval <= 0) return (FALSE);
    if (++checkCount >= intrCheckInterval)
    {
       XEvent	ev;
 
+      RedrawInterrupt ();
       checkCount = 0;
       if (XCheckWindowEvent (mainDisplay, dummyWindow1, ButtonPressMask, &ev))
       {
@@ -326,7 +437,7 @@ int DrawObj (Win, ObjPtr)
             if (!checkBBox || NeedToDraw (obj_ptr->bbox))
             {
                if (!DrawObj (Win, obj_ptr)) return (FALSE);
-               if (CheckInterrupt ())
+               if (execAnimatePixmap == None && CheckInterrupt ())
                {
                   Msg ("User interrupt.  Repaint aborted.");
                   return (FALSE);
@@ -349,7 +460,8 @@ int DrawObj (Win, ObjPtr)
    return (TRUE);
 }
 
-void DrawPaperBoundary ()
+void DrawPaperBoundary (Win)
+   Window	Win;
 {
    register int	x_end, y_end;
 
@@ -359,237 +471,280 @@ void DrawPaperBoundary ()
       if (drawOrigY+drawWinH > paperHeight)
       {
          y_end = OFFSET_Y(paperHeight);
-         XDrawLine (mainDisplay, drawWindow, defaultGC, x_end, 0, x_end, y_end);
-         XDrawLine (mainDisplay, drawWindow, defaultGC, 0, y_end, x_end, y_end);
+         XDrawLine (mainDisplay, Win, defaultGC, x_end, 0, x_end, y_end);
+         XDrawLine (mainDisplay, Win, defaultGC, 0, y_end, x_end, y_end);
       }
       else
-         XDrawLine (mainDisplay, drawWindow, defaultGC, x_end, 0, x_end,
+         XDrawLine (mainDisplay, Win, defaultGC, x_end, 0, x_end,
                ZOOMED_SIZE(drawWinH));
    }
    else if (drawOrigY+drawWinH > paperHeight)
    {
       y_end = OFFSET_Y(paperHeight);
-      XDrawLine (mainDisplay, drawWindow, defaultGC, 0, y_end,
+      XDrawLine (mainDisplay, Win, defaultGC, 0, y_end,
             ZOOMED_SIZE(drawWinW), y_end);
    }
 }
 
-void RedrawAnArea (BotObj, LtX, LtY, RbX, RbY)
-   struct ObjRec	* BotObj;
-   int			LtX, LtY, RbX, RbY;
+void RedrawAnArea(BotObj, LtX, LtY, RbX, RbY)
+   struct ObjRec *BotObj;
+   int LtX, LtY, RbX, RbY;
    /* LtX, LtY, RbX, RbY are absolute coordinates */
 {
-   register struct ObjRec	* obj_ptr;
+   register struct ObjRec *obj_ptr;
+   int x=OFFSET_X(LtX), y=OFFSET_Y(LtY);
+   int w=ZOOMED_SIZE(RbX-LtX)+1, h=ZOOMED_SIZE(RbY-LtY)+1;
 
-   SetRecVals (clipRecs[0], OFFSET_X(LtX), OFFSET_Y(LtY),
+   SetRecVals(clipRecs[0], OFFSET_X(LtX), OFFSET_Y(LtY),
          ZOOMED_SIZE(RbX-LtX)+1, ZOOMED_SIZE(RbY-LtY)+1);
    numClipRecs = 1;
    clipOrdering = YXBanded;
-   XSetClipRectangles (mainDisplay, drawGC, 0, 0, clipRecs, numClipRecs,
+   XSetClipRectangles(mainDisplay, drawGC, 0, 0, clipRecs, numClipRecs,
          clipOrdering);
 
-   XClearArea (mainDisplay, drawWindow, OFFSET_X(LtX), OFFSET_Y(LtY),
-         ZOOMED_SIZE(RbX-LtX)+1, ZOOMED_SIZE(RbY-LtY)+1, FALSE);
+   if (execAnimatePixmap != None) {
+      XGCValues values;
+      int real_w=(x+w >= execAnimatePixmapW ? execAnimatePixmapW-x : w);
+      int real_h=(y+h >= execAnimatePixmapH ? execAnimatePixmapH-y : h);
+
+      values.foreground = myBgPixel;
+      values.function = GXcopy;
+      values.fill_style = FillSolid;
+      XChangeGC(mainDisplay, drawGC,
+            GCForeground | GCFunction | GCFillStyle, &values);
+      XFillRectangle(mainDisplay, execAnimatePixmap, drawGC,
+            x, y, real_w, real_h);
+   } else {
+      XClearArea(mainDisplay, drawWindow, x, y, w, h, FALSE);
+   }
 
    if (paperWidth >= LtX && paperWidth <= RbX ||
-         paperHeight >= LtY && paperHeight <= RbY)
-      DrawPaperBoundary ();
-
-   DrawGridLines (drawWindow, OFFSET_X(LtX), OFFSET_Y(LtY),
-         ZOOMED_SIZE(RbX-LtX)+1, ZOOMED_SIZE(RbY-LtY)+1);
-   DrawPageLines (drawWindow, OFFSET_X(LtX), OFFSET_Y(LtY),
-         ZOOMED_SIZE(RbX-LtX)+1, ZOOMED_SIZE(RbY-LtY)+1);
+         paperHeight >= LtY && paperHeight <= RbY) {
+      DrawPaperBoundary(execAnimatePixmap==None ? drawWindow :
+            execAnimatePixmap);
+   }
+   if (execAnimatePixmap != None) {
+      DrawGridLines(execAnimatePixmap, x, y, w, h);
+      DrawPageLines(execAnimatePixmap, x, y, w, h);
+   } else {
+      DrawGridLines(drawWindow, x, y, w, h);
+      DrawPageLines(drawWindow, x, y, w, h);
+   }
 
    numRedrawBBox = 1;
    smallArea[0].ltx = LtX; smallArea[0].lty = LtY;
    smallArea[0].rbx = RbX; smallArea[0].rby = RbY;
-   ShowInterrupt ();
-   for (obj_ptr = BotObj; obj_ptr != NULL; obj_ptr = obj_ptr->prev)
-   {
-      if (BBoxIntersect (obj_ptr->bbox, drawWinBBox) &&
-            BBoxIntersect (obj_ptr->bbox, smallArea[0]))
-      {
-         if (!DrawObj (drawWindow, obj_ptr)) break;
-         if (CheckInterrupt ())
-         {
-            Msg ("User interrupt.  Repaint aborted.");
+   ShowInterrupt(DEF_CHECK_INTERVAL);
+   for (obj_ptr = BotObj; obj_ptr != NULL; obj_ptr = obj_ptr->prev) {
+      if (BBoxIntersect(obj_ptr->bbox, drawWinBBox) &&
+            BBoxIntersect(obj_ptr->bbox, smallArea[0])) {
+         if (!DrawObj(execAnimatePixmap==None ? drawWindow : execAnimatePixmap,
+               obj_ptr)) {
+            break;
+         }
+         if (execAnimatePixmap == None && CheckInterrupt()) {
+            Msg("User interrupt.  Repaint aborted.");
             break;
          }
       }
    }
-   HideInterrupt ();
-   SetDefaultDrawWinClipRecs ();
+   HideInterrupt();
+   SetDefaultDrawWinClipRecs();
+   if (execAnimatePixmap != None && execAnimateRedraw &&
+         x < execAnimatePixmapW && y < execAnimatePixmapH) {
+      int real_w=(x+w >= execAnimatePixmapW ? execAnimatePixmapW-x : w);
+      int real_h=(y+h >= execAnimatePixmapH ? execAnimatePixmapH-y : h);
+
+      XSetFunction(mainDisplay, drawGC, GXcopy);
+      XCopyArea(mainDisplay, execAnimatePixmap, drawWindow, drawGC,
+            x, y, real_w, real_h, x, y);
+   }
 }
 
-void RedrawAreas (BotObj, LtX1, LtY1, RbX1, RbY1, LtX2, LtY2, RbX2, RbY2)
-   struct ObjRec	* BotObj;
-   int			LtX1, LtY1, RbX1, RbY1, LtX2, LtY2, RbX2, RbY2;
+void RedrawAreas(BotObj, LtX1, LtY1, RbX1, RbY1, LtX2, LtY2, RbX2, RbY2)
+   struct ObjRec *BotObj;
+   int LtX1, LtY1, RbX1, RbY1, LtX2, LtY2, RbX2, RbY2;
    /* note:  these coordinates are absolute */
 {
-   register struct ObjRec	* obj_ptr;
-   int				ltx, lty, rbx, rby, rec1_slot;
-   struct BBRec			bbox1, bbox2;
+   register struct ObjRec *obj_ptr;
+   struct BBRec bbox1, bbox2;
+   int rec1_slot;
+   int x1=OFFSET_X(LtX1), y1=OFFSET_Y(LtY1);
+   int w1=ZOOMED_SIZE(RbX1-LtX1)+1, h1=ZOOMED_SIZE(RbY1-LtY1)+1;
+   int x2=OFFSET_X(LtX2), y2=OFFSET_Y(LtY2);
+   int w2=ZOOMED_SIZE(RbX2-LtX2)+1, h2=ZOOMED_SIZE(RbY2-LtY2)+1;
 
    bbox1.ltx = LtX1; bbox1.lty = LtY1;
    bbox1.rbx = RbX1; bbox1.rby = RbY1;
    bbox2.ltx = LtX2; bbox2.lty = LtY2;
    bbox2.rbx = RbX2; bbox2.rby = RbY2;
 
-   if (Inside (bbox1, bbox2))
-   {
-      RedrawAnArea (BotObj, LtX2, LtY2, RbX2, RbY2);
+   if (Inside(bbox1, bbox2)) {
+      RedrawAnArea(BotObj, LtX2, LtY2, RbX2, RbY2);
       return;
-   }
-   else if (Inside (bbox2, bbox1))
-   {
-      RedrawAnArea (BotObj, LtX1, LtY1, RbX1, RbY1);
+   } else if (Inside(bbox2, bbox1)) {
+      RedrawAnArea(BotObj, LtX1, LtY1, RbX1, RbY1);
       return;
    }
 
-   XClearArea (mainDisplay, drawWindow, OFFSET_X(LtX1), OFFSET_Y(LtY1),
-         ZOOMED_SIZE(RbX1-LtX1)+1, ZOOMED_SIZE(RbY1-LtY1)+1, FALSE);
+   if (execAnimatePixmap != None) {
+      XGCValues values;
+      int real_w=(x1+w1 >= execAnimatePixmapW ? execAnimatePixmapW-x1 : w1);
+      int real_h=(y1+h1 >= execAnimatePixmapH ? execAnimatePixmapH-y1 : h1);
 
-   if (BBoxIntersect (bbox1, bbox2))
-   {
-      ltx = min(LtX1,LtX2); lty = min(LtY1,LtY2);
-      rbx = max(RbX1,RbX2); rby = max(RbY1,RbY2);
-      RedrawAnArea (BotObj, ltx, lty, rbx, rby);
-      return;
+      values.foreground = myBgPixel;
+      values.function = GXcopy;
+      values.fill_style = FillSolid;
+      XChangeGC(mainDisplay, drawGC,
+            GCForeground | GCFunction | GCFillStyle, &values);
+      XFillRectangle(mainDisplay, execAnimatePixmap, drawGC,
+            x1, y1, real_w, real_h);
+   } else {
+      XClearArea(mainDisplay, drawWindow, x1, y1, w1, h1, FALSE);
    }
 
-   if (LtY1 == LtY2)
-   {
+   if (BBoxIntersect(bbox1, bbox2)) {
+      int union_ltx, union_lty, union_rbx, union_rby;
+
+      union_ltx = min(LtX1,LtX2); union_lty = min(LtY1,LtY2);
+      union_rbx = max(RbX1,RbX2); union_rby = max(RbY1,RbY2);
+      RedrawAnArea(BotObj, union_ltx, union_lty, union_rbx, union_rby);
+      return;
+   }
+   if (LtY1 == LtY2) {
       rec1_slot = (LtX1 <= LtX2) ? 0 : 1;
-      SetRecVals (clipRecs[rec1_slot], OFFSET_X(LtX1), OFFSET_Y(LtY1),
+      SetRecVals(clipRecs[rec1_slot], OFFSET_X(LtX1), OFFSET_Y(LtY1),
             ZOOMED_SIZE(RbX1-LtX1)+1, ZOOMED_SIZE(RbY1-LtY1)+1);
-      SetRecVals (clipRecs[!rec1_slot], OFFSET_X(LtX2), OFFSET_Y(LtY2),
+      SetRecVals(clipRecs[!rec1_slot], OFFSET_X(LtX2), OFFSET_Y(LtY2),
             ZOOMED_SIZE(RbX2-LtX2)+1, ZOOMED_SIZE(RbY2-LtY2)+1);
       numClipRecs = 2;
-   }
-   else
-   {
-      if (LtY1 < LtY2)
-      {
-         if (RbY1 <= LtY2)
-         {  /* y-extents do not intersect */
-            SetRecVals (clipRecs[0], OFFSET_X(LtX1), OFFSET_Y(LtY1),
+   } else {
+      if (LtY1 < LtY2) {
+         if (RbY1 <= LtY2) {  /* y-extents do not intersect */
+            SetRecVals(clipRecs[0], OFFSET_X(LtX1), OFFSET_Y(LtY1),
                   ZOOMED_SIZE(RbX1-LtX1)+1, ZOOMED_SIZE(RbY1-LtY1)+1);
-            SetRecVals (clipRecs[1], OFFSET_X(LtX2), OFFSET_Y(LtY2),
+            SetRecVals(clipRecs[1], OFFSET_X(LtX2), OFFSET_Y(LtY2),
                   ZOOMED_SIZE(RbX2-LtX2)+1, ZOOMED_SIZE(RbY2-LtY2)+1);
             numClipRecs = 2;
-         }
-         else if (RbY1 >= RbY2)
-         {  /* box 2's y-extents is inside box 1's y-extents */
+         } else if (RbY1 >= RbY2) {
+            /* box 2's y-extents is inside box 1's y-extents */
             rec1_slot = (LtX1 < LtX2) ? 0 : 1;  
-            SetRecVals (clipRecs[rec1_slot], OFFSET_X(LtX1), OFFSET_Y(LtY1),
+            SetRecVals(clipRecs[rec1_slot], OFFSET_X(LtX1), OFFSET_Y(LtY1),
                   ZOOMED_SIZE(RbX1-LtX1)+1, ZOOMED_SIZE(RbY1-LtY1)+1);
-            SetRecVals (clipRecs[!rec1_slot], OFFSET_X(LtX2), OFFSET_Y(LtY2),
+            SetRecVals(clipRecs[!rec1_slot], OFFSET_X(LtX2), OFFSET_Y(LtY2),
                   ZOOMED_SIZE(RbX2-LtX2)+1, ZOOMED_SIZE(RbY2-LtY2)+1);
             numClipRecs = 2;
-         }
-         else
-         {  
-            SetRecVals (clipRecs[0], OFFSET_X(LtX1), OFFSET_Y(LtY1),
+         } else {  
+            SetRecVals(clipRecs[0], OFFSET_X(LtX1), OFFSET_Y(LtY1),
                   ZOOMED_SIZE(RbX1-LtX1)+1, ZOOMED_SIZE(LtY2-LtY1)+1);
-            if (LtX1 < LtX2)
-            {
-               SetRecVals (clipRecs[1], OFFSET_X(LtX1), OFFSET_Y(LtY2),
+            if (LtX1 < LtX2) {
+               SetRecVals(clipRecs[1], OFFSET_X(LtX1), OFFSET_Y(LtY2),
                      ZOOMED_SIZE(RbX1-LtX1)+1, ZOOMED_SIZE(RbY1-LtY2)+1);
-               SetRecVals (clipRecs[2], OFFSET_X(LtX2), OFFSET_Y(LtY2),
+               SetRecVals(clipRecs[2], OFFSET_X(LtX2), OFFSET_Y(LtY2),
                      ZOOMED_SIZE(RbX2-LtX2)+1, ZOOMED_SIZE(RbY1-LtY2)+1);
-            }
-            else
-            {
-               SetRecVals (clipRecs[1], OFFSET_X(LtX2), OFFSET_Y(LtY2),
+            } else {
+               SetRecVals(clipRecs[1], OFFSET_X(LtX2), OFFSET_Y(LtY2),
                      ZOOMED_SIZE(RbX2-LtX2)+1, ZOOMED_SIZE(RbY1-LtY2)+1);
-               SetRecVals (clipRecs[2], OFFSET_X(LtX1), OFFSET_Y(LtY2),
+               SetRecVals(clipRecs[2], OFFSET_X(LtX1), OFFSET_Y(LtY2),
                      ZOOMED_SIZE(RbX1-LtX1)+1, ZOOMED_SIZE(RbY1-LtY2)+1);
             }
-            SetRecVals (clipRecs[3], OFFSET_X(LtX2), OFFSET_Y(RbY1),
+            SetRecVals(clipRecs[3], OFFSET_X(LtX2), OFFSET_Y(RbY1),
                   ZOOMED_SIZE(RbX2-LtX2)+1, ZOOMED_SIZE(RbY2-RbY1)+1);
             numClipRecs = 4;
          }
-      }
-      else
-      {
-         if (RbY2 <= LtY1)
-         {  /* y-extents do not intersect */
-            SetRecVals (clipRecs[0], OFFSET_X(LtX2), OFFSET_Y(LtY2),
+      } else {
+         if (RbY2 <= LtY1) {  /* y-extents do not intersect */
+            SetRecVals(clipRecs[0], OFFSET_X(LtX2), OFFSET_Y(LtY2),
                   ZOOMED_SIZE(RbX2-LtX2)+1, ZOOMED_SIZE(RbY2-LtY2)+1);
-            SetRecVals (clipRecs[1], OFFSET_X(LtX1), OFFSET_Y(LtY1),
+            SetRecVals(clipRecs[1], OFFSET_X(LtX1), OFFSET_Y(LtY1),
                   ZOOMED_SIZE(RbX1-LtX1)+1, ZOOMED_SIZE(RbY1-LtY1)+1);
             numClipRecs = 2;
-         }
-         else if (RbY2 >= RbY1)
-         {  /* box 1's y-extents is inside box 2's y-extents */
+         } else if (RbY2 >= RbY1) {
+            /* box 1's y-extents is inside box 2's y-extents */
             rec1_slot = (LtX1 < LtX2) ? 0 : 1;  
-            SetRecVals (clipRecs[rec1_slot], OFFSET_X(LtX1), OFFSET_Y(LtY1),
+            SetRecVals(clipRecs[rec1_slot], OFFSET_X(LtX1), OFFSET_Y(LtY1),
                   ZOOMED_SIZE(RbX1-LtX1)+1, ZOOMED_SIZE(RbY1-LtY1)+1);
-            SetRecVals (clipRecs[!rec1_slot], OFFSET_X(LtX2), OFFSET_Y(LtY2),
+            SetRecVals(clipRecs[!rec1_slot], OFFSET_X(LtX2), OFFSET_Y(LtY2),
                   ZOOMED_SIZE(RbX2-LtX2)+1, ZOOMED_SIZE(RbY2-LtY2)+1);
             numClipRecs = 2;
-         }
-         else
-         {  
-            SetRecVals (clipRecs[0], OFFSET_X(LtX2), OFFSET_Y(LtY2),
+         } else {  
+            SetRecVals(clipRecs[0], OFFSET_X(LtX2), OFFSET_Y(LtY2),
                   ZOOMED_SIZE(RbX2-LtX2)+1, ZOOMED_SIZE(LtY1-LtY2)+1);
-            if (LtX1 < LtX2)
-            {
-               SetRecVals (clipRecs[1], OFFSET_X(LtX1), OFFSET_Y(LtY1),
+            if (LtX1 < LtX2) {
+               SetRecVals(clipRecs[1], OFFSET_X(LtX1), OFFSET_Y(LtY1),
                      ZOOMED_SIZE(RbX1-LtX1)+1, ZOOMED_SIZE(RbY2-LtY1)+1);
-               SetRecVals (clipRecs[2], OFFSET_X(LtX2), OFFSET_Y(LtY1),
+               SetRecVals(clipRecs[2], OFFSET_X(LtX2), OFFSET_Y(LtY1),
                      ZOOMED_SIZE(RbX2-LtX2)+1, ZOOMED_SIZE(RbY2-LtY1)+1);
-            }
-            else
-            {
-               SetRecVals (clipRecs[1], OFFSET_X(LtX2), OFFSET_Y(LtY1),
+            } else {
+               SetRecVals(clipRecs[1], OFFSET_X(LtX2), OFFSET_Y(LtY1),
                      ZOOMED_SIZE(RbX2-LtX2)+1, ZOOMED_SIZE(RbY2-LtY1)+1);
-               SetRecVals (clipRecs[2], OFFSET_X(LtX1), OFFSET_Y(LtY1),
+               SetRecVals(clipRecs[2], OFFSET_X(LtX1), OFFSET_Y(LtY1),
                      ZOOMED_SIZE(RbX1-LtX1)+1, ZOOMED_SIZE(RbY2-LtY1)+1);
             }
-            SetRecVals (clipRecs[3], OFFSET_X(LtX1), OFFSET_Y(RbY2),
+            SetRecVals(clipRecs[3], OFFSET_X(LtX1), OFFSET_Y(RbY2),
                   ZOOMED_SIZE(RbX1-LtX1)+1, ZOOMED_SIZE(RbY1-RbY2)+1);
             numClipRecs = 4;
          }
       }
    }
    clipOrdering = YXSorted;
-   XSetClipRectangles (mainDisplay, drawGC, 0, 0, clipRecs, numClipRecs,
+   XSetClipRectangles(mainDisplay, drawGC, 0, 0, clipRecs, numClipRecs,
          clipOrdering);
 
    if (paperWidth >= LtX1 && paperWidth <= RbX1 ||
          paperHeight >= LtY1 && paperHeight <= RbY1 ||
          paperWidth >= LtX2 && paperWidth <= RbX2 ||
-         paperHeight >= LtY2 && paperHeight <= RbY2)
-      DrawPaperBoundary ();
-
-   DrawGridLines (drawWindow, OFFSET_X(LtX1), OFFSET_Y(LtY1),
-         ZOOMED_SIZE(RbX1-LtX1)+1, ZOOMED_SIZE(RbY1-LtY1)+1);
-   DrawPageLines (drawWindow, OFFSET_X(LtX1), OFFSET_Y(LtY1),
-         ZOOMED_SIZE(RbX1-LtX1)+1, ZOOMED_SIZE(RbY1-LtY1)+1);
+         paperHeight >= LtY2 && paperHeight <= RbY2) {
+      DrawPaperBoundary(execAnimatePixmap==None ? drawWindow :
+            execAnimatePixmap);
+   }
+   if (execAnimatePixmap != None) {
+      DrawGridLines(execAnimatePixmap, x1, y1, w1, h1);
+      DrawPageLines(execAnimatePixmap, x1, y1, w1, h1);
+   } else {
+      DrawGridLines(drawWindow, x1, y1, w1, h1);
+      DrawPageLines(drawWindow, x1, y1, w1, h1);
+   }
 
    numRedrawBBox = 2;
    smallArea[0].ltx = LtX1; smallArea[0].lty = LtY1;
    smallArea[0].rbx = RbX1; smallArea[0].rby = RbY1;
    smallArea[1].ltx = LtX2; smallArea[1].lty = LtY2;
    smallArea[1].rbx = RbX2; smallArea[1].rby = RbY2;
-   ShowInterrupt ();
-   for (obj_ptr = BotObj; obj_ptr != NULL; obj_ptr = obj_ptr->prev)
-   {
-      if (BBoxIntersect (obj_ptr->bbox, drawWinBBox) &&
-            (BBoxIntersect (obj_ptr->bbox, bbox1) ||
-            BBoxIntersect (obj_ptr->bbox, bbox2)))
-      {
-         if (!DrawObj (drawWindow, obj_ptr)) break;
-         if (CheckInterrupt ())
-         {
-            Msg ("User interrupt.  Repaint aborted.");
+   ShowInterrupt(DEF_CHECK_INTERVAL);
+   for (obj_ptr = BotObj; obj_ptr != NULL; obj_ptr = obj_ptr->prev) {
+      if (BBoxIntersect(obj_ptr->bbox, drawWinBBox) &&
+            (BBoxIntersect(obj_ptr->bbox, bbox1) ||
+            BBoxIntersect(obj_ptr->bbox, bbox2))) {
+         if (!DrawObj(execAnimatePixmap==None ? drawWindow : execAnimatePixmap,
+               obj_ptr)) {
+            break;
+         }
+         if (execAnimatePixmap == None && CheckInterrupt()) {
+            Msg("User interrupt.  Repaint aborted.");
             break;
          }
       }
    }
-   HideInterrupt ();
-   SetDefaultDrawWinClipRecs ();
+   HideInterrupt();
+   SetDefaultDrawWinClipRecs();
+   if (execAnimatePixmap != None && execAnimateRedraw) {
+      XSetFunction(mainDisplay, drawGC, GXcopy);
+      if (x1 < execAnimatePixmapW && y1 < execAnimatePixmapH) {
+         int real_w=(x1+w1 >= execAnimatePixmapW ? execAnimatePixmapW-x1 : w1);
+         int real_h=(y1+h1 >= execAnimatePixmapH ? execAnimatePixmapH-y1 : h1);
+
+         XCopyArea(mainDisplay, execAnimatePixmap, drawWindow, drawGC,
+               x1, y1, real_w, real_h, x1, y1);
+      }
+      if (x2 < execAnimatePixmapW && y2 < execAnimatePixmapH) {
+         int real_w=(x2+w2 >= execAnimatePixmapW ? execAnimatePixmapW-x2 : w2);
+         int real_h=(y2+h2 >= execAnimatePixmapH ? execAnimatePixmapH-y2 : h2);
+
+         XCopyArea(mainDisplay, execAnimatePixmap, drawWindow, drawGC,
+               x2, y2, real_w, real_h, x2, y2);
+      }
+   }
 }
 
 static
@@ -753,7 +908,7 @@ Pixmap DrawAllOnPixmap (LtX, LtY, W, H)
    AdjSplineVs ();
 
    checkBBox = FALSE;
-   ShowInterrupt ();
+   ShowInterrupt (DEF_CHECK_INTERVAL);
    for (obj_ptr = botObj; obj_ptr != NULL; obj_ptr = obj_ptr->prev)
    {
       if (!DrawObj (pixmap, obj_ptr))
@@ -762,7 +917,7 @@ Pixmap DrawAllOnPixmap (LtX, LtY, W, H)
          pixmap = None;
          break;
       }
-      if (CheckInterrupt ())
+      if (execAnimatePixmap == None && CheckInterrupt ())
       {
          Msg ("User interrupt.  Repaint aborted.");
          XFreePixmap (mainDisplay, pixmap);
@@ -770,7 +925,7 @@ Pixmap DrawAllOnPixmap (LtX, LtY, W, H)
          break;
       }
    }
-   HideInterrupt ();
+   HideInterrupt();
    checkBBox = TRUE;
 
    drawOrigX = saved_draw_orig_x; drawOrigY = saved_draw_orig_y;
@@ -789,46 +944,137 @@ void RedrawDrawWindow (BotObj)
 {
    register struct ObjRec	* obj_ptr;
 
-   DrawPaperBoundary ();
-   RedrawGridLines ();
-   RedrawPageLines ();
+   if (execAnimating) {
+      int already_animating=(execAnimatePixmap!=None);
+
+      if (execAnimatePixmap != None) {
+         XFreePixmap(mainDisplay, execAnimatePixmap);
+         execAnimatePixmap = None;
+      }
+      execAnimatePixmapW = ZOOMED_SIZE(drawWinW);
+      execAnimatePixmapH = ZOOMED_SIZE(drawWinH);
+      execAnimatePixmap = XCreatePixmap(mainDisplay, mainWindow,
+            execAnimatePixmapW, execAnimatePixmapH, mainDepth);
+      if (execAnimatePixmap == None) {
+         if (already_animating) {
+            sprintf(gszMsgBox,
+                  "Fail to create pixmap of size [%1dx%1d].\n\n%s.",
+                  execAnimatePixmapW, execAnimatePixmapH,
+                  "Can not cache animation");
+            MsgBox(gszMsgBox, TOOL_NAME, INFO_MB);
+         }
+         execAnimatePixmapW = execAnimatePixmapH = 0;
+      } else {
+         XGCValues values;
+
+         values.foreground = myBgPixel;
+         values.function = GXcopy;
+         values.fill_style = FillSolid;
+         XChangeGC(mainDisplay, drawGC,
+               GCForeground | GCFunction | GCFillStyle, &values);
+         XFillRectangle(mainDisplay, execAnimatePixmap, drawGC,
+               0, 0, execAnimatePixmapW, execAnimatePixmapH);
+      }
+   }
+   DrawPaperBoundary(execAnimatePixmap==None ? drawWindow : execAnimatePixmap);
+   RedrawGridLines(execAnimatePixmap==None ? drawWindow : execAnimatePixmap);
+   RedrawPageLines(execAnimatePixmap==None ? drawWindow : execAnimatePixmap);
 
    numRedrawBBox = 0;
-   ShowInterrupt ();
-   for (obj_ptr = BotObj; obj_ptr != NULL; obj_ptr = obj_ptr->prev)
-   {
-      if (BBoxIntersect (obj_ptr->bbox, drawWinBBox))
-      {
-         if (!DrawObj (drawWindow, obj_ptr)) break;
-         if (CheckInterrupt ())
-         {
-            Msg ("User interrupt.  Repaint aborted.");
+   ShowInterrupt(DEF_CHECK_INTERVAL);
+   for (obj_ptr = BotObj; obj_ptr != NULL; obj_ptr = obj_ptr->prev) {
+      if (BBoxIntersect(obj_ptr->bbox, drawWinBBox)) {
+         if (!DrawObj(execAnimatePixmap==None ? drawWindow : execAnimatePixmap,
+               obj_ptr)) {
+            break;
+         }
+         if (execAnimatePixmap == None && CheckInterrupt()) {
+            Msg("User interrupt.  Repaint aborted.");
             break;
          }
       }
    }
-   HideInterrupt ();
+   HideInterrupt();
+   if (execAnimatePixmap != None) {
+      XSetFunction(mainDisplay, drawGC, GXcopy);
+      XCopyArea(mainDisplay, execAnimatePixmap, drawWindow, drawGC,
+            0, 0, execAnimatePixmapW, execAnimatePixmapH, 0, 0);
+   }
 }
 
-void ClearAndRedrawDrawWindow ()
+void ClearAndRedrawDrawWindow()
 {
-   XClearWindow (mainDisplay, drawWindow);
+   XClearWindow(mainDisplay, drawWindow);
+   if (execAnimatePixmap != None) {
+      XGCValues values;
+
+      values.foreground = myBgPixel;
+      values.function = GXcopy;
+      values.fill_style = FillSolid;
+      XChangeGC(mainDisplay, drawGC,
+            GCForeground | GCFunction | GCFillStyle, &values);
+      XFillRectangle(mainDisplay, execAnimatePixmap, drawGC,
+            0, 0, execAnimatePixmapW, execAnimatePixmapH);
+   }
    somethingHighLighted = FALSE;
-   RedrawDrawWindow (botObj);
-   RedrawCurText ();
-   HighLightForward ();
+   RedrawDrawWindow(botObj);
+   RedrawCurText();
+   if (!execAnimating) HighLightForward();
 }
 
-void ClearAndRedrawDrawWindowDontDrawCurText ()
+void ClearAndRedrawDrawWindowNoCurT()
+   /* use to be ClearAndRedrawDrawWindowDontDrawCurText() */
 {
-   XClearWindow (mainDisplay, drawWindow);
+   XClearWindow(mainDisplay, drawWindow);
+   if (execAnimatePixmap != None) {
+      XGCValues values;
+
+      values.foreground = myBgPixel;
+      values.function = GXcopy;
+      values.fill_style = FillSolid;
+      XChangeGC(mainDisplay, drawGC,
+            GCForeground | GCFunction | GCFillStyle, &values);
+      XFillRectangle(mainDisplay, execAnimatePixmap, drawGC,
+            0, 0, execAnimatePixmapW, execAnimatePixmapH);
+   }
    somethingHighLighted = FALSE;
-   RedrawDrawWindow (botObj);
-   HighLightForward ();
+   RedrawDrawWindow(botObj);
+   HighLightForward();
+}
+
+int BeginExecAnimate()
+{
+   execAnimating = TRUE;
+   execAnimateRedraw = TRUE;
+   RedrawDrawWindow(botObj);
+
+   if (execAnimatePixmap == None) {
+      sprintf(gszMsgBox, "Fail to create pixmap of size [%1dx%1d].\n\n%s.",
+            execAnimatePixmapW, execAnimatePixmapH, "Can not begin animation");
+      MsgBox(gszMsgBox, TOOL_NAME, INFO_MB);
+      execAnimatePixmapW = execAnimatePixmapH = 0;
+      execAnimating = execAnimateRedraw = FALSE;
+      return FALSE;
+   }
+   return TRUE;
+}
+
+void EndExecAnimate()
+{
+   if (execAnimatePixmap != None) {
+      XFreePixmap(mainDisplay, execAnimatePixmap);
+      execAnimatePixmap = None;
+   }
+   execAnimatePixmapW = execAnimatePixmapH = 0;
+   execAnimating = execAnimateRedraw = FALSE;
 }
 
 void CleanUpDrawingWindow ()
 {
+   if (execAnimatePixmap != None) {
+      Msg("Forcing an end_animate().");
+      EndExecAnimate();
+   }
    SetCurChoice (NOTHING);
    if (topSel != NULL) { HighLightReverse (); RemoveAllSel (); }
    if (tgifObj != NULL && tgifObj->fattr != NULL)
@@ -978,7 +1224,7 @@ int ShortHand (input)
             case '$': /*#$*/ SetCurChoice (VERTEXMODE); break;
             case '&': /*#&*/ AlignSelToPage (); break;
             case '*': /*#**/ RedoCmd (); break;
-            case '(': /*#(*/ ImportEPSFile (); break;
+            case '(': /*#(*/ ImportEPSFile (FALSE); break;
             case ')': /*#)*/ ScaleAllSelObj (); break;
             case '<': /*#<*/ LockSelObj (); break;
             case '>': /*#>*/ UnlockSelObj (); break;
@@ -1105,6 +1351,42 @@ int ShortHand (input)
             ToggleShowMenubar ();
          else if (strcmp (name, "ToggleShowStatus()") == 0)
             ToggleShowStatus ();
+         else if (strcmp (name, "BrowseXBitmap()") == 0)
+            BrowseXBitmap ();
+         else if (strcmp (name, "BrowseXPixmap()") == 0)
+            BrowseXPixmap ();
+         else if (strcmp (name, "SpecifyPaperSize()") == 0)
+            SpecifyPaperSize ();
+         else if (strcmp (name, "ToggleOneMotionSelMove()") == 0)
+            ToggleOneMotionSelectMove ();
+         else if (strcmp (name, "GoBack()") == 0)
+            NavigateBack ();
+         else if (strcmp (name, "GoForward()") == 0)
+            NavigateForward ();
+         else if (strcmp (name, "RefreshCurrent()") == 0)
+            NavigateRefresh ();
+         else if (strcmp (name, "HotList()") == 0)
+            NavigateHotList ();
+         else if (strcmp (name, "AddCurrentToHotList()") == 0)
+            NavigateAddToHotList ();
+         else if (strcmp (name, "SessionHistory()") == 0)
+            NavigateSessionHistory ();
+         else if (strcmp (name, "ToggleHyperSpace()") == 0)
+            ToggleHyperSpace ();
+         else if (strcmp (name, "EmbedEPSFile()") == 0)
+            ImportEPSFile (TRUE);
+         else if (strcmp (name, "SetSelLineWidth()") == 0)
+            SetSelectedLineWidth ();
+         else if (strcmp (name, "AddColor()") == 0)
+            AddColor ();
+         else if (strcmp (name, "ImportAttrs()") == 0)
+            ImportAttrs ();
+         else if (strcmp (name, "ExportAttrs()") == 0)
+            ExportAttrs ();
+         else if (strcmp (name, "MergeWithTable()") == 0)
+            MergeWithTable ();
+         else if (strcmp (name, "ExportToTable()") == 0)
+            ExportToTable ();
       }
       return (BAD);
    }
@@ -1147,8 +1429,11 @@ int DrawingEventHandler (input)
       switch (curChoice)
       {
          case NOTHING:
-            SetMouseStatus ("select/move/resize objects", "Main Menu",
-                  "Mode Menu");
+            if (inHyperSpace)
+               SetMouseStatus ("(none)", "Main Menu", "Mode Menu");
+            else
+               SetMouseStatus ("select/move/resize objects", "Main Menu",
+                     "Mode Menu");
             break;
          case DRAWTEXT:
             SetMouseStatus ("enter text", "Main Menu", "Mode Menu");
@@ -1189,6 +1474,46 @@ int DrawingEventHandler (input)
       mouse_y = (input->xmotion).y;
       GridXY (mouse_x, mouse_y, &grid_x, &grid_y);
       MarkRulers (grid_x, grid_y);
+      if (inHyperSpace) {
+         struct ObjRec	* obj_ptr, * owner_obj;
+         struct AttrRec	* attr_ptr;
+
+         if ((obj_ptr=FindAnObj(mouse_x, mouse_y, &owner_obj)) != NULL) {
+            if (owner_obj != NULL) obj_ptr = owner_obj;
+            if ((attr_ptr=FindAttrWithName(obj_ptr, TELEPORT_ATTR, NULL)) !=
+                  NULL || (attr_ptr=FindAttrWithName(obj_ptr, "href=",
+                  NULL)) != NULL)
+            {
+               char	fname[MAXPATHLENGTH+1];
+
+               SetHyperSpaceCursor (drawWindow);
+               if (FormNewFileName (curDir, attr_ptr->s,
+                     (strcmp(attr_ptr->name,TELEPORT_ATTR)==0 ?
+                     OBJ_FILE_EXT : NULL), fname))
+                  SetStringStatus (fname);
+            }
+            else if ((allowLaunchInHyperSpace &&
+                  (attr_ptr=FindAttrWithName(obj_ptr, LAUNCH_ATTR, NULL)) !=
+                  NULL) || (attr_ptr=FindAttrWithName(obj_ptr, EXEC_ATTR,
+                  NULL)) != NULL)
+            {
+               SetHyperSpaceCursor (drawWindow);
+               sprintf (gszMsgBox, "%s%s", attr_ptr->name,
+                     (*attr_ptr->s=='\0' ? "..." : attr_ptr->s));
+               SetStringStatus (gszMsgBox);
+            }
+            else
+            {
+               ShowCursor ();
+               SetMouseStatus ("(none)", "Main Menu", "Mode Menu");
+            }
+         }
+         else
+         {
+            ShowCursor ();
+            SetMouseStatus ("(none)", "Main Menu", "Mode Menu");
+         }
+      }
       return (INVALID);
    }
 
@@ -1201,8 +1526,9 @@ int DrawingEventHandler (input)
          SetCurChoice (NOTHING);
          return (INVALID);
       }
-      else if ((button_ev->button == Button2) && curChoice == NOTHING &&
-            (button_ev->state & (ShiftMask | ControlMask)))
+      else if (((button_ev->button == Button2) && curChoice == NOTHING &&
+            (button_ev->state & (ShiftMask | ControlMask))) ||
+            (inHyperSpace && button_ev->button == Button1))
       {
          Teleport (button_ev);
          return (INVALID);
@@ -1232,3 +1558,4 @@ int DrawingEventHandler (input)
    }
    return (INVALID);
 }
+
