@@ -25,7 +25,7 @@
  */
 #ifndef lint
 static char RCSid[] =
-      "@(#)$Header: /amnt/maui/tangram/u/william/X11/TGIF2/RCS/dup.c,v 2.61 1994/03/21 04:54:08 william Exp $";
+      "@(#)$Header: /export/bourbon/multimedia/william/X11/TGIF2/RCS/dup.c,v 2.63 1994/11/07 04:01:57 william Exp $";
 #endif
 
 #include <stdio.h>
@@ -573,6 +573,10 @@ void DupTextObj (TextPtr, ObjPtr)
    ObjPtr->detail.t = text_ptr;
 }
 
+#ifdef _NO_EXTERN
+struct ObjRec	* DupObj ARGS_DECL((struct ObjRec *));
+#endif /* _NO_EXTERN */
+
 static
 void DupGroupObj (GroupPtr, ObjPtr)
    struct GroupRec	* GroupPtr;
@@ -654,6 +658,7 @@ struct ObjRec * DupObj (ObjPtr)
             strcpy (obj_ptr->detail.r->s, ObjPtr->detail.r->s);
             obj_ptr->detail.r->rotate = ObjPtr->detail.r->rotate;
             obj_ptr->detail.r->flip = ObjPtr->detail.r->flip;
+            obj_ptr->detail.r->deck_index = (-1);
          }
          break;
    }
