@@ -25,12 +25,15 @@
  */
 #ifndef lint
 static char RCSid[] =
-      "@(#)$Header: /amnt/maui/tangram/u/william/X11/TGIF2/RCS/edit.c,v 2.111 1994/05/01 23:17:42 william Exp $";
+      "@(#)$Header: /export/bourbon/multimedia/william/X11/TGIF2/RCS/edit.c,v 2.119 1994/11/30 06:13:15 william Exp $";
 #endif
 
 #include <stdio.h>
 #include <math.h>
+#include <sys/types.h>
 #include <X11/Xlib.h>
+#include <X11/Xutil.h>
+#include <X11/keysym.h>
 #include "const.h"
 #include "types.h"
 
@@ -101,7 +104,9 @@ char * editMenuStr[] =
         "ConvertIntSpline ",
         "Smooth <-> Hinge ",
         "MakeRegPolygon #\"",
-        "BreakUpText    ##"
+        "BreakUpText    ##",
+        "SetSelLineWidth  ",
+        "AddColor         "
       };
 
 static char * editMenuDescription[] =
@@ -131,6 +136,8 @@ static char * editMenuDescription[] =
         "Toggle smooth/hinge vertices for a selected poly/polygon/spline",
         "Make a selected polygon/closed spline into a regular one",
         "Break up selected text objects into one-character text objects",
+        "Change the line width (and arrow width/height) of selected objects",
+        "Add colors to the current palette",
         NULL
       };
 
@@ -338,6 +345,8 @@ void SelectModeToggleSmoothHinge ()
       return;
    }
    PrepareToRecord (CMD_REPLACE, topSel, botSel, numObjSelected);
+   SaveStatusStrings ();
+   SetMouseStatus ("Toggle smooth/hinge points", "Cancel", "Cancel");
    TwoLineMsg ("Click left mouse button to toggle Smooth/Hinge points.",
          "Click other buttons to quit.");
 
@@ -438,6 +447,7 @@ void SelectModeToggleSmoothHinge ()
          while (XCheckMaskEvent (mainDisplay, PointerMotionMask, &ev)) ;
       }
    }
+   RestoreStatusStrings ();
    if (pt_toggled)
       RecordCmd (CMD_REPLACE, NULL, topSel, botSel, numObjSelected);
    else
@@ -817,7 +827,7 @@ void MakeRegularPolygon ()
    }
 
    sprintf (msg, "Do you want a vertex at the 3 o'clock position?  [ync](y)");
-   if ((vertex_at_right = YesNoCancel (msg, CONFIRM_YES)) == CONFIRM_CANCEL)
+   if ((vertex_at_right = MsgBox (msg, TOOL_NAME, YNC_MB)) == MB_ID_CANCEL)
       return;
 
    tmpTopObj = tmpBotObj = NULL;
@@ -914,6 +924,8 @@ void DeletePoint ()
       case OBJ_POLY: poly_ptr = obj_ptr->detail.p; break;
       case OBJ_POLYGON: polygon_ptr = obj_ptr->detail.g; break;
    }
+   SaveStatusStrings ();
+   SetMouseStatus ("Delete a vertex", "Cancel", "Cancel");
    TwoLineMsg ("Click left mouse button to DELETE points.",
          "Click other buttons to quit.");
 
@@ -1062,6 +1074,7 @@ void DeletePoint ()
          while (XCheckMaskEvent (mainDisplay, PointerMotionMask, &ev)) ;
       }
    }
+   RestoreStatusStrings ();
    if (pt_deleted)
    {
       if (topSel == NULL)
@@ -1073,6 +1086,8 @@ void DeletePoint ()
       AbortPrepareCmd (CMD_REPLACE);
 }
 
+static XComposeStatus	c_stat;
+
 static
 int ContinueAddPolyPoint (ObjPtr, MouseX, MouseY, Index, PolyPtr,
       LastMouseX, LastMouseY)
@@ -1082,13 +1097,13 @@ int ContinueAddPolyPoint (ObjPtr, MouseX, MouseY, Index, PolyPtr,
    int			* LastMouseX, * LastMouseY;
    /* (MouseX,MouseY) is the mouse's origin in screen offsets */
 {
-   int		n = PolyPtr->n, sn = 0, curved=PolyPtr->curved;
-   int		already_moved = FALSE, done = FALSE, before = FALSE;
-   XPoint	* vs=PolyPtr->vlist, v[3], * sv = NULL, * pv=NULL;
+   int		n=PolyPtr->n, sn=0, curved=PolyPtr->curved;
+   int		already_moved=FALSE, done=FALSE, before=FALSE;
+   XPoint	* vs=PolyPtr->vlist, v[3], * sv=NULL, * pv=NULL;
    XPoint	* cntrlv=NULL;
    int		prev_x, prev_y, x, y, next_x, next_y, new_x, new_y;
    int		orig_x, orig_y, grid_x, grid_y, new_mouse_x, new_mouse_y;
-   int		sel_ltx, sel_lty, sel_rbx, sel_rby, num = 0, i, intn=0;
+   int		sel_ltx, sel_lty, sel_rbx, sel_rby, num=0, i, intn=0;
    char		buf[80], * smooth=PolyPtr->smooth, * tmp_smooth=NULL;
    double	prev_angle, next_angle, new_angle, theta_1, theta_2;
    XEvent	input, ev;
@@ -1151,8 +1166,6 @@ int ContinueAddPolyPoint (ObjPtr, MouseX, MouseY, Index, PolyPtr,
 
    GridXY (MouseX, MouseY, &orig_x, &orig_y);
    new_mouse_x = MouseX; new_mouse_y = MouseY;
-   /* XDrawString (mainDisplay, drawWindow, revDefaultGC, new_mouse_x+4,
-         new_mouse_y+defaultFontAsc, "ADD", 3); */
    sprintf (buf, "ADD: 0x0%+1d%+1d", x, y);
    StartShowMeasureCursor (OFFSET_X(x), OFFSET_Y(y), buf, FALSE);
 
@@ -1164,12 +1177,8 @@ int ContinueAddPolyPoint (ObjPtr, MouseX, MouseY, Index, PolyPtr,
          ExposeEventHandler (&input, TRUE);
       else if (input.type == MotionNotify)
       {
-         /* XDrawString (mainDisplay, drawWindow, revDefaultGC, new_mouse_x+4,
-               new_mouse_y+defaultFontAsc, "ADD", 3); */
          new_mouse_x = input.xmotion.x;
          new_mouse_y = input.xmotion.y;
-         /* XDrawString (mainDisplay, drawWindow, revDefaultGC, new_mouse_x+4,
-               new_mouse_y+defaultFontAsc, "ADD", 3); */
 
          GridXY (new_mouse_x, new_mouse_y, &grid_x, &grid_y);
          new_x = ABS_SIZE(new_mouse_x-MouseX) + x;
@@ -1232,8 +1241,8 @@ int ContinueAddPolyPoint (ObjPtr, MouseX, MouseY, Index, PolyPtr,
                            drawOrigX, drawOrigY, n+1, pv);
                      break;
                }
-               XDrawLines (mainDisplay, drawWindow, revDefaultGC,
-                     PolyPtr->svlist, PolyPtr->sn, CoordModeOrigin);
+               XDrawLines (mainDisplay, drawWindow, revDefaultGC, sv, sn,
+                     CoordModeOrigin);
             }
          }
          else
@@ -1290,27 +1299,31 @@ int ContinueAddPolyPoint (ObjPtr, MouseX, MouseY, Index, PolyPtr,
       {
          done = TRUE;
          *LastMouseX = new_mouse_x; *LastMouseY = new_mouse_y;
-         /* MARK(drawWindow, revDefaultGC, OFFSET_X(vs[Index].x),
-               OFFSET_Y(vs[Index].y)); */
          if (curved == LT_INTSPLINE || smooth == NULL || !smooth[Index])
             MARK(drawWindow, revDefaultGC, OFFSET_X(vs[Index].x),
                   OFFSET_Y(vs[Index].y));
          else
             MARKO(drawWindow, revDefaultGC, OFFSET_X(vs[Index].x),
                   OFFSET_Y(vs[Index].y));
-         /* XDrawString (mainDisplay, drawWindow, revDefaultGC, new_mouse_x+4,
-               new_mouse_y+defaultFontAsc, "ADD", 3); */
-         sprintf (buf, "ADD: %1dx%1d%+1d%+1d", abs(ABS_X(v[1].x)-x),
-               abs(ABS_Y(v[1].y)-y), ABS_X(v[1].x), ABS_Y(v[1].y));
-         EndShowMeasureCursor (v[1].x, v[1].y, buf, FALSE);
+         if (already_moved)
+         {
+            sprintf (buf, "ADD: %1dx%1d%+1d%+1d", abs(ABS_X(v[1].x)-x),
+                  abs(ABS_Y(v[1].y)-y), ABS_X(v[1].x), ABS_Y(v[1].y));
+            EndShowMeasureCursor (v[1].x, v[1].y, buf, FALSE);
+         }
+         else
+         {
+            sprintf (buf, "ADD: 0x0%+1d%+1d", x, y);
+            StartShowMeasureCursor (OFFSET_X(x), OFFSET_Y(y), buf, FALSE);
+         }
 
          if (!already_moved)
             return (FALSE);
          else
          {
             if (splineRubberband)
-               XDrawLines (mainDisplay, drawWindow, revDefaultGC,
-                     PolyPtr->svlist, PolyPtr->sn, CoordModeOrigin);
+               XDrawLines (mainDisplay, drawWindow, revDefaultGC, sv, sn,
+                     CoordModeOrigin);
             if (grid_x == orig_x && grid_y == orig_y)
                return (FALSE);
          }
@@ -1381,6 +1394,48 @@ int ContinueAddPolyPoint (ObjPtr, MouseX, MouseY, Index, PolyPtr,
          SetFileModified (TRUE);
          justDupped = FALSE;
       }
+      else if (input.type == KeyPress)
+      {
+         KeySym	key_sym;
+         char	s[80];
+
+         XLookupString (&(input.xkey), s, 80-1, &key_sym, &c_stat);
+         TranslateKeys (s, &key_sym);
+         if (s[0] == '\033' && (key_sym & 0xff) == '\033')
+         {
+            done = TRUE;
+            *LastMouseX = new_mouse_x; *LastMouseY = new_mouse_y;
+            if (curved == LT_INTSPLINE || smooth == NULL || !smooth[Index])
+               MARK(drawWindow, revDefaultGC, OFFSET_X(vs[Index].x),
+                     OFFSET_Y(vs[Index].y));
+            else
+               MARKO(drawWindow, revDefaultGC, OFFSET_X(vs[Index].x),
+                     OFFSET_Y(vs[Index].y));
+            if (already_moved)
+            {
+               sprintf (buf, "ADD: %1dx%1d%+1d%+1d", abs(ABS_X(v[1].x)-x),
+                     abs(ABS_Y(v[1].y)-y), ABS_X(v[1].x), ABS_Y(v[1].y));
+               EndShowMeasureCursor (v[1].x, v[1].y, buf, FALSE);
+            }
+            else
+            {
+               sprintf (buf, "ADD: 0x0%+1d%+1d", x, y);
+               StartShowMeasureCursor (OFFSET_X(x), OFFSET_Y(y), buf, FALSE);
+            }
+
+            if (already_moved)
+            {
+               if (splineRubberband)
+                  XDrawLines (mainDisplay, drawWindow, revDefaultGC, sv, sn,
+                        CoordModeOrigin);
+               else
+                  MyDashedLine (drawWindow, revDefaultGC, v, 3);
+               if (grid_x == orig_x && grid_y == orig_y)
+                  return (FALSE);
+            }
+            return (FALSE);
+         }
+      }
    }
    return (TRUE);
 }
@@ -1394,9 +1449,9 @@ int ContinueAddPolygonPoint (ObjPtr, MouseX, MouseY, Index, PolygonPtr,
    int			* LastMouseX, * LastMouseY;
    /* (MouseX,MouseY) is the mouse's origin in screen offsets */
 {
-   int		n = PolygonPtr->n, sn = 0, curved=PolygonPtr->curved;
-   int		already_moved = FALSE, done = FALSE, before = FALSE;
-   XPoint	* vs=PolygonPtr->vlist, v[3], * sv = NULL, * pv=NULL;
+   int		n=PolygonPtr->n, sn=0, curved=PolygonPtr->curved;
+   int		already_moved=FALSE, done=FALSE, before=FALSE;
+   XPoint	* vs=PolygonPtr->vlist, v[3], * sv=NULL, * pv=NULL;
    XPoint	* cntrlv=NULL;
    int		prev_x, prev_y, x, y, next_x, next_y, new_x, new_y;
    int		orig_x, orig_y, grid_x, grid_y, new_mouse_x, new_mouse_y;
@@ -1411,8 +1466,6 @@ int ContinueAddPolygonPoint (ObjPtr, MouseX, MouseY, Index, PolygonPtr,
    else
       MARKO(drawWindow, revDefaultGC, OFFSET_X(vs[Index].x),
             OFFSET_Y(vs[Index].y));
-   /* MARK(drawWindow, revDefaultGC, OFFSET_X(vs[Index].x),
-         OFFSET_Y(vs[Index].y)); */
 
    sel_ltx =  selLtX; sel_lty = selLtY;
    sel_rbx =  selRbX; sel_rby = selRbY;
@@ -1460,8 +1513,6 @@ int ContinueAddPolygonPoint (ObjPtr, MouseX, MouseY, Index, PolygonPtr,
 
    GridXY (MouseX, MouseY, &orig_x, &orig_y);
    new_mouse_x = MouseX; new_mouse_y = MouseY;
-   /* XDrawString (mainDisplay, drawWindow, revDefaultGC, new_mouse_x+4,
-         new_mouse_y+defaultFontAsc, "ADD", 3); */
    sprintf (buf, "ADD: 0x0%+1d%+1d", x, y);
    StartShowMeasureCursor (OFFSET_X(x), OFFSET_Y(y), buf, FALSE);
 
@@ -1473,12 +1524,8 @@ int ContinueAddPolygonPoint (ObjPtr, MouseX, MouseY, Index, PolygonPtr,
          ExposeEventHandler (&input, TRUE);
       else if (input.type == MotionNotify)
       {
-         /* XDrawString (mainDisplay, drawWindow, revDefaultGC, new_mouse_x+4,
-               new_mouse_y+defaultFontAsc, "ADD", 3); */
          new_mouse_x = input.xmotion.x;
          new_mouse_y = input.xmotion.y;
-         /* XDrawString (mainDisplay, drawWindow, revDefaultGC, new_mouse_x+4,
-               new_mouse_y+defaultFontAsc, "ADD", 3); */
 
          GridXY (new_mouse_x, new_mouse_y, &grid_x, &grid_y);
          new_x = ABS_SIZE(new_mouse_x-MouseX) + x;
@@ -1521,8 +1568,8 @@ int ContinueAddPolygonPoint (ObjPtr, MouseX, MouseY, Index, PolygonPtr,
                            drawOrigX, drawOrigY, n+1, pv);
                      break;
                }
-               XDrawLines (mainDisplay, drawWindow, revDefaultGC,
-                     PolygonPtr->svlist, PolygonPtr->sn, CoordModeOrigin);
+               XDrawLines (mainDisplay, drawWindow, revDefaultGC, sv, sn,
+                     CoordModeOrigin);
             }
          }
          else
@@ -1595,21 +1642,25 @@ int ContinueAddPolygonPoint (ObjPtr, MouseX, MouseY, Index, PolygonPtr,
          else
             MARKO(drawWindow, revDefaultGC, OFFSET_X(vs[Index].x),
                   OFFSET_Y(vs[Index].y));
-         /* MARK(drawWindow, revDefaultGC, OFFSET_X(vs[Index].x),
-               OFFSET_Y(vs[Index].y)); */
-         /* XDrawString (mainDisplay, drawWindow, revDefaultGC, new_mouse_x+4,
-               new_mouse_y+defaultFontAsc, "ADD", 3); */
-         sprintf (buf, "ADD: %1dx%1d%+1d%+1d", abs(ABS_X(v[1].x)-x),
-               abs(ABS_Y(v[1].y)-y), ABS_X(v[1].x), ABS_Y(v[1].y));
-         EndShowMeasureCursor (v[1].x, v[1].y, buf, FALSE);
+         if (already_moved)
+         {
+            sprintf (buf, "ADD: %1dx%1d%+1d%+1d", abs(ABS_X(v[1].x)-x),
+                  abs(ABS_Y(v[1].y)-y), ABS_X(v[1].x), ABS_Y(v[1].y));
+            EndShowMeasureCursor (v[1].x, v[1].y, buf, FALSE);
+         }
+         else
+         {
+            sprintf (buf, "ADD: 0x0%+1d%+1d", x, y);
+            StartShowMeasureCursor (OFFSET_X(x), OFFSET_Y(y), buf, FALSE);
+         }
 
          if (!already_moved)
             return (FALSE);
          else
          {
             if (splineRubberband)
-               XDrawLines (mainDisplay, drawWindow, revDefaultGC,
-                     PolygonPtr->svlist, PolygonPtr->sn, CoordModeOrigin);
+               XDrawLines (mainDisplay, drawWindow, revDefaultGC, sv, sn,
+                     CoordModeOrigin);
             if (grid_x == orig_x && grid_y == orig_y)
                return (FALSE);
          }
@@ -1705,6 +1756,48 @@ int ContinueAddPolygonPoint (ObjPtr, MouseX, MouseY, Index, PolygonPtr,
          SetFileModified (TRUE);
          justDupped = FALSE;
       }
+      else if (input.type == KeyPress)
+      {
+         KeySym	key_sym;
+         char	s[80];
+
+         XLookupString (&(input.xkey), s, 80-1, &key_sym, &c_stat);
+         TranslateKeys (s, &key_sym);
+         if (s[0] == '\033' && (key_sym & 0xff) == '\033')
+         {
+            done = TRUE;
+            *LastMouseX = new_mouse_x; *LastMouseY = new_mouse_y;
+            if (curved == LT_INTSPLINE || smooth == NULL || !smooth[Index])
+               MARK(drawWindow, revDefaultGC, OFFSET_X(vs[Index].x),
+                     OFFSET_Y(vs[Index].y));
+            else
+               MARKO(drawWindow, revDefaultGC, OFFSET_X(vs[Index].x),
+                     OFFSET_Y(vs[Index].y));
+            if (already_moved)
+            {
+               sprintf (buf, "ADD: %1dx%1d%+1d%+1d", abs(ABS_X(v[1].x)-x),
+                     abs(ABS_Y(v[1].y)-y), ABS_X(v[1].x), ABS_Y(v[1].y));
+               EndShowMeasureCursor (v[1].x, v[1].y, buf, FALSE);
+            }
+            else
+            {
+               sprintf (buf, "ADD: 0x0%+1d%+1d", x, y);
+               StartShowMeasureCursor (OFFSET_X(x), OFFSET_Y(y), buf, FALSE);
+            }
+
+            if (already_moved)
+            {
+               if (splineRubberband)
+                  XDrawLines (mainDisplay, drawWindow, revDefaultGC, sv, sn,
+                        CoordModeOrigin);
+               else
+                  MyDashedLine (drawWindow, revDefaultGC, v, 3);
+               if (grid_x == orig_x && grid_y == orig_y)
+                  return (FALSE);
+            }
+            return (FALSE);
+         }
+      }
    }
    return (TRUE);
 }
@@ -1744,6 +1837,8 @@ void AddPoint ()
       case OBJ_POLY: poly_ptr = obj_ptr->detail.p; break;
       case OBJ_POLYGON: polygon_ptr = obj_ptr->detail.g; break;
    }
+   SaveStatusStrings ();
+   SetMouseStatus ("Add a vertex", "Cancel", "Cancel");
    TwoLineMsg ("Drag left mouse button to ADD points.",
          "Click other buttons to quit.");
 
@@ -1809,6 +1904,7 @@ void AddPoint ()
          while (XCheckMaskEvent (mainDisplay, PointerMotionMask, &ev)) ;
       }
    }
+   RestoreStatusStrings ();
    if (pt_added)
       RecordCmd (CMD_REPLACE, NULL, topSel, botSel, numObjSelected);
    else
@@ -1966,6 +2062,8 @@ void EditSubMenu (index)
       case EDIT_SMOOTH_HINGE: ToggleSmoothHinge (); break;
       case EDIT_MAKE_REGULAR: MakeRegularPolygon (); break;
       case EDIT_BREAK_TEXT: BreakUpText (); break;
+      case EDIT_SET_SEL_LINEWIDTH: SetSelectedLineWidth (); break;
+      case EDIT_ADD_COLOR: AddColor (); break;
    }
 }
 
