@@ -25,7 +25,7 @@
  */
 #ifndef lint
 static char RCSid[] =
-      "@(#)$Header: /amnt/maui/tangram/u/william/X11/TGIF2/RCS/eps.c,v 2.16 1994/03/21 04:54:18 william Exp $";
+      "@(#)$Header: /export/bourbon/multimedia/william/X11/TGIF2/RCS/eps.c,v 2.21 1994/12/24 22:22:58 william Exp $";
 #endif
 
 #include <sys/types.h>
@@ -695,6 +695,8 @@ void DumpEPSObj (FP, ObjPtr)
    fprintf (FP, "/showpage {} def\n");
    fprintf (FP, "/letter {} def\n");
    fprintf (FP, "/legal {} def\n");
+   fprintf (FP, "0 setgray 0 setlinecap 1 setlinewidth\n");
+   fprintf (FP, "0 setlinejoin 10 setmiterlimit [] 0 setdash newpath\n");
    fprintf (FP, "1 %1d %1d div %1d mul 100 div div dup neg scale\n",
          psDotsPerInch, PIX_PER_INCH, printMag);
    fprintf (FP, "%1d %s mul neg %1d %s mul neg translate\n", psDotsPerInch,
@@ -723,7 +725,7 @@ struct ObjRec * CreateEPSObj (FileName, ImageW, ImageH, bitmap, image,
    int		ImageW, ImageH, num_lines, epsf_level;
    Pixmap	bitmap;
    XImage	* image;
-   float	llx, lly, urx, ury;
+   float	* llx, * lly, * urx, * ury;
 {
    struct XBmRec	* xbm_ptr;
    struct ObjRec	* obj_ptr;
@@ -742,8 +744,10 @@ struct ObjRec * CreateEPSObj (FileName, ImageW, ImageH, bitmap, image,
 
    if (bitmap == None)
    {
-      xbm_ptr->eps_w = w = (urx >= llx) ? (int)(urx-llx) : (int)(llx-urx);
-      xbm_ptr->eps_h = h = (ury >= lly) ? (int)(ury-lly) : (int)(lly-ury);
+      xbm_ptr->eps_w = w = ((*urx) >= (*llx)) ? (int)((*urx)-(*llx)) :
+            (int)((*llx)-(*urx));
+      xbm_ptr->eps_h = h = ((*ury) >= (*lly)) ? (int)((*ury)-(*lly)) :
+            (int)((*lly)-(*ury));
    }
    else
    {
@@ -754,10 +758,10 @@ struct ObjRec * CreateEPSObj (FileName, ImageW, ImageH, bitmap, image,
    xbm_ptr->fill = objFill;
    xbm_ptr->rotate = xbm_ptr->cached_rotate = ROTATE0;
    xbm_ptr->flip = xbm_ptr->cached_flip = 0;
-   xbm_ptr->llx = (int)(llx*1000.0);
-   xbm_ptr->lly = (int)(lly*1000.0);
-   xbm_ptr->urx = (int)(urx*1000.0);
-   xbm_ptr->ury = (int)(ury*1000.0);
+   xbm_ptr->llx = (int)((*llx)*1000.0);
+   xbm_ptr->lly = (int)((*lly)*1000.0);
+   xbm_ptr->urx = (int)((*urx)*1000.0);
+   xbm_ptr->ury = (int)((*ury)*1000.0);
    xbm_ptr->cached_zoom = 0;
    xbm_ptr->cached_bitmap = None;
    xbm_ptr->cached_w = xbm_ptr->cached_h = 0;
@@ -825,7 +829,7 @@ void UpdateEPS ()
    HighLightReverse ();
    PrepareToReplaceAnObj (obj_ptr);
    new_obj_ptr = CreateEPSObj (xbm_ptr->filename, image_w, image_h, bitmap,
-         image, num_lines, lines, epsf_level, llx, lly, urx, ury,
+         image, num_lines, lines, epsf_level, &llx, &lly, &urx, &ury,
          write_date);
 
    new_obj_ptr->x = obj_ptr->x;
