@@ -25,7 +25,7 @@
  */
 #ifndef lint
 static char RCSid[] =
-      "@(#)$Header: /amnt/maui/tangram/u/william/X11/TGIF2/RCS/font.c,v 2.87 1994/04/18 04:34:41 william Exp $";
+      "@(#)$Header: /export/bourbon/multimedia/william/X11/TGIF2/RCS/font.c,v 2.96 1994/12/14 05:34:52 william Exp $";
 #endif
 
 #include <stdio.h>
@@ -105,6 +105,10 @@ int	curSize = 20;
 int	curStyle = STYLE_NR;
 int	curRotate = ROTATE0;
 
+static int	defaultCurFont=(-1);
+static int	defaultCurSize=(-1);
+static int	defaultCurStyle=(-1);
+
 static char	* initSizeMenuStr[] = {
       "8 ", "10", "11", "12", "14", "17", "18", "20", "24", "25", "34" };
 
@@ -169,49 +173,108 @@ static char	* charCodeToName[] =
 {
 /* \200 */ "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "",
 /* \220 */ "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "",
-/* \240 */ "x", "", "", "", "\\250", "", "x", "",
-           "x", "x", "\\343", "", "x", "x", "x", "x",
-/* \260 */ "x", "x", "x", "x", "\\302", "x", "", "\\264",
-           "\\313", "x", "\\353", "", "x", "x", "x", "",
-/* \300 */ "8#260 /Agrave", "8#265 /Aacute", "8#276 /Acircumflex",
-        /* "8#300 /Atilde", "8#311 /Adieresis", "8#314 /Aring", "\\341", */
-           "8#300 /Atilde", "8#311 /Adieresis", "8#314 /Aring", "8#306 /AE",
-           "8#321 /Ccedilla", "8#322 /Egrave", "8#323 /Eacute",
-           "8#324 /Ecircumflex", "8#325 /Edieresis", "8#326 /Igrave",
-           "8#327 /Iacute", "8#330 /Icircumflex", "8#331 /Idieresis",
-/* \320    "x", "8#332 /Ntilde", "8#333 /Ograve", "8#334 /Oacute", */
-/* \320 */ "8#320 /Eth", "8#332 /Ntilde", "8#333 /Ograve", "8#334 /Oacute",
-        /* "8#335 /Ocircumflex", "8#336 /Otilde", "8#337 /Odieresis", "x", */
-           "8#335 /Ocircumflex", "8#336 /Otilde", "8#337 /Odieresis",
+/* \240 */ "x",
+           "8#241 /exclamdown",
+           "8#242 /cent",
+           "8#243 /sterling",
+           "8#244 /currency",
+           "8#245 /yen",
+           "8#246 /bar",
+           "8#247 /section",
+           "8#250 /dieresis",
+           "8#251 /copyright",
+           "8#252 /ordfeminine",
+           "8#253 /guillemotleft",
+           "8#254 /logicalnot",
+           "8#255 /emdash",
+           "8#256 /registered",
+           "8#257 /macron",
+/* \260 */ "8#260 /degree",
+           "8#261 /plusminus",
+           "x",
+           "x",
+           "8#264 /acute",
+           "8#265 /mu",
+           "8#266 /paragraph",
+           "8#267 /periodcentered",
+           "8#270 /cedilla",
+           "x",
+           "8#272 /ordmasculine",
+           "8#273 /guillemotright",
+           "x",
+           "x",
+           "x",
+           "8#277 /questiondown",
+/* \300 */ "8#300 /Agrave",
+           "8#301 /Aacute",
+           "8#302 /Acircumflex",
+           "8#303 /Atilde",
+           "8#304 /Adieresis",
+           "8#305 /Aring",
+           "8#306 /AE",
+           "8#307 /Ccedilla",
+           "8#310 /Egrave",
+           "8#311 /Eacute",
+           "8#312 /Ecircumflex",
+           "8#313 /Edieresis",
+           "8#314 /Igrave",
+           "8#315 /Iacute",
+           "8#316 /Icircumflex",
+           "8#317 /Idieresis",
+/* \320 */ "8#320 /Eth",
+           "8#321 /Ntilde",
+           "8#322 /Ograve",
+           "8#323 /Oacute",
+           "8#324 /Ocircumflex",
+           "8#325 /Otilde",
+           "8#326 /Odieresis",
            "8#327 /multiply",
-        /* "\\351", "8#340 /Ugrave", "8#342 /Uacute", "8#344 /Ucircumflex", */
-           "8#330 /Oslash", "8#340 /Ugrave", "8#342 /Uacute",
-           "8#344 /Ucircumflex",
-        /* "8#345 /Udieresis", "x", "x", "\\373", */
-           "8#345 /Udieresis", "8#335 /Yacute", "8#336 /Thorn",
-           "8#373 /germandbls",
-/* \340 */ "8#346 /agrave", "8#347 /aacute", "8#354 /acircumflex",
-        /* "8#355 /atilde", "8#356 /adieresis", "8#357 /aring", "\\361", */
-           "8#355 /atilde", "8#356 /adieresis", "8#357 /aring", "8#346 /ae",
-           "8#360 /ccedilla", "8#362 /egrave", "8#363 /eacute",
-           "8#364 /ecircumflex", "8#366 /edieresis", "8#367 /igrave",
-           "8#374 /iacute", "8#375 /icircumflex", "8#376 /idieresis",
-/* \360    "x", "8#254 /ntilde", "8#255 /ograve", "8#256 /oacute", */
-/* \360 */ "8#360 /eth", "8#254 /ntilde", "8#255 /ograve", "8#256 /oacute",
-        /* "8#257 /ocircumflex", "8#271 /otilde", "8#274 /odieresis", "x", */
-           "8#257 /ocircumflex", "8#271 /otilde", "8#274 /odieresis",
+           "8#330 /Oslash",
+           "8#331 /Ugrave",
+           "8#332 /Uacute",
+           "8#333 /Ucircumflex",
+           "8#334 /Udieresis",
+           "8#335 /Yacute",
+           "8#336 /Thorn",
+           "8#337 /germandbls",
+/* \340 */ "8#340 /agrave",
+           "8#341 /aacute",
+           "8#342 /acircumflex",
+           "8#343 /atilde",
+           "8#344 /adieresis",
+           "8#345 /aring",
+           "8#346 /ae",
+           "8#347 /ccedilla",
+           "8#350 /egrave",
+           "8#351 /eacute",
+           "8#352 /ecircumflex",
+           "8#353 /edieresis",
+           "8#354 /igrave",
+           "8#355 /iacute",
+           "8#356 /icircumflex",
+           "8#357 /idieresis",
+/* \360 */ "8#360 /eth",
+           "8#361 /ntilde",
+           "8#362 /ograve",
+           "8#363 /oacute",
+           "8#364 /ocircumflex",
+           "8#365 /otilde",
+           "8#366 /odieresis",
            "8#367 /divide",
-        /* "\\371", "8#275 /ugrave", "8#350 /uacute", "8#352 /ucircumflex", */
-           "8#370 /oslash", "8#275 /ugrave", "8#350 /uacute",
-           "8#352 /ucircumflex",
-        /* "8#370 /udieresis", "x", "x", "8#372 /ydieresis" */
-           "8#370 /udieresis", "8#375 /yacute", "8#376 /thorn",
-           "8#372 /ydieresis"
+           "8#370 /oslash",
+           "8#371 /ugrave",
+           "8#372 /uacute",
+           "8#373 /ucircumflex",
+           "8#374 /udieresis",
+           "8#375 /yacute",
+           "8#376 /thorn",
+           "8#377 /ydieresis"
 };
 
 #define InfoIndex(FontIdx,StyleIdx) (((FontIdx)*MAXFONTSTYLES+(StyleIdx))*3)
 
 static int	debugScalableFonts=FALSE; /* debug scalable font */
+static char	gszAttemptedFontName[MAXSTRING+1];
 
 static
 struct FontSizeRec * FindFontInfo (font_index, style_index, size)
@@ -219,10 +282,10 @@ struct FontSizeRec * FindFontInfo (font_index, style_index, size)
 {
    struct FontSizeRec	* fs_ptr, * prev_fs=NULL, * new_fs_ptr;
    int			info_index, watch_cursor;
-   char			font_name[MAXSTRING+1], msg[MAXSTRING];
    XFontStruct		* xfs=NULL;
 
    canvasFontIsFaked = FALSE;
+   *gszAttemptedFontName = '\0';
    for (fs_ptr=fontFamilies[font_index].fr[style_index].next; fs_ptr != NULL;
          fs_ptr = fs_ptr->next)
    {
@@ -237,9 +300,9 @@ struct FontSizeRec * FindFontInfo (font_index, style_index, size)
    }
    info_index = InfoIndex (font_index, style_index);
    if (strstr (fontInfoStr[info_index], "%d") != NULL)
-      sprintf (font_name, fontInfoStr[info_index], size);
+      sprintf (gszAttemptedFontName, fontInfoStr[info_index], size);
    else
-      sprintf (font_name, "%s-%s-*-%1d-*-*-*-*-*-%s", fontNamePrefix,
+      sprintf (gszAttemptedFontName, "%s-%s-*-%1d-*-*-*-*-*-%s", fontNamePrefix,
             fontInfoStr[info_index], size, fontInfoStr[info_index+1]);
 
    watch_cursor = watchCursorOnMainWindow;
@@ -248,7 +311,7 @@ struct FontSizeRec * FindFontInfo (font_index, style_index, size)
       SetWatchCursor (drawWindow);
       SetWatchCursor (mainWindow);
    }
-   xfs = XLoadQueryFont (mainDisplay, font_name);
+   xfs = XLoadQueryFont (mainDisplay, gszAttemptedFontName);
    if (xfs != NULL && debugScalableFonts)
    {
       int	i;
@@ -268,32 +331,63 @@ struct FontSizeRec * FindFontInfo (font_index, style_index, size)
       xfs = fontFamilies[font_index].fr[style_index].xfs;
       if (xfs == NULL)
       {
+         char font_name[MAXSTRING+1];
+
          if (strstr (fontInfoStr[info_index], "%d") != NULL)
-            sprintf (font_name, fontInfoStr[info_index], defaultFontSize);
+            sprintf (font_name, fontInfoStr[info_index],
+                  defaultFontSize);
          else
-            sprintf (font_name, "%s-%s-*-%1d-*-*-*-*-*-%s", fontNamePrefix,
-                  fontInfoStr[info_index], defaultFontSize,
+            sprintf (font_name, "%s-%s-*-%1d-*-*-*-*-*-%s",
+                  fontNamePrefix, fontInfoStr[info_index], defaultFontSize,
                   fontInfoStr[info_index+1]);
          xfs = XLoadQueryFont (mainDisplay, font_name);
          fontFamilies[font_index].fr[style_index].xfs = xfs;
          fontFamilies[font_index].fr[style_index].size = defaultFontSize;
+         if (defaultCurFont == (-1))
+         {
+            defaultCurFont = font_index;
+            defaultCurStyle = style_index;
+            defaultCurSize = defaultFontSize;
+         }
          if (xfs == NULL)
          {
             if (!attemptingToSetFontProperty)
             {
-               sprintf (msg, "Can not open '%s'!  Abort!", font_name);
-               Error ("OpenFont()", msg );
+               int default_info_index=InfoIndex(defaultCurFont,defaultCurStyle);
+
+               if (strstr (fontInfoStr[default_info_index], "%d") != NULL)
+                  sprintf (font_name, fontInfoStr[default_info_index],
+                        defaultCurSize);
+               else
+                  sprintf (font_name, "%s-%s-*-%1d-*-*-*-*-*-%s",
+                        fontNamePrefix, fontInfoStr[default_info_index],
+                        defaultCurSize, fontInfoStr[default_info_index+1]);
+               xfs = XLoadQueryFont(mainDisplay, font_name);
+               fontFamilies[font_index].fr[style_index].xfs = xfs;
+               fontFamilies[font_index].fr[style_index].size = defaultCurSize;
+               if (xfs == NULL) return NULL;
+               sprintf(gszMsgBox, "%s %1d substituted for %s %1d.\n\n%s",
+                     fontInfoStr[default_info_index+2], defaultCurSize,
+                     fontInfoStr[info_index+2], size,
+                     "Such text can not be edited and most likely look ugly.");
+               MsgBox(gszMsgBox, TOOL_NAME, INFO_MB);
             }
          }
          else if (changingFontSizeFromRead)
          {
-            sprintf (msg, "%s %1d substituted for %s %1d...",
+            sprintf (gszMsgBox, "%s %1d substituted for %s %1d...",
                   fontInfoStr[info_index+2], defaultFontSize,
                   fontInfoStr[info_index+2], size);
-            TwoLineMsg (msg, "    such text can not be edited.");
+            TwoLineMsg (gszMsgBox, "    such text can not be edited.");
          }
       }
    }
+   else if (defaultCurFont == (-1))
+   {
+      defaultCurFont = font_index;
+      defaultCurStyle = style_index;
+      defaultCurSize = size;
+   }
    if (!watch_cursor && mainWindow!=None)
    {
       SetDefaultCursor (mainWindow);
@@ -317,16 +411,16 @@ int ValidCharCode (c_ptr)
    char	* c_ptr;
 {
    register int	index = (int)(*c_ptr - '\200');
-   char		msg[MAXSTRING];
 
    if (*charCodeToName[index] == '\0' || *charCodeToName[index] == '8' ||
          *charCodeToName[index] == '\\')
       return (TRUE);
    else
    {
-      sprintf (msg, "Unrecognized character code \\%o.  Character discarded!",
+      sprintf (gszMsgBox,
+         "Unrecognized character code \\%o.  Character discarded!",
          (*c_ptr)&0xff);
-      Msg (msg);
+      Msg (gszMsgBox);
       return (FALSE);
    }
 }
@@ -425,7 +519,8 @@ void PrepareObjFontInfo (ObjPtr)
          case OBJ_TEXT: PrepareText (obj_ptr->detail.t); break;
 
          case OBJ_GROUP:
-         case OBJ_SYM: PrepareObjFontInfo (obj_ptr->detail.r->last); break;
+         case OBJ_SYM:
+         case OBJ_ICON: PrepareObjFontInfo (obj_ptr->detail.r->last); break;
       }
       for (attr_ptr=obj_ptr->lattr; attr_ptr!=NULL; attr_ptr=attr_ptr->prev)
          PrepareText (attr_ptr->obj->detail.t);
@@ -472,7 +567,8 @@ void PrepareEightBitFontInfo ()
             case OBJ_TEXT: PrepareText (obj_ptr->detail.t); break;
 
             case OBJ_GROUP:
-            case OBJ_SYM: PrepareObjFontInfo (obj_ptr->detail.r->last); break;
+            case OBJ_SYM:
+            case OBJ_ICON: PrepareObjFontInfo (obj_ptr->detail.r->last); break;
          }
          for (attr_ptr=obj_ptr->lattr; attr_ptr!=NULL; attr_ptr=attr_ptr->prev)
             PrepareText (attr_ptr->obj->detail.t);
@@ -624,7 +720,7 @@ void DumpEightBitFontInfo (FP)
                {
                   for (k = 0; k < 8; k++)
                      if (flag & (1<<k))
-                        fprintf (FP, " %s\n", charCodeToName[j*8+k]);
+                        fprintf (FP, " %s\n", charCodeToName[(j<<3)+k]);
                }
             }
             fprintf (FP, " ] def\n");
@@ -651,10 +747,14 @@ int GetCompatibleSize (font_dpi, font_size)
 
 void SetCanvasFont ()
 {
+   register int		i;
    struct FontSizeRec	* fs_ptr;
 
    fs_ptr = FindFontInfo (curFont, curStyle, curSize);
 
+   if (!attemptingToSetFontProperty && fs_ptr == NULL) {
+      fs_ptr = FindFontInfo(defaultCurFont, defaultCurStyle, defaultCurSize);
+   }
    if (attemptingToSetFontProperty && fs_ptr == NULL)
    {
       canvasFontSize = INVALID;
@@ -671,7 +771,6 @@ void SetCanvasFont ()
    textCursorH = canvasFontHeight;
 }
 
-static
 void TrimBlanks (Str)
    char	* Str;
 {
@@ -1014,6 +1113,11 @@ void InitFonts ()
       }
    }
    fs_ptr = FindFontInfo (curFont, curStyle, curSize);
+   if (fs_ptr == NULL) {
+      sprintf (gszMsgBox, "Can not open the Default(Msg)Font '%s'!  Abort!",
+            gszAttemptedFontName);
+      Error ("OpenFont()", gszMsgBox);
+   }
 
    defaultFontPtr = fs_ptr->xfs;
    defaultFontWidth = defaultFontPtr->max_bounds.width;
@@ -1021,7 +1125,18 @@ void InitFonts ()
    defaultFontDes = defaultFontPtr->max_bounds.descent;
    defaultFontHeight = defaultFontAsc + defaultFontDes;
 
+   if (defaultFontWidth < 1 || defaultFontHeight < 1)
+      fprintf (stderr, "%s (%1d/%1d).\n\t%s.\n",
+            "Warning:  Very small default font width/height",
+            defaultFontWidth, defaultFontHeight,
+            "Possibly a problem with the font path");
+
    fs_ptr = FindFontInfo (curFont, curStyle, ruler_font_size);
+   if (fs_ptr == NULL) {
+      sprintf (gszMsgBox, "Can not open the RulerFont '%s'!  Abort!",
+            gszAttemptedFontName);
+      Error ("OpenFont()", gszMsgBox);
+   }
 
    rulerFontPtr = fs_ptr->xfs;
    rulerFontWidth = rulerFontPtr->max_bounds.width;
@@ -1042,7 +1157,7 @@ void InitFonts ()
    if ((c_ptr=XGetDefault (mainDisplay, TOOL_NAME, "InitialFontStyle")) != NULL)
    {
       len = strlen (c_ptr);
-      if (len < strlen (styleMenuStr[0]))
+      if (len < ((int) strlen (styleMenuStr[0])))
       {
          for (i = 0; i < MAXFONTSTYLES; i++)
             if (strncmp (c_ptr, styleMenuStr[i], len) == 0)
@@ -1060,7 +1175,7 @@ void InitFonts ()
    if ((c_ptr=XGetDefault (mainDisplay, TOOL_NAME, "InitialFontJust")) != NULL)
    {
       len = strlen (c_ptr);
-      if (len < strlen (styleMenuStr[0]))
+      if (len < ((int) strlen (styleMenuStr[0])))
       {
          for (i = MAXFONTSTYLES+1; i < MAXFONTSTYLES+1+MAXJUSTS; i++)
             if (strncmp (c_ptr, styleMenuStr[i], len) == 0)
