@@ -25,7 +25,7 @@
  */
 #ifndef lint
 static char RCSid[] =
-      "@(#)$Header: /amnt/maui/tangram/u/william/X11/TGIF2/RCS/grid.c,v 2.94 1994/05/03 04:49:10 william Exp $";
+      "@(#)$Header: /export/bourbon/multimedia/william/X11/TGIF2/RCS/grid.c,v 2.102 1994/11/17 05:27:43 william Exp $";
 #endif
 
 #include <stdio.h>
@@ -67,32 +67,35 @@ int	whereToPrint = LATEX_FIG;
 int	moveMode = UNCONST_MOVE;
 int	gridShown = TRUE;
 int	mapShown = TRUE;
+int	usePaperSizeStoredInFile = FALSE;
+int	oneMotionSelectMove = FALSE;
 
 char * layoutMenuStr[] =
-      {  "+Grid        #i",
-         "-Grid        #d",
-         "HideGrid     #g",
-         "SnapOff     ^#g",
-         "ScrollToOrig #^",
-         "SaveOrigin     ",
-         "ZoomIn       #z",
-         "ZoomOut      #o",
-         "DefaultZoom  #:",
-         "ZoomWayOut   #`",
-         "Landscape",
-         "Portrait",
-         "SetReduction #%",
-         "RawPSFile   ^#x",
-         "ColorPS     ^#k",
-         "ConstMove    #@",
-         "MetricGrid  ^#-",
-         "HideBit/Pixmap ",
-         "UseGrayScale   ",
-         "ShowMeasurement",
-         "ShowMenubar    ",
-         "ShowStatus     "
+      {  "+Grid          #i",
+         "-Grid          #d",
+         "HideGrid       #g",
+         "SnapOff       ^#g",
+         "ScrollToOrig   #^",
+         "SaveOrigin       ",
+         "ZoomIn         #z",
+         "ZoomOut        #o",
+         "DefaultZoom    #:",
+         "ZoomWayOut     #`",
+         "Landscape        ",
+         "Portrait         ",
+         "SetReduction   #%",
+         "RawPSFile     ^#x",
+         "ColorPS       ^#k",
+         "ConstMove      #@",
+         "MetricGrid    ^#-",
+         "HideBit/Pixmap   ",
+         "UseGrayScale     ",
+         "ShowMeasurement  ",
+         "ShowMenubar      ",
+         "ShowStatus       ",
+         "ClickSelClickMove"
       };
-#define MAX_LAYOUT_STR 16 /* longest string length in layoutMenuStr[] */
+#define MAX_LAYOUT_STR 18 /* longest string length in layoutMenuStr[] */
 static char * layoutMenuDescription[] =
       {  "Increment grid size",
          "Decrement grid size",
@@ -116,6 +119,7 @@ static char * layoutMenuDescription[] =
          "Show cursor measurements",
          "Show menubar window",
          "Show status window",
+         "Toggle between click-select-move and click-select-click-move modes",
          NULL
       };
 #define MAX_LAYOUT_DESC 80 /* longest string length in layoutMenuStr[] */
@@ -154,7 +158,8 @@ void MyVDotLine (Win, X, YStart, YEnd)
 
 #define FAKE_CM 80
 
-void RedrawGridLines ()
+void RedrawGridLines (Win)
+   Window	Win;
 {
    register int	i, inc = 0, abs_grid = 0;
    int		x_start, y_start, x_end, y_end, x_grid_start, y_grid_start;
@@ -206,9 +211,9 @@ void RedrawGridLines ()
       y_grid_start = ((int)(drawOrigY / step) + 1) * step - drawOrigY;
 
    for (i = x_start; i < x_end; i += inc)
-      MyVDotLine (drawWindow, i, y_grid_start, y_end);
+      MyVDotLine (Win, i, y_grid_start, y_end);
    for (i = y_start; i < y_end; i += inc)
-      MyHDotLine (drawWindow, i, x_grid_start, x_end);
+      MyHDotLine (Win, i, x_grid_start, x_end);
 }
 
 void DrawGridLines (Win, LtX, LtY, W, H)
@@ -281,7 +286,8 @@ void DrawGridLines (Win, LtX, LtY, W, H)
       MyHDotLine (Win, i, x_grid_start, x_grid_end);
 }
 
-void RedrawPageLines ()
+void RedrawPageLines (Win)
+   Window	Win;
 {
    register int	i, x, y;
    int		end;
@@ -293,7 +299,7 @@ void RedrawPageLines ()
    {
       x = OFFSET_X(i);
       if (i >= drawOrigX && i < drawOrigX+drawWinW)
-         XDrawLine (mainDisplay, drawWindow, defaultGC, x, 0, x, end);
+         XDrawLine (mainDisplay, Win, defaultGC, x, 0, x, end);
       if (i > drawOrigX+drawWinW)
          break;
    }
@@ -302,7 +308,7 @@ void RedrawPageLines ()
    {
       y = OFFSET_Y(i);
       if (i >= drawOrigY && i < drawOrigY+drawWinH)
-         XDrawLine (mainDisplay, drawWindow, defaultGC, 0, y, end, y);
+         XDrawLine (mainDisplay, Win, defaultGC, 0, y, end, y);
       if (i > drawOrigY+drawWinH)
          break;
    }
@@ -323,7 +329,7 @@ void DrawPageLines (Win, LtX, LtY, W, H)
    {
       x = OFFSET_X(i);
       if (x >= LtX && x < LtX+W)
-         XDrawLine (mainDisplay, drawWindow, defaultGC, x, start, x, end);
+         XDrawLine (mainDisplay, Win, defaultGC, x, start, x, end);
       if (i > drawOrigX+drawWinW)
          break;
    }
@@ -333,7 +339,7 @@ void DrawPageLines (Win, LtX, LtY, W, H)
    {
       y = OFFSET_Y(i);
       if (y >= LtY && y < LtY+H)
-         XDrawLine (mainDisplay, drawWindow, defaultGC, start, y, end, y);
+         XDrawLine (mainDisplay, Win, defaultGC, start, y, end, y);
       if (i > drawOrigY+drawWinH)
          break;
    }
@@ -391,20 +397,23 @@ static char useGrayDesc[]        =
 static char noGrayDesc[]         =
       "Use tiling patterns as is (may be slow for printing";
 
-static char hideMeasureStr[]     = "HideMeasurement";
-static char showMeasureStr[]     = "ShowMeasurement";
-static char hideMeasureDesc[]    = "Hide cursor measurements";
-static char showMeasureDesc[]    = "Show cursor measurements";
+static char hideMeasureStr[]      = "HideMeasurement";
+static char showMeasureStr[]      = "ShowMeasurement";
+static char hideMeasureDesc[]     = "Hide cursor measurements";
+static char showMeasureDesc[]     = "Show cursor measurements";
 
-static char hideMenubarStr[]     = "HideMenubar";
-static char showMenubarStr[]     = "ShowMenubar";
-static char hideMenubarDesc[]    = "Hide menubar window";
-static char showMenubarDesc[]    = "Show menubar window";
+static char hideMenubarStr[]      = "HideMenubar";
+static char showMenubarStr[]      = "ShowMenubar";
+static char hideMenubarDesc[]     = "Hide menubar window";
+static char showMenubarDesc[]     = "Show menubar window";
 
-static char hideStatusStr[]      = "HideStatus";
-static char showStatusStr[]      = "ShowStatus";
-static char hideStatusDesc[]     = "Hide status window";
-static char showStatusDesc[]     = "Show status window";
+static char hideStatusStr[]       = "HideStatus";
+static char showStatusStr[]       = "ShowStatus";
+static char hideStatusDesc[]      = "Hide status window";
+static char showStatusDesc[]      = "Show status window";
+
+static char oneMotionSelMoveStr[] = "OneMotionSelMove";
+static char clickSelClickMoveStr[]= "ClickSelClickMove";
 
 void ToggleGridSystem ()
 {
@@ -575,7 +584,7 @@ void ToggleWhereToPrint ()
       case PS_FILE: Msg ("Will print raw PostScript file."); break;
       case XBM_FILE:
          if (colorDump)
-            Msg ("Will print X11 Pitmap file.");
+            Msg ("Will print X11 Pixmap file.");
          else
             Msg ("Will print X11 Bitmap file.");
          break;
@@ -588,6 +597,16 @@ void ToggleWhereToPrint ()
    UpdateSubMenu (MENU_LAYOUT);
 }
 
+void ToggleOneMotionSelectMove ()
+{
+   oneMotionSelectMove = !oneMotionSelectMove;
+   if (oneMotionSelectMove)
+      Msg ("Click-select-move in one motion mode selected.");
+   else
+      Msg ("Click-select-click-move mode selected.");
+   UpdateSubMenu (MENU_LAYOUT);
+}
+
 static
 void PostZoom (xc, yc)
    int	xc, yc;
@@ -685,7 +704,8 @@ void PostZoom (xc, yc)
    AdjSplineVs ();
    AdjCaches ();
    ShowZoom ();
-   ClearAndRedrawDrawWindowDontDrawCurText ();
+   /* use to be ClearAndRedrawDrawWindowDontDrawCurText() */
+   ClearAndRedrawDrawWindowNoCurT ();
    SetDefaultCursor (mainWindow);
    ShowCursor ();
    justDupped = FALSE;
@@ -865,6 +885,32 @@ void PreciseZoom (zoomed_in, zoom_scale, force_zoom)
    if (curChoice==DRAWTEXT && textCursorShown) PostZoomCurText (xc, yc);
 }
 
+void SetPSPageWidthHeight ()
+{
+   if (mainWindow != None)
+   {
+      sprintf (psYOffStr[0], "%.2f",
+            (float)(((float)onePageHeight)/((float)PIX_PER_INCH)));
+      psYOff[0] = ((float)onePageHeight)/((float)PIX_PER_INCH);
+      psPageWidthInInch[0]=((float)onePageWidth)/((float)PIX_PER_INCH);
+      psPageWidthInInch[1]=((float)onePageHeight)/((float)PIX_PER_INCH);
+      psPageHeightInInch[0]=((float)onePageHeight)/((float)PIX_PER_INCH);
+      psPageHeightInInch[1]=((float)onePageWidth)/((float)PIX_PER_INCH);
+   }
+}
+
+void ResetOnePageSize ()
+{  /* reset 8.5in x 11in */
+#ifndef A4PAPER
+   onePageWidth = (85*PIX_PER_INCH)/10;
+   onePageHeight = 11*PIX_PER_INCH;
+#else /* A4PAPER */
+   onePageWidth = (825*PIX_PER_INCH)/100;
+   onePageHeight = (117*PIX_PER_INCH)/10;
+#endif /* A4PAPER */
+   SetPSPageWidthHeight ();
+}
+
 int UpdPageStyle (NewPageStyle)
    int	NewPageStyle;
 {
@@ -872,9 +918,9 @@ int UpdPageStyle (NewPageStyle)
 
    pageStyle = NewPageStyle;
    onePageWidth =
-         (int)(psPageWidthInInch[NewPageStyle]*PIX_PER_INCH*100/printMag);
+         round(psPageWidthInInch[NewPageStyle]*PIX_PER_INCH*100/printMag);
    onePageHeight =
-         (int)(psPageHeightInInch[NewPageStyle]*PIX_PER_INCH*100/printMag);
+         round(psPageHeightInInch[NewPageStyle]*PIX_PER_INCH*100/printMag);
    paperWidth = onePageWidth * paperCol;
    paperHeight = onePageHeight * paperRow;
    if (drawOrigX + drawWinW > paperWidth || drawOrigX + drawWinW > old_w)
@@ -1049,6 +1095,7 @@ void LayoutSubMenu (index)
       case LAYOUT_TOGGLE_SHOW_MEASURE: ToggleShowMeasurement (); break;
       case LAYOUT_TOGGLE_SHOW_MENUBAR: ToggleShowMenubar (); break;
       case LAYOUT_TOGGLE_SHOW_STATUS: ToggleShowStatus (); break;
+      case LAYOUT_TOGGLE_ONEMOTION_SELMOVE: ToggleOneMotionSelectMove (); break;
    }
 }
 
@@ -1207,6 +1254,12 @@ void SetUpLayoutMenuStr (menu_strings, menu_description)
       strcpy (menu_strings[LAYOUT_TOGGLE_SHOW_STATUS], hideStatusStr);
       strcpy (menu_description[LAYOUT_TOGGLE_SHOW_STATUS], hideStatusDesc);
    }
+   if (oneMotionSelectMove)
+      strcpy (menu_strings[LAYOUT_TOGGLE_ONEMOTION_SELMOVE],
+            clickSelClickMoveStr);
+   else
+      strcpy (menu_strings[LAYOUT_TOGGLE_ONEMOTION_SELMOVE],
+            oneMotionSelMoveStr);
 }
 
 int LayoutMenu (X, Y, TrackMenubar)
