@@ -25,7 +25,7 @@
  */
 #ifndef lint
 static char RCSid[] =
-      "@(#)$Header: /amnt/maui/tangram/u/william/X11/TGIF2/RCS/group.c,v 2.27 1994/03/21 04:54:57 william Exp $";
+      "@(#)$Header: /export/bourbon/multimedia/william/X11/TGIF2/RCS/group.c,v 2.28 1994/11/07 04:00:03 william Exp $";
 #endif
 
 #include <stdio.h>
@@ -61,6 +61,9 @@ void CreateGroupObj (TopObjPtr, BotObjPtr)
    group_ptr = (struct GroupRec *) calloc (1, sizeof(struct GroupRec));
    group_ptr->first = TopObjPtr;
    group_ptr->last = BotObjPtr;
+   group_ptr->rotate = ROTATE0;
+   group_ptr->flip = NO_FLIP;
+   group_ptr->deck_index = (-1);
    obj_ptr = (struct ObjRec *) calloc (1, sizeof(struct ObjRec));
    obj_ptr->x = selObjLtX; /* note:  selLtX, selLtY are absolute */
    obj_ptr->y = selObjLtY;
@@ -245,6 +248,7 @@ void ReadGroupObj (FP, ObjType, ObjPtr)
    group_ptr->last = bot_obj;
    group_ptr->rotate = rotate;
    group_ptr->flip = flip;
+   group_ptr->deck_index = (-1);
    if (ObjType == OBJ_ICON) strcpy (group_ptr->s, tmp_str);
 
    ltx = top_obj->bbox.ltx;
