@@ -25,12 +25,14 @@
  */
 #ifndef lint
 static char RCSid[] =
-      "@(#)$Header: /amnt/maui/tangram/u/william/X11/TGIF2/RCS/mainloop.c,v 2.100 1994/04/04 00:49:06 william Exp $";
+      "@(#)$Header: /export/bourbon/multimedia/william/X11/TGIF2/RCS/mainloop.c,v 2.119 1995/01/02 21:38:15 william Exp $";
 #endif
 
 #include <stdio.h>
 #include <signal.h>
+#include <sys/wait.h>
 #include <string.h>
+#include <sys/types.h>
 #include <X11/Xlib.h>
 #include <X11/Xutil.h>
 #ifdef USE_XT_INITIALIZE
@@ -49,6 +51,7 @@ static char RCSid[] =
 #include "cursor.e"
 #include "dialog.e"
 #include "drawing.e"
+#include "expr.e"
 #include "file.e"
 #include "font.e"
 #include "grid.e"
@@ -59,9 +62,11 @@ static char RCSid[] =
 #include "menu.e"
 #include "msg.e"
 #include "names.e"
+#include "navigate.e"
 #include "obj.e"
 #include "page.e"
 #include "raster.e"
+#include "remote.e"
 #include "ruler.e"
 #include "scroll.e"
 #include "select.e"
@@ -76,7 +81,11 @@ static char RCSid[] =
 extern int	atoi ARGS_DECL((char *));
 #ifndef _NO_EXTERN
 extern int	fork ARGS_DECL((void));
+#if defined(IRIX) || defined(linux)
+extern int	system ARGS_DECL((const char *));
+#else /* ~(defined(IRIX) || defined(linux)) */
 extern int	system ARGS_DECL((char *));
+#endif /* defined(IRIX) || defined(linux) */
 #endif
 
 #ifdef USE_XAPPLRESDIR
@@ -169,9 +178,15 @@ static
 void ExecWithFile (CmdName, FileName)
    char	* CmdName, * FileName;
 {
-   int	pid;
    char	s[255];
 
+#ifdef _BACKGROUND_DONT_FORK
+   sprintf (s, "xterm -bd red -e %s %s", CmdName, FileName);
+   strcat(s, " &");
+   system(s);
+#else /* ~_BACKGROUND_DONT_FORK */
+   int	pid;
+
    sprintf (s, "xterm -bd red -e %s %s", CmdName, FileName);
    pid = fork ();
    if (pid == 0)
@@ -179,6 +194,7 @@ void ExecWithFile (CmdName, FileName)
       system (s);
       exit (0);
    }
+#endif /* _BACKGROUND_DONT_FORK */
 }
 
 void DeallocStrings (FStr, Str1, Menu1, Str2, Menu2, Str3, Menu3)
@@ -237,6 +253,12 @@ void AllocStrings (FStr, Str1, Menu1, Str2, Menu2, Str3, Menu3)
 void CleanUp ()
 {
    TieLooseEnds ();
+   CleanUpExec ();
+   CleanUpCmds ();
+   CleanUpExpr ();
+   CleanUpNavigate ();
+   CleanUpRemote ();
+
    CleanUpExtraWinInfo ();
    CleanUpDrawingWindow ();
    CleanUpPage ();
@@ -260,7 +282,6 @@ void CleanUp ()
    CleanUpXBm ();
    CleanUpXPm ();
    CleanUpMsg ();
-   CleanUpCmds ();
    CleanUpShortCut ();
 
    if (iconWindowCreated)
@@ -406,6 +427,10 @@ void MainLoop (Op, FileName, FuncStr, Str1, Menu1, Str2, Menu2, Str3, Menu3)
 
    if (strcmp (Op, "init") == 0 || strcmp (Op, "justinit") == 0)
    {
+      int	file_is_remote=FALSE, remote_buf_sz=0, is_html=FALSE;
+      char	remote_fname[MAXPATHLENGTH+1], *tmp_remote_fname=NULL;
+      char	* remote_buf=NULL;
+
       if (TGIF_PATCHLEVEL == 0)
          sprintf (initMsg1, "%s Version %s", TOOL_NAME, version_string);
       else
@@ -503,34 +528,109 @@ void MainLoop (Op, FileName, FuncStr, Str1, Menu1, Str2, Menu2, Str3, Menu3)
       InitExtraWinInfo ();
       quitDraw = FALSE;
 
-      if (FileName[0] != '\0')
+      *remote_fname = '\0';
+      if (*FileName != '\0' && FileIsRemote (FileName))
+      {
+         if (!FormNewFileName (curDir, FileName, NULL, remote_fname))
+         {
+            sprintf(s, "Invalid remote file name '%s'.", FileName);
+            Dialog (s, NULL, NULL);
+            *FileName = '\0';
+         }
+         else
+         {
+            char *content_type=NULL;
+
+            SaveStatusStrings ();
+            rc = LoadRemoteFileInMem(remote_fname, &remote_buf,
+                  &content_type, &remote_buf_sz, &is_html);
+            RestoreStatusStrings ();
+            if (rc && remote_buf != NULL &&
+                  (tmp_remote_fname=WriteRemoteFileIntoTemp(remote_buf,
+                  remote_buf_sz)) != NULL)
+            {
+               int	url_is_html=UrlIsHtml(remote_fname);
+
+               if ((url_is_html || !is_html) && (UseViewer(remote_fname,
+                     content_type, tmp_remote_fname) != FALSE))
+               {
+                  unlink (tmp_remote_fname);
+                  if (remote_buf != NULL) FreeRemoteBuf (remote_buf);
+                  if (tmp_remote_fname != NULL)
+                     FreeRemoteBuf (tmp_remote_fname);
+                  remote_buf = tmp_remote_fname = NULL;
+                  file_is_remote = FALSE;
+                  *FileName = '\0';
+               }
+               else
+                  file_is_remote = TRUE;
+            }
+            else
+            {
+               if (remote_buf != NULL) FreeRemoteBuf (remote_buf);
+               remote_buf = NULL;
+               *FileName = '\0';
+            }
+            if (content_type != NULL) FreeRemoteBuf(content_type);
+         }
+      }
+      if (*FileName != '\0')
       {
          int obj_file=TRUE;
 
          sprintf (obj_ext_str, ".%s", OBJ_FILE_EXT);
          obj_ext_len = strlen (obj_ext_str);
-         len = strlen (FileName);
 
-         if (len >= obj_ext_len &&
-               strcmp (&FileName[len-obj_ext_len], obj_ext_str) == 0)
-            strcpy (file_name, FileName);
-         else
+         *file_name = '\0';
+         if (file_is_remote)
          {
-            sprintf (obj_ext_str, ".%s", SYM_FILE_EXT);
-            obj_ext_len = strlen (obj_ext_str);
-
-            if (len >= obj_ext_len &&
-                  strcmp (&FileName[len-obj_ext_len], obj_ext_str) == 0)
+            if ((fp=fopen(tmp_remote_fname, "r")) == NULL)
             {
-               strcpy (file_name, FileName);
-               obj_file = FALSE;
+               sprintf (s, "Can not read tmp file '%s'.", tmp_remote_fname);
+               Dialog (s, NULL, NULL);
+
+               unlink (tmp_remote_fname);
+               FreeRemoteBuf (remote_buf);
+               FreeRemoteBuf (tmp_remote_fname);
+               remote_buf = tmp_remote_fname = NULL;
+               file_is_remote = FALSE;
             }
             else
-               sprintf (file_name, "%s.%s", FileName, OBJ_FILE_EXT);
+            {
+               len = strlen (remote_fname);
+               sprintf (obj_ext_str, ".%s", SYM_FILE_EXT);
+               obj_ext_len = strlen (obj_ext_str);
+               if (len >= obj_ext_len &&
+                     strcmp (&FileName[len-obj_ext_len], obj_ext_str) == 0)
+                  obj_file = FALSE;
+               else
+                  obj_file = TRUE;
+            }
+         }
+         else
+         {
+            len = strlen (FileName);
+            if (len >= obj_ext_len &&
+                  strcmp (&FileName[len-obj_ext_len], obj_ext_str) == 0)
+               strcpy (file_name, FileName);
+            else
+            {
+               sprintf (obj_ext_str, ".%s", SYM_FILE_EXT);
+               obj_ext_len = strlen (obj_ext_str);
+
+               if (len >= obj_ext_len &&
+                     strcmp (&FileName[len-obj_ext_len], obj_ext_str) == 0)
+               {
+                  strcpy (file_name, FileName);
+                  obj_file = FALSE;
+               }
+               else
+                  sprintf (file_name, "%s.%s", FileName, OBJ_FILE_EXT);
+            }
+            if ((short_name = IsPrefix (bootDir, file_name, &rest))) ++rest;
          }
 
-         if ((short_name = IsPrefix (bootDir, file_name, &rest))) ++rest;
-         if ((fp = fopen (file_name, "r")) == NULL)
+         if (*file_name != '\0' && (fp = fopen (file_name, "r")) == NULL)
          {
             lastPageNum = 1;
             InitPage ();
@@ -562,18 +662,26 @@ void MainLoop (Op, FileName, FuncStr, Str1, Menu1, Str2, Menu2, Str3, Menu3)
                sprintf (s, "Current (empty) file is '%s'.", file_name);
             Msg (s);
          }
-         else
+         else if (file_is_remote || *file_name != '\0')
          {
             SetFileModified (FALSE);
-            if (short_name)
-               sprintf (s, "Loading '%s' ...", rest);
+            if (file_is_remote)
+               sprintf (s, "Loading '%s' ...", remote_fname);
             else
-               sprintf (s, "Loading '%s' ...", file_name);
+            {
+               if (short_name)
+                  sprintf (s, "Loading '%s' ...", rest);
+               else
+                  sprintf (s, "Loading '%s' ...", file_name);
+            }
             Msg (s);
 
             strcpy (tmp_filename, scanFileName);
             tmp_linenum = scanLineNum;
-            strcpy (scanFileName, (short_name ? rest : file_name));
+            if (file_is_remote)
+               strcpy (scanFileName, tmp_remote_fname);
+            else
+               strcpy (scanFileName, (short_name ? rest : file_name));
             scanLineNum = 0;
 
             SetWatchCursor (drawWindow);
@@ -581,6 +689,7 @@ void MainLoop (Op, FileName, FuncStr, Str1, Menu1, Str2, Menu2, Str3, Menu3)
 
             readingPageNum = 0;
             loadedCurPageNum = 0;
+            foundGoodStateObject = FALSE;
             while ((read_status = ReadObj (fp, &obj_ptr)) == TRUE)
                if (obj_ptr != NULL)
                {
@@ -593,42 +702,65 @@ void MainLoop (Op, FileName, FuncStr, Str1, Menu1, Str2, Menu2, Str3, Menu3)
 
             fclose (fp);
 
-            if (read_status == INVALID)
-            {
-               sprintf (s, "File version too large (=%1d).  Load aborted!",
-                     fileVersion);
-               Msg (s);
-            }
-            else
-            {
-               if (*file_name == '/')
-                  strcpy (full_name, file_name);
-               else
-                  sprintf (full_name, "%s/%s", curDir, file_name);
-
-               if (obj_file)
-               {
-                  SetCurDir (full_name);
-                  *curSymDir = '\0';
-               }
-               else
-                  SetCurSymDir (full_name);
-               curFileDefined = TRUE;
-
-               if (short_name)
-                  sprintf (s, "Current file is '%s'.", rest);
-               else
-                  sprintf (s, "Current file is '%s'.", file_name);
-               Msg (s);
-            }
             if (loadedCurPageNum <= 0)
             {
                loadedCurPageNum = curPageNum = lastPageNum = 1;
                InitPage ();
             }
+            if (read_status == INVALID)
+            {
+               sprintf (s, "File version too large (=%1d).  Load aborted!",
+                     fileVersion);
+               Msg (s);
+               sprintf (s, "    You may need a more recent version of %s",
+                     TOOL_NAME);
+               Msg (s);
+            }
+            else
+            {
+               curFileDefined = TRUE;
+               if (file_is_remote)
+               {
+                  SetCurDir (remote_fname);
+                  if (!foundGoodStateObject) PasteString (remote_buf);
+               }
+               else
+               {
+                  if (*file_name == '/')
+                     strcpy (full_name, file_name);
+                  else
+                     sprintf (full_name, "%s/%s", curDir, file_name);
+                  if (obj_file)
+                  {
+                     SetCurDir (full_name);
+                     *curSymDir = '\0';
+                  }
+                  else
+                     SetCurSymDir (full_name);
+               }
+               if (file_is_remote)
+                  sprintf (s, "Current file is '%s'.", remote_fname);
+               else
+               {
+               if (short_name)
+                  sprintf (s, "Current file is '%s'.", rest);
+               else
+                  sprintf (s, "Current file is '%s'.", file_name);
+               }
+               Msg (s);
+            }
             GotoPageNum (loadedCurPageNum);
             SetDefaultCursor (mainWindow);
             SetDefaultCursor (drawWindow);
+
+            if (file_is_remote)
+            {
+               unlink (tmp_remote_fname);
+               if (remote_buf != NULL) FreeRemoteBuf (remote_buf);
+               if (tmp_remote_fname != NULL) FreeRemoteBuf (tmp_remote_fname);
+               remote_buf = tmp_remote_fname = NULL;
+            }
+            if (foundGoodStateObject) CommitNavigate ();
          }
       }
       else
