@@ -26,10 +26,11 @@
  */
 #ifndef lint
 static char RCSid[] =
-      "@(#)$Header: /amnt/maui/tangram/u/william/X11/TGIF2/RCS/mainmenu.c,v 2.7 1994/04/04 00:49:06 william Exp $";
+      "@(#)$Header: /export/bourbon/multimedia/william/X11/TGIF2/RCS/mainmenu.c,v 2.13 1994/11/30 06:13:15 william Exp $";
 #endif
 
 #include <stdio.h>
+#include <sys/types.h>
 #include <X11/Xlib.h>
 #include <X11/Xutil.h>
 #include "const.h"
@@ -49,6 +50,7 @@ static char RCSid[] =
 #endif
 #include "menu.e"
 #include "msg.e"
+#include "navigate.e"
 #include "page.e"
 #include "pattern.e"
 #include "raster.e"
@@ -315,6 +317,7 @@ int MainMenuEventHandler (input)
                case MENU_FILL: FillMenu (x, y, FALSE); break;
                case MENU_PEN: PenMenu (x, y, FALSE); break;
                case MENU_COLOR: ColorMenu (x, y, FALSE); break;
+               case MENU_NAVIGATE: NavigateMenu (x, y, FALSE); break;
                case MENU_SPECIAL: SpecialMenu (x, y, FALSE); break;
                case MENU_PAGE: PageMenu (x, y, FALSE); break;
                case MENU_PAGELAYOUT: PageLayoutMenu (x, y, FALSE); break;
@@ -719,6 +722,13 @@ void RedrawSubMenu (menu_index)
                maxColors, fore_colors, pixmap, init_rv, MULTICOLOR);
          cfree (pixmap);
          break;
+      case MENU_NAVIGATE:
+         DefaultColorArrays (MAXNAVIGATEMENUS, &fore_colors, &valid, &init_rv,
+               NULL);
+         PaintTextSubMenu (win, subMenuInfo[menu_index].w,
+               inHyperSpace ? navigateMenuStrInHyperSpace : navigateMenuStr,
+               MAXNAVIGATEMENUS, fore_colors, valid, init_rv);
+         break;
       case MENU_SPECIAL:
          DefaultColorArrays (MAXSPECIALMENUS, &fore_colors, &valid, &init_rv,
                NULL);
@@ -760,6 +770,7 @@ void GetRowCol (menu_index, x, y, row, col, unit_w, unit_h, num_rows)
       case MENU_FONT:
       case MENU_LAYOUT:
       case MENU_ARRANGE:
+      case MENU_NAVIGATE:
       case MENU_SPECIAL:
       case MENU_PAGE:
          *unit_w = subMenuInfo[menu_index].w;
@@ -898,19 +909,23 @@ int SubMenuEventHandler (input)
                         case MENU_HORIALIGN: HoriAlignSubMenu (up_row); break;
                         case MENU_VERTALIGN: VertAlignSubMenu (up_row); break;
                         case MENU_FILL:
-                           ChangeAllSelFill (up_col*num_rows+up_row); break;
+                           ChangeAllSelFill (up_col*num_rows+up_row, TRUE);
+                           break;
                         case MENU_LINEWIDTH:
-                           ChangeAllSelLineWidth (up_row); break;
+                           ChangeAllSelLineWidth (up_row, TRUE); break;
                         case MENU_LINESTYLE:
-                           ChangeAllSelLineStyle (up_row); break;
+                           ChangeAllSelLineStyle (up_row, TRUE); break;
                         case MENU_LINETYPE:
-                           ChangeAllSelLineType (up_row); break;
+                           ChangeAllSelLineType (up_row, TRUE); break;
                         case MENU_LINEDASH:
-                           ChangeAllSelDashes (up_row); break;
+                           ChangeAllSelDashes (up_row, TRUE); break;
                         case MENU_PEN:
-                           ChangeAllSelPen (up_col*num_rows+up_row); break;
+                           ChangeAllSelPen (up_col*num_rows+up_row, TRUE);
+                           break;
                         case MENU_COLOR:
-                           ChangeAllSelColor (up_col*num_rows+up_row); break;
+                           ChangeAllSelColor (up_col*num_rows+up_row, TRUE);
+                           break;
+                        case MENU_NAVIGATE: NavigateSubMenu (up_row); break;
                         case MENU_SPECIAL: SpecialSubMenu (up_row); break;
                         case MENU_PAGE: PageSubMenu (up_row); break;
                         case MENU_PAGELAYOUT: PageLayoutSubMenu (up_row); break;
