@@ -25,11 +25,12 @@
  */
 #ifndef lint
 static char RCSid[] =
-      "@(#)$Header: /amnt/maui/tangram/u/william/X11/TGIF2/RCS/menu.c,v 2.94 1994/05/01 23:19:48 william Exp $";
+      "@(#)$Header: /export/bourbon/multimedia/william/X11/TGIF2/RCS/menu.c,v 2.109 1994/12/12 00:11:40 william Exp $";
 #endif
 
 #include <stdio.h>
 #include <string.h>
+#include <sys/types.h>
 #include <X11/Xlib.h>
 #include <X11/Xutil.h>
 #include "const.h"
@@ -61,6 +62,7 @@ static char RCSid[] =
 #include "page.e"
 #include "pattern.e"
 #include "raster.e"
+#include "remote.e"
 #include "select.e"
 #include "setup.e"
 #include "special.e"
@@ -77,13 +79,13 @@ int	importingIconFile = FALSE;
 int	showVersion = TRUE;
 int	activeMenu = INVALID;
 
-char	* mainMenuStr[MAXMENUS] =
+char	* mainMenuStr[MAXMENUS+1] =
       {
          "Mode", "File", "Edit", "Layout", "MoveMode", "Arrange",
          "Page", "PageLayout", "HoriAlign", "VertAlign",
          "Font", "TextStyle", "TextSize",
          "LineDash", "LineStyle", "LineType", "LineWidth",
-         "Fill", "Pen", "Color", "Special"
+         "Fill", "Pen", "Color", "Navigate", "Special", NULL
       };
 static char * mainMenuDescription[] =
       {
@@ -107,6 +109,7 @@ static char * mainMenuDescription[] =
          "Fill Menu",
          "Pen Menu",
          "Color Menu",
+         "Navigate Menu",
          "Special Menu",
          NULL
       };
@@ -142,9 +145,10 @@ static int		specialEscapeForGotoPageUnderViewMultiMenu=INVALID;
 #define MENUBARMENU_TEXT 6
 #define MENUBARMENU_GRAPHICS 7
 #define MENUBARMENU_COLOR 8
-#define MENUBARMENU_SPECIAL 9
+#define MENUBARMENU_NAVIGATE 9
+#define MENUBARMENU_SPECIAL 10
 
-#define MAXMENUBARMENUS 10
+#define MAXMENUBARMENUS 11
 
 static char * mainMenubarMenuStr[MAXMENUBARMENUS] =
 {
@@ -157,6 +161,7 @@ static char * mainMenubarMenuStr[MAXMENUBARMENUS] =
    "Text",
    "Graphics",
    "Color",
+   "Navigate",
    "Special"
 };
 
@@ -999,6 +1004,7 @@ int MainMenu ()
       case MENU_FILL: FillMenu (x, y, FALSE); break;
       case MENU_PEN: PenMenu (x, y, FALSE); break;
       case MENU_COLOR: ColorMenu (x, y, FALSE); break;
+      case MENU_NAVIGATE: NavigateMenu (x, y, FALSE); break;
       case MENU_SPECIAL: SpecialMenu (x, y, FALSE); break;
       case MENU_PAGE: PageMenu (x, y, FALSE); break;
       case MENU_PAGELAYOUT: PageLayoutMenu (x, y, FALSE); break;
@@ -1105,20 +1111,21 @@ void RedrawIconWindow ()
       struct AttrRec	* attr_ptr;
 
       justIconified = FALSE;
-      if ((attr_ptr=FindAttrWithName(iconTgifObj,"icon_init_exec_obj=")) !=
-            NULL && (obj_ptr=FindObjWithName(iconBotObj,attr_ptr->s)) != NULL &&
-            (attr_ptr=FindAttrWithName(obj_ptr,EXEC_ATTR)) != NULL)
+      if ((attr_ptr=FindAttrWithName(iconTgifObj,"icon_init_exec_obj=",NULL)) !=
+            NULL && (obj_ptr=FindObjWithName(iconBotObj,iconTgifObj,
+            attr_ptr->s,FALSE,FALSE,NULL,NULL)) != NULL &&
+            (attr_ptr=FindAttrWithName(obj_ptr,EXEC_ATTR,NULL)) != NULL)
       {
          int	saved_intr_check_interval=intrCheckInterval;
          int	saved_history_depth=historyDepth;
 
          intrCheckInterval = 1;
          historyDepth = 0;
-         ShowInterrupt ();
+         ShowInterrupt (1);
 
          DoExec (attr_ptr, obj_ptr);
 
-         HideInterrupt ();
+         HideInterrupt();
          intrCheckInterval = saved_intr_check_interval;
          historyDepth = saved_history_depth;
       }
@@ -1168,7 +1175,7 @@ void InitIcon ()
       else
          strcat (s, iconFileName);
    }
-   else if (strlen (c_ptr) >= 200)
+   else if (((int) strlen (c_ptr)) >= 200)
       /* too long, must be an error */
       strcat (s, iconFileName);
    else if (*c_ptr == '/')
@@ -1757,6 +1764,7 @@ int PullDownFromMenubar (index, x, y, text_bbox)
                rc = GraphicsMultiMenu (x, y, TRUE);
                break;
             case MENUBARMENU_COLOR: rc = ColorMenu (x, y, TRUE); break;
+            case MENUBARMENU_NAVIGATE: rc = NavigateMenu (x, y, TRUE); break;
             case MENUBARMENU_SPECIAL: rc = SpecialMenu (x, y, TRUE); break;
          }
       }
@@ -1782,6 +1790,7 @@ int PullDownFromMenubar (index, x, y, text_bbox)
             case MENU_FILL: rc = FillMenu (x, y, TRUE); break;
             case MENU_PEN: rc = PenMenu (x, y, TRUE); break;
             case MENU_COLOR: rc = ColorMenu (x, y, TRUE); break;
+            case MENU_NAVIGATE: rc = NavigateMenu (x, y, TRUE); break;
             case MENU_SPECIAL: rc = SpecialMenu (x, y, TRUE); break;
             case MENU_PAGE: rc = PageMenu (x, y, TRUE); break;
             case MENU_PAGELAYOUT: rc = PageLayoutMenu (x, y, TRUE); break;
@@ -1839,6 +1848,7 @@ static struct MouseStatusStrRec menubarMouseStatus[] = {
    { "", "Fill Menu", "" },
    { "", "Pen Menu", "" },
    { "", "Color Menu", "" },
+   { "", "Navigate Menu", "" },
    { "", "Special Menu", "" }
 };
 static struct MouseStatusStrRec shortMenubarMouseStatus[] = {
@@ -1851,6 +1861,7 @@ static struct MouseStatusStrRec shortMenubarMouseStatus[] = {
    { "", "Text MultiMenu", "" },
    { "", "Graphics MultiMenu", "" },
    { "", "Color Menu", "" },
+   { "", "Navigate Menu", "" },
    { "", "Special Menu", "" }
 };
 
@@ -1898,7 +1909,7 @@ int MenubarEventHandler (input)
       else
       {
          SaveStatusStrings ();
-         rc = PullDownFromMenubar (index, win_x, win_y, text_bbox);
+         rc = PullDownFromMenubar (index, win_x, win_y, &text_bbox);
          RestoreStatusStrings ();
          SetMouseStatus (NULL, NULL, NULL);
       }
