@@ -25,7 +25,7 @@
  */
 #ifndef lint
 static char RCSid[] =
-      "@(#)$Header: /amnt/maui/tangram/u/william/X11/TGIF2/RCS/msg.c,v 2.47 1994/05/01 23:20:15 william Exp $";
+      "@(#)$Header: /export/bourbon/multimedia/william/X11/TGIF2/RCS/msg.c,v 2.58 1994/11/06 22:06:21 william Exp $";
 #endif
 
 #include <stdio.h>
@@ -48,6 +48,7 @@ static char RCSid[] =
 #endif
 #include "raster.e"
 #include "setup.e"
+#include "util.e"
 
 #define MSG_ROWS 2
 
@@ -67,6 +68,13 @@ static int		firstCharIndex = 0;
 
 static int		reverseMouseStatusButtons=FALSE;
 
+int FailAllocMessage ()
+{
+   fprintf (stderr, "Out of virtual memory and can not calloc().\n");
+   fflush (stderr);
+   return (FALSE);
+}
+
 static
 void AddMsg (Msg)
    char	* Msg;
@@ -252,7 +260,6 @@ void PrintMsgBuffer ()
 
 #include "xbm/btn1.xbm"
 
-static GC	statusGC=NULL;
 static int	oneLineStatus=FALSE;
 static char	btnStatusStr[MAX_STATUS_BTNS][MAXSTRING+1];
 static char	oneLineStatusStr[MAXSTRING+1];
@@ -262,7 +269,7 @@ void RedrawStatusStrings ()
 {
    register int	i;
 
-   if (statusWindow == None) return;
+   if (PRTGIF || noStatusWindow) return;
 
    if (oneLineStatus)
    {
@@ -297,7 +304,7 @@ void RedrawStatusWindow ()
    register int	i;
    XEvent	ev;
 
-   if (statusWindow == None) return;
+   if (PRTGIF || noStatusWindow) return;
 
    if (!oneLineStatus)
    {
@@ -344,7 +351,7 @@ void SetMouseStatus (Left, Middle, Right)
       right_index = 2;
    }
 
-   if (statusWindow == None) return;
+   if (PRTGIF || noStatusWindow) return;
 
    if (oneLineStatus)
    {
@@ -367,23 +374,36 @@ void SetMouseStatus (Left, Middle, Right)
    }
    if (Left != NULL || Middle != NULL || Right != NULL)
    {
-      if (Left != NULL)
-         strcpy (btnStatusStr[left_index], Left);
-      else
+      if (Left != NULL) {
+         if (Left != btnStatusStr[left_index] &&
+               strcmp(Left, btnStatusStr[left_index]) != 0) {
+            UtilStrCpy (btnStatusStr[left_index], MAXSTRING+1, Left);
+         }
+      } else {
          *btnStatusStr[left_index] = '\0';
-      if (Middle != NULL)
-         strcpy (btnStatusStr[1], Middle);
-      else
+      }
+      if (Middle != NULL) {
+         if (Middle != btnStatusStr[1] &&
+               strcmp(Left, btnStatusStr[1]) != 0) {
+            UtilStrCpy (btnStatusStr[1], MAXSTRING+1, Middle);
+         }
+      } else {
          *btnStatusStr[1] = '\0';
-      if (Right != NULL)
-         strcpy (btnStatusStr[right_index], Right);
-      else
+      }
+      if (Right != NULL) {
+         if (Right != btnStatusStr[right_index] &&
+               strcmp(Left, btnStatusStr[right_index]) != 0) {
+            UtilStrCpy (btnStatusStr[right_index], MAXSTRING+1, Right);
+         }
+      } else {
          *btnStatusStr[right_index] = '\0';
+      }
    }
-   if (force_redraw)
+   if (force_redraw) {
       RedrawStatusWindow ();
-   else
+   } else {
       RedrawStatusStrings ();
+   }
 }
 
 void SetStringStatus (StatusStr)
@@ -393,7 +413,7 @@ void SetStringStatus (StatusStr)
    register int	i;
    int		force_redraw=FALSE;
 
-   if (statusWindow == None) return;
+   if (PRTGIF || noStatusWindow) return;
 
    if (!oneLineStatus)
    {
@@ -406,7 +426,7 @@ void SetStringStatus (StatusStr)
    else if (StatusStr != NULL && strcmp (StatusStr, oneLineStatusStr) == 0)
       return;
 
-   if (StatusStr != NULL) strcpy (oneLineStatusStr, StatusStr);
+   if (StatusStr != NULL) UtilStrCpy (oneLineStatusStr, MAXSTRING+1, StatusStr);
    if (force_redraw)
       RedrawStatusWindow ();
    else
@@ -421,10 +441,10 @@ void SaveStatusStrings ()
 {
    register int	i;
 
-   if (statusWindow == None) return;
+   if (PRTGIF || noStatusWindow) return;
    for (i=0; i < MAX_STATUS_BTNS; i++)
-      strcpy (savedBtnStatusStr[i], btnStatusStr[i]);
-   strcpy (savedOneLineStatusStr, oneLineStatusStr);
+      UtilStrCpy (savedBtnStatusStr[i], MAXSTRING+1, btnStatusStr[i]);
+   UtilStrCpy (savedOneLineStatusStr, MAXSTRING+1, oneLineStatusStr);
    savedOneLineStatus = oneLineStatus;
 }
 
@@ -432,10 +452,10 @@ void RestoreStatusStrings ()
 {
    register int	i;
 
-   if (statusWindow == None) return;
+   if (PRTGIF || noStatusWindow) return;
    for (i=0; i < MAX_STATUS_BTNS; i++)
-      strcpy (btnStatusStr[i], savedBtnStatusStr[i]);
-   strcpy (oneLineStatusStr, savedOneLineStatusStr);
+      UtilStrCpy (btnStatusStr[i], MAXSTRING+1, savedBtnStatusStr[i]);
+   UtilStrCpy (oneLineStatusStr, MAXSTRING+1, savedOneLineStatusStr);
    if (savedOneLineStatus != oneLineStatus)
    {
       if (savedOneLineStatus)
@@ -447,6 +467,44 @@ void RestoreStatusStrings ()
       RedrawStatusWindow ();
 }
 
+void SaveStatusStringsIntoBuf(ppsz_buf, pn_one_line)
+   char ppsz_buf[MAX_STATUS_BTNS+1][MAXSTRING+1];
+   int *pn_one_line;
+   /* dimension of ppsz_buf must be [MAX_STATUS_BTNS+1][MAXSTRING+1] */
+{
+   register int	i;
+
+   if (PRTGIF || noStatusWindow) return;
+   for (i=0; i < MAX_STATUS_BTNS; i++) {
+      UtilStrCpy(ppsz_buf[i], MAXSTRING+1, btnStatusStr[i]);
+   }
+   UtilStrCpy(ppsz_buf[i], MAXSTRING+1, oneLineStatusStr);
+   *pn_one_line = oneLineStatus;
+}
+
+void RestoreStatusStringsFromBuf(ppsz_buf, one_line)
+   char ppsz_buf[MAX_STATUS_BTNS+1][MAXSTRING+1];
+   int one_line;
+   /* dimension of ppsz_buf must be [MAX_STATUS_BTNS+1][MAXSTRING+1] */
+{
+   register int	i;
+
+   if (PRTGIF || noStatusWindow) return;
+   for (i=0; i < MAX_STATUS_BTNS; i++) {
+      UtilStrCpy(btnStatusStr[i], MAXSTRING+1, ppsz_buf[i]);
+   }
+   UtilStrCpy(oneLineStatusStr, MAXSTRING+1, ppsz_buf[i]);
+   if (one_line != oneLineStatus) {
+      if (one_line) {
+         SetStringStatus(oneLineStatusStr);
+      } else {
+         SetMouseStatus(btnStatusStr[0], btnStatusStr[1], btnStatusStr[2]);
+      }
+   } else {
+      RedrawStatusWindow();
+   }
+}
+
 void MsgEventHandler (input)
    XEvent	* input;
 {
@@ -543,7 +601,7 @@ void MsgEventHandler (input)
 void StatusEventHandler (input)
    XEvent	* input;
 {
-   if (statusWindow == None) return;
+   if (PRTGIF || noStatusWindow) return;
 
    if (input->type == Expose)
       RedrawStatusWindow ();
@@ -553,16 +611,6 @@ void InitStatus ()
 {
    register int	i;
    char		* c_ptr;
-   XGCValues	values;
-
-   if (statusWindow == None) return;
-
-   values.foreground = myFgPixel;
-   values.background = myBgPixel;
-   values.fill_style = FillOpaqueStippled;
-   values.font = defaultFontPtr->fid;
-   /* statusGC = XCreateGC (mainDisplay, statusWindow,
-         GCForeground | GCBackground | GCFillStyle | GCFont, &values); */
 
    oneLineStatus = FALSE;
    *oneLineStatusStr = '\0';
@@ -577,8 +625,6 @@ void InitStatus ()
 
 void CleanUpStatus()
 {
-   if (statusGC != NULL) XFreeGC (mainDisplay, statusGC);
-   statusGC = NULL;
 }
 
 static char	* scanVal = NULL;
@@ -596,6 +642,7 @@ char * GetString ()
 {
    char	* c_ptr;
 
+   if (scanVal == NULL || scanSep == NULL) return (NULL);
    while (*scanVal!='\0' && strchr(scanSep,*scanVal)!=NULL) *scanVal++ = '\0';
    if (*scanVal == '\0') return (NULL);
 
@@ -612,6 +659,7 @@ int ScanValue (fmt, v, item, stype)
 {
    char	* c_ptr, msg[MAXPATHLENGTH];
 
+   if (scanVal == NULL) return (INVALID);
 #ifdef DEBUG
    printf ("get %s for %s from %s:  ", item, stype, scanVal);
 #endif
@@ -796,6 +844,7 @@ void Usage (tool_name)
    fprintf (stderr, "\t[-bd <color>] \\\n");
    fprintf (stderr, "\t[-rv] [-nv] \\\n");
    fprintf (stderr, "\t[-bw] \\\n");
+   fprintf (stderr, "\t[-cwo] \\\n");
    fprintf (stderr, "\t[-geometry <geom>] [=<geom>] \\\n");
    fprintf (stderr, "\t[<filename>[.obj]]\n");
    exit (-1);
