@@ -25,7 +25,7 @@
  */
 #ifndef lint
 static char RCSid[] =
-      "@(#)$Header: /amnt/maui/tangram/u/william/X11/TGIF2/RCS/names.c,v 2.107 1994/03/21 04:55:29 william Exp $";
+      "@(#)$Header: /export/bourbon/multimedia/william/X11/TGIF2/RCS/names.c,v 2.123 1994/11/30 06:13:15 william Exp $";
 #endif
 
 #include <sys/types.h>
@@ -73,6 +73,7 @@ static char RCSid[] =
 #include "auxtext.e"
 #include "box.e"
 #include "button.e"
+#include "copypaste.e"
 #include "cursor.e"
 #include "file.e"
 #include "font.e"
@@ -83,21 +84,25 @@ static char RCSid[] =
 #ifndef _NO_EXTERN
 #include "names.e"
 #endif
+#include "navigate.e"
 #include "raster.e"
 #include "rect.e"
+#include "remote.e"
 #include "setup.e"
 
 extern char	* getenv ARGS_DECL((char *));
 extern int	atoi ARGS_DECL((char *));
 
-char	curDomainName[MAXPATHLENGTH];
-char	curDomainPath[MAXPATHLENGTH];
-char	curDir[MAXPATHLENGTH];
-char	curImportDir[MAXPATHLENGTH];
+char	curDomainName[MAXPATHLENGTH+1];
+char	curDomainPath[MAXPATHLENGTH+1];
+char	curDir[MAXPATHLENGTH+1];
+char	curLocalDir[MAXPATHLENGTH+1];
+char	curImportDir[MAXPATHLENGTH+1];
 char	curSymDir[MAXPATHLENGTH];
 
 int	doubleClickInterval;
-int	importFromLibrary = FALSE;
+int	importFromLibrary=FALSE;
+int	curDirIsLocal=TRUE;
 
 Window		nameBaseWin;
 Window		nameDspWin;
@@ -118,7 +123,7 @@ int	nameDspWinW;		/* width of display window */
 int	justClicked=FALSE;
 int	domainInResource=TRUE;
 
-int	ignoreDirectoryFlagInMakeNameDspItemArray=FALSE;
+int	ignoreDirectoryFlag=FALSE;
 
 static int	nameScrollAreaH;	/* heighr of the grey scroll area */
 
@@ -227,7 +232,13 @@ DspList * SymbolListing ()
    for (path_index = 0; path_index < symPathNumEntries; path_index++)
    {
       strcpy (path, symPath[path_index]);
-      if (strcmp (".", path) == 0) strcpy (path, curDir);
+      if (strcmp (".", path) == 0)
+      {
+         if (curDirIsLocal)
+            strcpy (path, curDir);
+         else
+            strcpy (path, curLocalDir);
+      }
 
       if ((dirp = opendir (path)) == NULL)
          continue;
@@ -315,6 +326,28 @@ void BuildSymbolList ()
    topOfSymLinkList = NULL;
 }
 
+static
+int ExtensionMatch (Spec, DirName)
+   char	* Spec, * DirName;
+{
+   register char	* c_ptr;
+   char			other_ext_str[MAXSTRING];
+   int			len;
+
+   if (*Spec == '\0') return FALSE;
+   len = strlen(DirName);
+   strcpy (other_ext_str, Spec);
+   for (c_ptr=strtok(other_ext_str,";"); c_ptr != NULL; c_ptr=strtok(NULL, ";"))
+   {
+      int	other_ext_len=strlen(c_ptr);
+
+      if (len > other_ext_len &&
+            strcmp (c_ptr, &DirName[len-other_ext_len]) == 0)
+         return TRUE;
+   }
+   return FALSE;
+}
+
 static
 DspList * DirListing (Path, ExtStr, OtherExtStr)
    char	* Path, * ExtStr, * OtherExtStr;
@@ -322,9 +355,9 @@ DspList * DirListing (Path, ExtStr, OtherExtStr)
    DspList		* dsp_ptr, * head_ptr, * tail_ptr, * p, * p1;
    DIR			* dirp;
    DIR_ENTRY		* d;
-   int			len, ext_len, other_ext_len, count = 0;
+   int			len, ext_len, count = 0;
    char			path[MAXPATHLENGTH], s[MAXPATHLENGTH];
-   char			ext_str[MAXSTRING], other_ext_str[MAXSTRING];
+   char			ext_str[MAXSTRING];
    struct stat		stat_buf;
 
    if (*Path == '\0')
@@ -349,16 +382,6 @@ DspList * DirListing (Path, ExtStr, OtherExtStr)
       *ext_str = '\0';
       ext_len = 0;
    }
-   if (*OtherExtStr == '\0')
-   {
-      *other_ext_str = '\0';
-      other_ext_len = 0;
-   }
-   else
-   {
-      sprintf (other_ext_str, ".%s", OtherExtStr);
-      other_ext_len = strlen (other_ext_str);
-   }
 
 #ifdef VMS
    /* Fake the .. entry for VMS */
@@ -374,7 +397,6 @@ DspList * DirListing (Path, ExtStr, OtherExtStr)
    while ((d = readdir (dirp)) != NULL)
    {
       len = strlen (d->d_name);
-      if (*OtherExtStr == '\0') other_ext_len = len;
       if (ext_len == 0)
       {
 #ifdef VMS
@@ -406,9 +428,9 @@ DspList * DirListing (Path, ExtStr, OtherExtStr)
             strcpy (dsp_ptr->itemstr, d->d_name);
          }
       }
-      else if (len > ext_len && strcmp (ext_str,&d->d_name[len-ext_len]) == 0 ||
-            len > other_ext_len &&
-            strcmp (other_ext_str,&d->d_name[len-other_ext_len]) == 0)
+      else if ((len > ext_len &&
+            strcmp (ext_str,&d->d_name[len-ext_len]) == 0) ||
+            ExtensionMatch(OtherExtStr, d->d_name))
       {
          dsp_ptr = (DspList *) calloc (1, sizeof(DspList));
          dsp_ptr->directory = FALSE;
@@ -529,7 +551,7 @@ void InitNames ()
    nameDspWinH = nameDspH + 2 * brdrW;
    nameScrollAreaH = nameDspH;
 
-   ignoreDirectoryFlagInMakeNameDspItemArray = FALSE;
+   ignoreDirectoryFlag = FALSE;
 
    *curDomainName = '\0';
    *curDomainPath = '\0';
@@ -595,7 +617,7 @@ void InitNames ()
       if (domainInResource)
          ParseSymPath (c_ptr);
       else if ((c_ptr=getenv (sym_path)) == NULL ||
-            strlen (c_ptr) >= MAXPATHLENGTH-1)
+            ((int) strlen (c_ptr)) >= MAXPATHLENGTH-1)
          ParseSymPath (".");
       else
          ParseSymPath (c_ptr);
@@ -617,7 +639,13 @@ void UpdateDirInfo ()
 {
    SetWatchCursor (drawWindow);
    SetWatchCursor (mainWindow);
-   if ((topOfDirLinkList = DirListing (curDir, OBJ_FILE_EXT, "")) != NULL)
+   if (curDirIsLocal)
+   {
+      if ((topOfDirLinkList = DirListing (curDir, OBJ_FILE_EXT, "")) != NULL)
+         BuildDirList ();
+   }
+   else if ((topOfDirLinkList = DirListing (curLocalDir, OBJ_FILE_EXT, "")) !=
+         NULL)
       BuildDirList ();
    SetDefaultCursor (mainWindow);
    SetDefaultCursor (drawWindow);
@@ -640,7 +668,7 @@ void CleanUpNames ()
    if (symbolList != NULL) cfree (symbolList);
    if (dirList != NULL) cfree (dirList);
    symbolList = dirList = NULL;
-   ignoreDirectoryFlagInMakeNameDspItemArray = FALSE;
+   ignoreDirectoryFlag = FALSE;
 }
 
 void RedrawNameScrollWin ()
@@ -784,7 +812,7 @@ char * * MakeNameDspItemArray (Entries, DLPtr)
    {
       dsp_ptr[i] = c_ptr;
       len = strlen (DLPtr->itemstr);
-      if (!ignoreDirectoryFlagInMakeNameDspItemArray && !DLPtr->directory)
+      if (!ignoreDirectoryFlag && !DLPtr->directory)
       {
          for (j = len; j >= 0 && DLPtr->itemstr[j] != '/'; j--) ;
          if (j >= 0)
@@ -1092,6 +1120,7 @@ int DirNames (TopStr, ExtStr, SelStr, JustSetDir)
    int			button_selected = INVALID, change_to_root;
    int			faking_dot_dot, tabbed_from_root=TRUE;
    int			just_tabbed_from_root=FALSE;
+   int			name_is_remote=FALSE;
    XEvent		input, ev;
    int			looping = TRUE, changing, name_index, pop_from_root;
    char			buf[80], name[MAXPATHLENGTH], full_name[MAXPATHLENGTH];
@@ -1107,6 +1136,7 @@ int DirNames (TopStr, ExtStr, SelStr, JustSetDir)
    XSetWindowAttributes	win_attrs;
    int			win_x, win_y;
    DspList		* dsp_ptr, * next_dsp;
+   struct BBRec		edit_bbox;
 
    dsp_w = DisplayWidth (mainDisplay, mainScreen);
    dsp_h = DisplayHeight (mainDisplay, mainScreen);
@@ -1173,6 +1203,8 @@ int DirNames (TopStr, ExtStr, SelStr, JustSetDir)
 #else
    XSetWMNormalHints (mainDisplay, nameBaseWin, &sizehints);
 #endif
+   sprintf (buf, "%s - %s", TOOL_NAME, TopStr);
+   XStoreName (mainDisplay, nameBaseWin, buf);
 
    XSetTransientForHint (mainDisplay, nameBaseWin, mainWindow);
 
@@ -1205,8 +1237,10 @@ int DirNames (TopStr, ExtStr, SelStr, JustSetDir)
 
    if (importingFile)
       strcpy (dir_name, curImportDir);
-   else
+   else if (curDirIsLocal)
       strcpy (dir_name, curDir);
+   else
+      strcpy (dir_name, curLocalDir);
 
    for (dsp_ptr = topOfDirLinkList; dsp_ptr != NULL; dsp_ptr = next_dsp)
    {
@@ -1220,7 +1254,7 @@ int DirNames (TopStr, ExtStr, SelStr, JustSetDir)
    SetWatchCursor (nameBaseWin);
    Msg ("Generating list of file names, please wait ...");
    if (ExtStr != NULL && strcmp (ExtStr, EPSF_FILE_EXT) == 0)
-      strcpy (other_ext_str, PS_FILE_EXT);
+      sprintf (other_ext_str, ".%s;.epsi", PS_FILE_EXT);
    else if (ExtStr != NULL && strcmp (ExtStr, OBJ_FILE_EXT) == 0)
       strcpy (other_ext_str, SYM_FILE_EXT);
    else
@@ -1235,6 +1269,11 @@ int DirNames (TopStr, ExtStr, SelStr, JustSetDir)
    SetDefaultCursor (drawWindow);
    SetDrawCursor (nameBaseWin);
 
+   edit_bbox.ltx = graph_start;
+   edit_bbox.lty = 3*ROW_HEIGHT;
+   edit_bbox.rbx = graph_start+ITEM_LEN*defaultFontWidth+8;
+   edit_bbox.rby = 3*ROW_HEIGHT+defaultFontHeight+4;
+
    justClicked = FALSE;
    *saved_dir_name = '\0';
    faking_dot_dot = FALSE;
@@ -1494,9 +1533,14 @@ int DirNames (TopStr, ExtStr, SelStr, JustSetDir)
             {
                tabbed_from_root = TRUE;
                just_tabbed_from_root = TRUE;
-               sprintf (full_name, "%s/%s", dir_name, name);
-               strcpy (tmp_name, name);
-               ParseFileName (full_name, dir_name, name);
+               if (FileIsRemote (name))
+                  name_is_remote = TRUE;
+               else
+               {
+                  sprintf (full_name, "%s/%s", dir_name, name);
+                  strcpy (tmp_name, name);
+                  ParseFileName (full_name, dir_name, name);
+               }
                changing = FALSE;
                index = INVALID;
             }
@@ -1659,6 +1703,19 @@ int DirNames (TopStr, ExtStr, SelStr, JustSetDir)
             {
                if (PointInBBox (button_ev->x, button_ev->y, buttonBBox[0]))
                {
+                  if (!tabbed_from_root)
+                  {
+                     if (FileIsRemote (name))
+                        name_is_remote = TRUE;
+                     else
+                     {
+                        tabbed_from_root = TRUE;
+                        just_tabbed_from_root = TRUE;
+                        sprintf (full_name, "%s/%s", dir_name, name);
+                        strcpy (tmp_name, name);
+                        ParseFileName (full_name, dir_name, name);
+                     }
+                  }
                   changing = FALSE;
                   index = nameMarked;
                   button_selected = BUTTON_OK;
@@ -1687,6 +1744,33 @@ int DirNames (TopStr, ExtStr, SelStr, JustSetDir)
                   index = INVALID;
                   button_selected = BUTTON_CANCEL;
                }
+               else if (!tabbed_from_root && button_ev->button == Button2 &&
+                     PointInBBox (button_ev->x, button_ev->y, edit_bbox))
+               {
+                  int	buf_len=0;
+                  char	* cut_buffer=FetchCutBuffer (&buf_len);
+
+                  if (cut_buffer != NULL)
+                  {
+                     char	* c_ptr=cut_buffer;
+                     int	max_len=sizeof(full_name)-strlen(dir_name)-3;
+
+                     while (name_index < max_len &&
+                           *c_ptr >= '\040' && *c_ptr < '\177')
+                        name[name_index++] = *c_ptr++;
+
+                     name[name_index] = '\0';
+                     sprintf (full_name, "%s/%s", dir_name, name);
+                     if (exposed)
+                     {
+                        RedrawNamePath (full_name, graph_start,
+                              3*ROW_HEIGHT+defaultFontAsc+2);
+                        RedrawNameScrollWin ();
+                        RedrawDspWindow ();
+                     }
+                     XFree (cut_buffer);
+                  }
+               }
             }
             else if (button_ev->window == nameScrollWin)
                NameScrollHandler (button_ev);
@@ -1772,6 +1856,8 @@ int DirNames (TopStr, ExtStr, SelStr, JustSetDir)
          *sel_str = '\0';
          if (dirList != NULL) { cfree (dirList); dirList = NULL; }
       }
+      else if (name_is_remote)
+         break;
       else
       {
          SetWatchCursor (drawWindow);
@@ -1811,21 +1897,38 @@ int DirNames (TopStr, ExtStr, SelStr, JustSetDir)
       DisplayButton (nameBaseWin, buttonStr[button_selected], 8,
             &buttonBBox[button_selected], BUTTON_INVERT);
 
+   XDestroyWindow (mainDisplay, nameBaseWin);
+   if (warpToWinCenter)
+      XWarpPointer (mainDisplay, None, drawWindow, 0, 0, 0, 0,
+            (int)(ZOOMED_SIZE(drawWinW)>>1), (int)(ZOOMED_SIZE(drawWinH)>>1));
+
    if (button_selected == BUTTON_SETDIR)
    {
-      sprintf (SelStr, "%s", dir_name);
-      *JustSetDir = TRUE;
+      if (DirIsRemote (dir_name))
+      {
+         *SelStr = '\0';
+         *JustSetDir = FALSE;
+         sprintf (name, "SETDIR not supported for '%s'.", dir_name);
+         Dialog (name, NULL, NULL);
+      }
+      else
+      {
+         sprintf (SelStr, "%s", dir_name);
+         *JustSetDir = TRUE;
+      }
+   }
+   else if (name_is_remote)
+   {
+      /* index != INVALID means SelStr is valid */
+      index = 0;
+      strcpy (SelStr, name);
+      *JustSetDir = FALSE;
    }
    else
    {
       sprintf (SelStr, "%s/%s", dir_name, sel_str);
       *JustSetDir = FALSE;
    }
-
-   XDestroyWindow (mainDisplay, nameBaseWin);
-   if (warpToWinCenter)
-      XWarpPointer (mainDisplay, None, drawWindow, 0, 0, 0, 0,
-            (int)(ZOOMED_SIZE(drawWinW)>>1), (int)(ZOOMED_SIZE(drawWinH)>>1));
    return (index);
 }
 
@@ -1911,6 +2014,8 @@ int ChooseAName (TopStr)
 #else
    XSetWMNormalHints (mainDisplay, nameBaseWin, &sizehints);
 #endif
+   sprintf (buf, "%s - %s", TOOL_NAME, TopStr);
+   XStoreName (mainDisplay, nameBaseWin, buf);
 
    XSetTransientForHint (mainDisplay, nameBaseWin, mainWindow);
 
@@ -2228,7 +2333,10 @@ int SelectFileName (MsgStr, SelStr)
    DspList	* dsp_ptr, * next_dsp;
    char		saved_cur_dir[MAXPATHLENGTH];
 
-   strcpy (saved_cur_dir, curDir);
+   if (curDirIsLocal)
+      strcpy (saved_cur_dir, curDir);
+   else
+      strcpy (saved_cur_dir, curLocalDir);
    saved_num_dir_entries = numDirEntries;
    if ((index = DirNames (MsgStr,OBJ_FILE_EXT,SelStr,&just_set_dir)) == INVALID)
    {
@@ -2266,7 +2374,10 @@ int SelectFileNameToPaste (MsgStr, SelStr)
    DspList	* dsp_ptr, * next_dsp;
    char		saved_cur_dir[MAXPATHLENGTH];
 
-   strcpy (saved_cur_dir, curDir);
+   if (curDirIsLocal)
+      strcpy (saved_cur_dir, curDir);
+   else
+      strcpy (saved_cur_dir, curLocalDir);
    saved_num_dir_entries = numDirEntries;
    if ((index = DirNames (MsgStr,NULL,SelStr,&just_set_dir)) == INVALID)
    {
@@ -2629,10 +2740,8 @@ static
 DspList * BitmapListing (ExtStr,OtherExtStr)
    char	* ExtStr, * OtherExtStr;
 {
-   int			len, path_index, count = 0, reject;
-   int			ext_len, other_ext_len;
-   char			path[MAXPATHLENGTH];
-   char			ext_str[MAXSTRING], other_ext_str[MAXSTRING];
+   int			len, path_index, count = 0, reject, ext_len;
+   char			path[MAXPATHLENGTH], ext_str[MAXSTRING];
    DspList		* dsp_ptr, * head_ptr, * tail_ptr, * p, * p1;
    DIR			* dirp;
    DIR_ENTRY		* d;
@@ -2641,20 +2750,16 @@ DspList * BitmapListing (ExtStr,OtherExtStr)
 
    sprintf (ext_str, ".%s", ExtStr);
    ext_len = strlen (ext_str);
-   if (*OtherExtStr == '\0')
-   {
-      *other_ext_str = '\0';
-      other_ext_len = 0;
-   }
-   else
-   {
-      sprintf (other_ext_str, ".%s", OtherExtStr);
-      other_ext_len = strlen (other_ext_str);
-   }
    for (path_index = 0; path_index < symPathNumEntries; path_index++)
    {
       strcpy (path, symPath[path_index]);
-      if (strcmp (".", path) == 0) strcpy (path, curDir);
+      if (strcmp (".", path) == 0)
+      {
+         if (curDirIsLocal)
+            strcpy (path, curDir);
+         else
+            strcpy (path, curLocalDir);
+      }
 
       if ((dirp = opendir (path)) == NULL)
          continue;
@@ -2662,10 +2767,9 @@ DspList * BitmapListing (ExtStr,OtherExtStr)
       while ((d = readdir (dirp)) != NULL)
       {
          len = strlen (d->d_name);
-         if (*OtherExtStr == '\0') other_ext_len = len;
-         if (!(len > ext_len && strcmp (ext_str,&d->d_name[len-ext_len]) == 0 ||
-               len > other_ext_len &&
-               strcmp (other_ext_str,&d->d_name[len-other_ext_len]) == 0))
+         if (!((len > ext_len &&
+               strcmp (ext_str,&d->d_name[len-ext_len]) == 0) ||
+               ExtensionMatch(OtherExtStr, d->d_name)))
             continue;
 
          if (head_ptr == NULL)
@@ -2740,7 +2844,7 @@ int SelectFromLibrary (MsgStr, ExtStr, SelStr, PathStr)
    else if (strcmp (ExtStr, EPSF_FILE_EXT) == 0)
    {
       Msg ("Generating list of EPS file names, please wait ...");
-      strcpy (other_ext_str, PS_FILE_EXT);
+      sprintf (other_ext_str, ".%s;.epsi", PS_FILE_EXT);
    }
    else
    {
@@ -2864,6 +2968,17 @@ void SetCurDir (FileName)
    strcpy (file_name, FileName);
    ShrinkName (file_name);
 
+   if (curDirIsLocal && FileIsRemote (FileName))
+   {
+      strcpy (curLocalDir, curDir);
+      if (autoHyperSpaceOnRemote) {
+         inHyperSpace = TRUE;
+         Msg ("Entering hyperspace...");
+      }
+   }
+   else if (!curDirIsLocal && !FileIsRemote (FileName))
+      *curLocalDir = '\0';
+
    for (i = strlen(file_name)-1; i>=0 && file_name[i]!='/'; i--) ;
 
    if (i < 0)
@@ -2879,6 +2994,8 @@ void SetCurDir (FileName)
       file_name[i] = '\0';
       strcpy (curDir, file_name);
    }
+   curDirIsLocal = (!DirIsRemote (curDir));
+   RedrawDummyWindow1 ();
 }
 
 void SetCurSymDir (FileName)
