@@ -25,7 +25,7 @@
  */
 #ifndef lint
 static char RCSid[] =
-      "@(#)$Header: /amnt/maui/tangram/u/william/X11/TGIF2/RCS/obj.c,v 2.44 1994/03/21 04:55:35 william Exp $";
+      "@(#)$Header: /export/bourbon/multimedia/william/X11/TGIF2/RCS/obj.c,v 2.45 1994/10/20 04:10:47 william Exp $";
 #endif
 
 #include <stdio.h>
@@ -549,3 +549,11 @@ void AdjCaches ()
             AdjObjCache (attr_ptr->obj);
    }
 }
+
+void AdjHotSpots()
+{
+}
+
+void AdjObjHotSpot()
+{
+}
