@@ -25,12 +25,15 @@
  */
 #ifndef lint
 static char RCSid[] =
-      "@(#)$Header: /amnt/maui/tangram/u/william/X11/TGIF2/RCS/oval.c,v 2.49 1994/03/21 04:55:42 william Exp $";
+      "@(#)$Header: /export/bourbon/multimedia/william/X11/TGIF2/RCS/oval.c,v 2.55 1994/11/30 06:13:15 william Exp $";
 #endif
 
 #include <stdio.h>
 #include <math.h>
+#include <sys/types.h>
 #include <X11/Xlib.h>
+#include <X11/Xutil.h>
+#include <X11/keysym.h>
 #include "const.h"
 #include "types.h"
 
@@ -340,13 +343,14 @@ void CreateOvalObj (BBox)
    obj_ptr->fattr = obj_ptr->lattr = NULL;
    AddObj (NULL, topObj, obj_ptr);
 }
+
+static XComposeStatus	c_stat;
  
 static
 void ContinueOval (OrigX, OrigY)
    int	OrigX, OrigY;
 {
-   int 			end_x, end_y, grid_x, grid_y;
-   int 			done = FALSE;
+   int 			end_x, end_y, grid_x, grid_y, done=FALSE, abort=FALSE;
    int			xor_pixel;
    char			buf[80];
    struct BBRec		bbox;
@@ -442,8 +446,28 @@ void ContinueOval (OrigX, OrigY)
          DrawIntervalRulers (OrigX, OrigY, grid_x, grid_y);
          while (XCheckMaskEvent (mainDisplay, PointerMotionMask, &ev)) ;
       }
+      else if (input.type == KeyPress)
+      {
+         KeySym	key_sym;
+         char	s[80];
+
+         XLookupString (&(input.xkey), s, 80-1, &key_sym, &c_stat);
+         TranslateKeys (s, &key_sym);
+         if (s[0] == '\033' && (key_sym & 0xff) == '\033')
+         {
+            XUngrabPointer (mainDisplay, CurrentTime);
+            MyOval (drawWindow, drawGC, bbox);
+            EndIntervalRulers (grid_x, grid_y);
+            sprintf (buf, "%1dx%1d%+1d%+1d", ABS_SIZE(abs(bbox.rbx-OrigX)),
+                  ABS_SIZE(abs(bbox.rby-OrigY)), ABS_X(bbox.rbx),
+                  ABS_Y(bbox.rby));
+            EndShowMeasureCursor (bbox.rbx, bbox.rby, buf, TRUE);
+            abort = TRUE;
+            done = TRUE;
+         }
+      }
    }
-   if (OrigX != grid_x && OrigY != grid_y)
+   if (!abort && OrigX != grid_x && OrigY != grid_y)
    {
       if (bbox.ltx > bbox.rbx)
       {
@@ -502,14 +526,14 @@ void ReadOvalObj (Inbuf, ObjPtr)
    struct ObjRec	* * ObjPtr;
 {
    struct OvalRec	* oval_ptr;
-   char			color_str[20], * s;
+   char			color_str[40], * s;
    int			ltx, lty, rbx, rby, fill, width, pen, dash, w, id=0;
    int			new_alloc, rotation, locked=FALSE;
 
    *ObjPtr = NULL;
 
    s = FindChar ('(', Inbuf);
-   s = ParseStr (s, ',', color_str);
+   s = ParseStr (s, ',', color_str, sizeof(color_str));
 
    InitScan (s, "\n\t, ");
 
@@ -637,7 +661,7 @@ void ReadOvalObj (Inbuf, ObjPtr)
    oval_ptr->dash = dash;
    (*ObjPtr)->x = ltx;
    (*ObjPtr)->y = lty;
-   (*ObjPtr)->color = QuickFindColorIndex (color_str, &new_alloc);
+   (*ObjPtr)->color = QuickFindColorIndex (color_str, &new_alloc, TRUE);
    (*ObjPtr)->dirty = FALSE;
    (*ObjPtr)->id = id;
    (*ObjPtr)->rotation = rotation;
