@@ -25,11 +25,12 @@
  */
 #ifndef lint
 static char RCSid[] =
-      "@(#)$Header: /amnt/maui/tangram/u/william/X11/TGIF2/RCS/page.c,v 2.20 1994/04/27 06:28:25 william Exp $";
+      "@(#)$Header: /export/bourbon/multimedia/william/X11/TGIF2/RCS/page.c,v 2.32 1994/12/21 07:16:06 william Exp $";
 #endif
 
 #include <stdio.h>
 #include <string.h>
+#include <sys/types.h>
 #include <X11/Xlib.h>
 #include <X11/Xutil.h>
 #include <X11/keysym.h>
@@ -73,14 +74,15 @@ int		pageLayoutMode=PAGE_STACK;
 int		pageLineShownInTileMode=TRUE;
 
 char * pageStackMenuStr[] =
-      { "NextPage     ",
-        "PrevPage     ",
-        "NamePages    ",
-        "GotoPage     ",
-        "AddPageBefore",
-        "AddPageAfter ",
-        "DeleteCurPage",
-        "PrintOnePage "
+      { "NextPage         ",
+        "PrevPage         ",
+        "NamePages        ",
+        "GotoPage         ",
+        "AddPageBefore    ",
+        "AddPageAfter     ",
+        "DeleteCurPage    ",
+        "PrintOnePage     ",
+        "SpecifyPaperSize "
       };
 static char * pageStackMenuDescription[] =
       { "Go to next page",
@@ -91,18 +93,21 @@ static char * pageStackMenuDescription[] =
         "Add a page after the current page",
         "Delete the current page",
         "Print the curret page",
+        "Set the physical size of the paper",
         NULL
       };
 
 char * pageTileMenuStr[] =
       { "TogglePageLineShown",
         "SpecifyDrawingSize ",
-        "PrintOnePage       "
+        "PrintOnePage       ",
+        "SpecifyPaperSize   "
       };
 static char * pageTileMenuDescription[] =
       { "Toggle the showing of page boundaries",
         "Change the drawing size",
         "Select and print one of the pages",
+        "Set the physical size of the paper",
         NULL
       };
 
@@ -154,8 +159,8 @@ void GotoPageNum (new_page_num)
    if (curPage->draw_orig_x != drawOrigX || curPage->draw_orig_y != drawOrigY ||
          curPage->zoom_scale != zoomScale || curPage->zoomed_in != zoomedIn)
    {
-      AdjSplineVs();
-      AdjCaches();
+      if (!PRTGIF) AdjSplineVs();
+      if (!PRTGIF) AdjCaches();
       curPage->draw_orig_x = drawOrigX;
       curPage->draw_orig_y = drawOrigY;
       curPage->zoom_scale = zoomScale;
@@ -320,6 +325,13 @@ int EditOrSelectPageNames (TopStr, Which)
 #else
    XSetWMNormalHints (mainDisplay, nameBaseWin, &sizehints);
 #endif
+   switch (Which)
+   {
+      case PAGE_GOTO: sprintf (buf, "%s - Goto Page", TOOL_NAME); break;
+      case PAGE_NAME: sprintf (buf, "%s - Name Pages", TOOL_NAME); break;
+      default: sprintf (buf, "%s - Page Information", TOOL_NAME); break;
+   }
+   XStoreName (mainDisplay, nameBaseWin, buf);
 
    XSetTransientForHint (mainDisplay, nameBaseWin, mainWindow);
 
@@ -605,9 +617,9 @@ void NamePages ()
    MakeQuiescent ();
 
    dsp_ptr = PageNameListing ();
-   ignoreDirectoryFlagInMakeNameDspItemArray = TRUE;
+   ignoreDirectoryFlag = TRUE;
    nameDspPtr = MakeNameDspItemArray (nameEntries, dsp_ptr);
-   ignoreDirectoryFlagInMakeNameDspItemArray = FALSE;
+   ignoreDirectoryFlag = FALSE;
    nameFirst = 0;
    nameMarked = curPageNum-1;
    if (EditOrSelectPageNames ("Edit page names...", PAGE_NAME))
@@ -663,7 +675,6 @@ void NamePages ()
 
 void GotoPage ()
 {
-   char		msg[MAXSTRING+1];
    DspList	* dsp_ptr;
    int		rc;
 
@@ -675,9 +686,9 @@ void GotoPage ()
    MakeQuiescent ();
 
    dsp_ptr = PageNameListing ();
-   ignoreDirectoryFlagInMakeNameDspItemArray = TRUE;
+   ignoreDirectoryFlag = TRUE;
    nameDspPtr = MakeNameDspItemArray (nameEntries, dsp_ptr);
-   ignoreDirectoryFlagInMakeNameDspItemArray = FALSE;
+   ignoreDirectoryFlag = FALSE;
    nameFirst = 0;
    nameMarked = curPageNum-1;
 
@@ -697,20 +708,20 @@ void GotoPage ()
       }
       if (nameMarked < 1 || nameMarked > lastPageNum)
       {
-         sprintf (msg, "Invalid page number '%1d'.", nameMarked);
-         Msg (msg);
+         sprintf (gszMsgBox, "Invalid page number '%1d'.", nameMarked);
+         Msg (gszMsgBox);
          return;
       }
       if (nameMarked == curPageNum)
       {
-         sprintf (msg, "Already at page %1d.", curPageNum);
-         Msg (msg);
+         sprintf (gszMsgBox, "Already at page %1d.", curPageNum);
+         Msg (gszMsgBox);
          return;
       }
       SetCurPage (nameMarked);
 
-      sprintf (msg, "Current page is page %1d.", curPageNum);
-      Msg (msg);
+      sprintf (gszMsgBox, "Current page is page %1d.", curPageNum);
+      Msg (gszMsgBox);
    }
    else
       Msg ("");
@@ -723,7 +734,6 @@ static
 void AddPage (AfterCurPage)
    int AfterCurPage;
 {
-   char			msg[MAXSTRING+1];
    int			n;
    struct PageRec	* page_ptr;
 
@@ -734,10 +744,10 @@ void AddPage (AfterCurPage)
    }
    if (firstCmd != NULL)
    {
-      sprintf (msg, "Adding a page %s page %1d can not be undone.  %s",
+      sprintf (gszMsgBox, "Adding a page %s page %1d can not be undone.  %s",
             (AfterCurPage ? "after" : "before"), curPageNum,
             "Ok to proceed? [ync](y)");
-      if (YesNoCancel (msg, CONFIRM_YES) != CONFIRM_YES) return;
+      if (MsgBox (gszMsgBox, TOOL_NAME, YNC_MB) != MB_ID_YES) return;
       CleanUpCmds ();
    }
    n = (AfterCurPage ? curPageNum : curPageNum-1);
@@ -779,8 +789,8 @@ void AddPage (AfterCurPage)
    SetFileModified (TRUE);
    ShowPage ();
 
-   sprintf (msg, "Page %1d added.", curPageNum);
-   Msg (msg);
+   sprintf (gszMsgBox, "Page %1d added.", curPageNum);
+   Msg (gszMsgBox);
 }
 
 void AddPageBefore ()
@@ -795,7 +805,6 @@ void AddPageAfter ()
 
 void DeleteCurPage ()
 {
-   char			msg[MAXSTRING+1];
    int			n;
    struct PageRec	* page_ptr;
 
@@ -811,9 +820,9 @@ void DeleteCurPage ()
    }
    if (firstCmd != NULL || topObj != NULL)
    {
-      sprintf (msg, "Deleting page %1d can not be undone.  %s",
+      sprintf (gszMsgBox, "Deleting page %1d can not be undone.  %s",
             curPageNum, "Ok to proceed? [ync](y)");
-      if (YesNoCancel (msg, CONFIRM_YES) != CONFIRM_YES) return;
+      if (MsgBox (gszMsgBox, TOOL_NAME, YNC_MB) != MB_ID_YES) return;
       CleanUpCmds ();
    }
    n = curPageNum;
@@ -847,8 +856,8 @@ void DeleteCurPage ()
    RedrawTitleWindow ();
    ShowPage ();
 
-   sprintf (msg, "Page %1d deleted.", n);
-   Msg (msg);
+   sprintf (gszMsgBox, "Page %1d deleted.", n);
+   Msg (gszMsgBox);
 }
 
 void TogglePageLineShown ()
@@ -865,7 +874,7 @@ void TogglePageLineShown ()
 static
 int JustSpecifyDrawingSize ()
 {
-   char	spec[MAXSTRING+1], * c_ptr, * part_1, msg[MAXSTRING+1];
+   char	spec[MAXSTRING+1], * c_ptr, * part_1;
    int	cols=0, rows=0;
 
    if (Dialog ("Please enter drawing size specification: [W x H]",
@@ -874,9 +883,9 @@ int JustSpecifyDrawingSize ()
    if ((part_1=strtok(spec, " ,xX")) == NULL) return (FALSE);
    if ((c_ptr=strtok(NULL, " ,xX")) == NULL)
    {
-      sprintf (msg, "Invalid drawing size specification.  %s.",
+      sprintf (gszMsgBox, "Invalid drawing size specification.\n\n%s.",
             "Must specify both Width and Height");
-      Dialog (msg, "( <CR> or <ESC> to continue )", spec);
+      MsgBox (gszMsgBox, TOOL_NAME, INFO_MB);
       return (FALSE);
    }
    else
@@ -888,9 +897,10 @@ int JustSpecifyDrawingSize ()
    {
       if (cols*rows < lastPageNum)
       {
-         sprintf (msg, "Invalid drawing size:  W times H should be >= %1d",
+         sprintf (gszMsgBox,
+               "Invalid drawing size:  W times H should be >= %1d",
                lastPageNum);
-         Dialog (msg, "( <CR> or <ESC> to continue )", spec);
+         MsgBox (gszMsgBox, TOOL_NAME, INFO_MB);
       }
       else
       {
@@ -901,20 +911,19 @@ int JustSpecifyDrawingSize ()
    }
    else if (cols == 0 || rows == 0)
    {
-      sprintf (msg, "Invalid drawing size specification.  %s.",
+      sprintf (gszMsgBox, "Invalid drawing size specification.\n\n%s.",
             "Must specify both Width and Height");
-      Dialog (msg, "( <CR> or <ESC> to continue )", spec);
+      MsgBox (gszMsgBox, TOOL_NAME, INFO_MB);
       return (FALSE);
    }
    else if (cols*rows < lastPageNum)
    {
-      sprintf (msg, "Invalid drawing size:  W times H should be >= %1d",
+      sprintf (gszMsgBox, "Invalid drawing size:  W times H should be >= %1d",
             lastPageNum);
-      Dialog (msg, "( <CR> or <ESC> to continue )", spec);
+      MsgBox (gszMsgBox, TOOL_NAME, INFO_MB);
    }
    else
-      Dialog ("Invalid drawing size specified.",
-            "( <CR> or <ESC> to continue )", spec);
+      MsgBox ("Invalid drawing size specified.", TOOL_NAME, INFO_MB);
    return (FALSE);
 }
 
@@ -940,13 +949,12 @@ void SpecifyDrawingSize ()
 void PrintOnePage ()
 {
    int	x, y;
-   char	msg[81];
 
    if (whereToPrint == XBM_FILE)
    {
-      sprintf (msg, "Can not do PrintOnePage() in %s format.",
+      sprintf (gszMsgBox, "Can not do PrintOnePage() in %s format.",
             colorDump ? "XPM" : "XBM");
-      Msg (msg);
+      Msg (gszMsgBox);
       return;
    }
    MakeQuiescent ();
@@ -980,6 +988,117 @@ void PrintOnePage ()
    }
 }
 
+#ifdef __hpux
+extern double   atof ARGS_DECL((const char *));
+#else /* !__hpux */
+extern double   atof ARGS_DECL((char *));
+#endif /* __hpux */
+
+static
+int GetDimension (Spec, NumPixels)
+   char	* Spec;
+   int	* NumPixels;
+{
+   char		* c_ptr;
+   double	val;
+
+   if ((c_ptr=strstr(Spec, "in")) != NULL ||
+         (c_ptr=strstr(Spec, "In")) != NULL ||
+         (c_ptr=strstr(Spec, "IN")) != NULL)
+   {
+      *c_ptr = '\0';
+      val = atof (Spec) * ((double)PIX_PER_INCH);
+      *NumPixels = round(val);
+      if (*NumPixels <= 0) return FALSE;
+   }
+   else if ((c_ptr=strstr(Spec, "cm")) != NULL ||
+         (c_ptr=strstr(Spec, "Cm")) != NULL ||
+         (c_ptr=strstr(Spec, "CM")) != NULL)
+   {
+      *c_ptr = '\0';
+      val = atof (Spec) * ((double)ONE_CM);
+      *NumPixels = round(val);
+      if (*NumPixels <= 0) return FALSE;
+   }
+   else if (sscanf(Spec, "%d", NumPixels) != 1)
+   {
+      return FALSE;
+   }
+   return TRUE;
+}
+
+int SetPaperSize (Spec)
+   char	* Spec;
+{
+   int	ok=TRUE;
+   char	* c_ptr;
+
+   TrimBlanks (Spec);
+   if ((c_ptr=strstr(Spec, " x ")) == NULL &&
+         (c_ptr=strstr(Spec, " X ")) == NULL)
+      ok = FALSE;
+   else
+   {
+      int	w_in_pixel, h_in_pixel;
+
+      *c_ptr = '\0';
+      TrimBlanks (&c_ptr[3]);
+      if (GetDimension(Spec, &w_in_pixel) &&
+            GetDimension(&c_ptr[3], &h_in_pixel))
+      {
+         onePageWidth = w_in_pixel;
+         onePageHeight = h_in_pixel;
+         SetPSPageWidthHeight ();
+      }
+      else
+         ok = FALSE;
+   }
+   if (ok)
+   {
+      if (mainWindow != None)
+      {
+         sprintf (gszMsgBox,
+               "New paper size: %.2fin x %.2fin (%.2fcm x %.2fcm).",
+               (float)(((float)onePageWidth)/((float)PIX_PER_INCH)),
+               (float)(((float)onePageHeight)/((float)PIX_PER_INCH)),
+               (float)(((float)onePageWidth)/((float)ONE_CM)),
+               (float)(((float)onePageHeight)/((float)ONE_CM)));
+         Msg (gszMsgBox);
+      }
+   }
+   else
+   {
+      if (mainWindow == None)
+      {
+         sprintf (gszMsgBox, "Invalid %s*InitialPaperSize: '%s'.", TOOL_NAME,
+               Spec);
+         fprintf (stderr, "%s\n", gszMsgBox);
+      }
+      else
+      {
+         sprintf (gszMsgBox, "Invalid paper size specification: '%s'.", Spec);
+         MsgBox (gszMsgBox, TOOL_NAME, INFO_MB);
+      }
+   }
+   return (ok);
+}
+
+void SpecifyPaperSize ()
+{
+   char	spec[MAXSTRING+1];
+
+   MakeQuiescent ();
+   if (Dialog ("Please enter physical paper size: [W x H] (e.g., 8.5in x 11in)",
+         "( <CR>: accept, <ESC>: cancel )", spec) == INVALID)
+      return;
+   if (SetPaperSize (spec))
+   {
+      UpdPageStyle (pageStyle);
+      SetFileModified (TRUE);
+      ClearAndRedrawDrawWindow ();
+   }
+}
+
 void PageSubMenu (index)
    int	index;
 {
@@ -996,6 +1115,7 @@ void PageSubMenu (index)
             case PAGE_ADDAFTER: AddPageAfter (); break;
             case PAGE_DEL: DeleteCurPage (); break;
             case PAGE_PRINT_ONE_IN_STACK: PrintOnePage (); break;
+            case PAGE_PAPER_SIZE_IN_STACK: SpecifyPaperSize (); break;
          }
          break;
       case PAGE_TILE:
@@ -1004,6 +1124,7 @@ void PageSubMenu (index)
             case PAGE_TOGGLELINE: TogglePageLineShown (); break;
             case PAGE_SIZING: SpecifyDrawingSize (); break;
             case PAGE_PRINT_ONE: PrintOnePage (); break;
+            case PAGE_PAPER_SIZE: SpecifyPaperSize (); break;
          }
          break;
    }
@@ -1064,7 +1185,7 @@ int AnyObjOnPageBoundary ()
 {
    register struct ObjRec	* obj_ptr;
    int				ltx, lty, rbx, rby, next_page_x, next_page_y;
-   char				msg[MAXSTRING+1], dummy_str[MAXSTRING+1];
+   char				dummy_str[MAXSTRING+1];
 
    for (obj_ptr = topObj; obj_ptr != NULL; obj_ptr = obj_ptr->next)
    {
@@ -1073,10 +1194,10 @@ int AnyObjOnPageBoundary ()
       if (ltx < 0 || lty < 0 || rbx >= onePageWidth*paperCol ||
             rby >= onePageHeight*paperRow)
       {
-         sprintf (msg, "%s.  %s.",
+         sprintf (gszMsgBox, "%s.\n\n%s.",
                "There are objects outside the page boundary",
                "Can not switch to TILED mode");
-         Dialog (msg, "( <CR> or <ESC> to continue )", dummy_str);
+         MsgBox (gszMsgBox, TOOL_NAME, INFO_MB);
          return (TRUE);
       }
       next_page_x = ((ltx % onePageWidth) == 0) ?
@@ -1087,10 +1208,10 @@ int AnyObjOnPageBoundary ()
             onePageHeight * (((int)(lty / onePageHeight)) + 1);
       if (rbx >= next_page_x || rby >= next_page_y)
       {
-         sprintf (msg, "%s.  %s.",
+         sprintf (gszMsgBox, "%s.\n\n%s.",
                "There are objects on the page boundary",
                "Can not switch to TILED mode");
-         Dialog (msg, "( <CR> or <ESC> to continue )", dummy_str);
+         MsgBox (gszMsgBox, TOOL_NAME, INFO_MB);
          return (TRUE);
       }
    }
@@ -1206,11 +1327,9 @@ int OkToFlushUndoBuffer ()
 {
    if (firstCmd != NULL)
    {
-      char	msg[MAXSTRING+1];
-
-      sprintf (msg, "%s.  %s? [ync](y)",
+      sprintf (gszMsgBox, "%s.  %s? [ync](y)",
             "Changing the page layout mode can not be undone", "Ok to proceed");
-      if (YesNoCancel (msg, CONFIRM_YES) == CONFIRM_YES)
+      if (MsgBox (gszMsgBox, TOOL_NAME, YNC_MB) == MB_ID_YES)
       {
          CleanUpCmds ();
          return (TRUE);
@@ -1230,12 +1349,10 @@ int PreservePageNames ()
          break;
    if (page_ptr != NULL)
    {
-      char msg[MAXSTRING];
-
-      sprintf (msg, "%s  %s [ync](y)",
+      sprintf (gszMsgBox, "%s  %s [ync](y)",
             "Switching from TILED to STACK mode loses all page names.",
             "Ok to proceed?" );
-      if (YesNoCancel (msg, CONFIRM_YES) != CONFIRM_YES)
+      if (MsgBox (gszMsgBox, TOOL_NAME, YNC_MB) != MB_ID_YES)
          return (FALSE);
    }
    return (TRUE);
