@@ -25,10 +25,11 @@
  */
 #ifndef lint
 static char RCSid[] =
-      "@(#)$Header: /amnt/maui/tangram/u/william/X11/TGIF2/RCS/pattern.c,v 2.46 1994/04/04 00:49:06 william Exp $";
+      "@(#)$Header: /export/bourbon/multimedia/william/X11/TGIF2/RCS/pattern.c,v 2.49 1994/11/17 06:51:58 william Exp $";
 #endif
 
 #include <stdio.h>
+#include <string.h>
 #include <X11/Xlib.h>
 #include "const.h"
 #include "types.h"
@@ -55,6 +56,7 @@ static char RCSid[] =
 #include "setup.e"
 #include "spline.e"
 #include "text.e"
+#include "util.e"
 
 int     objFill = NONEPAT;
 int     lineStyle = LS_RIGHT;
@@ -306,8 +308,8 @@ int ChangeObjFill (ObjPtr, FillIndex)
    return (changed);
 }
 
-void ChangeAllSelFill (FillIndex)
-   int	FillIndex;
+void ChangeAllSelFill (FillIndex, HighLight)
+   int	FillIndex, HighLight;
 {
    register struct SelRec	* sel_ptr;
    int				changed=FALSE, ltx, lty, rbx, rby;
@@ -320,7 +322,7 @@ void ChangeAllSelFill (FillIndex)
       if (topSel == NULL) return;
    }
 
-   HighLightReverse ();
+   if (HighLight) HighLightReverse ();
    StartCompositeCmd ();
    for (sel_ptr = botSel; sel_ptr != NULL; sel_ptr = sel_ptr->prev)
    {
@@ -345,7 +347,7 @@ void ChangeAllSelFill (FillIndex)
             selLtY-GRID_ABS_SIZE(1), selRbX+GRID_ABS_SIZE(1),
             selRbY+GRID_ABS_SIZE(1));
    }
-   HighLightForward ();
+   if (HighLight) HighLightForward ();
 }
 
 static char * fillMenuDescription[] = { "\nSet fill pattern\n", NULL };
@@ -362,7 +364,7 @@ int FillMenu (X, Y, TrackMenubar)
          fore_colors, patPixmap, init_rv, fillMenuDescription, SINGLECOLOR,
          TrackMenubar);
 
-   if (index >= 0) ChangeAllSelFill (index);
+   if (index >= 0) ChangeAllSelFill (index, TRUE);
    return (index);
 }
 
@@ -409,8 +411,8 @@ int ChangeObjLineStyle (ObjPtr, StyleIndex)
    return (changed);
 }
 
-void ChangeAllSelLineStyle (StyleIndex)
-   int	StyleIndex;
+void ChangeAllSelLineStyle (StyleIndex, HighLight)
+   int	StyleIndex, HighLight;
 {
    register struct SelRec	* sel_ptr;
    int				ltx, lty, rbx, rby, changed=FALSE;
@@ -423,7 +425,7 @@ void ChangeAllSelLineStyle (StyleIndex)
       if (topSel == NULL) return;
    }
 
-   HighLightReverse ();
+   if (HighLight) HighLightReverse ();
    StartCompositeCmd ();
    for (sel_ptr = botSel; sel_ptr != NULL; sel_ptr = sel_ptr->prev)
    {
@@ -448,7 +450,7 @@ void ChangeAllSelLineStyle (StyleIndex)
             selLtY-GRID_ABS_SIZE(1), selRbX+GRID_ABS_SIZE(1),
             selRbY+GRID_ABS_SIZE(1));
    }
-   HighLightForward ();
+   if (HighLight) HighLightForward ();
 }
 
 static
@@ -651,8 +653,8 @@ int ChangeObjLineType (ObjPtr, TypeIndex)
    return (changed);
 }
 
-void ChangeAllSelLineType (TypeIndex)
-   int	TypeIndex;
+void ChangeAllSelLineType (TypeIndex, HighLight)
+   int	TypeIndex, HighLight;
 {
    register struct SelRec	* sel_ptr;
    int				ltx, lty, rbx, rby, changed=FALSE;
@@ -665,7 +667,7 @@ void ChangeAllSelLineType (TypeIndex)
       if (topSel == NULL) return;
    }
 
-   HighLightReverse ();
+   if (HighLight) HighLightReverse ();
    StartCompositeCmd ();
    for (sel_ptr = botSel; sel_ptr != NULL; sel_ptr = sel_ptr->prev)
    {
@@ -690,7 +692,7 @@ void ChangeAllSelLineType (TypeIndex)
             selLtY-GRID_ABS_SIZE(1), selRbX+GRID_ABS_SIZE(1),
             selRbY+GRID_ABS_SIZE(1));
    }
-   HighLightForward ();
+   if (HighLight) HighLightForward ();
 }
 
 static
@@ -704,12 +706,13 @@ int ChangeObjLineWidth (ObjPtr, W, AW, AH)
    switch (ObjPtr->type)
    {
       case OBJ_POLY:
-         if (ObjPtr->detail.p->width != W || ObjPtr->detail.p->aw != AW ||
-               ObjPtr->detail.p->ah != AH)
+         if (ObjPtr->detail.p->width != W ||
+               (AW != (-1) && ObjPtr->detail.p->aw != AW) ||
+               (AH != (-1) && ObjPtr->detail.p->ah != AH))
          {
             ObjPtr->detail.p->width = W;
-            ObjPtr->detail.p->aw = AW;
-            ObjPtr->detail.p->ah = AH;
+            if (AW != (-1)) ObjPtr->detail.p->aw = AW;
+            if (AH != (-1)) ObjPtr->detail.p->ah = AH;
             changed = TRUE;
             AdjObjSplineVs (ObjPtr);
             if (ObjPtr->detail.p->curved != LT_INTSPLINE)
@@ -742,12 +745,13 @@ int ChangeObjLineWidth (ObjPtr, W, AW, AH)
          }
          break;
       case OBJ_ARC:
-         if (ObjPtr->detail.a->width != W || ObjPtr->detail.a->aw != AW ||
-               ObjPtr->detail.a->ah != AH)
+         if (ObjPtr->detail.a->width != W ||
+               (AW != (-1) && ObjPtr->detail.a->aw != AW) ||
+               (AH != (-1) && ObjPtr->detail.a->ah != AH))
          {
             ObjPtr->detail.a->width = W;
-            ObjPtr->detail.a->aw = AW;
-            ObjPtr->detail.a->ah = AH;
+            if (AW != (-1)) ObjPtr->detail.a->aw = AW;
+            if (AH != (-1)) ObjPtr->detail.a->ah = AH;
             changed = TRUE;
          }
          break;
@@ -771,8 +775,8 @@ int ChangeObjLineWidth (ObjPtr, W, AW, AH)
    return (changed);
 }
 
-void ChangeAllSelLineWidth (WidthIndex)
-   int	WidthIndex;
+void ChangeAllSelLineWidth (WidthIndex, HighLight)
+   int	WidthIndex, HighLight;
 {
    register struct SelRec	* sel_ptr;
    int				ltx, lty, rbx, rby, changed=FALSE;
@@ -790,7 +794,7 @@ void ChangeAllSelLineWidth (WidthIndex)
    aw = curArrowHeadW[WidthIndex];
    ah = curArrowHeadH[WidthIndex];
 
-   HighLightReverse ();
+   if (HighLight) HighLightReverse ();
    StartCompositeCmd ();
    for (sel_ptr = botSel; sel_ptr != NULL; sel_ptr = sel_ptr->prev)
    {
@@ -815,7 +819,7 @@ void ChangeAllSelLineWidth (WidthIndex)
             selLtY-GRID_ABS_SIZE(1), selRbX+GRID_ABS_SIZE(1),
             selRbY+GRID_ABS_SIZE(1));
    }
-   HighLightForward ();
+   if (HighLight) HighLightForward ();
 }
 
 static
@@ -882,8 +886,8 @@ int ChangeObjDashes (ObjPtr, DashIndex)
    return (changed);
 }
 
-void ChangeAllSelDashes (DashIndex)
-   int	DashIndex;
+void ChangeAllSelDashes (DashIndex, HighLight)
+   int	DashIndex, HighLight;
 {
    register struct SelRec	* sel_ptr;
    int				changed=FALSE;
@@ -896,7 +900,7 @@ void ChangeAllSelDashes (DashIndex)
       if (topSel == NULL) return;
    }
 
-   HighLightReverse ();
+   if (HighLight) HighLightReverse ();
    StartCompositeCmd ();
    for (sel_ptr = botSel; sel_ptr != NULL; sel_ptr = sel_ptr->prev)
    {
@@ -917,7 +921,7 @@ void ChangeAllSelDashes (DashIndex)
       RedrawAnArea (botObj, selLtX-GRID_ABS_SIZE(1), selLtY-GRID_ABS_SIZE(1),
             selRbX+GRID_ABS_SIZE(1), selRbY+GRID_ABS_SIZE(1));
    }
-   HighLightForward ();
+   if (HighLight) HighLightForward ();
 }
 
 int LineWidthMenu (X, Y, TrackMenubar)
@@ -943,7 +947,7 @@ int LineWidthMenu (X, Y, TrackMenubar)
          maxLineWidths, fore_colors, lineWidthPixmap, init_rv, desc,
          SINGLECOLOR, TrackMenubar);
 
-   if (index >= 0) ChangeAllSelLineWidth (index);
+   if (index >= 0) ChangeAllSelLineWidth (index, TRUE);
    if (desc != NULL)
    {
       for (i=0; i < maxLineWidths; i++)
@@ -975,7 +979,7 @@ int LineStyleMenu (X, Y, TrackMenubar)
          MAXLINESTYLES, fore_colors, lineStylePixmap, init_rv,
          lineStyleMenuDescription, SINGLECOLOR, TrackMenubar);
 
-   if (index >= 0) ChangeAllSelLineStyle (index);
+   if (index >= 0) ChangeAllSelLineStyle (index, TRUE);
    return (index);
 }
 
@@ -1000,7 +1004,7 @@ int LineTypeMenu (X, Y, TrackMenubar)
          MAXLINETYPES, fore_colors, lineTypePixmap, init_rv,
          lineTypeMenuDescription, SINGLECOLOR, TrackMenubar);
 
-   if (index >= 0) ChangeAllSelLineType (index);
+   if (index >= 0) ChangeAllSelLineType (index, TRUE);
    return (index);
 }
 
@@ -1022,7 +1026,7 @@ int LineDashMenu (X, Y, TrackMenubar)
          MAXDASHES, fore_colors, dashPixmap, init_rv, lineDashMenuDescription,
          SINGLECOLOR, TrackMenubar);
 
-   if (index >= 0) ChangeAllSelDashes (index);
+   if (index >= 0) ChangeAllSelDashes (index, TRUE);
    return (index);
 }
 
@@ -1097,8 +1101,8 @@ int ChangeObjPen (ObjPtr, PenIndex)
    return (changed);
 }
 
-void ChangeAllSelPen (PenIndex)
-   int	PenIndex;
+void ChangeAllSelPen (PenIndex, HighLight)
+   int	PenIndex, HighLight;
 {
    register struct SelRec	* sel_ptr;
    int				changed=FALSE;
@@ -1111,7 +1115,7 @@ void ChangeAllSelPen (PenIndex)
       if (topSel == NULL) return;
    }
 
-   HighLightReverse ();
+   if (HighLight) HighLightReverse ();
    StartCompositeCmd ();
    for (sel_ptr = botSel; sel_ptr != NULL; sel_ptr = sel_ptr->prev)
    {
@@ -1132,7 +1136,7 @@ void ChangeAllSelPen (PenIndex)
       RedrawAnArea (botObj, selLtX-GRID_ABS_SIZE(1), selLtY-GRID_ABS_SIZE(1),
             selRbX+GRID_ABS_SIZE(1), selRbY+GRID_ABS_SIZE(1));
    }
-   HighLightForward ();
+   if (HighLight) HighLightForward ();
 }
 
 static char * penMenuDescription[] = { "\nSet pen pattern\n", NULL };
@@ -1149,7 +1153,7 @@ int PenMenu (X, Y, TrackMenubar)
          fore_colors, patPixmap, init_rv, penMenuDescription, SINGLECOLOR,
          TrackMenubar);
 
-   if (index >= 0) ChangeAllSelPen (index);
+   if (index >= 0) ChangeAllSelPen (index, TRUE);
    return (index);
 }
 
@@ -1366,3 +1370,74 @@ void UpdateSelObjs ()
    }
    HighLightForward ();
 }
+
+void ChangeAllSelRealLineWidth (Width, AW, AH, HighLight)
+   int	Width, AW, AH, HighLight;
+{
+   register struct SelRec	* sel_ptr;
+   int				changed=FALSE;
+
+   if (topSel == NULL) return;
+
+   if (HighLight) HighLightReverse ();
+   StartCompositeCmd ();
+   for (sel_ptr = botSel; sel_ptr != NULL; sel_ptr = sel_ptr->prev)
+   {
+      PrepareToReplaceAnObj (sel_ptr->obj);
+      if (ChangeObjLineWidth (sel_ptr->obj, Width, AW, AH))
+      {
+         changed = TRUE;
+         RecordReplaceAnObj (sel_ptr->obj);
+      }
+      else
+         AbortPrepareCmd (CMD_REPLACE);
+   }
+   EndCompositeCmd ();
+
+   if (changed)
+   {
+      int ltx, lty, rbx, rby;
+
+      SetFileModified (TRUE);
+      ltx = selLtX; lty = selLtY; rbx = selRbX; rby = selRbY;
+      UpdSelBBox ();
+      RedrawAreas (botObj, ltx-GRID_ABS_SIZE(1), lty-GRID_ABS_SIZE(1),
+            rbx+GRID_ABS_SIZE(1), rby+GRID_ABS_SIZE(1), selLtX-GRID_ABS_SIZE(1),
+            selLtY-GRID_ABS_SIZE(1), selRbX+GRID_ABS_SIZE(1),
+            selRbY+GRID_ABS_SIZE(1));
+   }
+   if (HighLight) HighLightForward ();
+}
+
+void SetSelectedLineWidth()
+{
+   char spec[MAXSTRING+1], *w_str=NULL, *aw_str=NULL, *ah_str=NULL;
+   int w=(-1), aw=(-1), ah=(-1);
+
+   if (topSel == NULL) {
+      MsgBox("No object selected!", TOOL_NAME, INFO_MB);
+      return;
+   }
+   *spec = '\0';
+   Dialog("Please enter line width (arrow width and height are optional):",
+         "( <CR>: accept, <ESC>: cancel )", spec);
+   UtilTrimBlanks(spec);
+   if (*spec == '\0') return;
+   if ((w_str=strtok(spec, ", \t\n\r")) == NULL) return;
+   w = atoi(w_str);
+   if (w < 0) {
+      sprintf(gszMsgBox, "Invalid width: '%s'.", w_str);
+      MsgBox(gszMsgBox, TOOL_NAME, INFO_MB);
+      return;
+   }
+   if ((aw_str=strtok(NULL, ", \t\n\r")) != NULL &&
+         (ah_str=strtok(NULL, ", \t\n\r")) != NULL) {
+      if ((aw=atoi(aw_str)) < 0 || (ah=atoi(ah_str)) < 0) {
+         sprintf(gszMsgBox, "Invalid arrow width and height: '%s %s'.",
+               aw_str, ah_str);
+         MsgBox(gszMsgBox, TOOL_NAME, INFO_MB);
+         return;
+      }
+   }
+   ChangeAllSelRealLineWidth(w, aw, ah, TRUE);
+}
