@@ -25,13 +25,16 @@
  */
 #ifndef lint
 static char RCSid[] =
-      "@(#)$Header: /amnt/maui/tangram/u/william/X11/TGIF2/RCS/poly.c,v 2.86 1994/05/01 23:42:04 william Exp $";
+      "@(#)$Header: /export/bourbon/multimedia/william/X11/TGIF2/RCS/poly.c,v 2.100 1994/12/09 06:45:33 william Exp $";
 #endif
 
 #include <math.h>
 #include <stdio.h>
 #include <string.h>
+#include <sys/types.h>
 #include <X11/Xlib.h>
+#include <X11/Xutil.h>
+#include <X11/keysym.h>
 #include "const.h"
 #include "types.h"
 
@@ -286,19 +289,23 @@ void CreatePolyObj (NumPts, CreateAbsolute)
    AddObj (NULL, topObj, obj_ptr);
 }
 
+static XComposeStatus	c_stat;
+
 static
 void ContinuePoly (OrigX, OrigY)
    int 	OrigX, OrigY;
    /* OrigX and OrigY are screen coordinates (scaled and translated). */
    /* OrigX and OrigY are also on grid. */
 {
+   register int		i;
    XGCValues		values;
    XEvent		input, ev;
    XButtonEvent		* button_ev;
    XMotionEvent		* motion_ev;
-   int			xor_pixel;
-   int 			end_x, end_y, grid_x, grid_y, done = FALSE, num_pts = 1;
+   int			xor_pixel, abort=FALSE;
+   int 			end_x, end_y, grid_x, grid_y, done=FALSE, num_pts=1;
    int			last_x=OrigX, last_y=OrigY, n=2, sn=0, max_n=40, intn=0;
+   int			ltx=OrigX, lty=OrigY, rbx=OrigX, rby=OrigY;
    char			buf[80];
    struct PtRec		* pt_ptr;
    XPoint		* v=NULL, * sv=NULL, * cntrlv=NULL;
@@ -335,6 +342,13 @@ void ContinuePoly (OrigX, OrigY)
          case LT_INTSPLINE:
             sv = MakeIntSplinePolyVertex (&sn, &intn, &cntrlv,
                   drawOrigX, drawOrigY, n, v);
+            for (i=0; i < sn; i++)
+            {
+               if (sv[i].x < ltx) ltx = sv[i].x;
+               if (sv[i].y < lty) lty = sv[i].y;
+               if (sv[i].x > rbx) rbx = sv[i].x;
+               if (sv[i].y > rby) rby = sv[i].y;
+            }
             break;
       }
    }
@@ -342,6 +356,8 @@ void ContinuePoly (OrigX, OrigY)
    if (curChoice == FREEHAND)
    {
       Msg ("Release button to terminate freehand.");
+      sprintf (buf, "%+1d%+1d", ABS_X(grid_x), ABS_Y(grid_y));
+      StartShowMeasureCursor (grid_x, grid_y, buf, TRUE);
       XGrabPointer (mainDisplay, drawWindow, FALSE,
             PointerMotionMask | ButtonReleaseMask,
             GrabModeAsync, GrabModeAsync, None, handCursor, CurrentTime);
@@ -365,12 +381,12 @@ void ContinuePoly (OrigX, OrigY)
          ExposeEventHandler (&input, TRUE);
       else if (input.type == MotionNotify)
       {
-         if (curChoice != FREEHAND)
-         {
+         if (curChoice == FREEHAND)
+            sprintf (buf, "%+1d%+1d", ABS_X(grid_x), ABS_Y(grid_y));
+         else
             sprintf (buf, "%1dx%1d%+1d%+1d", ABS_SIZE(abs(grid_x-OrigX)),
                   ABS_SIZE(abs(grid_y-OrigY)), ABS_X(grid_x), ABS_Y(grid_y));
-            ShowMeasureCursor (grid_x, grid_y, buf, TRUE);
-         }
+         ShowMeasureCursor (grid_x, grid_y, buf, TRUE);
          if (curSpline != LT_STRAIGHT && splineRubberband)
          {
 #ifdef HP_LINE_BUG
@@ -417,6 +433,13 @@ void ContinuePoly (OrigX, OrigY)
                      if (cntrlv != NULL) cfree (cntrlv);
                      sv = MakeIntSplinePolyVertex (&sn, &intn, &cntrlv,
                            drawOrigX, drawOrigY, n, v);
+                     for (i=0; i < sn; i++)
+                     {
+                        if (sv[i].x < ltx) ltx = sv[i].x;
+                        if (sv[i].y < lty) lty = sv[i].y;
+                        if (sv[i].x > rbx) rbx = sv[i].x;
+                        if (sv[i].y > rby) rby = sv[i].y;
+                     }
                      break;
                }
 #ifdef HP_LINE_BUG
@@ -490,6 +513,13 @@ void ContinuePoly (OrigX, OrigY)
                         if (cntrlv != NULL) cfree (cntrlv);
                         sv = MakeIntSplinePolyVertex (&sn, &intn, &cntrlv,
                               drawOrigX, drawOrigY, n, v);
+                        for (i=0; i < sn; i++)
+                        {
+                           if (sv[i].x < ltx) ltx = sv[i].x;
+                           if (sv[i].y < lty) lty = sv[i].y;
+                           if (sv[i].x > rbx) rbx = sv[i].x;
+                           if (sv[i].y > rby) rby = sv[i].y;
+                        }
                         break;
                   }
 #ifdef HP_LINE_BUG
@@ -513,6 +543,8 @@ void ContinuePoly (OrigX, OrigY)
                }
             }
             OrigX = grid_x; OrigY = grid_y;
+            sprintf (buf, "%+1d%+1d", ABS_X(grid_x), ABS_Y(grid_y));
+            ShowMeasureCursor (grid_x, grid_y, buf, TRUE);
          }
          else
          {
@@ -532,6 +564,13 @@ void ContinuePoly (OrigX, OrigY)
                      cfree (cntrlv);
                      sv = MakeIntSplinePolyVertex (&sn, &intn, &cntrlv,
                            drawOrigX, drawOrigY, n, v);
+                     for (i=0; i < sn; i++)
+                     {
+                        if (sv[i].x < ltx) ltx = sv[i].x;
+                        if (sv[i].y < lty) lty = sv[i].y;
+                        if (sv[i].x > rbx) rbx = sv[i].x;
+                        if (sv[i].y > rby) rby = sv[i].y;
+                     }
                      break;
                }
 #ifdef HP_LINE_BUG
@@ -621,6 +660,13 @@ void ContinuePoly (OrigX, OrigY)
                      if (cntrlv != NULL) cfree (cntrlv);
                      sv = MakeIntSplinePolyVertex (&sn, &intn, &cntrlv,
                            drawOrigX, drawOrigY, n, v);
+                     for (i=0; i < sn; i++)
+                     {
+                        if (sv[i].x < ltx) ltx = sv[i].x;
+                        if (sv[i].y < lty) lty = sv[i].y;
+                        if (sv[i].x > rbx) rbx = sv[i].x;
+                        if (sv[i].y > rby) rby = sv[i].y;
+                     }
                      break;
                }
 #ifdef HP_LINE_BUG
@@ -657,6 +703,9 @@ void ContinuePoly (OrigX, OrigY)
       }
       else if (input.type == ButtonRelease && curChoice == FREEHAND)
       {
+         sprintf (buf, "%+1d%+1d", ABS_X(grid_x), ABS_Y(grid_y));
+         EndShowMeasureCursor (grid_x, grid_y, buf, TRUE);
+
          button_ev = &(input.xbutton);
 
          if (grid_x != last_x || grid_y != last_y)
@@ -670,6 +719,25 @@ void ContinuePoly (OrigX, OrigY)
          }
          done = TRUE;
       }
+      else if (input.type == KeyPress)
+      {
+         KeySym	key_sym;
+         char	s[80];
+
+         XLookupString (&(input.xkey), s, 80-1, &key_sym, &c_stat);
+         TranslateKeys (s, &key_sym);
+         if (s[0] == '\033' && (key_sym & 0xff) == '\033')
+         {
+            if (curChoice == FREEHAND)
+               sprintf (buf, "%+1d%+1d", ABS_X(grid_x), ABS_Y(grid_y));
+            else
+               sprintf (buf, "%1dx%1d%+1d%+1d", ABS_SIZE(abs(grid_x-OrigX)),
+                     ABS_SIZE(abs(grid_y-OrigY)), ABS_X(grid_x), ABS_Y(grid_y));
+            EndShowMeasureCursor (grid_x, grid_y, buf, TRUE);
+            abort = TRUE;
+            done = TRUE;
+         }
+      }
    }
    XUngrabPointer (mainDisplay, CurrentTime);
    RestoreStatusStrings ();
@@ -682,7 +750,7 @@ void ContinuePoly (OrigX, OrigY)
       if (curSpline == LT_INTSPLINE && cntrlv != NULL) cfree (cntrlv);
    }
 
-   if (num_pts > 1)
+   if (!abort && num_pts > 1)
    {
       CreatePolyObj (num_pts, CREATE_RELATIVE);
       RecordNewObjCmd ();
@@ -693,6 +761,33 @@ void ContinuePoly (OrigX, OrigY)
       polyDrawn = TRUE;
       SetFileModified (TRUE);
    }
+   else
+   {
+      struct PtRec	* pt_ptr, * next_pt;
+
+      if (curSpline != LT_INTSPLINE)
+      {
+         ltx = rbx = grid_x;
+         lty = rby = grid_y;
+      }
+      for (pt_ptr=lastPtPtr; pt_ptr != NULL; pt_ptr=next_pt)
+      {
+         next_pt = pt_ptr->next;
+         if (curSpline != LT_INTSPLINE)
+         {
+            if (pt_ptr->x < ltx) ltx = pt_ptr->x;
+            if (pt_ptr->y < lty) lty = pt_ptr->y;
+            if (pt_ptr->x > rbx) rbx = pt_ptr->x;
+            if (pt_ptr->y > rby) rby = pt_ptr->y;
+         }
+         cfree (pt_ptr);
+      }
+      RedrawAnArea (botObj, ABS_X(ltx)-GRID_ABS_SIZE(1),
+            ABS_Y(lty)-GRID_ABS_SIZE(1), ABS_X(rbx)+GRID_ABS_SIZE(1),
+            ABS_Y(rby)+GRID_ABS_SIZE(1));
+      lastPtPtr = NULL;
+      polyDrawn = FALSE;
+   }
 }
 
 void DrawPoly (input)
@@ -708,7 +803,12 @@ void DrawPoly (input)
    {
       mouse_x = input->xbutton.x;
       mouse_y = input->xbutton.y;
-      GridXY (mouse_x, mouse_y, &grid_x, &grid_y);
+      if (curChoice == FREEHAND) {
+         grid_x = mouse_x;
+         grid_y = mouse_y;
+      } else {
+         GridXY (mouse_x, mouse_y, &grid_x, &grid_y);
+      }
       lastPtPtr = (struct PtRec *) calloc (1, sizeof(struct PtRec));
       lastPtPtr->x = grid_x;
       lastPtPtr->y = grid_y;
@@ -719,11 +819,13 @@ void DrawPoly (input)
 
 void InputPolyPts ()
 {
-   char		inbuf[MAXSTRING+1];
-   int		more_poly;
+   char			inbuf[MAXSTRING+1];
+   int			more_poly=FALSE, num_polys=0;
+   int			started_composite=FALSE;
+   struct ObjRec	* saved_top_obj=topObj;
 
    MakeQuiescent ();
-
+   XSync (mainDisplay, False);
    do
    {
       int		len, ok=TRUE, num_pts=0, eof=TRUE;
@@ -765,6 +867,7 @@ void InputPolyPts ()
                   ok = FALSE;
                   Dialog ("Error reading integer for poly points.",
                         "( <CR> or <ESC> to continue )", inbuf);
+                  XSync (mainDisplay, False);
                   break;
                }
                lastPtPtr = pt_ptr;
@@ -779,6 +882,7 @@ void InputPolyPts ()
                ok = FALSE;
                Dialog ("Error reading integer for poly points.",
                      "( <CR> or <ESC> to continue )", inbuf);
+               XSync (mainDisplay, False);
             }
          }
          printf ("> ");
@@ -788,24 +892,50 @@ void InputPolyPts ()
       if (eof) rewind (stdin);
       if (ok && num_pts > 1)
       {
+         num_polys++;
          CreatePolyObj (num_pts, CREATE_ABSOLUTE);
-         RecordNewObjCmd ();
-         RedrawAnArea (botObj, topObj->bbox.ltx-GRID_ABS_SIZE(1),
-               topObj->bbox.lty-GRID_ABS_SIZE(1),
-               topObj->bbox.rbx+GRID_ABS_SIZE(1),
-               topObj->bbox.rby+GRID_ABS_SIZE(1));
-         SelectTopObj ();
-         SetFileModified (TRUE);
-         justDupped = FALSE;
+         if (more_poly || num_polys > 1)
+         {
+            if (num_polys <= 1)
+            {
+               StartCompositeCmd ();
+               started_composite = TRUE;
+            }
+            RecordNewObjCmd ();
+            DrawObj (drawWindow, topObj);
+         }
+         else
+         {
+            RecordNewObjCmd ();
+            RedrawAnArea (botObj, topObj->bbox.ltx-GRID_ABS_SIZE(1),
+                  topObj->bbox.lty-GRID_ABS_SIZE(1),
+                  topObj->bbox.rbx+GRID_ABS_SIZE(1),
+                  topObj->bbox.rby+GRID_ABS_SIZE(1));
+            SelectTopObj ();
+            SetFileModified (TRUE);
+            justDupped = FALSE;
+         }
       }
       if (ok && num_pts <= 1)
+      {
          Dialog ("Too few points.", "( <CR> or <ESC> to continue )", inbuf);
+         XSync (mainDisplay, False);
+      }
       for ( ; lastPtPtr != NULL; lastPtPtr=pt_ptr)
       {
          pt_ptr = lastPtPtr->next;
          cfree (pt_ptr);
       }
    } while (more_poly);
+   if (num_polys > 1 || started_composite)
+   {
+      SelectAndHighLightNewObjects (saved_top_obj);
+      GroupSelObj ();
+      EndCompositeCmd ();
+
+      SetFileModified (TRUE);
+      justDupped = FALSE;
+   }
 }
 
 static
@@ -967,9 +1097,11 @@ void DumpPolyPath (FP, ObjPtr, Vs, NumPts, Smooth, Style, Width, ArrowWidth,
       dx = Vs[1].x - Vs[0].x;
       dy = Vs[1].y - Vs[0].y;
       if (dx != 0 || dy != 0)
+      {
          for (i=0; i < Indent+6; i++) fprintf (FP, " ");
          fprintf (FP, "%1d %1d atan dup cos %1d mul exch sin %1d %s\n",
                dy, dx, aw, aw, "mul rmoveto");
+      }
    }
    if (Style & LS_RIGHT)
    {
@@ -1012,7 +1144,12 @@ void DumpPolyPath (FP, ObjPtr, Vs, NumPts, Smooth, Style, Width, ArrowWidth,
    else if (Curved == LT_INTSPLINE)
    {
       DumpCurvedPolyPoints (FP, NumPts, Vs, Indent+6);
-      if (NumPts != 2)
+      if (NumPts <= 2)
+      {
+         for (i=0; i < Indent+6; i++) fprintf (FP, " ");
+         fprintf (FP, "%1d %1d lineto\n",Vs[NumPts-1].x,Vs[NumPts-1].y);
+      }
+      else
       {
          for (i=0; i < Indent+6; i++) fprintf (FP, " ");
          fprintf (FP, "%1d %1d curveto\n",Vs[NumPts-1].x,Vs[NumPts-1].y);
@@ -1448,7 +1585,7 @@ void SavePolyObj (FP, ObjPtr)
          poly_ptr->vlist[n-1].y) == EOF)
       writeFileFailed = TRUE;
 
-   if (fprintf (FP, "%1d,%1d,%1d,%1d,%1d,%1d,%1d,%1d,%1d,%1d,%1d,[",
+   if (fprintf (FP, "%1d,%1d,%1d,%1d,%1d,%1d,%1d,%1d,%1d,%1d,%1d,",
          poly_ptr->style, poly_ptr->width, poly_ptr->pen, ObjPtr->id,
          poly_ptr->curved, poly_ptr->fill, poly_ptr->dash, ObjPtr->rotation,
          poly_ptr->aw, poly_ptr->ah, ObjPtr->locked) == EOF)
@@ -1485,7 +1622,7 @@ int ReadSmoothHinge (FP, Curved, NumPts, Smooth)
    c_ptr++;
    for (j = 0; j < num_nibbles; j++)
    {
-      int	data;
+      int	data=0;
 
       if (nibble_count++ == 64)
       {
@@ -1518,7 +1655,7 @@ void ReadPolyObj (FP, Inbuf, ObjPtr)
    register int		i;
    struct PolyRec	* poly_ptr;
    XPoint		* v;
-   char			color_str[20], * s, inbuf[MAXSTRING+1], msg[MAXSTRING];
+   char			color_str[40], * s, inbuf[MAXSTRING+1], msg[MAXSTRING];
    int			num_pts, ltx=0, lty=0, rbx=0, rby=0, x, y, id=0;
    int			initialized, rotation, count, new_alloc;
    int			style, width=0, pen, curved, fill, dash, locked=FALSE;
@@ -1528,7 +1665,7 @@ void ReadPolyObj (FP, Inbuf, ObjPtr)
    *ObjPtr = NULL;
 
    s = FindChar ('(', Inbuf);
-   s = ParseStr (s, ',', color_str);
+   s = ParseStr (s, ',', color_str, sizeof(color_str));
 
    InitScan (s, "\t\n, []");
 
@@ -1888,7 +2025,7 @@ void ReadPolyObj (FP, Inbuf, ObjPtr)
 
    (*ObjPtr)->x = ltx;
    (*ObjPtr)->y = lty;
-   (*ObjPtr)->color = QuickFindColorIndex (color_str, &new_alloc);
+   (*ObjPtr)->color = QuickFindColorIndex (color_str, &new_alloc, TRUE);
    (*ObjPtr)->dirty = FALSE;
    (*ObjPtr)->id = id;
    (*ObjPtr)->rotation = rotation;
