@@ -25,12 +25,13 @@
  */
 #ifndef lint
 static char RCSid[] =
-      "@(#)$Header: /amnt/maui/tangram/u/william/X11/TGIF2/RCS/polygon.c,v 2.82 1994/04/18 04:34:13 william Exp $";
+      "@(#)$Header: /export/bourbon/multimedia/william/X11/TGIF2/RCS/polygon.c,v 2.93 1994/11/30 06:13:15 william Exp $";
 #endif
 
 #include <math.h>
 #include <stdio.h>
 #include <string.h>
+#include <sys/types.h>
 #include <X11/Xlib.h>
 #include <X11/Xutil.h>
 #include <X11/keysym.h>
@@ -598,15 +599,17 @@ void ContinuePolygon (OrigX, OrigY)
    int 	OrigX, OrigY;
    /* OrigX and OrigY are screen coordinates (scaled and translated). */
 {
+   register int		i;
    XGCValues		values;
    XEvent		input, ev;
    XButtonEvent		* button_ev;
    XMotionEvent		* motion_ev;
    KeySym		key_sym;
    char			s[80], buf[80];
-   int 			end_x, end_y, grid_x, grid_y, done = FALSE;
-   int 			saved_x, saved_y, closed=FALSE;
+   int 			end_x, end_y, grid_x, grid_y, done=FALSE;
+   int 			saved_x, saved_y, closed=FALSE, abort=FALSE;
    int			xor_pixel, num_pts=1, n=2, sn=0, max_n=40, intn=0;
+   int			ltx=OrigX, lty=OrigY, rbx=OrigX, rby=OrigY;
    struct PtRec		* pt_ptr;
    XPoint		* v = NULL, * sv = NULL, * cntrlv=NULL;
 
@@ -642,6 +645,13 @@ void ContinuePolygon (OrigX, OrigY)
          case LT_INTSPLINE:
             sv = MakeIntSplinePolygonVertex (&sn, &intn, &cntrlv,
                   drawOrigX, drawOrigY, n+1, v);
+            for (i=0; i < sn; i++)
+            {
+               if (sv[i].x < ltx) ltx = sv[i].x;
+               if (sv[i].y < lty) lty = sv[i].y;
+               if (sv[i].x > rbx) rbx = sv[i].x;
+               if (sv[i].y > rby) rby = sv[i].y;
+            }
             break;
       }
    }
@@ -661,17 +671,26 @@ void ContinuePolygon (OrigX, OrigY)
 
       if (input.type == Expose || input.type == VisibilityNotify)
          ExposeEventHandler (&input, TRUE);
-      else if ((input.type == KeyPress || input.type == KeyRelease) &&
-            num_pts != 1 && !(curSpline != LT_STRAIGHT && splineRubberband))
+      else if (input.type == KeyPress || input.type == KeyRelease)
       {
          XLookupString (&(input.xkey), s, 80-1, &key_sym, &c_stat);
-         if (key_sym == XK_Shift_L || key_sym == XK_Shift_R ||
-               key_sym == XK_Control_L || key_sym == XK_Control_R)
+         if (num_pts != 1 && !(curSpline != LT_STRAIGHT && splineRubberband) &&
+               (key_sym == XK_Shift_L || key_sym == XK_Shift_R ||
+               key_sym == XK_Control_L || key_sym == XK_Control_R))
          {
             XDrawLine (mainDisplay, drawWindow, drawGC, saved_x, saved_y,
                   grid_x, grid_y);
             closed = !closed;
          }
+         else if (input.type == KeyPress && s[0] == '\033' &&
+               (key_sym & 0xff) == '\033')
+         {
+            sprintf (buf, "%1dx%1d%+1d%+1d", ABS_SIZE(abs(grid_x-OrigX)),
+                  ABS_SIZE(abs(grid_y-OrigY)), ABS_X(grid_x), ABS_Y(grid_y));
+            EndShowMeasureCursor (grid_x, grid_y, buf, TRUE);
+            abort = TRUE;
+            done = TRUE;
+         }
       }
       else if (input.type == MotionNotify)
       {
@@ -708,6 +727,13 @@ void ContinuePolygon (OrigX, OrigY)
                   if (cntrlv != NULL) cfree (cntrlv);
                   sv = MakeIntSplinePolygonVertex (&sn, &intn, &cntrlv,
                         drawOrigX, drawOrigY, n+1, v);
+                  for (i=0; i < sn; i++)
+                  {
+                     if (sv[i].x < ltx) ltx = sv[i].x;
+                     if (sv[i].y < lty) lty = sv[i].y;
+                     if (sv[i].x > rbx) rbx = sv[i].x;
+                     if (sv[i].y > rby) rby = sv[i].y;
+                  }
                   break;
             }
             XDrawLines (mainDisplay, drawWindow, drawGC, sv, sn,
@@ -780,6 +806,13 @@ void ContinuePolygon (OrigX, OrigY)
                      if (cntrlv != NULL) cfree (cntrlv);
                      sv = MakeIntSplinePolygonVertex (&sn, &intn, &cntrlv,
                            drawOrigX, drawOrigY, n+1, v);
+                     for (i=0; i < sn; i++)
+                     {
+                        if (sv[i].x < ltx) ltx = sv[i].x;
+                        if (sv[i].y < lty) lty = sv[i].y;
+                        if (sv[i].x > rbx) rbx = sv[i].x;
+                        if (sv[i].y > rby) rby = sv[i].y;
+                     }
                      break;
                }
                XDrawLines (mainDisplay, drawWindow, drawGC, sv, sn,
@@ -862,26 +895,43 @@ void ContinuePolygon (OrigX, OrigY)
       cfree (v); cfree (sv);
       if (curSpline == LT_INTSPLINE && cntrlv != NULL) cfree (cntrlv);
    }
-
-   switch (num_pts)
+   if (!abort && num_pts > 3)
    {
-      case 1: cfree (lastPtPtr); break;
-      case 2: cfree (lastPtPtr->next); cfree (lastPtPtr); break;
-      case 3:
-         cfree (lastPtPtr->next->next);
-         cfree (lastPtPtr->next);
-         cfree (lastPtPtr);
-         break;
-      default:
-         CreatePolygonObj (num_pts, CREATE_RELATIVE);
-         RecordNewObjCmd ();
-         RedrawAnArea (botObj, topObj->bbox.ltx-GRID_ABS_SIZE(1),
-               topObj->bbox.lty-GRID_ABS_SIZE(1),
-               topObj->bbox.rbx+GRID_ABS_SIZE(1),
-               topObj->bbox.rby+GRID_ABS_SIZE(1));
-         polygonDrawn = TRUE;
-         SetFileModified (TRUE);
-         break;
+      CreatePolygonObj (num_pts, CREATE_RELATIVE);
+      RecordNewObjCmd ();
+      RedrawAnArea (botObj, topObj->bbox.ltx-GRID_ABS_SIZE(1),
+            topObj->bbox.lty-GRID_ABS_SIZE(1),
+            topObj->bbox.rbx+GRID_ABS_SIZE(1),
+            topObj->bbox.rby+GRID_ABS_SIZE(1));
+      polygonDrawn = TRUE;
+      SetFileModified (TRUE);
+   }
+   else
+   {
+      struct PtRec	* pt_ptr, * next_pt;
+
+      if (curSpline != LT_INTSPLINE)
+      {
+         ltx = rbx = grid_x;
+         lty = rby = grid_y;
+      }
+      for (pt_ptr=lastPtPtr; pt_ptr != NULL; pt_ptr=next_pt)
+      {
+         next_pt = pt_ptr->next;
+         if (curSpline != LT_INTSPLINE)
+         {
+            if (pt_ptr->x < ltx) ltx = pt_ptr->x;
+            if (pt_ptr->y < lty) lty = pt_ptr->y;
+            if (pt_ptr->x > rbx) rbx = pt_ptr->x;
+            if (pt_ptr->y > rby) rby = pt_ptr->y;
+         }
+         cfree (pt_ptr);
+      }
+      RedrawAnArea (botObj, ABS_X(ltx)-GRID_ABS_SIZE(1),
+            ABS_Y(lty)-GRID_ABS_SIZE(1), ABS_X(rbx)+GRID_ABS_SIZE(1),
+            ABS_Y(rby)+GRID_ABS_SIZE(1));
+      lastPtPtr = NULL;
+      polygonDrawn = FALSE;
    }
 }
 
@@ -907,13 +957,34 @@ void DrawPolygon (input)
    } 
 }
 
+void SelectAndHighLightNewObjects (PrevTopObj)
+   struct ObjRec	* PrevTopObj;
+{
+   register struct ObjRec	* obj_ptr;
+   register struct SelRec	* sel_ptr;
+
+   if (topSel != NULL) { HighLightReverse (); RemoveAllSel(); }
+   for (obj_ptr = topObj; obj_ptr != PrevTopObj; obj_ptr = obj_ptr->next)
+   {
+      sel_ptr = (struct SelRec *) calloc (1, sizeof(struct SelRec));
+      if (sel_ptr == NULL) fprintf (stderr, "Can not calloc().\n");
+      sel_ptr->obj = obj_ptr;
+      sel_ptr->prev = sel_ptr->next = NULL;
+      AddSel (botSel, NULL, sel_ptr);
+   }
+   UpdSelBBox ();
+   HighLightForward ();
+}
+
 void InputPolygonPts ()
 {
-   char	inbuf[MAXSTRING+1];
-   int	more_polygon=FALSE;
+   char			inbuf[MAXSTRING+1];
+   int			more_polygon=FALSE, num_polygons=0;
+   int			started_composite=FALSE;
+   struct ObjRec	* saved_top_obj=topObj;
 
    MakeQuiescent ();
-
+   XSync (mainDisplay, False);
    do
    {
       int		len, ok=TRUE, num_pts=0;
@@ -956,6 +1027,7 @@ void InputPolygonPts ()
                   ok = FALSE;
                   Dialog ("Error reading integer for poly points.",
                         "( <CR> or <ESC> to continue )", inbuf);
+                  XSync (mainDisplay, False);
                   break;
                }
                if (lastPtPtr == NULL)
@@ -975,6 +1047,7 @@ void InputPolygonPts ()
                ok = FALSE;
                Dialog ("Error reading integer for poly points.",
                      "( <CR> or <ESC> to continue )", inbuf);
+               XSync (mainDisplay, False);
             }
          }
          printf ("> ");
@@ -984,6 +1057,7 @@ void InputPolygonPts ()
       if (eof) rewind (stdin);
       if (ok && num_pts > 2)
       {
+         num_polygons++;
          if (lastPtPtr->x != first_x || lastPtPtr->y != first_y)
          {
             num_pts++;
@@ -994,23 +1068,48 @@ void InputPolygonPts ()
             lastPtPtr = pt_ptr;
          }
          CreatePolygonObj (num_pts, CREATE_ABSOLUTE);
-         RecordNewObjCmd ();
-         RedrawAnArea (botObj, topObj->bbox.ltx-GRID_ABS_SIZE(1),
-               topObj->bbox.lty-GRID_ABS_SIZE(1),
-               topObj->bbox.rbx+GRID_ABS_SIZE(1),
-               topObj->bbox.rby+GRID_ABS_SIZE(1));
-         SelectTopObj ();
-         SetFileModified (TRUE);
-         justDupped = FALSE;
+         if (more_polygon || num_polygons > 1)
+         {
+            if (num_polygons <= 1)
+            {
+               StartCompositeCmd ();
+               started_composite = TRUE;
+            }
+            RecordNewObjCmd ();
+            DrawObj (drawWindow, topObj);
+         }
+         else
+         {
+            RecordNewObjCmd ();
+            RedrawAnArea (botObj, topObj->bbox.ltx-GRID_ABS_SIZE(1),
+                  topObj->bbox.lty-GRID_ABS_SIZE(1),
+                  topObj->bbox.rbx+GRID_ABS_SIZE(1),
+                  topObj->bbox.rby+GRID_ABS_SIZE(1));
+            SelectTopObj ();
+            SetFileModified (TRUE);
+            justDupped = FALSE;
+         }
       }
       if (ok && num_pts <= 2)
+      {
          Dialog ("Too few points.", "( <CR> or <ESC> to continue )", inbuf);
+         XSync (mainDisplay, False);
+      }
       for ( ; lastPtPtr != NULL; lastPtPtr=pt_ptr)
       {
          pt_ptr = lastPtPtr->next;
          cfree (pt_ptr);
       }
    } while (more_polygon);
+   if (num_polygons > 1 || started_composite)
+   {
+      SelectAndHighLightNewObjects (saved_top_obj);
+      GroupSelObj ();
+      EndCompositeCmd ();
+
+      SetFileModified (TRUE);
+      justDupped = FALSE;
+   }
 }
 
 void SavePolygonObj (FP, ObjPtr)
@@ -1040,7 +1139,7 @@ void SavePolygonObj (FP, ObjPtr)
          polygon_ptr->vlist[n-1].x, polygon_ptr->vlist[n-1].y) == EOF)
       writeFileFailed = TRUE;
 
-   if (fprintf (FP, "%1d,%1d,%1d,%1d,%1d,%1d,%1d,%1d,[",
+   if (fprintf (FP, "%1d,%1d,%1d,%1d,%1d,%1d,%1d,%1d,",
          polygon_ptr->fill, polygon_ptr->width, polygon_ptr->pen,
          polygon_ptr->curved, ObjPtr->id, polygon_ptr->dash, ObjPtr->rotation,
          ObjPtr->locked) == EOF)
@@ -1063,7 +1162,7 @@ void ReadPolygonObj (FP, Inbuf, ObjPtr)
    register int		i;
    struct PolygonRec	* polygon_ptr;
    XPoint		* v;
-   char			color_str[20], * s, inbuf[MAXSTRING], msg[MAXSTRING];
+   char			color_str[40], * s, inbuf[MAXSTRING], msg[MAXSTRING];
    int			num_pts, ltx=0, lty=0, rbx=0, rby=0, x, y, id=0;
    int			fill, width, pen, w, new_alloc, locked=FALSE;
    int			curved, dash, initialized, rotation, count;
@@ -1072,7 +1171,7 @@ void ReadPolygonObj (FP, Inbuf, ObjPtr)
    *ObjPtr = NULL;
 
    s = FindChar ('(', Inbuf);
-   s = ParseStr (s, ',', color_str);
+   s = ParseStr (s, ',', color_str, sizeof(color_str));
 
    InitScan (s, "\t\n, []");
 
@@ -1385,7 +1484,7 @@ void ReadPolygonObj (FP, Inbuf, ObjPtr)
 
    (*ObjPtr)->x = ltx;
    (*ObjPtr)->y = lty;
-   (*ObjPtr)->color = QuickFindColorIndex (color_str, &new_alloc);
+   (*ObjPtr)->color = QuickFindColorIndex (color_str, &new_alloc, TRUE);
    (*ObjPtr)->dirty = FALSE;
    (*ObjPtr)->id = id;
    (*ObjPtr)->rotation = rotation;
