@@ -25,7 +25,7 @@
  */
 #ifndef lint
 static char RCSid[] =
-      "@(#)$Header: /amnt/maui/tangram/u/william/X11/TGIF2/RCS/prtgif.c,v 2.33 1994/03/21 04:56:06 william Exp $";
+      "@(#)$Header: /export/bourbon/multimedia/william/X11/TGIF2/RCS/prtgif.c,v 2.34 1995/01/02 01:42:14 william Exp $";
 #endif
 
 #include <stdio.h>
@@ -97,7 +97,7 @@ int main (argc, argv)
    }
    *(--c_ptr) = '\0';
 
-   if ((fp = popen (cmd, "r")) == NULL)
+   if ((fp = (FILE*) popen (cmd, "r")) == NULL)
    {
       fprintf(stderr, "Can not popen() to tgif.  Abort!\n");
       exit (-1);
