@@ -25,7 +25,7 @@
  */
 #ifndef lint
 static char RCSid[] =
-      "@(#)$Header: /amnt/maui/tangram/u/william/X11/TGIF2/RCS/raster.c,v 2.53 1994/04/19 05:04:28 william Exp $";
+      "@(#)$Header: /export/bourbon/multimedia/william/X11/TGIF2/RCS/raster.c,v 2.57 1994/11/10 05:24:43 william Exp $";
 #endif
 
 #include <stdio.h>
@@ -176,6 +176,8 @@ static char RCSid[] =
 #include "xbm/unconst_mv.xbm"
 #include "xbm/edit.xbm"
 #include "xbm/intr.xbm"
+#include "xbm/intr90.xbm"
+#include "xbm/trek.xbm"
 #include "xbm/stack.xbm"
 #include "xbm/tile.xbm"
 
@@ -188,6 +190,10 @@ static char RCSid[] =
 #include "xbm/btn2.xbm"
 #include "xbm/btn3.xbm"
 
+#include "xbm/stop.xbm"
+#include "xbm/question.xbm"
+#include "xbm/info.xbm"
+
 extern int	atoi ARGS_DECL((char *));
 
 GC	rasterGC;
@@ -219,7 +225,8 @@ Pixmap	vspacePixmap;
 Pixmap	rcbRadiusPixmap;
 Pixmap	moveModePixmap[MAXMOVEMODES];
 Pixmap	editPixmap;
-Pixmap	intrPixmap;
+Pixmap	intrPixmap[MAXINTRS];
+Pixmap	trekPixmap;
 Pixmap	pageLayoutPixmap[MAXPAGELAYOUTMODES];
 
 Pixmap	rotatePixmap[4];
@@ -229,6 +236,7 @@ Pixmap	shortLineStylePixmap[MAXLINESTYLES];
 Pixmap	shortDashPixmap[MAXDASHES];
 
 Pixmap	statusBtnPixmap[MAX_STATUS_BTNS];
+Pixmap	msgBoxPixmap[MAX_MB_ICONS];
 
 int	maxLineWidths = MAXLINEWIDTHS;
 
@@ -410,8 +418,9 @@ void InitPattern ()
          GCBackground | GCGraphicsExposures, &values);
 
    values.font = defaultFontPtr->fid;
+   values.line_width = 1;
    defaultGC = XCreateGC (mainDisplay, mainWindow,
-         GCForeground | GCBackground | GCFont, &values);
+         GCForeground | GCBackground | GCFont | GCLineWidth, &values);
 
    values.foreground = xorOne;
    values.background = xorZero;
@@ -580,8 +589,13 @@ void InitPattern ()
    editPixmap = XCreateBitmapFromData (mainDisplay, mainWindow,
          edit_bits, edit_width, edit_height);
 
-   intrPixmap = XCreateBitmapFromData (mainDisplay, mainWindow,
+   intrPixmap[0] = XCreateBitmapFromData (mainDisplay, mainWindow,
          intr_bits, intr_width, intr_height);
+   intrPixmap[1] = XCreateBitmapFromData (mainDisplay, mainWindow,
+         intr90_bits, intr90_width, intr90_height);
+
+   trekPixmap = XCreateBitmapFromData (mainDisplay, mainWindow,
+         trek_bits, trek_width, trek_height);
 
    pageLayoutPixmap[PAGE_STACK] = XCreateBitmapFromData (mainDisplay,
          mainWindow, stack_bits, stack_width, stack_height);
@@ -680,6 +694,13 @@ void InitPattern ()
    statusBtnPixmap[2] = XCreateBitmapFromData (mainDisplay, mainWindow,
          btn3_bits, btn3_width, btn3_height);
 
+   msgBoxPixmap[0] = XCreateBitmapFromData (mainDisplay, mainWindow,
+         stop_bits, stop_width, stop_height);
+   msgBoxPixmap[1] = XCreateBitmapFromData (mainDisplay, mainWindow,
+         question_bits, question_width, question_height);
+   msgBoxPixmap[2] = XCreateBitmapFromData (mainDisplay, mainWindow,
+         info_bits, info_width, info_height);
+
    InitLines();
 }
 
@@ -741,7 +762,9 @@ void CleanUpRasters ()
    for (i = 0; i < MAXMOVEMODES; i++)
       XFreePixmap (mainDisplay, moveModePixmap[i]);
    XFreePixmap (mainDisplay, editPixmap);
-   XFreePixmap (mainDisplay, intrPixmap);
+   for (i = 0; i < MAXINTRS; i++)
+      XFreePixmap (mainDisplay, intrPixmap[i]);
+   XFreePixmap (mainDisplay, trekPixmap);
    for (i = 0; i < MAXPAGELAYOUTMODES; i++)
       XFreePixmap (mainDisplay, pageLayoutPixmap[i]);
    for (i = 0; i < 4; i++)
@@ -758,6 +781,9 @@ void CleanUpRasters ()
    for (i = 0; i < MAX_STATUS_BTNS; i++)
       XFreePixmap (mainDisplay, statusBtnPixmap[i]);
 
+   for (i = 0; i < MAX_MB_ICONS; i++)
+      XFreePixmap (mainDisplay, msgBoxPixmap[i]);
+
    if (newLineWidthAllocated)
    {
       for (i = 0; i < maxLineWidths; i++)
