@@ -25,11 +25,14 @@
  */
 #ifndef lint
 static char RCSid[] =
-      "@(#)$Header: /amnt/maui/tangram/u/william/X11/TGIF2/RCS/rcbox.c,v 2.48 1994/03/21 04:56:19 william Exp $";
+      "@(#)$Header: /export/bourbon/multimedia/william/X11/TGIF2/RCS/rcbox.c,v 2.54 1994/11/30 06:13:15 william Exp $";
 #endif
 
 #include <stdio.h>
+#include <sys/types.h>
 #include <X11/Xlib.h>
+#include <X11/Xutil.h>
+#include <X11/keysym.h>
 #include "const.h"
 #include "types.h"
 
@@ -452,13 +455,15 @@ void CreateRCBoxObj (X1, Y1, X2, Y2)
    obj_ptr->fattr = obj_ptr->lattr = NULL;
    AddObj (NULL, topObj, obj_ptr);
 }
+
+static XComposeStatus	c_stat;
  
 static
 void ContinueRCBox (OrigX, OrigY)
    int 	OrigX, OrigY;
 {
    int 			end_x, end_y, grid_x, grid_y, saved_x, saved_y;
-   int 			done = FALSE;
+   int 			done=FALSE, abort=FALSE;
    int			xor_pixel, radius=ZOOMED_SIZE(rcbRadius);
    char			buf[80];
    XGCValues		values;
@@ -551,8 +556,27 @@ void ContinueRCBox (OrigX, OrigY)
          DrawIntervalRulers (OrigX, OrigY, grid_x, grid_y);
          while (XCheckMaskEvent (mainDisplay, PointerMotionMask, &ev)) ;
       }
+      else if (input.type == KeyPress)
+      {
+         KeySym	key_sym;
+         char	s[80];
+
+         XLookupString (&(input.xkey), s, 80-1, &key_sym, &c_stat);
+         TranslateKeys (s, &key_sym);
+         if (s[0] == '\033' && (key_sym & 0xff) == '\033')
+         {
+            XUngrabPointer (mainDisplay, CurrentTime);
+            MyRCBox (drawWindow,drawGC,OrigX,OrigY,saved_x,saved_y,radius);
+            EndIntervalRulers (grid_x, grid_y);
+            sprintf (buf, "%1dx%1d%+1d%+1d", ABS_SIZE(abs(saved_x-OrigX)),
+                  ABS_SIZE(abs(saved_y-OrigY)), ABS_X(saved_x), ABS_Y(saved_y));
+            EndShowMeasureCursor (saved_x, saved_y, buf, TRUE);
+            abort = TRUE;
+            done = TRUE;
+         }
+      }
    }
-   if (OrigX != grid_x && OrigY != grid_y)
+   if (!abort && OrigX != grid_x && OrigY != grid_y)
    {
       CreateRCBoxObj (OrigX, OrigY, grid_x, grid_y);
       RecordNewObjCmd ();
@@ -605,14 +629,14 @@ void ReadRCBoxObj (Inbuf, ObjPtr)
    struct ObjRec	* * ObjPtr;
 {
    struct RCBoxRec	* rcbox_ptr;
-   char			color_str[20], * s, msg[MAXSTRING];
+   char			color_str[40], * s, msg[MAXSTRING];
    int			ltx, lty, rbx, rby, fill, width, pen, dash, radius;
    int			rotation, new_alloc, id=0, w, locked=FALSE;
 
    *ObjPtr = NULL;
 
    s = FindChar ('(', Inbuf);
-   s = ParseStr (s, ',', color_str);
+   s = ParseStr (s, ',', color_str, sizeof(color_str));
 
    InitScan (s, ", \t\n");
 
@@ -704,7 +728,7 @@ void ReadRCBoxObj (Inbuf, ObjPtr)
    rcbox_ptr->radius = radius;
    (*ObjPtr)->x = ltx;
    (*ObjPtr)->y = lty;
-   (*ObjPtr)->color = QuickFindColorIndex (color_str, &new_alloc);
+   (*ObjPtr)->color = QuickFindColorIndex (color_str, &new_alloc, TRUE);
    (*ObjPtr)->dirty = FALSE;
    (*ObjPtr)->id = id;
    (*ObjPtr)->rotation = rotation;
