@@ -25,7 +25,7 @@
  */
 #ifndef lint
 static char RCSid[] =
-      "@(#)$Header: /amnt/maui/tangram/u/william/X11/TGIF2/RCS/scroll.c,v 2.41 1994/04/04 00:49:06 william Exp $";
+      "@(#)$Header: /export/bourbon/multimedia/william/X11/TGIF2/RCS/scroll.c,v 2.42 1994/05/22 05:53:04 william Exp $";
 #endif
 
 #include <stdio.h>
@@ -326,50 +326,53 @@ void ScrollUp (button_ev)
    }
 }
 
-void ScrollDown (button_ev)
-   XButtonEvent	* button_ev;
+void ForceScrollDown (ScrollAFullWindow)
+   int	ScrollAFullWindow;
 {
    int	adjustment = 0;
 
+   switch (gridSystem)
+   {
+      case ENGLISH_GRID:
+         if (ScrollAFullWindow)
+            adjustment = ((int)(ZOOMED_SIZE(drawWinH)/HALF_INCH))*HALF_INCH;
+         else
+            adjustment = HALF_INCH;
+         break;
+      case METRIC_GRID:
+         if (zoomedIn && zoomScale > 1)
+         {
+            if (ScrollAFullWindow)
+               adjustment = ((int)(ZOOMED_SIZE(drawWinH)/FAKE_CM))*FAKE_CM;
+            else
+               adjustment = FAKE_CM;
+         }
+         else
+         {
+            if (ScrollAFullWindow)
+               adjustment = ((int)(ZOOMED_SIZE(drawWinH)/ONE_CM))*ONE_CM;
+            else
+               adjustment = ONE_CM;
+         }
+         break;
+   }
+   drawOrigY += ABS_SIZE(adjustment);
+   RedrawVScrollWindow ();
+   UpdDrawWinBBox ();
+   AdjSplineVs ();
+   AdjustCurText (0, -adjustment);
+   RedrawRulers ();
+   ClearAndRedrawDrawWindow ();
+}
+
+void ScrollDown (button_ev)
+   XButtonEvent	* button_ev;
+{
    if (paperHeight <= drawWinH) return;
 
    if (drawOrigY+drawWinH < paperHeight)
-   {
-      switch (gridSystem)
-      {
-         case ENGLISH_GRID:
-            if (button_ev!=NULL && (button_ev->state & (ShiftMask|ControlMask)))
-               adjustment = ((int)(ZOOMED_SIZE(drawWinH)/HALF_INCH))*HALF_INCH;
-            else
-               adjustment = HALF_INCH;
-            break;
-         case METRIC_GRID:
-            if (zoomedIn && zoomScale > 1)
-            {
-               if (button_ev!=NULL &&
-                     (button_ev->state & (ShiftMask|ControlMask)))
-                  adjustment = ((int)(ZOOMED_SIZE(drawWinH)/FAKE_CM))*FAKE_CM;
-               else
-                  adjustment = FAKE_CM;
-            }
-            else
-            {
-               if (button_ev!=NULL &&
-                     (button_ev->state & (ShiftMask|ControlMask)))
-                  adjustment = ((int)(ZOOMED_SIZE(drawWinH)/ONE_CM))*ONE_CM;
-               else
-                  adjustment = ONE_CM;
-            }
-            break;
-      }
-      drawOrigY += ABS_SIZE(adjustment);
-      RedrawVScrollWindow ();
-      UpdDrawWinBBox ();
-      AdjSplineVs ();
-      AdjustCurText (0, -adjustment);
-      RedrawRulers ();
-      ClearAndRedrawDrawWindow ();
-   }
+      ForceScrollDown (button_ev != NULL &&
+            (button_ev->state & (ShiftMask|ControlMask)));
 }
 
 static
