@@ -25,10 +25,12 @@
  */
 #ifndef lint
 static char RCSid[] =
-      "@(#)$Header: /amnt/maui/tangram/u/william/X11/TGIF2/RCS/select.c,v 2.86 1994/03/21 04:56:34 william Exp $";
+      "@(#)$Header: /export/bourbon/multimedia/william/X11/TGIF2/RCS/select.c,v 2.113 1995/01/02 22:21:48 william Exp $";
 #endif
 
 #include <stdio.h>
+#include <string.h>
+#include <sys/types.h>
 #include <X11/Xlib.h>
 #include <X11/Xutil.h>
 #include <X11/keysym.h>
@@ -58,12 +60,14 @@ static char RCSid[] =
 #include "poly.e"
 #include "raster.e"
 #include "rect.e"
+#include "remote.e"
 #include "ruler.e"
 #include "scroll.e"
 #ifndef _NO_EXTERN
 #include "select.e"
 #endif
 #include "setup.e"
+#include "stk.e"
 #include "stretch.e"
 
 extern int	atoi ARGS_DECL((char *));
@@ -1213,16 +1217,38 @@ void Select (input)
             return;
          }
       }
-      else if (curChoice == NOTHING && topSel != NULL)
+      else if (curChoice == NOTHING)
       {
-         if ((sel_ptr = PtInSelMark (mouse_x, mouse_y, &corner)) != NULL)
+         if (topSel != NULL)
          {
-            StretchSel (grid_x, grid_y, sel_ptr->obj, corner);
-            return;
+            if (oneMotionSelectMove &&
+                  PtInSelMark (mouse_x, mouse_y, &corner) == NULL &&
+                  PtInSelected (mouse_x, mouse_y) == NULL)
+            {  /* may be pointing in not already selected object */
+               HighLightReverse ();
+               RemoveAllSel ();
+               if (SelectOneObj (mouse_x, mouse_y) != NULL)
+               {
+                  HighLightForward ();
+                  MoveSel (grid_x, grid_y, topSel->obj);
+                  return;
+               }
+            }
+            else if ((sel_ptr=PtInSelMark(mouse_x,mouse_y,&corner)) != NULL)
+            {
+               StretchSel (grid_x, grid_y, sel_ptr->obj, corner);
+               return;
+            }
+            else if ((obj_ptr = PtInSelected (mouse_x, mouse_y)) != NULL)
+            {
+               MoveSel (grid_x, grid_y, obj_ptr);
+               return;
+            }
          }
-         else if ((obj_ptr = PtInSelected (mouse_x, mouse_y)) != NULL)
+         else if (oneMotionSelectMove && SelectOneObj(mouse_x,mouse_y) != NULL)
          {
-            MoveSel (grid_x, grid_y, obj_ptr);
+            HighLightForward ();
+            MoveSel (grid_x, grid_y, topSel->obj);
             return;
          }
       }
@@ -1231,41 +1257,107 @@ void Select (input)
    }
 }
 
-struct AttrRec * FindAttrWithName (ObjPtr, AttrName)
-   struct ObjRec	* ObjPtr;
-   char			* AttrName;
+struct AttrRec *FindFileAttrWithName(AttrName)
+   char *AttrName;
 {
-   register struct AttrRec	* attr_ptr;
-   struct AttrRec		* found_attr=NULL;
-   int				count=1;
+   struct AttrRec *attr_ptr, *found_attr=NULL;
+   int count=1;
 
-   for (attr_ptr=ObjPtr->fattr; attr_ptr!=NULL; attr_ptr=attr_ptr->next)
-      if (strcmp (attr_ptr->name, AttrName) == 0)
-      {
+   if (tgifObj == NULL) return NULL;
+
+   for (attr_ptr=tgifObj->fattr; attr_ptr != NULL; attr_ptr=attr_ptr->next) {
+      if (strcmp(attr_ptr->name, AttrName) == 0) {
          found_attr = attr_ptr;
          break;
       }
-   if (attr_ptr == NULL) return (NULL);
-
-   for (attr_ptr=found_attr->next; attr_ptr!=NULL; attr_ptr=attr_ptr->next)
-      if (strcmp (attr_ptr->name, AttrName) == 0)
-         if (attr_ptr->obj->color == colorIndex)
-            break;
-         else
-            count++;
-
-   if (attr_ptr != NULL)
-      found_attr = attr_ptr;
-   else if (count != 1)
-   {
-      char	msg[MAXSTRING+1];
-
-      sprintf (msg, "Can not find '%s' attribute with color %s.",
-            AttrName, colorMenuItems[colorIndex]);
-      Msg (msg);
-      return (NULL);
    }
-   return (found_attr);
+   if (attr_ptr == NULL) return NULL;
+
+   for (attr_ptr=found_attr->next; attr_ptr!=NULL; attr_ptr=attr_ptr->next) {
+      if (strcmp(attr_ptr->name, AttrName) == 0) {
+         if (attr_ptr->obj->color == colorIndex) {
+            break;
+         } else {
+            count++;
+         }
+      }
+   }
+   if (attr_ptr != NULL) {
+      found_attr = attr_ptr;
+   } else if (count != 1) {
+      sprintf(gszMsgBox, "Can not find '%s' file attribute with color %s.",
+            AttrName, colorMenuItems[colorIndex]);
+      Msg(gszMsgBox);
+      return NULL;
+   }
+   return found_attr;
+}
+
+static
+struct AttrRec *FindObjAttrWithName(ObjPtr, AttrName)
+   struct ObjRec *ObjPtr;
+   char *AttrName;
+   /* AttrName here must not contain '.' */
+{
+   register struct AttrRec *attr_ptr;
+   struct AttrRec *found_attr=NULL;
+   int count=1;
+
+   if (ObjPtr == NULL) return NULL;
+
+   for (attr_ptr=ObjPtr->fattr; attr_ptr!=NULL; attr_ptr=attr_ptr->next) {
+      if (strcmp(attr_ptr->name, AttrName) == 0) {
+         found_attr = attr_ptr;
+         break;
+      }
+   }
+   if (attr_ptr == NULL) return NULL;
+
+   for (attr_ptr=found_attr->next; attr_ptr!=NULL; attr_ptr=attr_ptr->next) {
+      if (strcmp(attr_ptr->name, AttrName) == 0) {
+         if (attr_ptr->obj->color == colorIndex) {
+            break;
+         } else {
+            count++;
+         }
+      }
+   }
+   if (attr_ptr != NULL) {
+      found_attr = attr_ptr;
+   } else if (count != 1) {
+      sprintf(gszMsgBox, "Can not find '%s' attribute with color %s.",
+            AttrName, colorMenuItems[colorIndex]);
+      Msg(gszMsgBox);
+      return NULL;
+   }
+   return found_attr;
+}
+
+struct AttrRec *FindAttrWithName(ObjPtr, AttrName, pp_top_owner)
+   struct ObjRec *ObjPtr, **pp_top_owner;
+   char *AttrName;
+{
+   struct ObjRec *obj_ptr;
+   char *dot_ptr;
+
+   if ((dot_ptr=strchr(AttrName, '.')) == NULL) {
+      if (pp_top_owner != NULL) *pp_top_owner = ObjPtr;
+      return FindObjAttrWithName(ObjPtr, AttrName);
+   }
+   *dot_ptr = '\0';
+   if (strcmp(AttrName, "!") == 0) {
+      *dot_ptr++ = '.';
+      if (pp_top_owner != NULL) *pp_top_owner = tgifObj;
+      return FindFileAttrWithName(dot_ptr);
+   }
+   if ((obj_ptr=FindObjWithName(botObj, ObjPtr, AttrName, FALSE,
+         FALSE, NULL, pp_top_owner)) == NULL) {
+      *dot_ptr++ = '.';
+      return NULL;
+   }
+   *dot_ptr++ = '.';
+   if (pp_top_owner != NULL) *pp_top_owner = ObjPtr;
+   return FindObjAttrWithName(obj_ptr, dot_ptr);
 }
 
 struct AttrRec * ValidAttrArg (c_ptr, obj_ptr, new_c_ptr)
@@ -1289,7 +1381,7 @@ struct AttrRec * ValidAttrArg (c_ptr, obj_ptr, new_c_ptr)
                *name_ptr++ = '=';
                *name_ptr = '\0';
                *new_c_ptr = c_ptr;
-               attr_ptr = FindAttrWithName (obj_ptr, name);
+               attr_ptr = FindAttrWithName (obj_ptr, name, NULL);
                if (attr_ptr == NULL)
                {
                   char	msg[MAXSTRING+1];
@@ -1306,78 +1398,102 @@ struct AttrRec * ValidAttrArg (c_ptr, obj_ptr, new_c_ptr)
    return (NULL);
 }
 
-static
-void DoTeleport (teleport_attr)
-   struct AttrRec	* teleport_attr;
+int DoTeleport(teleport_attr)
+   struct AttrRec *teleport_attr;
 {
-   char	file_name[MAXPATHLENGTH];
-   int	do_not_save=FALSE;
+   char file_name[MAXPATHLENGTH+1], msg[MAXSTRING+1];
+   int do_not_save=FALSE, rc=TRUE;
 
-   while (fileModified)
-   {
-      switch (YesNoCancel (
-            "File modified, save file before open? [ync](y)",
-            CONFIRM_YES))
-      {
-         case CONFIRM_YES: SaveFile (); break;
-         case CONFIRM_NO:
-               do_not_save = TRUE;
-               SetFileModified (FALSE);
-               break;
-         case CONFIRM_CANCEL: return;
+   while (!DirIsRemote(curDir) && fileModified) {
+      switch (MsgBox("File modified.\n\nsave file before open? [ync](y)",
+            TOOL_NAME, YNC_MB)) {
+      case MB_ID_YES: SaveFile(); break;
+      case MB_ID_NO: do_not_save = TRUE; SetFileModified(FALSE); break;
+      case MB_ID_CANCEL: return FALSE;
       }
    }
-   if (*(teleport_attr->s) == '/')
-   {
-      sprintf (file_name, "%s.%s", teleport_attr->s, OBJ_FILE_EXT);
-      if (!LoadFile (file_name, TRUE) && do_not_save)
-         SetFileModified (TRUE);
-   }
-   else
-   {
-      sprintf (file_name, "%s/%s.%s", curDir, teleport_attr->s, OBJ_FILE_EXT);
-      if (!LoadFile (file_name, TRUE) && do_not_save)
-         SetFileModified (TRUE);
+   if (!FormNewFileName(curDir, teleport_attr->s,
+         (strcmp(teleport_attr->name,TELEPORT_ATTR)==0 ? OBJ_FILE_EXT : NULL),
+         file_name)) {
+      sprintf(msg, "Invalid teleport destination '%s'.", teleport_attr->s);
+      MsgBox(msg, TOOL_NAME, INFO_MB);
+      if (do_not_save) SetFileModified(TRUE);
+      rc = FALSE;
+   } else if (FileIsRemote(file_name)) {
+      char *buf=NULL, *content_type=NULL;
+      int rc, buf_sz=0, is_html=FALSE;
+
+      SetWatchCursor(drawWindow);
+      SetWatchCursor(mainWindow);
+      SaveStatusStrings();
+      rc = LoadRemoteFileInMem(file_name, &buf, &content_type, &buf_sz,
+            &is_html);
+      RestoreStatusStrings();
+      SetDefaultCursor(mainWindow);
+      ShowCursor();
+      if (rc && buf != NULL) {
+         LoadRemoteFileFromMem(file_name, buf, content_type, buf_sz, is_html);
+      } else {
+         if (do_not_save) SetFileModified(TRUE);
+         rc = FALSE;
+      }
+      if (content_type != NULL) FreeRemoteBuf(content_type);
+      if (buf != NULL) FreeRemoteBuf(buf);
+   } else {
+      if (!LoadFile(file_name, TRUE)) {
+         if (do_not_save) SetFileModified(TRUE);
+         rc = FALSE;
+      }
    }
+   return rc;
 }
 
 #define DO_PAGE_BY_NUM 0
 #define DO_PAGE_BY_NAME 1
 
-static
-void DoPageTeleport (teleport_attr, do_by_page_name)
-   struct AttrRec	* teleport_attr;
-   int			do_by_page_name;
+int DoPageTeleport(teleport_attr, do_by_page_name)
+   struct AttrRec *teleport_attr;
+   int do_by_page_name;
 {
-   int	i;
-   char	msg[MAXSTRING+1], dummy_str[MAXSTRING+1];
+   int i, rc=TRUE;
+   char msg[MAXSTRING+1];
 
-   if (do_by_page_name)
-   {
-      struct PageRec	* page_ptr;
+   if (do_by_page_name) {
+      struct PageRec *page_ptr;
 
       for (i=1, page_ptr=firstPage; page_ptr!=NULL;
-            page_ptr=page_ptr->next, i++)
-         if (page_ptr->name != NULL && strcmp (page_ptr->name,
-               teleport_attr->s) == 0)
-         {
-            if (curPageNum != i) SetCurPage (i);
-            return;
+            page_ptr=page_ptr->next, i++) {
+         if (page_ptr->name != NULL && strcmp(page_ptr->name,
+               teleport_attr->s) == 0) {
+            if (curPageNum != i) SetCurPage(i);
+            return TRUE;
          }
-      sprintf (msg, "Can not find page \"%s\" to warp to.", teleport_attr->s);
-      Dialog (msg, "( <CR> or <ESC> to continue )", dummy_str);
-   }
-   else
-   {
-      i=atoi(teleport_attr->s);
-      if (i >= 1 && i <= lastPageNum)
-      {
-         if (curPageNum != i) SetCurPage (i);
       }
-      else
-      {
-         sprintf (msg, "Can not find page %1d to warp to.", i);
-         Dialog (msg, "( <CR> or <ESC> to continue )", dummy_str);
+      sprintf(msg, "Can not find page \"%s\" to warp to.", teleport_attr->s);
+      MsgBox(msg, TOOL_NAME, INFO_MB);
+      rc = FALSE;
+   } else {
+      i = atoi(teleport_attr->s);
+      if (i >= 1 && i <= lastPageNum) {
+         if (curPageNum != i) SetCurPage(i);
+      } else {
+         sprintf(msg, "Can not find page %1d to warp to.", i);
+         MsgBox(msg, TOOL_NAME, INFO_MB);
+         rc = FALSE;
+      }
+   }
+   return rc;
+}
+
+static
+void ResetDeckIndices()
+{
+   register struct ObjRec *obj_ptr;
+
+   for (obj_ptr=botObj; obj_ptr != NULL; obj_ptr=obj_ptr->prev) {
+      if (obj_ptr->type == OBJ_GROUP || obj_ptr->type == OBJ_SYM ||
+            obj_ptr->type == OBJ_ICON) {
+         obj_ptr->detail.r->deck_index = (-1);
       }
    }
 }
@@ -1395,7 +1511,10 @@ void Teleport (button_ev)
 
    if (owner_obj != NULL) obj_ptr = owner_obj;
 
-   teleport_attr = FindAttrWithName (obj_ptr, TELEPORT_ATTR);
+   teleport_attr = FindAttrWithName (obj_ptr, TELEPORT_ATTR, NULL);
+   if (teleport_attr != NULL) { DoTeleport (teleport_attr); return; }
+
+   teleport_attr = FindAttrWithName (obj_ptr, "href=", NULL);
    if (teleport_attr != NULL) { DoTeleport (teleport_attr); return; }
 
    strcpy (buf, TELEPORT_ATTR);
@@ -1403,7 +1522,7 @@ void Teleport (button_ev)
    if (buf[len-1] == '=')
    {
       sprintf (&buf[len-1], "_page#=");
-      teleport_attr = FindAttrWithName (obj_ptr, buf);
+      teleport_attr = FindAttrWithName (obj_ptr, buf, NULL);
       if (teleport_attr != NULL)
       {
          DoPageTeleport (teleport_attr, DO_PAGE_BY_NUM);
@@ -1411,7 +1530,7 @@ void Teleport (button_ev)
       }
 
       sprintf (&buf[len-1], "_page=");
-      teleport_attr = FindAttrWithName (obj_ptr, buf);
+      teleport_attr = FindAttrWithName (obj_ptr, buf, NULL);
       if (teleport_attr != NULL)
       {
          DoPageTeleport (teleport_attr, DO_PAGE_BY_NAME);
@@ -1419,22 +1538,50 @@ void Teleport (button_ev)
       }
    }
 
-   launch_attr = FindAttrWithName (obj_ptr, LAUNCH_ATTR);
+   launch_attr = FindAttrWithName (obj_ptr, LAUNCH_ATTR, NULL);
    if (launch_attr != NULL) { DoLaunch (launch_attr, obj_ptr); return; }
 
-   exec_attr = FindAttrWithName (obj_ptr, EXEC_ATTR);
-   if (exec_attr != NULL)
+   exec_attr = FindAttrWithName (obj_ptr, EXEC_ATTR, NULL);
+   while (exec_attr != NULL)
    {
       int	saved_intr_check_interval=intrCheckInterval;
+      int	one_line_status=FALSE, exec_rc;
+      int	saved_history_depth=historyDepth;
+      char	status_buf[MAX_STATUS_BTNS+1][MAXSTRING+1];
 
+      MakeQuiescent ();
       intrCheckInterval = 1;
-      ShowInterrupt ();
+      ShowInterrupt (1);
 
-      DoExec (exec_attr, obj_ptr);
+      ResetDeckIndices ();
+      SaveStatusStringsIntoBuf (status_buf, &one_line_status);
+      if (cmdToExecAfterHyperJump != NULL) {
+         cfree(cmdToExecAfterHyperJump);
+         cmdToExecAfterHyperJump = NULL;
+      }
+      warpToAttr = NULL;
+      exec_rc = DoExec (exec_attr, obj_ptr);
+      exec_attr = NULL;
+      RemoveAllSel();
+      EndExecAnimate();
+      if (saved_history_depth != historyDepth) RestoreDefaultHistoryDepth();
+      if (exec_rc == TRUE && warpToAttr != NULL) DoTeleport (warpToAttr);
+      RestoreStatusStringsFromBuf (status_buf, one_line_status);
 
-      HideInterrupt ();
+      while (HideInterrupt() > 0) ;
       intrCheckInterval = saved_intr_check_interval;
 
+      if (exec_rc==TRUE && warpToAttr!=NULL && cmdToExecAfterHyperJump!=NULL) {
+         exec_attr = FindAttrWithName (NULL, cmdToExecAfterHyperJump, NULL);
+         if (exec_attr == NULL) {
+            sprintf(gszMsgBox, "Can not find the '%s' %s '%s' command.",
+                  cmdToExecAfterHyperJump, "attribute while executing the",
+                  "hyperjump_then_exec");
+            MsgBox(gszMsgBox, TOOL_NAME, INFO_MB);
+         } else {
+            continue;
+         }
+      }
       return;
    }
 
