@@ -25,10 +25,11 @@
  */
 #ifndef lint
 static char RCSid[] =
-      "@(#)$Header: /amnt/maui/tangram/u/william/X11/TGIF2/RCS/setup.c,v 2.119 1994/05/01 23:42:23 william Exp $";
+      "@(#)$Header: /export/bourbon/multimedia/william/X11/TGIF2/RCS/setup.c,v 2.134 1994/11/30 06:13:15 william Exp $";
 #endif
 
 #include <stdio.h>
+#include <sys/types.h>
 #include <X11/Xlib.h>
 #include <X11/Xutil.h>
 #include "const.h"
@@ -481,6 +482,37 @@ int mainWinEventHandler (input)
    return (INVALID);
 }
 
+void InitPaperSize ()
+{
+   psYOffStr = (char * *) calloc (MAXPAGESTYLES, sizeof(char *));
+   psYOffStr[0] = (char *) calloc (20, sizeof(char));
+   psYOffStr[1] = (char *) calloc (2, sizeof(char));
+   sprintf (psYOffStr[0], "%.2f",
+         (float)(((float)onePageHeight)/((float)PIX_PER_INCH)));
+   sprintf (psYOffStr[1], "0");
+   psYOff = (float *) calloc (MAXPAGESTYLES, sizeof(float));
+   psYOff[0] = ((float)onePageHeight)/((float)PIX_PER_INCH);
+   psYOff[1] = (float)0.0;
+   psPageWidthInInch = (float *) calloc (MAXPAGESTYLES, sizeof(float));
+   psPageWidthInInch[0] = ((float)onePageWidth)/((float)PIX_PER_INCH);
+   psPageWidthInInch[1] = ((float)onePageHeight)/((float)PIX_PER_INCH);
+   psPageHeightInInch = (float *) calloc (MAXPAGESTYLES, sizeof(float));
+   psPageHeightInInch[0] = ((float)onePageHeight)/((float)PIX_PER_INCH);
+   psPageHeightInInch[1] = ((float)onePageWidth)/((float)PIX_PER_INCH);
+}
+
+void CleanUpPaperSize ()
+{
+   cfree (psPageHeightInInch);
+   cfree (psPageWidthInInch);
+   cfree (psYOff);
+   cfree (psYOffStr[0]);
+   cfree (psYOffStr[1]);
+   cfree (psYOffStr);
+   psYOffStr = NULL;
+   psYOff = psPageWidthInInch = psPageHeightInInch = NULL;
+}
+
 static Window	savedMainWindow=None, savedDrawWindow=None;
 
 void Setup ()
@@ -488,7 +520,7 @@ void Setup ()
    int		bitmask = 0, x_neg_in_def = FALSE, y_neg_in_def = FALSE;
    int		x_in_cmdline=FALSE, y_in_cmdline=FALSE, cur_y=0, i;
    char		* c_ptr;
-   int		def_x_neg = 0, def_y_neg = 0;
+   int		def_x_neg = 0, def_y_neg = 0, paper_size_set;
    XWMHints	wmhints;
    XSizeHints	sizehints;
 
@@ -556,25 +588,6 @@ void Setup ()
          autoPan = FALSE;
    }
 
-   printMag = 100;
-   if ((c_ptr = XGetDefault (mainDisplay,TOOL_NAME,"PercentPrintReduction")) !=
-         NULL)
-   {
-      int	print_reduction = atoi (c_ptr);
-
-      if (print_reduction <= 0)
-      {
-         fprintf (stderr, "Invalid %s*PercentPrintReduction: '%s', %s.\n",
-               TOOL_NAME, c_ptr, "100 is used");
-         print_reduction = 100;
-      }
-      if (printMag != print_reduction)
-      {
-         printMag = print_reduction;
-         UpdPageStyle (pageStyle);
-      }
-   }
-
    moveMode = UNCONST_MOVE;
    if ((c_ptr = XGetDefault (mainDisplay,TOOL_NAME,"ConstrainedMove")) != NULL)
    {
@@ -681,6 +694,8 @@ void Setup ()
    if ((c_ptr = XGetDefault (mainDisplay, TOOL_NAME, "HistoryDepth")) != NULL)
       historyDepth = atoi (c_ptr);
 
+   defaultHistoryDepth = historyDepth;
+
    canvasWindowOnly = FALSE;
    if (cmdLineCWO)
       canvasWindowOnly = TRUE;
@@ -785,6 +800,59 @@ void Setup ()
          noStatusWindow = TRUE;
    }
 
+   scrollBarW = SCROLLBAR_W;
+   if ((c_ptr = XGetDefault (mainDisplay, TOOL_NAME, "ScrollBarWidth")) != NULL)
+   {
+      scrollBarW = atoi (c_ptr);
+      if (scrollBarW < 2 || scrollBarW > 16)
+      {
+         fprintf (stderr, "Invalid %s*ScrollBarWidth: '%s', %s, %1d is used.\n",
+               TOOL_NAME, c_ptr, "(must be between 2 and 16)", SCROLLBAR_W);
+         splineTol = SCROLLBAR_W;
+      }
+   }
+
+   paper_size_set = FALSE;
+   if ((c_ptr=XGetDefault(mainDisplay,TOOL_NAME,"InitialPaperSize")) != NULL)
+      paper_size_set = SetPaperSize (c_ptr);
+
+   InitPaperSize ();
+
+   printMag = 100;
+   if ((c_ptr = XGetDefault (mainDisplay,TOOL_NAME,"PercentPrintReduction")) !=
+         NULL)
+   {
+      int	print_reduction = atoi (c_ptr);
+
+      if (print_reduction <= 0)
+      {
+         fprintf (stderr, "Invalid %s*PercentPrintReduction: '%s', %s.\n",
+               TOOL_NAME, c_ptr, "100 is used");
+         print_reduction = 100;
+      }
+      if (printMag != print_reduction)
+      {
+         printMag = print_reduction;
+         paper_size_set = TRUE;
+      }
+   }
+   if (paper_size_set) UpdPageStyle (pageStyle);
+
+   usePaperSizeStoredInFile = FALSE;
+   if ((c_ptr=XGetDefault(mainDisplay,TOOL_NAME,"UsePaperSizeStoredInFile")) !=
+         NULL)
+   {
+      if (strcmp ("true", c_ptr) == 0 || strcmp ("True", c_ptr) == 0)
+         usePaperSizeStoredInFile = TRUE;
+   }
+
+   oneMotionSelectMove = FALSE;
+   if ((c_ptr=XGetDefault(mainDisplay,TOOL_NAME,"OneMotionSelMove")) != NULL)
+   {
+      if (strcmp ("true", c_ptr) == 0 || strcmp ("True", c_ptr) == 0)
+         oneMotionSelectMove = TRUE;
+   }
+
    initialMenubarWindowH = MENUBAR_WINDOW_H;
 
    strcpy (scanFileName, "No File");
@@ -1154,7 +1222,7 @@ void Setup ()
       XMapWindow (mainDisplay, dummyWindow2);
       XSelectInput (mainDisplay, dummyWindow2, EnterWindowMask);
 
-      if (statusWindow != None)
+      if (!noStatusWindow)
       {
          XMapWindow (mainDisplay, statusWindow);
          XSelectInput (mainDisplay, statusWindow, ExposureMask |
@@ -1162,6 +1230,9 @@ void Setup ()
          for (i=0; i < MAX_STATUS_BTNS; i++)
             XMapWindow (mainDisplay, statusSubWindow[i]);
       }
+      else
+         XSelectInput (mainDisplay, statusWindow, ExposureMask |
+               EnterWindowMask);
    }
 #else
    if (canvasWindowOnly)
@@ -1215,7 +1286,7 @@ void Setup ()
       XSelectInput (mainDisplay, dummyWindow2, EnterWindowMask);
       XMapWindow (mainDisplay, dummyWindow2);
 
-      if (statusWindow != None)
+      if (!noStatusWindow)
       {
          XSelectInput (mainDisplay, statusWindow, ExposureMask |
                EnterWindowMask);
@@ -1223,6 +1294,9 @@ void Setup ()
          for (i=0; i < MAX_STATUS_BTNS; i++)
             XMapWindow (mainDisplay, statusSubWindow[i]);
       }
+      else
+         XSelectInput (mainDisplay, statusWindow, ExposureMask |
+               EnterWindowMask);
    }
 #endif
 
@@ -1287,11 +1361,13 @@ void Setup ()
    XSetWMHints (mainDisplay, drawWindow, &wmhints);
 
    InitText ();
+   InitRemote ();
 }
 
 void CleanUpResiduals ()
 {
    if (savedMainWindow != None) XDestroyWindow (mainDisplay, savedMainWindow);
+   CleanUpPaperSize ();
 }
 
 int TieLooseEnds ()
