@@ -25,7 +25,7 @@
  */
 #ifndef lint
 static char RCSid[] =
-      "@(#)$Header: /amnt/maui/tangram/u/william/X11/TGIF2/RCS/stk.c,v 2.52 1994/05/02 06:08:33 william Exp $";
+      "@(#)$Header: /export/bourbon/multimedia/william/X11/TGIF2/RCS/stk.c,v 2.65 1994/11/26 00:35:55 william Exp $";
 #endif
 
 #include <stdio.h>
@@ -52,6 +52,7 @@ static char RCSid[] =
 #include "menu.e"
 #include "msg.e"
 #include "names.e"
+#include "navigate.e"
 #include "obj.e"
 #include "page.e"
 #include "pattern.e"
@@ -67,8 +68,7 @@ static char RCSid[] =
 #include "text.e"
 
 struct ObjRec	* tgifObj=NULL;
-
-static struct StkRec	* topStk = NULL;
+struct StkRec	* topStk=NULL;
 
 int AncesterModified ()
 {
@@ -108,58 +108,15 @@ void InitStk ()
    InitTgifObj ();
 }
 
-void PushIcon ()
+struct StkRec * SaveFileInfo ()
 {
    struct StkRec	* stk_ptr;
-   struct ObjRec	* obj_ptr;
-   char			sym_name[MAXPATHLENGTH], path_name[MAXPATHLENGTH];
-   char			file_name[MAXPATHLENGTH], s[MAXPATHLENGTH], * rest;
-   int			read_status, short_name;
-   FILE			* fp;
-   int			tmp_linenum, interrupted;
-   char			tmp_filename[MAXPATHLENGTH];
-
-   if (topSel == NULL || topSel != botSel || topSel->obj->type != OBJ_ICON)
-   {
-      Msg ("Please select one ICON object to push into.");
-      return;
-   }
-
-   strcpy (sym_name, topSel->obj->detail.r->s);
-   if (!GetSymbolPath (sym_name, path_name))
-   {
-      sprintf (s, "Can not find '%s.%s' in %s", sym_name, SYM_FILE_EXT,
-            curDomainPath);
-      Msg (s);
-      return;
-   }
-
-   strcat (sym_name, ".");
-   strcat (sym_name, SYM_FILE_EXT);
-   sprintf (file_name, "%s/%s", path_name, sym_name);
-
-   if ((short_name = IsPrefix (bootDir, file_name, &rest))) ++rest;
-
-   if ((fp = fopen (file_name, "r")) == NULL)
-   {
-      if (short_name)
-         sprintf (s, "Can not open '%s', icon not pushed into.", rest);
-      else
-         sprintf (s, "Can not open '%s', icon not pushed into.", file_name);
-      Msg (s);
-      return;
-   }
-
-   strcpy (tmp_filename, scanFileName);
-   tmp_linenum = scanLineNum;
-   strcpy (scanFileName, (short_name ? rest : file_name));
-   scanLineNum = 0;
-
-   HighLightReverse ();
 
    stk_ptr = (struct StkRec *) calloc (1, sizeof(struct StkRec));
+   if (stk_ptr == NULL) fprintf (stderr, "Can not calloc().\n");
+
    stk_ptr->next = topStk;
-   stk_ptr->sel = topSel->obj;
+   stk_ptr->sel = (topSel == NULL ? NULL : topSel->obj);
    stk_ptr->first = topObj;
    stk_ptr->last = botObj;
    stk_ptr->file_mod = fileModified;
@@ -193,8 +150,6 @@ void PushIcon ()
    stk_ptr->rcb_radius = rcbRadius;
    stk_ptr->first_file_attr = tgifObj->fattr;
    stk_ptr->last_file_attr = tgifObj->lattr;
-   tgifObj->fattr = NULL;
-   tgifObj->lattr = NULL;
 
    stk_ptr->first_page = firstPage;
    stk_ptr->last_page = lastPage;
@@ -206,11 +161,31 @@ void PushIcon ()
    stk_ptr->page_layout_mode = pageLayoutMode;
 
    stk_ptr->color_dump = colorDump;
+   stk_ptr->one_page_width = onePageWidth;
+   stk_ptr->one_page_height = onePageHeight;
 
    stk_ptr->first_cmd = firstCmd;
    stk_ptr->last_cmd = lastCmd;
    stk_ptr->cur_cmd = curCmd;
    stk_ptr->history_count = historyCount;
+
+   strcpy (stk_ptr->dir, curDir);
+   stk_ptr->name_valid = curFileDefined;
+   if (stk_ptr->name_valid) strcat (stk_ptr->name, curFileName);
+   strcpy (stk_ptr->sym_dir, curSymDir);
+   strcpy (stk_ptr->domain, curDomainName);
+
+   stk_ptr->saved_comments = savedComments;
+   stk_ptr->saved_comments_len = savedCommentsLen;
+
+   return (stk_ptr);
+}
+
+void ResetFileInfo ()
+{
+   tgifObj->fattr = NULL;
+   tgifObj->lattr = NULL;
+
    firstCmd = lastCmd = curCmd = NULL;
    historyCount = 0;
 
@@ -219,17 +194,64 @@ void PushIcon ()
    lastPageNum = 0;
    InitPage ();
 
-   strcpy (stk_ptr->dir, curDir);
-   if ((stk_ptr->name_valid = curFileDefined))
-      strcat (stk_ptr->name, curFileName);
-   strcpy (stk_ptr->sym_dir, curSymDir);
-   strcpy (stk_ptr->domain, curDomainName);
-
-   stk_ptr->saved_comments = savedComments;
-   stk_ptr->saved_comments_len = savedCommentsLen;
    savedComments = NULL;
    savedCommentsLen = 0;
 
+   if (usePaperSizeStoredInFile) ResetOnePageSize ();
+}
+
+int PushIcon ()
+{
+   struct StkRec	* stk_ptr;
+   struct ObjRec	* obj_ptr;
+   char			sym_name[MAXPATHLENGTH], path_name[MAXPATHLENGTH];
+   char			file_name[MAXPATHLENGTH], s[MAXPATHLENGTH], * rest;
+   int			read_status, short_name;
+   FILE			* fp;
+   int			tmp_linenum, interrupted;
+   char			tmp_filename[MAXPATHLENGTH];
+
+   if (topSel == NULL || topSel != botSel || topSel->obj->type != OBJ_ICON)
+   {
+      Msg ("Please select one ICON object to push into.");
+      return (FALSE);
+   }
+
+   strcpy (sym_name, topSel->obj->detail.r->s);
+   if (!GetSymbolPath (sym_name, path_name))
+   {
+      sprintf (s, "Can not find '%s.%s' in %s", sym_name, SYM_FILE_EXT,
+            curDomainPath);
+      Msg (s);
+      return (FALSE);
+   }
+
+   strcat (sym_name, ".");
+   strcat (sym_name, SYM_FILE_EXT);
+   sprintf (file_name, "%s/%s", path_name, sym_name);
+
+   if ((short_name = IsPrefix (bootDir, file_name, &rest))) ++rest;
+
+   if ((fp = fopen (file_name, "r")) == NULL)
+   {
+      if (short_name)
+         sprintf (s, "Can not open '%s', icon not pushed into.", rest);
+      else
+         sprintf (s, "Can not open '%s', icon not pushed into.", file_name);
+      Msg (s);
+      return (FALSE);
+   }
+
+   strcpy (tmp_filename, scanFileName);
+   tmp_linenum = scanLineNum;
+   strcpy (scanFileName, (short_name ? rest : file_name));
+   scanLineNum = 0;
+
+   HighLightReverse ();
+
+   stk_ptr = SaveFileInfo ();
+   ResetFileInfo ();
+
    topStk = stk_ptr;
 
    if (short_name)
@@ -245,8 +267,9 @@ void PushIcon ()
    SetWatchCursor (drawWindow);
    SetWatchCursor (mainWindow);
    numRedrawBBox = 0;
-   ShowInterrupt ();
+   ShowInterrupt (1);
    interrupted = FALSE;
+   foundGoodStateObject = FALSE;
    while ((read_status = ReadObj (fp, &obj_ptr)) == TRUE)
    {
       if (obj_ptr != NULL)
@@ -263,7 +286,7 @@ void PushIcon ()
       }
    }
    if (interrupted) Msg ("User interrupt.  Drawing aborted.");
-   HideInterrupt ();
+   HideInterrupt();
 
    strcpy (scanFileName, tmp_filename);
    scanLineNum = tmp_linenum;
@@ -277,14 +300,20 @@ void PushIcon ()
       sprintf (s, "File version too large (=%1d).  Push aborted!",
             fileVersion);
       Msg (s);
-      DrawPaperBoundary ();
-      RedrawGridLines ();
-      return;
+      DrawPaperBoundary (drawWindow);
+      RedrawGridLines (drawWindow);
+      return (FALSE);
    }
 
    strcpy (curFileName, sym_name);
    strcpy (curSymDir, path_name);
    curFileDefined = TRUE;
+   if (!curDirIsLocal)
+   {
+      strcpy (curDir, curLocalDir);
+      *curLocalDir = '\0';
+      curDirIsLocal = TRUE;
+   }
 
    if (short_name)
       sprintf (s, "Current file is '%s'.", rest);
@@ -298,12 +327,108 @@ void PushIcon ()
 
    RedrawTitleWindow ();
    UpdateAllSubMenus ();
+   if (foundGoodStateObject && !navigatingBackAndForth) CommitNavigate ();
+   return (foundGoodStateObject);
+}
+
+void RestoreFileInfo (StkPtr)
+   struct StkRec	* StkPtr;
+{
+   char	msg[MAXPATHLENGTH], fname[MAXPATHLENGTH], * rest;
+
+   fileModified = StkPtr->file_mod;
+   objId = StkPtr->id;
+
+   CleanUpComments ();
+   CleanUpDrawingWindow ();
+
+   drawOrigX = StkPtr->orig_x;
+   drawOrigY = StkPtr->orig_y;
+   zoomScale = StkPtr->zoom;
+   zoomedIn = StkPtr->zoomed;
+   gridSystem = StkPtr->grid_system;
+   xyEnglishGrid = StkPtr->english_grid;
+   xyMetricGrid = StkPtr->metric_grid;
+   gridOn = StkPtr->grid_on;
+   colorIndex = StkPtr->color;
+   horiAlign = StkPtr->h_align;
+   vertAlign = StkPtr->v_align;
+   lineWidth = StkPtr->line_w;
+   lineStyle = StkPtr->line_s;
+   objFill = StkPtr->fill;
+   penPat = StkPtr->pen;
+   textJust = StkPtr->just;
+   textVSpace = StkPtr->v_space;
+   curFont = StkPtr->font;
+   curStyle = StkPtr->f_style;
+   curSize = StkPtr->f_size;
+   printMag = StkPtr->print_mag;
+   gridShown = StkPtr->grid_shown;
+   moveMode = StkPtr->move_mode;
+   curRotate = StkPtr->text_rotate;
+   rcbRadius = StkPtr->rcb_radius;
+   tgifObj->fattr = StkPtr->first_file_attr;
+   tgifObj->lattr = StkPtr->last_file_attr;
+
+   CleanUpCmds ();
+   firstCmd = StkPtr->first_cmd;
+   lastCmd = StkPtr->last_cmd;
+   curCmd = StkPtr->cur_cmd;
+   historyCount = StkPtr->history_count;
+
+   firstPage = StkPtr->first_page;
+   lastPage = StkPtr->last_page;
+   curPage = StkPtr->cur_page;
+   curPageNum = StkPtr->cur_page_num;
+   lastPageNum = StkPtr->last_page_num;
+   paperCol = StkPtr->cols;
+   paperRow = StkPtr->rows;
+   pageLayoutMode = StkPtr->page_layout_mode;
+
+   colorDump = StkPtr->color_dump;
+   onePageWidth = StkPtr->one_page_width;
+   onePageHeight = StkPtr->one_page_height;
+
+   if (usePaperSizeStoredInFile) SetPSPageWidthHeight ();
+
+   curPage->top = topObj = StkPtr->first;
+   curPage->bot = botObj = StkPtr->last;
+   strcpy (curDomainName, StkPtr->domain);
+   strcpy (curSymDir, StkPtr->sym_dir);
+   curFileDefined = StkPtr->name_valid;
+   if (curFileDefined)
+   {
+      strcpy (curFileName, StkPtr->name);
+      if (*curSymDir == '\0')
+         sprintf (fname, "%s/%s", StkPtr->dir, curFileName);
+      else
+         sprintf (fname, "%s/%s", curSymDir, curFileName);
+      if (IsPrefix (bootDir, fname, &rest))
+         sprintf (msg, "Pop back to '%s'.", ++rest);
+      else
+         sprintf (msg, "Pop back to '%s'.", fname);
+      Msg (msg);
+
+      SetCurDir (fname);
+   }
+   else
+   {
+      sprintf (msg, "Poping back to parent level.  Current file undefined.");
+      Msg (msg);
+   }
+
+   if (strcmp (curDir, StkPtr->dir) != 0)
+   {
+      strcpy (curDir, StkPtr->dir);
+      UpdateDirInfo ();
+   }
+
+   savedComments = StkPtr->saved_comments;
+   savedCommentsLen = StkPtr->saved_comments_len;
 }
 
 void PopIcon ()
 {
-   char	msg[MAXPATHLENGTH], dummy[MAXPATHLENGTH], * rest;
-
    if (topStk == NULL)
    {
       Msg ("Already at top level.");
@@ -311,97 +436,16 @@ void PopIcon ()
    }
    while (fileModified)
    {
-      switch (YesNoCancel ("File modified, save file before quit? [ync](y)",
-            CONFIRM_YES))
+      switch (MsgBox ("File modified, save file before quit? [ync](y)",
+            TOOL_NAME, YNC_MB))
       {
-         case CONFIRM_YES: SaveFile (); break;
-         case CONFIRM_NO: SetFileModified (FALSE); break;
-         case CONFIRM_CANCEL: return;
+         case MB_ID_YES: SaveFile (); break;
+         case MB_ID_NO: SetFileModified (FALSE); break;
+         case MB_ID_CANCEL: return;
       }
    }
-   fileModified = topStk->file_mod;
-   objId = topStk->id;
-
-   CleanUpComments ();
-   CleanUpDrawingWindow ();
-
-   drawOrigX = topStk->orig_x;
-   drawOrigY = topStk->orig_y;
-   zoomScale = topStk->zoom;
-   zoomedIn = topStk->zoomed;
-   gridSystem = topStk->grid_system;
-   xyEnglishGrid = topStk->english_grid;
-   xyMetricGrid = topStk->metric_grid;
-   gridOn = topStk->grid_on;
-   colorIndex = topStk->color;
-   horiAlign = topStk->h_align;
-   vertAlign = topStk->v_align;
-   lineWidth = topStk->line_w;
-   lineStyle = topStk->line_s;
-   objFill = topStk->fill;
-   penPat = topStk->pen;
-   textJust = topStk->just;
-   textVSpace = topStk->v_space;
-   curFont = topStk->font;
-   curStyle = topStk->f_style;
-   curSize = topStk->f_size;
-   printMag = topStk->print_mag;
-   gridShown = topStk->grid_shown;
-   moveMode = topStk->move_mode;
-   curRotate = topStk->text_rotate;
-   rcbRadius = topStk->rcb_radius;
-   tgifObj->fattr = topStk->first_file_attr;
-   tgifObj->lattr = topStk->last_file_attr;
-
-   CleanUpCmds ();
-   firstCmd = topStk->first_cmd;
-   lastCmd = topStk->last_cmd;
-   curCmd = topStk->cur_cmd;
-   historyCount = topStk->history_count;
-
-   firstPage = topStk->first_page;
-   lastPage = topStk->last_page;
-   curPage = topStk->cur_page;
-   curPageNum = topStk->cur_page_num;
-   lastPageNum = topStk->last_page_num;
-   paperCol = topStk->cols;
-   paperRow = topStk->rows;
-   pageLayoutMode = topStk->page_layout_mode;
-
-   colorDump = topStk->color_dump;
-
-   curPage->top = topObj = topStk->first;
-   curPage->bot = botObj = topStk->last;
-   strcpy (curDomainName, topStk->domain);
-   strcpy (curSymDir, topStk->sym_dir);
-   if ((curFileDefined = topStk->name_valid))
-   {
-      strcpy (curFileName, topStk->name);
-      if (*curSymDir == '\0')
-         sprintf (dummy, "%s/%s", topStk->dir, curFileName);
-      else
-         sprintf (dummy, "%s/%s", curSymDir, curFileName);
-      if (IsPrefix (bootDir, dummy, &rest))
-         sprintf (msg, "Pop back to '%s'.", ++rest);
-      else
-         sprintf (msg, "Pop back to '%s'.", dummy);
-      Msg (msg);
-   }
-   else
-   {
-      sprintf (dummy, "Poping back to parent level.  Current file undefined.");
-      Msg (dummy);
-   }
-   if (strcmp (curDir, topStk->dir) != 0)
-   {
-      strcpy (curDir, topStk->dir);
-      UpdateDirInfo ();
-   }
-   else
-      strcpy (curDir, topStk->dir);
-
-   savedComments = topStk->saved_comments;
-   savedCommentsLen = topStk->saved_comments_len;
+   AdjustNavigate ();
+   RestoreFileInfo (topStk);
 
    topSel = botSel = (struct SelRec *) calloc (1, sizeof(struct SelRec));
    topSel->next = NULL;
