@@ -25,13 +25,16 @@
  */
 #ifndef lint
 static char RCSid[] =
-      "@(#)$Header: /amnt/maui/tangram/u/william/X11/TGIF2/RCS/stretch.c,v 2.87 1994/03/21 04:57:03 william Exp $";
+      "@(#)$Header: /export/bourbon/multimedia/william/X11/TGIF2/RCS/stretch.c,v 2.95 1994/12/06 04:26:33 william Exp $";
 #endif
 
 #include <math.h>
 #include <stdio.h>
 #include <string.h>
+#include <sys/types.h>
 #include <X11/Xlib.h>
+#include <X11/Xutil.h>
+#include <X11/keysym.h>
 #include "const.h"
 #include "types.h"
 
@@ -274,10 +277,10 @@ void StretchPoly (XGridOff, YGridOff, ObjPtr, NumPts, V, Index)
    struct ObjRec	* ObjPtr;
 {
    register int	i;
-   int		x, y, dx, dy, num = 0, stretching = TRUE;
+   int		x, y, dx, dy, num=0, stretching=TRUE;
    int		ltx, lty, rbx, rby, curved=LT_STRAIGHT;
-   int		grid_x = XGridOff, grid_y = YGridOff, sn, intn;
-   int		auto_center_attr = AutoCenterAttr (ObjPtr);
+   int		grid_x=XGridOff, grid_y=YGridOff, sn, intn;
+   int		auto_center_attr=AutoCenterAttr (ObjPtr);
    char		buf[80], * smooth=NULL;
    XEvent	input, ev;
    XPoint	* pv=NULL, * sv=NULL, * cntrlv=NULL;
@@ -420,93 +423,6 @@ void StretchPoly (XGridOff, YGridOff, ObjPtr, NumPts, V, Index)
       {
          XUngrabPointer (mainDisplay, CurrentTime);
          stretching = FALSE;
-         sprintf (buf, "dx=%+1d,dy=%+1d", ABS_SIZE(grid_x-XGridOff),
-               ABS_SIZE(grid_y-YGridOff));
-         EndShowMeasureCursor (grid_x, grid_y, buf, TRUE);
-
-         if (dx != 0 || dy != 0)
-         {
-            if (splineRubberband)
-               XDrawLines (mainDisplay, drawWindow, revDefaultGC, sv, sn,
-                     CoordModeOrigin);
-            else
-               MyDashedLine (drawWindow, revDefaultGC, v, num);
-
-            HighLightReverse ();
-
-            PrepareToReplaceAnObj (ObjPtr);
-
-            dx = ABS_SIZE(dx);
-            dy = ABS_SIZE(dy);
-            switch (ObjPtr->type)
-            {
-               case OBJ_POLY:
-                  V[Index].x += dx; V[Index].y += dy;
-                  AdjObjSplineVs (ObjPtr);
-                  if (ObjPtr->detail.p->curved != LT_INTSPLINE)
-                     UpdPolyBBox (ObjPtr, NumPts, V);
-                  else
-                     UpdPolyBBox (ObjPtr, ObjPtr->detail.p->intn,
-                           ObjPtr->detail.p->intvlist);
-                  break;
-               case OBJ_POLYGON:
-                  V[Index].x += dx; V[Index].y += dy;
-                  if (Index == 0)
-                  {
-                     V[NumPts-1].x += dx; V[NumPts-1].y += dy;
-                  }
-                  else if (Index == NumPts-1)
-                  {
-                     V[0].x += dx; V[0].y += dy;
-                  }
-                  AdjObjSplineVs (ObjPtr);
-                  if (ObjPtr->detail.g->curved != LT_INTSPLINE)
-                     UpdPolyBBox (ObjPtr, NumPts, V);
-                  else
-                     UpdPolyBBox (ObjPtr, ObjPtr->detail.g->intn,
-                           ObjPtr->detail.g->intvlist);
-                  break;
-            }
-            if (auto_center_attr)
-            {
-               struct AttrRec	* attr_ptr=ObjPtr->fattr;
-               int		modified=FALSE;
-
-               for ( ; attr_ptr != NULL; attr_ptr = attr_ptr->next)
-                  if (attr_ptr->shown)
-                  {
-                     struct BBRec	bbox;
-
-                     CenterObjInOBBox (attr_ptr->obj, ObjPtr->obbox, &bbox);
-                     if (bbox.ltx < ltx) ltx = bbox.ltx;
-                     if (bbox.lty < lty) lty = bbox.lty;
-                     if (bbox.rbx > rbx) rbx = bbox.rbx;
-                     if (bbox.rby > rby) rby = bbox.rby;
-                     modified = TRUE;
-                  }
-               if (modified) AdjObjBBox (ObjPtr);
-            }
-            RecordReplaceAnObj (ObjPtr);
-
-            UpdSelBBox ();
-            RedrawAreas (botObj, ltx-GRID_ABS_SIZE(1), lty-GRID_ABS_SIZE(1),
-                  rbx+GRID_ABS_SIZE(1), rby+GRID_ABS_SIZE(1),
-                  ObjPtr->bbox.ltx-GRID_ABS_SIZE(1),
-                  ObjPtr->bbox.lty-GRID_ABS_SIZE(1),
-                  ObjPtr->bbox.rbx+GRID_ABS_SIZE(1),
-                  ObjPtr->bbox.rby+GRID_ABS_SIZE(1));
-            SetFileModified (TRUE);
-            justDupped = FALSE;
-
-            HighLightForward ();
-         }
-         if (splineRubberband)
-         {
-            cfree (sv);
-            cfree (pv);
-            if (smooth != NULL) cfree (smooth);
-            if (curved == LT_INTSPLINE && cntrlv != NULL) cfree (cntrlv);
-         }
       }
       else if (input.type == MotionNotify)
       {
@@ -586,6 +502,92 @@ void StretchPoly (XGridOff, YGridOff, ObjPtr, NumPts, V, Index)
          while (XCheckMaskEvent (mainDisplay, PointerMotionMask, &ev)) ;
       }
    }
+   sprintf (buf, "dx=%+1d,dy=%+1d", ABS_SIZE(grid_x-XGridOff),
+         ABS_SIZE(grid_y-YGridOff));
+   EndShowMeasureCursor (grid_x, grid_y, buf, TRUE);
+   if (dx != 0 || dy != 0)
+   {
+      if (splineRubberband)
+         XDrawLines (mainDisplay, drawWindow, revDefaultGC, sv, sn,
+               CoordModeOrigin);
+      else
+         MyDashedLine (drawWindow, revDefaultGC, v, num);
+
+      HighLightReverse ();
+
+      PrepareToReplaceAnObj (ObjPtr);
+
+      dx = ABS_SIZE(dx);
+      dy = ABS_SIZE(dy);
+      switch (ObjPtr->type)
+      {
+         case OBJ_POLY:
+            V[Index].x += dx; V[Index].y += dy;
+            AdjObjSplineVs (ObjPtr);
+            if (ObjPtr->detail.p->curved != LT_INTSPLINE)
+               UpdPolyBBox (ObjPtr, NumPts, V);
+            else
+               UpdPolyBBox (ObjPtr, ObjPtr->detail.p->intn,
+                     ObjPtr->detail.p->intvlist);
+            break;
+         case OBJ_POLYGON:
+            V[Index].x += dx; V[Index].y += dy;
+            if (Index == 0)
+            {
+               V[NumPts-1].x += dx; V[NumPts-1].y += dy;
+            }
+            else if (Index == NumPts-1)
+            {
+               V[0].x += dx; V[0].y += dy;
+            }
+            AdjObjSplineVs (ObjPtr);
+            if (ObjPtr->detail.g->curved != LT_INTSPLINE)
+               UpdPolyBBox (ObjPtr, NumPts, V);
+            else
+               UpdPolyBBox (ObjPtr, ObjPtr->detail.g->intn,
+                     ObjPtr->detail.g->intvlist);
+            break;
+      }
+      if (auto_center_attr)
+      {
+         struct AttrRec	* attr_ptr=ObjPtr->fattr;
+         int		modified=FALSE;
+
+         for ( ; attr_ptr != NULL; attr_ptr = attr_ptr->next)
+            if (attr_ptr->shown)
+            {
+               struct BBRec	bbox;
+
+               CenterObjInOBBox (attr_ptr->obj, ObjPtr->obbox, &bbox);
+               if (bbox.ltx < ltx) ltx = bbox.ltx;
+               if (bbox.lty < lty) lty = bbox.lty;
+               if (bbox.rbx > rbx) rbx = bbox.rbx;
+               if (bbox.rby > rby) rby = bbox.rby;
+               modified = TRUE;
+            }
+         if (modified) AdjObjBBox (ObjPtr);
+      }
+      RecordReplaceAnObj (ObjPtr);
+
+      UpdSelBBox ();
+      RedrawAreas (botObj, ltx-GRID_ABS_SIZE(1), lty-GRID_ABS_SIZE(1),
+            rbx+GRID_ABS_SIZE(1), rby+GRID_ABS_SIZE(1),
+            ObjPtr->bbox.ltx-GRID_ABS_SIZE(1),
+            ObjPtr->bbox.lty-GRID_ABS_SIZE(1),
+            ObjPtr->bbox.rbx+GRID_ABS_SIZE(1),
+            ObjPtr->bbox.rby+GRID_ABS_SIZE(1));
+      SetFileModified (TRUE);
+      justDupped = FALSE;
+
+      HighLightForward ();
+   }
+   if (splineRubberband)
+   {
+      cfree (sv);
+      cfree (pv);
+      if (smooth != NULL) cfree (smooth);
+      if (curved == LT_INTSPLINE && cntrlv != NULL) cfree (cntrlv);
+   }
 }
 
 static double	multX, multY;
@@ -962,7 +964,7 @@ int ConstrainedStretchAllSel (Corner, ltx, lty, rbx, rby)
    int	* ltx, * lty, * rbx, * rby;
 {
    register struct ObjRec	* obj_ptr;
-   int				something_stretched = FALSE, num_pts;
+   int				something_stretched=FALSE, num_pts;
    int				x_off, y_off, move_first, move_last, x, y;
    XPoint			* v;
 
@@ -1183,7 +1185,7 @@ void StretchBox (XGridOff, YGridOff, ObjPtr, Corner)
    int			XGridOff, YGridOff, Corner;
    struct ObjRec	* ObjPtr;
 {
-   int		x, y, stretching = TRUE;
+   int		x, y, stretching=TRUE;
    int		ltx, lty, rbx, rby, sel_ltx, sel_lty, sel_rbx, sel_rby;
    int		stretched_ltx, stretched_lty, stretched_rbx, stretched_rby;
    int		stretched_sel_ltx, stretched_sel_lty, stretched_sel_rbx;
@@ -1269,23 +1271,6 @@ void StretchBox (XGridOff, YGridOff, ObjPtr, Corner)
       {
          XUngrabPointer (mainDisplay, CurrentTime);
          stretching = FALSE;
-
-         EndIntervalRulers (grid_x, grid_y);
-         GetMeasurement (ObjPtr, buf);
-         EndShowMeasureCursor (grid_x, grid_y, buf, TRUE);
-         SelBox (drawWindow, revDefaultGC, stretched_ltx, stretched_lty,
-               stretched_rbx, stretched_rby);
-         SelBox (drawWindow, revDefaultGC, stretched_sel_ltx-2,
-               stretched_sel_lty-2, stretched_sel_rbx+2, stretched_sel_rby+2);
-
-         if (multX != (double)1.0 || multY != (double)1.0)
-         {
-            HighLightReverse ();
-            StretchAllSel (Corner);
-            HighLightForward ();
-            SetFileModified (TRUE);
-            justDupped = FALSE;
-         }
       }
       else if (input.type == MotionNotify)
       {
@@ -1367,6 +1352,21 @@ void StretchBox (XGridOff, YGridOff, ObjPtr, Corner)
          while (XCheckMaskEvent (mainDisplay, PointerMotionMask, &ev)) ;
       }
    }
+   EndIntervalRulers (grid_x, grid_y);
+   GetMeasurement (ObjPtr, buf);
+   EndShowMeasureCursor (grid_x, grid_y, buf, TRUE);
+   SelBox (drawWindow, revDefaultGC, stretched_ltx, stretched_lty,
+         stretched_rbx, stretched_rby);
+   SelBox (drawWindow, revDefaultGC, stretched_sel_ltx-2,
+         stretched_sel_lty-2, stretched_sel_rbx+2, stretched_sel_rby+2);
+   if (multX != (double)1.0 || multY != (double)1.0)
+   {
+      HighLightReverse ();
+      StretchAllSel (Corner);
+      HighLightForward ();
+      SetFileModified (TRUE);
+      justDupped = FALSE;
+   }
 }
 
 void StretchSel (XGridOff, YGridOff, ObjPtr, Corner)
@@ -1400,11 +1400,11 @@ void StretchSel (XGridOff, YGridOff, ObjPtr, Corner)
 
 void ScaleAnEPSObj (ObjPtr, ScalingFactor)
    struct ObjRec	* ObjPtr;
-   float		ScalingFactor;
+   float		* ScalingFactor;
 {
    struct BBRec	* obbox = &(ObjPtr->obbox);
 
-   multX = multY = (double) ScalingFactor;
+   multX = multY = (double) (*ScalingFactor);
    changeX = changeY = (fabs(multX-1.0) > 1.0e-6);
    if (!changeX && !changeY) return;;
 
@@ -1630,6 +1630,56 @@ void ScaleAllSelObj ()
    justDupped = FALSE;
 }
 
+/*
+ * The following code is INCORRECT because redo does not work!
+ *	The main problem with redo is that for a composite cmd,
+ *	if the first command inside the composite is a move or a
+ *	stretch, it is treated as if the move comes from the side
+ *	effect of a constrained move!
+ */
+
+void MoveSizeAllSelObj(AbsX, AbsY, AbsW, AbsH)
+   int AbsX, AbsY, AbsW, AbsH;
+   /* These are supposed to be points corresponding to obbox */
+{
+   int move=FALSE, stretch=FALSE;
+
+   if (topSel == NULL) return;
+
+   if (AbsX != selObjLtX || AbsY != selObjLtY) move = TRUE;
+   if (AbsW != selObjRbX-selObjLtX || AbsH != selObjRbY-selObjLtY) {
+      stretch = TRUE;
+   }
+   if (!move && !stretch) return;
+
+   if (move && stretch) {
+      StartCompositeCmd();
+   }
+   if (move) {
+      MoveAllSel(AbsX-selObjLtX, AbsY-selObjLtY);
+      UpdSelBBox();
+   }
+   if (stretch) {
+      int saved_h_align=horiAlign, saved_v_align=vertAlign;
+
+      horiAlign = ALIGN_L;
+      vertAlign = ALIGN_T;
+      multX = ((double)AbsW) / ((double)selObjRbX-selObjLtX);
+      multY = ((double)AbsH) / ((double)selObjRbY-selObjLtY);
+      changeX = (fabs(multX-1.0) > 1.0e-6);
+      changeY = (fabs(multY-1.0) > 1.0e-6);
+      ScaleAllSel(CORNER_RB);
+      horiAlign = saved_h_align;
+      vertAlign = saved_v_align;
+   }
+   if (move && stretch) {
+      EndCompositeCmd();
+   }
+   UpdSelBBox();
+   SetFileModified(TRUE);
+   justDupped = FALSE;
+}
+
 void FlipObjHorizontal (ObjPtr)
    struct ObjRec	* ObjPtr;
 {
