@@ -25,11 +25,12 @@
  */
 #ifndef lint
 static char RCSid[] =
-      "@(#)$Header: /amnt/maui/tangram/u/william/X11/TGIF2/RCS/text.c,v 2.133 1994/03/21 04:57:13 william Exp $";
+      "@(#)$Header: /export/bourbon/multimedia/william/X11/TGIF2/RCS/text.c,v 2.144 1994/12/15 06:01:58 william Exp $";
 #endif
 
 #include <stdio.h>
 #include <string.h>
+#include <sys/types.h>
 #include <X11/Xlib.h>
 #include <X11/Xutil.h>
 #include <X11/keysym.h>
@@ -114,6 +115,11 @@ static struct StrRec	* endStr = NULL;
 
 static int	escPressed = FALSE;
 
+#ifdef _NO_EXTERN
+void	DrawTextObj ARGS_DECL((Window, int XOff, int YOff, struct ObjRec *));
+void	RedrawCurText ARGS_DECL((void));
+#endif /* _NO_EXTERN */
+
 #define BLUR 32
 
 void BlurText (Win, gc, XOff, YOff, W, H)
@@ -228,7 +234,7 @@ void PaintText (Win,Str,Just,Rotate,XOff,YOff,xfs,ColorIndex,Pen)
    text_bbox.lty = ABS_Y(YOff);
    text_bbox.rbx = ABS_X(XOff+w);
    text_bbox.rby = ABS_Y(YOff+ZOOMED_SIZE(orig_h));
-   if (!BBoxIntersect (text_bbox, drawWinBBox)) return;
+   if (checkBBox && !BBoxIntersect (text_bbox, drawWinBBox)) return;
 
    values.foreground = colorPixels[ColorIndex];
    values.function = GXcopy;
@@ -458,7 +464,10 @@ int CreateTextObj ()
             FreeAttr (attr_ptr);
          }
          else
+         {
+            curTextObj->detail.t->lines = 1;
             UpdateAttr(curTextObj->detail.t, attr_ptr);
+         }
 
          AdjObjBBox (attr_ptr->owner);
          if (outerSel != NULL) AdjAncestorsBBox ();
@@ -1084,7 +1093,8 @@ void HandleButton (Button_Ev)
          {
             obj_ptr = curTextObj;
             curStr = obj_ptr->detail.t->first;
-            if (textJustClicked && (click_time-lastClickTime) < doubleClickInterval)
+            if (textJustClicked && (click_time-lastClickTime) <
+                  doubleClickInterval)
                double_clicked = TRUE;
          }
          else if (!PrepareEditExistingText (obj_ptr,abs_x,abs_y,&x_off,&y_off))
@@ -1363,7 +1373,7 @@ void HandleButton (Button_Ev)
                if (attr_ptr != NULL && attr_ptr->inherited &&
                      textEndY == textOrigY &&
                      attr_ptr->shown && attr_ptr->nameshown &&
-                     textEndIndex < strlen (attr_ptr->name))
+                     textEndIndex < ((int) strlen (attr_ptr->name)))
                {
                   textEndIndex = strlen (attr_ptr->name);
                   textEndX = left + XTextWidth (canvasFontPtr,
@@ -1476,7 +1486,7 @@ void HandleButton (Button_Ev)
       attr_ptr = obj_ptr->detail.t->attr;
       if (attr_ptr != NULL && attr_ptr->inherited && textEndY == textOrigY &&
             attr_ptr->shown && attr_ptr->nameshown &&
-            textEndIndex < strlen (attr_ptr->name))
+            textEndIndex < ((int) strlen (attr_ptr->name)))
       {
          textEndIndex = strlen (attr_ptr->name);
          textEndX = left + XTextWidth (canvasFontPtr, attr_ptr->name,
@@ -1705,7 +1715,7 @@ void HandleBS (ColorIndex)
             INVALID, INVALID);
 
       s = curStr->s;
-      for (i = textCurIndex; i <= strlen (curStr->s); i++) s[i-1] = s[i];
+      for (i = textCurIndex; i <= ((int)strlen(curStr->s)); i++) s[i-1] = s[i];
          textCurIndex--;
 
       curStrW = XTextWidth (canvasFontPtr, curStr->s, strlen (curStr->s));
@@ -1816,7 +1826,7 @@ void HandleChar (Str, ColorIndex)
       JustDeleteHighlightedText ();
    }
 
-   if (textCurIndex+strlen (&(curStr->s[textCurIndex])) >= MAXSTRING)
+   if (textCurIndex+((int) strlen (&(curStr->s[textCurIndex]))) >= MAXSTRING)
    {
       sprintf (msg, "String length exceeds %1d.  Character ignored.",MAXSTRING);
       Msg (msg);
@@ -1942,7 +1952,7 @@ void HandleTAB (key_ev, ColorIndex)
    attr_ptr = obj_ptr->detail.t->attr;
    if (attr_ptr != NULL && attr_ptr->inherited && textCurY == textOrigY &&
          attr_ptr->shown && attr_ptr->nameshown &&
-         textCurIndex < strlen (attr_ptr->name))
+         textCurIndex < ((int) strlen (attr_ptr->name)))
    {  /* clicked in the name of an inherited attribute */
       textCurIndex = strlen (attr_ptr->name);
       textCurX = left + XTextWidth (canvasFontPtr, attr_ptr->name,
@@ -1981,7 +1991,7 @@ void HandleTAB (key_ev, ColorIndex)
    attr_ptr = obj_ptr->detail.t->attr;
    if (attr_ptr != NULL && attr_ptr->inherited && textEndY == textOrigY &&
          attr_ptr->shown && attr_ptr->nameshown &&
-         textEndIndex < strlen (attr_ptr->name))
+         textEndIndex < ((int) strlen (attr_ptr->name)))
    {
       textEndIndex = strlen (attr_ptr->name);
       textEndX = left + XTextWidth (canvasFontPtr,
@@ -2080,7 +2090,7 @@ void HandleUp ()
 
    attr_ptr = curTextObj->detail.t->attr;
    if (curStr == firstStr && attr_ptr != NULL && attr_ptr->inherited &&
-         attr_ptr->nameshown && textCurIndex < strlen(attr_ptr->name))
+         attr_ptr->nameshown && textCurIndex < ((int) strlen(attr_ptr->name)))
    {
       textCurIndex = strlen (attr_ptr->name);
       textCurX = left + XTextWidth (canvasFontPtr, attr_ptr->name,
@@ -2168,8 +2178,6 @@ void HandleDown ()
    ScrollTo (textCurX, textCurY);
 }
 
-#define TGIF_HEADER 0x80
- 
 static XComposeStatus	c_stat;
 
 void DrawText (input)
@@ -2177,6 +2185,7 @@ void DrawText (input)
 {
    register char	* c_ptr;
    char			s[80], * cut_buffer=NULL;
+   int			xfree_cut_buffer=FALSE;
    XKeyEvent		* key_ev;
    KeySym		key_sym;
 
@@ -2223,6 +2232,7 @@ void DrawText (input)
             {
                sprintf (msg, "Can not open '%s' for read.", pasteFromFileName);
                Dialog (msg, NULL, NULL);
+               cfree (cut_buffer);
                return;
             }
             len = 0;
@@ -2232,8 +2242,9 @@ void DrawText (input)
          }
          else
          {
-            cut_buffer = (char *) XFetchBytes (mainDisplay, &len);
+            cut_buffer = (char *) FetchCutBuffer (&len);
             if (len == 0) { Msg ("Cut buffer is empty."); return; }
+            xfree_cut_buffer = TRUE;
          }
 
          if (escPressed)
@@ -2262,6 +2273,10 @@ void DrawText (input)
          PutTextCursor ();
          MarkRulers (textCurX, textCurY);
          SetFileModified (TRUE);
+         if (xfree_cut_buffer)
+            XFree (cut_buffer);
+         else
+            cfree (cut_buffer);
          return;
       }
       else if (copyInDrawTextMode)
@@ -2383,7 +2398,8 @@ void DrawText (input)
             (key_sym==XK_Tab && (key_ev->state & (ShiftMask | Mod1Mask))) ||
             (s[0]=='\033' && (key_sym & 0xff)=='\033') ||
             (s[0]=='\177' && (key_sym & 0x7f)=='\177') ||
-            ((s[0]&0xff) && key_sym>='\040' && key_sym<='\177') ||
+            ((key_ev->state & ControlMask)==0 && (s[0]&0xff)!=0 &&
+            key_sym>='\040' && key_sym<='\177') ||
             ((s[0]&0xff) && key_sym>0xa0 && key_sym<=0xff)))
          return;
 
@@ -2641,7 +2657,7 @@ void ReadTextObj (FP, Inbuf, ObjPtr)
    *ObjPtr = NULL;
 
    s = FindChar ('(', Inbuf);
-   s = ParseStr (s, ',', color_str);
+   s = ParseStr (s, ',', color_str, sizeof(color_str));
 
    InitScan (s, ", \t\n");
 
@@ -3082,7 +3098,7 @@ void ReadTextObj (FP, Inbuf, ObjPtr)
             num_lines*textCursorH+(num_lines-1)*textVSpace, rotate);
 
    (*ObjPtr)->type = OBJ_TEXT;
-   (*ObjPtr)->color = QuickFindColorIndex (color_str, &new_alloc);
+   (*ObjPtr)->color = QuickFindColorIndex (color_str, &new_alloc, TRUE);
    (*ObjPtr)->dirty = FALSE;
    (*ObjPtr)->id = id;
    (*ObjPtr)->rotation = rotation;
