@@ -25,7 +25,7 @@
  */
 #ifndef lint
 static char RCSid[] =
-      "@(#)$Header: /amnt/maui/tangram/u/william/X11/TGIF2/RCS/tgif.c,v 2.29 1994/04/27 05:29:39 william Exp $";
+      "@(#)$Header: /export/bourbon/multimedia/william/X11/TGIF2/RCS/tgif.c,v 2.34 1994/12/15 06:54:05 william Exp $";
 #endif
 
 #include <stdio.h>
@@ -35,12 +35,14 @@ static char RCSid[] =
 #include "patchlevel.h"
 #include "types.h"
 
+#include "color.e"
 #include "file.e"
 #include "grid.e"
 #include "mainloop.e"
 #include "msg.e"
 #include "obj.e"
 #include "pattern.e"
+#include "page.e"
 #include "ps.e"
 #include "setup.e"
 #include "version.e"
@@ -186,6 +188,14 @@ int PrTgifLoad (FileName)
             fileVersion);
       return (FALSE);
    }
+   if (cmdLineHasPageNum) {
+      if (cmdLinePageNum > lastPageNum) {
+         fprintf(stderr, "%s is skipped because it only contains %1d pages!\n",
+               FileName, lastPageNum);
+         return (FALSE);
+      }
+      GotoPageNum(cmdLinePageNum);
+   }
    return (TRUE);
 }
 
@@ -212,7 +222,7 @@ void DoPrTgif (argc, argv, from_prtgif)
       if ((c_ptr = getenv ("TGIFPATH")) == NULL)
          strcpy (drawPath, TGIF_PATH);
       else
-         if (strlen (c_ptr) >= MAXSTRING)
+         if (((int) strlen (c_ptr)) >= MAXSTRING)
             /* must be an error */
             strcpy (drawPath, TGIF_PATH);
          else
@@ -250,8 +260,31 @@ void DoPrTgif (argc, argv, from_prtgif)
             whereToPrint = PS_FILE;
          else if (strcmp (*argv, "-text") == 0)
             whereToPrint = TEXT_FILE;
+         else if (strcmp (*argv, "-page") == 0)
+         {
+            argc--; argv++;
+            if (argc <= 0)
+            { fprintf (stderr, "Missing page number.\n"); Usage (TOOL_NAME); }
+            cmdLineHasPageNum = TRUE;
+            strcpy(cmdLinePageNumStr, *argv);
+            if (sscanf (cmdLinePageNumStr, "%d", &cmdLinePageNum) != 1 ||
+                  cmdLinePageNum <= 0) {
+               cmdLineHasPageNum = FALSE;
+               fprintf(stderr, "Invalid page number.\n");
+               Usage(TOOL_NAME);
+            }
+         }
          else if (strcmp (*argv, "-gray") == 0)
             useGray = TRUE;
+         else if (strcmp (*argv, "-bw") == 0)
+         {
+         }
+         else if (strcmp (*argv, "-color") == 0)
+         {
+            fprintf (stderr, "Sorry, -color is not supported.\n");
+            fflush(stderr);
+            return;
+         }
          else if (strcmp (*argv, "-adobe") == 0)
             usePsAdobeString = TRUE;
          else if (strncmp (*argv, "-adobe=", 7) == 0)
@@ -270,6 +303,8 @@ void DoPrTgif (argc, argv, from_prtgif)
          else if (strcmp (*argv, "-tgif_path") == 0)
          {
             argc--; argv++;
+            if (argc <= 0)
+            { fprintf (stderr, "Missing tgif_path.\n"); Usage (TOOL_NAME); }
             len = strlen(*argv);
             if (**argv == '"' && (*argv)[len-1] == '"')
             {
@@ -284,6 +319,8 @@ void DoPrTgif (argc, argv, from_prtgif)
          else if (strcmp (*argv, "-print_cmd") == 0)
          {
             argc--; argv++;
+            if (argc <= 0)
+            { fprintf (stderr, "Missing print_cmd.\n"); Usage (TOOL_NAME); }
             len = strlen(*argv);
             if (**argv == '"' && (*argv)[len-1] == '"')
             {
@@ -309,7 +346,7 @@ void DoPrTgif (argc, argv, from_prtgif)
       else
          break;
    }
-
+   InitPaperSize ();
    switch (argc)
    {
       case 0:
@@ -323,6 +360,7 @@ void DoPrTgif (argc, argv, from_prtgif)
                if (inbuf[--len] == '\n') inbuf[len] = '\0';
                if (PrTgifLoad (inbuf))
                {
+                  colorDump = FALSE;
                   Dump (inbuf);
                   DelAllObj ();
                }
@@ -337,11 +375,13 @@ void DoPrTgif (argc, argv, from_prtgif)
             lastFile = (argc == 1);
             if (PrTgifLoad (*argv))
             {
+               colorDump = FALSE;
                Dump (*argv);
                DelAllObj ();
             }
          }
    }
+   CleanUpPaperSize ();
 }
 
 /*
@@ -389,7 +429,7 @@ void DoTgif (argc, argv)
       if (strcmp (argv[i], "-display") == 0)
       {
          if (++i == argc)
-         { fprintf (stderr, "Missing display name.\n"); Usage ("tgif"); }
+         { fprintf (stderr, "Missing display name.\n"); Usage (TOOL_NAME); }
          displayName = argv[i];
       }
       else if (strcmp (argv[i], "-bw") == 0)
@@ -403,28 +443,28 @@ void DoTgif (argc, argv)
       else if (strcmp (argv[i], "-fg") == 0)
       {
          if (++i == argc)
-         { fprintf (stderr, "Missing foreground color.\n"); Usage ("tgif"); }
+         { fprintf (stderr, "Missing foreground color.\n"); Usage (TOOL_NAME); }
          cmdLineForeground = (char *) calloc (80, sizeof(char));
          strcpy (cmdLineForeground, argv[i]);
       }
       else if (strcmp (argv[i], "-bg") == 0)
       {
          if (++i == argc)
-         { fprintf (stderr, "Missing background color.\n"); Usage ("tgif"); }
+         { fprintf (stderr, "Missing background color.\n"); Usage (TOOL_NAME); }
          cmdLineBackground = (char *) calloc (80, sizeof(char));
          strcpy (cmdLineBackground, argv[i]);
       }
       else if (strcmp (argv[i], "-bd") == 0)
       {
          if (++i == argc)
-         { fprintf (stderr, "Missing border color.\n"); Usage ("tgif"); }
+         { fprintf (stderr, "Missing border color.\n"); Usage (TOOL_NAME); }
          cmdLineBorder = (char *) calloc (80, sizeof(char));
          strcpy (cmdLineBorder, argv[i]);
       }
       else if (strcmp (argv[i], "-geometry") == 0)
       {
          if (++i == argc)
-         { fprintf (stderr, "Missing geometry spec.\n"); Usage ("tgif"); }
+         { fprintf (stderr, "Missing geometry spec.\n"); Usage (TOOL_NAME); }
          strcat (geometrySpec, argv[i]);
          geometrySpecified = TRUE;
       }
@@ -436,7 +476,7 @@ void DoTgif (argc, argv)
       else if (*argv[i] != '-')
          strcpy (file_name, argv[i]);
       else
-         Usage ("tgif");
+         Usage (TOOL_NAME);
    }
    if (file_name[0] == '\0')
       MainLoop ("init", "", &func_strp,
