@@ -23,7 +23,7 @@
  * OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
  * PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /amnt/maui/tangram/u/william/X11/TGIF2/RCS/types.h,v 2.97 1994/05/02 06:08:33 william Exp $
+ * @(#)$Header: /export/bourbon/multimedia/william/X11/TGIF2/RCS/types.h,v 2.101 1994/11/18 05:41:24 william Exp $
  */
 
 #ifndef _TGIF_TYPES_H_
@@ -45,7 +45,7 @@ typedef struct PtRec {
 } * PtRecPtr;
 
 typedef struct ObjRec {
-   int			x, y, type, color, id, dirty;
+   int			x, y, type, color, id, dirty, hot_spot, visible;
    int			rotation; /* degrees times 64 */
    XPoint		rotated_obbox[5], rotated_bbox[5];
    short		marked, locked;
@@ -81,7 +81,7 @@ typedef struct AttrRec {
 typedef struct GroupRec {
    struct ObjRec	* first, * last;
    char			s[MAXSTRING+1];
-   int			rotate, flip;
+   int			rotate, flip, deck_index;
 } * GroupRecPtr;
 
 typedef struct PolyRec {
@@ -193,6 +193,7 @@ typedef struct StkRec {
    struct PageRec	* first_page, * last_page, * cur_page;
    int			cur_page_num, last_page_num, cols, rows;
    int			page_layout_mode, color_dump;
+   int			one_page_width, one_page_height;
 } * StkRecPtr;
 
 typedef struct ArcRec {
@@ -263,4 +264,13 @@ typedef struct MouseStatusStrRec {
    char	* l, * m, * r;
 } MouseStatusStrPtr;
 
+typedef struct VRec {
+   int vtype;
+   union {
+      int i;
+      double d;
+      char *s;
+   } val;
+} VRecPtr;
+
 #endif /*_TGIF_TYPES_H_*/
